@@ -26,6 +26,7 @@ package history
 
 import (
 	"context"
+	"fmt"
 	"sync"
 	"time"
 
@@ -239,6 +240,35 @@ func (c *clientImpl) CloseShard(
 	return response, nil
 }
 
+func (c *clientImpl) GetShard(
+	ctx context.Context,
+	request *historyservice.GetShardRequest,
+	opts ...grpc.CallOption) (*historyservice.GetShardResponse, error) {
+
+	var err error
+	var client historyservice.HistoryServiceClient
+	if request.ShardId != 0 {
+		client, err = c.getClientForShardID(request.GetShardId())
+		if err != nil {
+			return nil, err
+		}
+	}
+	var response *historyservice.GetShardResponse
+	op := func(ctx context.Context, client historyservice.HistoryServiceClient) error {
+		var err error
+		ctx, cancel := c.createContext(ctx)
+		defer cancel()
+		response, err = client.GetShard(ctx, request, opts...)
+		return err
+	}
+
+	err = c.executeWithRedirect(ctx, client, op)
+	if err != nil {
+		return nil, err
+	}
+	return response, nil
+}
+
 func (c *clientImpl) DescribeMutableState(
 	ctx context.Context,
 	request *historyservice.DescribeMutableStateRequest,
@@ -997,6 +1027,73 @@ func (c *clientImpl) RefreshWorkflowTasks(
 	return response, nil
 }
 
+func (c *clientImpl) GenerateLastHistoryReplicationTasks(
+	ctx context.Context,
+	request *historyservice.GenerateLastHistoryReplicationTasksRequest,
+	opts ...grpc.CallOption,
+) (*historyservice.GenerateLastHistoryReplicationTasksResponse, error) {
+	client, err := c.getClientForWorkflowID(request.NamespaceId, request.GetExecution().GetWorkflowId())
+	var response *historyservice.GenerateLastHistoryReplicationTasksResponse
+	op := func(ctx context.Context, client historyservice.HistoryServiceClient) error {
+		var err error
+		ctx, cancel := c.createContext(ctx)
+		defer cancel()
+		response, err = client.GenerateLastHistoryReplicationTasks(ctx, request, opts...)
+		return err
+	}
+	err = c.executeWithRedirect(ctx, client, op)
+	if err != nil {
+		return nil, err
+	}
+	return response, nil
+}
+
+func (c *clientImpl) GetReplicationStatus(
+	ctx context.Context,
+	request *historyservice.GetReplicationStatusRequest,
+	opts ...grpc.CallOption,
+) (*historyservice.GetReplicationStatusResponse, error) {
+	clients, err := c.clients.GetAllClients()
+	if err != nil {
+		return nil, err
+	}
+
+	respChan := make(chan *historyservice.GetReplicationStatusResponse, len(clients))
+	errChan := make(chan error, 1)
+	var wg sync.WaitGroup
+	wg.Add(len(clients))
+	for _, client := range clients {
+		historyClient := client.(historyservice.HistoryServiceClient)
+		go func(client historyservice.HistoryServiceClient) {
+			defer wg.Done()
+			resp, err := historyClient.GetReplicationStatus(ctx, request, opts...)
+			if err != nil {
+				select {
+				case errChan <- err:
+				default:
+				}
+			} else {
+				respChan <- resp
+			}
+		}(historyClient)
+	}
+	wg.Wait()
+	close(respChan)
+	close(errChan)
+
+	response := &historyservice.GetReplicationStatusResponse{}
+	for resp := range respChan {
+		response.Shards = append(response.Shards, resp.Shards...)
+	}
+
+	if len(errChan) > 0 {
+		err = <-errChan
+		return response, err
+	}
+
+	return response, nil
+}
+
 func (c *clientImpl) createContext(parent context.Context) (context.Context, context.CancelFunc) {
 	return context.WithTimeout(parent, c.timeout)
 }
@@ -1007,6 +1104,9 @@ func (c *clientImpl) getClientForWorkflowID(namespaceID, workflowID string) (his
 }
 
 func (c *clientImpl) getClientForShardID(shardID int32) (historyservice.HistoryServiceClient, error) {
+	if shardID <= 0 {
+		return nil, serviceerror.NewInvalidArgument(fmt.Sprintf("Invalid ShardID: %d", shardID))
+	}
 	client, err := c.clients.GetClientForKey(convert.Int32ToString(shardID))
 	if err != nil {
 		return nil, err
