@@ -29,7 +29,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
-	"io/ioutil"
+	"io"
 	"sort"
 	"strconv"
 	"strings"
@@ -43,7 +43,7 @@ import (
 	"github.com/golang/mock/gomock"
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
-	"github.com/uber-go/tally"
+	"github.com/uber-go/tally/v4"
 	historypb "go.temporal.io/api/history/v1"
 	"go.temporal.io/api/serviceerror"
 
@@ -104,7 +104,7 @@ func (s *historyArchiverSuite) SetupTest() {
 	s.Assertions = require.New(s.T())
 	s.container = &archiver.HistoryBootstrapContainer{
 		Logger:        log.NewNoopLogger(),
-		MetricsClient: metrics.NewClient(scope, metrics.HistoryArchiverScope),
+		MetricsClient: metrics.NewClient(&metrics.ClientConfig{}, scope, metrics.HistoryArchiverScope),
 	}
 
 	s.controller = gomock.NewController(s.T())
@@ -221,7 +221,7 @@ func setupFsEmulation(s3cli *mocks.MockS3API) {
 			}
 
 			return &s3.GetObjectOutput{
-				Body: ioutil.NopCloser(bytes.NewReader(fs[*input.Bucket+*input.Key])),
+				Body: io.NopCloser(bytes.NewReader(fs[*input.Bucket+*input.Key])),
 			}, nil
 		}).AnyTimes()
 }
