@@ -26,15 +26,18 @@ package config
 
 import (
 	"bytes"
+	"strings"
 	"time"
 
 	"gopkg.in/yaml.v3"
 
 	"go.temporal.io/server/common/auth"
+	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/masker"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/persistence/visibility/store/elasticsearch/client"
 )
 
 type (
@@ -47,7 +50,7 @@ type (
 		// Log is the logging config
 		Log log.Config `yaml:"log"`
 		// ClusterMetadata is the config containing all valid clusters and active cluster
-		ClusterMetadata *ClusterMetadata `yaml:"clusterMetadata"`
+		ClusterMetadata *cluster.Config `yaml:"clusterMetadata"`
 		// DCRedirectionPolicy contains the frontend datacenter redirection policy
 		DCRedirectionPolicy DCRedirectionPolicy `yaml:"dcRedirectionPolicy"`
 		// Services is a map of service name to service config items
@@ -58,7 +61,7 @@ type (
 		PublicClient PublicClient `yaml:"publicClient"`
 		// DynamicConfigClient is the config for setting up the file based dynamic config client
 		// Filepath should be relative to the root directory
-		DynamicConfigClient dynamicconfig.FileBasedClientConfig `yaml:"dynamicConfigClient"`
+		DynamicConfigClient *dynamicconfig.FileBasedClientConfig `yaml:"dynamicConfigClient"`
 		// NamespaceDefaults is the default config for every namespace
 		NamespaceDefaults NamespaceDefaults `yaml:"namespaceDefaults"`
 	}
@@ -172,6 +175,10 @@ type (
 		// Optional - A list of base64 PEM-encoded public keys of the Certificate Authorities that are used to validate the server's TLS certificate.
 		// You cannot specify both RootCAFiles and RootCAData
 		RootCAData []string `yaml:"rootCaData"`
+
+		// Optional - Use TLS even is neither client certificate nor root CAs are configured
+		// This is for non-mTLS cases when client validates serve against a set of trusted CA certificates configured in the environment
+		ForceTLS bool `yaml:"forceTLS"`
 	}
 
 	// WorkerTLS contains TLS configuration for system workers within the Temporal Cluster to connect to Temporal frontend.
@@ -222,8 +229,6 @@ type (
 		NumHistoryShards int32 `yaml:"numHistoryShards" validate:"nonzero"`
 		// DataStores contains the configuration for all datastores
 		DataStores map[string]DataStore `yaml:"datastores"`
-		// VisibilityConfig is config for visibility sampling
-		VisibilityConfig *VisibilityConfig `yaml:"-" json:"-"`
 		// TransactionSizeLimit is the largest allowed transaction size
 		TransactionSizeLimit dynamicconfig.IntPropertyFn `yaml:"-" json:"-"`
 	}
@@ -239,30 +244,13 @@ type (
 		// Custom contains the config for custom datastore implementation
 		CustomDataStoreConfig *CustomDatastoreConfig `yaml:"customDatastore"`
 		// ElasticSearch contains the config for a ElasticSearch datastore
-		ElasticSearch *Elasticsearch `yaml:"elasticsearch"`
+		Elasticsearch *client.Config `yaml:"elasticsearch"`
 	}
 
 	FaultInjection struct {
 		Rate float64 `yaml:"rate"`
 	}
 
-	// VisibilityConfig is config for visibility sampling
-	VisibilityConfig struct {
-		// EnableSampling for visibility
-		EnableSampling dynamicconfig.BoolPropertyFn `yaml:"-" json:"-"`
-		// VisibilityOpenMaxQPS max QPS for record open workflows
-		VisibilityOpenMaxQPS dynamicconfig.IntPropertyFnWithNamespaceFilter `yaml:"-" json:"-"`
-		// VisibilityClosedMaxQPS max QPS for record closed workflows
-		VisibilityClosedMaxQPS dynamicconfig.IntPropertyFnWithNamespaceFilter `yaml:"-" json:"-"`
-		// VisibilityListMaxQPS max QPS for list workflow
-		VisibilityListMaxQPS dynamicconfig.IntPropertyFnWithNamespaceFilter `yaml:"-" json:"-"`
-		// MaxQPS is overall max QPS
-		MaxQPS dynamicconfig.IntPropertyFn `yaml:"-" json:"-"`
-		// ESProcessorAckTimeout is the timeout that store will wait to get ack signal from ES processor.
-		// Should be at least ESProcessorFlushInterval+<time to process request>.
-		ESProcessorAckTimeout dynamicconfig.DurationPropertyFn `yaml:"-" json:"-"`
-	}
-
 	// Cassandra contains configuration to connect to Cassandra cluster
 	Cassandra struct {
 		// Hosts is a csv of cassandra endpoints
@@ -345,28 +333,6 @@ type (
 	// Replicator describes the configuration of replicator
 	Replicator struct{}
 
-	// ClusterMetadata contains the all cluster which participated in cross DC
-	ClusterMetadata struct {
-		EnableGlobalNamespace bool `yaml:"enableGlobalNamespace"`
-		// FailoverVersionIncrement is the increment of each cluster version when failover happens
-		FailoverVersionIncrement int64 `yaml:"failoverVersionIncrement"`
-		// MasterClusterName is the master cluster name, only the master cluster can register / update namespace
-		// all clusters can do namespace failover
-		MasterClusterName string `yaml:"masterClusterName"`
-		// CurrentClusterName is the name of the current cluster
-		CurrentClusterName string `yaml:"currentClusterName"`
-		// ClusterInformation contains all cluster names to corresponding information about that cluster
-		ClusterInformation map[string]ClusterInformation `yaml:"clusterInformation"`
-	}
-
-	// ClusterInformation contains the information about each cluster which participated in cross DC
-	ClusterInformation struct {
-		Enabled                bool  `yaml:"enabled"`
-		InitialFailoverVersion int64 `yaml:"initialFailoverVersion"`
-		// Address indicate the remote service address(Host:Port). Host can be DNS name.
-		RPCAddress string `yaml:"rpcAddress"`
-	}
-
 	// ReplicationTaskProcessorConfig is the config for replication task processor.
 	ReplicationTaskProcessorConfig struct {
 		NoTaskInitialWaitIntervalSecs int     `yaml:"noTaskInitialWaitIntervalSecs"`
@@ -519,5 +485,19 @@ func (c *Config) String() string {
 }
 
 func (r *GroupTLS) IsEnabled() bool {
-	return r.Server.KeyFile != "" || r.Server.KeyData != ""
+	return r.Server.KeyFile != "" || r.Server.KeyData != "" ||
+		len(r.Client.RootCAFiles) > 0 || len(r.Client.RootCAData) > 0 ||
+		r.Client.ForceTLS
+}
+
+func (p *JWTKeyProvider) HasSourceURIsConfigured() bool {
+	if len(p.KeySourceURIs) == 0 {
+		return false
+	}
+	for _, uri := range p.KeySourceURIs {
+		if strings.TrimSpace(uri) != "" {
+			return true
+		}
+	}
+	return false
 }
