@@ -27,36 +27,33 @@ package metrics
 import (
 	"time"
 
-	"github.com/uber-go/tally"
+	"github.com/uber-go/tally/v4"
 )
 
 type tallyScope struct {
 	scope             tally.Scope
-	rootScope         tally.Scope
+	rootScope         internalScope
 	defs              map[int]metricDefinition
 	isNamespaceTagged bool
+	perUnitBuckets    map[MetricUnit]tally.Buckets
 }
 
-func newTallyScope(
-	rootScope tally.Scope,
+func newTallyScopeInternal(
+	rootScope internalScope,
 	scope tally.Scope,
 	defs map[int]metricDefinition,
 	isNamespace bool,
-) Scope {
+	perUnitBuckets map[MetricUnit]tally.Buckets,
+) internalScope {
+	if perUnitBuckets == nil {
+		perUnitBuckets = make(map[MetricUnit]tally.Buckets)
+	}
 	return &tallyScope{
 		scope:             scope,
 		rootScope:         rootScope,
 		defs:              defs,
 		isNamespaceTagged: isNamespace,
-	}
-}
-
-// NoopScope returns a noop scope of metrics
-func NoopScope(serviceIdx ServiceIdx) Scope {
-	return &tallyScope{
-		scope:     tally.NoopScope,
-		rootScope: tally.NoopScope,
-		defs:      getMetricDefs(serviceIdx),
+		perUnitBuckets:    perUnitBuckets,
 	}
 }
 
@@ -68,10 +65,14 @@ func (m *tallyScope) AddCounter(id int, delta int64) {
 	def := m.defs[id]
 	m.scope.Counter(def.metricName.String()).Inc(delta)
 	if !def.metricRollupName.Empty() {
-		m.rootScope.Counter(def.metricRollupName.String()).Inc(delta)
+		m.rootScope.AddCounterInternal(def.metricRollupName.String(), delta)
 	}
 }
 
+func (m *tallyScope) AddCounterInternal(name string, delta int64) {
+	m.scope.Counter(name).Inc(delta)
+}
+
 func (m *tallyScope) UpdateGauge(id int, value float64) {
 	def := m.defs[id]
 	m.scope.Gauge(def.metricName.String()).Update(value)
@@ -82,66 +83,60 @@ func (m *tallyScope) UpdateGauge(id int, value float64) {
 
 func (m *tallyScope) StartTimer(id int) Stopwatch {
 	def := m.defs[id]
-	timer := m.scope.Timer(def.metricName.String())
+	timer := NewStopwatch(m.scope.Timer(def.metricName.String()))
 	switch {
 	case !def.metricRollupName.Empty():
-		return NewStopwatch(timer, m.rootScope.Timer(def.metricRollupName.String()))
+		return NewCompositeStopwatch(timer, m.rootScope.StartTimerInternal(def.metricRollupName.String()))
 	case m.isNamespaceTagged:
-		// remove next line in v1.20.0
-		timerAllDeprecated := m.scope.Tagged(map[string]string{namespace: namespaceAllValue}).Timer(def.metricName.String())
-		timerAll := m.rootScope.Timer(def.metricName.String() + totalMetricSuffix)
-		return NewStopwatch(timer, timerAll, timerAllDeprecated)
+		timerAll := m.rootScope.StartTimerInternal(def.metricName.String() + totalMetricSuffix)
+		return NewCompositeStopwatch(timer, timerAll)
 	default:
-		return NewStopwatch(timer)
+		return timer
 	}
 }
 
+func (m *tallyScope) StartTimerInternal(timer string) Stopwatch {
+	return NewStopwatch(m.scope.Timer(timer))
+}
+
 func (m *tallyScope) RecordTimer(id int, d time.Duration) {
 	def := m.defs[id]
 	m.scope.Timer(def.metricName.String()).Record(d)
 	switch {
 	case !def.metricRollupName.Empty():
-		m.rootScope.Timer(def.metricRollupName.String()).Record(d)
+		m.rootScope.RecordTimerInternal(def.metricRollupName.String(), d)
 	case m.isNamespaceTagged:
-		// remove next line in v1.20.0
-		m.scope.Tagged(map[string]string{namespace: namespaceAllValue}).Timer(def.metricName.String()).Record(d)
-		m.rootScope.Timer(def.metricName.String() + totalMetricSuffix).Record(d)
+		m.rootScope.RecordTimerInternal(def.metricName.String()+totalMetricSuffix, d)
 	}
 }
 
+func (m *tallyScope) RecordTimerInternal(name string, d time.Duration) {
+	m.scope.Timer(name).Record(d)
+}
+
 func (m *tallyScope) RecordDistribution(id int, d int) {
-	dist := time.Duration(d * distributionToTimerRatio)
 	def := m.defs[id]
-	m.scope.Timer(def.metricName.String()).Record(dist)
+	m.RecordDistributionInternal(def.metricName.String(), def.unit, d)
+
 	switch {
 	case !def.metricRollupName.Empty():
-		m.rootScope.Timer(def.metricRollupName.String()).Record(dist)
+		m.rootScope.RecordDistributionInternal(def.metricRollupName.String(), def.unit, d)
 	case m.isNamespaceTagged:
-		// remove next line in v1.20.0
-		m.scope.Tagged(map[string]string{namespace: namespaceAllValue}).Timer(def.metricName.String()).Record(dist)
-		m.rootScope.Timer(def.metricName.String() + totalMetricSuffix).Record(dist)
+		m.rootScope.RecordDistributionInternal(def.metricName.String()+totalMetricSuffix, def.unit, d)
 	}
 }
 
-// Deprecated: not used
-func (m *tallyScope) RecordHistogramDuration(id int, value time.Duration) {
-	def := m.defs[id]
-	m.scope.Histogram(def.metricName.String(), m.getBuckets(id)).RecordDuration(value)
-	if !def.metricRollupName.Empty() {
-		m.rootScope.Histogram(def.metricRollupName.String(), m.getBuckets(id)).RecordDuration(value)
-	}
-}
-
-// Deprecated: not used
-func (m *tallyScope) RecordHistogramValue(id int, value float64) {
-	def := m.defs[id]
-	m.scope.Histogram(def.metricName.String(), m.getBuckets(id)).RecordValue(value)
-	if !def.metricRollupName.Empty() {
-		m.rootScope.Histogram(def.metricRollupName.String(), m.getBuckets(id)).RecordValue(value)
-	}
+func (m *tallyScope) RecordDistributionInternal(name string, unit MetricUnit, d int) {
+	buckets := m.perUnitBuckets[unit]
+	// if buckets == nil, Histogram will fall back to default buckets
+	m.scope.Histogram(name, buckets).RecordValue(float64(d))
 }
 
 func (m *tallyScope) Tagged(tags ...Tag) Scope {
+	return m.TaggedInternal(tags...)
+}
+
+func (m *tallyScope) TaggedInternal(tags ...Tag) internalScope {
 	namespaceTagged := m.isNamespaceTagged
 	tagMap := make(map[string]string, len(tags))
 	for _, tag := range tags {
@@ -150,7 +145,7 @@ func (m *tallyScope) Tagged(tags ...Tag) Scope {
 		}
 		tagMap[tag.Key()] = tag.Value()
 	}
-	return newTallyScope(m.rootScope, m.scope.Tagged(tagMap), m.defs, namespaceTagged)
+	return newTallyScopeInternal(m.rootScope, m.scope.Tagged(tagMap), m.defs, namespaceTagged, m.perUnitBuckets)
 }
 
 // todo: Add root service definition in metrics and remove after. See use in server.go.
@@ -160,13 +155,6 @@ func (m *tallyScope) GetScope() tally.Scope {
 	return m.scope
 }
 
-func (m *tallyScope) getBuckets(id int) tally.Buckets {
-	if m.defs[id].buckets != nil {
-		return m.defs[id].buckets
-	}
-	return tally.DefaultBuckets
-}
-
 func isNamespaceTagged(tag Tag) bool {
 	return tag.Key() == namespace && tag.Value() != namespaceAllValue
 }
