@@ -27,7 +27,7 @@ package metrics
 import (
 	"time"
 
-	"github.com/uber-go/tally"
+	"github.com/uber-go/tally/v4"
 )
 
 type tallyUserScope struct {
