@@ -51,22 +51,10 @@ import (
 	"go.temporal.io/server/common/primitives/timestamp"
 )
 
-// validateRetentionDuration ensures that retention duration can't be set below a sane minimum.
-func validateRetentionDuration(retention time.Duration, isGlobalNamespace bool) error {
-	min := MinRetentionLocal
-	max := common.MaxWorkflowRetentionPeriod
-	if isGlobalNamespace {
-		min = MinRetentionGlobal
-	}
-	if retention < min || retention > max {
-		return errInvalidRetentionPeriod
-	}
-	return nil
-}
-
 type (
 	// Handler is the namespace operation handler
 	Handler interface {
+		// Deprecated.
 		DeprecateNamespace(
 			ctx context.Context,
 			deprecateRequest *workflowservice.DeprecateNamespaceRequest,
@@ -102,6 +90,7 @@ type (
 	}
 )
 
+var ErrInvalidNamespaceStateUpdate = serviceerror.NewInvalidArgument("invalid namespace state update")
 var _ Handler = (*HandlerImpl)(nil)
 
 // NewHandler create a new namespace handler
@@ -394,7 +383,8 @@ func (d *HandlerImpl) UpdateNamespace(
 	configVersion := getResponse.Namespace.ConfigVersion
 	failoverVersion := getResponse.Namespace.FailoverVersion
 	failoverNotificationVersion := getResponse.Namespace.FailoverNotificationVersion
-	isGlobalNamespace := getResponse.IsGlobalNamespace
+	isGlobalNamespace := getResponse.IsGlobalNamespace || updateRequest.PromoteNamespace
+	needsNamespacePromotion := !getResponse.IsGlobalNamespace && updateRequest.PromoteNamespace
 
 	currentHistoryArchivalState := &ArchivalState{
 		State: config.HistoryArchivalState,
@@ -454,6 +444,13 @@ func (d *HandlerImpl) UpdateNamespace(
 			// only do merging
 			info.Data = d.mergeNamespaceData(info.Data, updatedInfo.Data)
 		}
+		if updatedInfo.State != enumspb.NAMESPACE_STATE_UNSPECIFIED && info.State != updatedInfo.State {
+			configurationChanged = true
+			if err := validateStateUpdate(getResponse, updateRequest); err != nil {
+				return nil, err
+			}
+			info.State = updatedInfo.State
+		}
 	}
 	if updateRequest.Config != nil {
 		updatedConfig := updateRequest.Config
@@ -513,6 +510,14 @@ func (d *HandlerImpl) UpdateNamespace(
 			}
 			replicationConfig.Clusters = clustersNew
 		}
+		if updateReplicationConfig.State != enumspb.REPLICATION_STATE_UNSPECIFIED &&
+			updateReplicationConfig.State != replicationConfig.State {
+			if err := validateReplicationStateUpdate(getResponse, updateRequest); err != nil {
+				return nil, err
+			}
+			configurationChanged = true
+			replicationConfig.State = updateReplicationConfig.State
+		}
 
 		if updateReplicationConfig.GetActiveClusterName() != "" {
 			activeClusterChanged = true
@@ -529,6 +534,10 @@ func (d *HandlerImpl) UpdateNamespace(
 		); err != nil {
 			return nil, err
 		}
+		if !d.clusterMetadata.IsGlobalNamespaceEnabled() {
+			return nil, serviceerror.NewInvalidArgument(fmt.Sprintf("global namespace is not enabled on this "+
+				"cluster, cannot update global namespace or promote local namespace: %v", updateRequest.Namespace))
+		}
 	} else {
 		if err := d.namespaceAttrValidator.validateNamespaceReplicationConfigForLocalNamespace(
 			replicationConfig,
@@ -539,7 +548,7 @@ func (d *HandlerImpl) UpdateNamespace(
 
 	if configurationChanged && activeClusterChanged && isGlobalNamespace {
 		return nil, errCannotDoNamespaceFailoverAndUpdate
-	} else if configurationChanged || activeClusterChanged {
+	} else if configurationChanged || activeClusterChanged || needsNamespacePromotion {
 		if configurationChanged && isGlobalNamespace && !d.clusterMetadata.IsMasterCluster() {
 			return nil, errNotMasterCluster
 		}
@@ -565,6 +574,7 @@ func (d *HandlerImpl) UpdateNamespace(
 				FailoverVersion:             failoverVersion,
 				FailoverNotificationVersion: failoverNotificationVersion,
 			},
+			IsGlobalNamespace:   isGlobalNamespace,
 			NotificationVersion: notificationVersion,
 		}
 		err = d.metadataMgr.UpdateNamespace(updateReq)
@@ -599,6 +609,7 @@ func (d *HandlerImpl) UpdateNamespace(
 }
 
 // DeprecateNamespace deprecates a namespace
+// Deprecated.
 func (d *HandlerImpl) DeprecateNamespace(
 	ctx context.Context,
 	deprecateRequest *workflowservice.DeprecateNamespaceRequest,
@@ -636,6 +647,7 @@ func (d *HandlerImpl) DeprecateNamespace(
 			FailoverNotificationVersion: getResponse.Namespace.FailoverNotificationVersion,
 		},
 		NotificationVersion: notificationVersion,
+		IsGlobalNamespace:   getResponse.IsGlobalNamespace,
 	}
 	err = d.metadataMgr.UpdateNamespace(updateReq)
 	if err != nil {
@@ -780,3 +792,85 @@ func (d *HandlerImpl) validateVisibilityArchivalURI(URIString string) error {
 
 	return archiver.ValidateURI(URI)
 }
+
+// validateRetentionDuration ensures that retention duration can't be set below a sane minimum.
+func validateRetentionDuration(retention time.Duration, isGlobalNamespace bool) error {
+	min := MinRetentionLocal
+	max := common.MaxWorkflowRetentionPeriod
+	if isGlobalNamespace {
+		min = MinRetentionGlobal
+	}
+	if retention < min || retention > max {
+		return errInvalidRetentionPeriod
+	}
+	return nil
+}
+
+func validateReplicationStateUpdate(existingNamespace *persistence.GetNamespaceResponse, nsUpdateRequest *workflowservice.UpdateNamespaceRequest) error {
+	if nsUpdateRequest.ReplicationConfig == nil ||
+		nsUpdateRequest.ReplicationConfig.State == enumspb.REPLICATION_STATE_UNSPECIFIED ||
+		nsUpdateRequest.ReplicationConfig.State == existingNamespace.Namespace.ReplicationConfig.State {
+		return nil // no change
+	}
+
+	nsState := existingNamespace.Namespace.Info.State
+	if nsState != enumspb.NAMESPACE_STATE_REGISTERED {
+		return serviceerror.NewInvalidArgument(fmt.Sprintf("update ReplicationState is only supported when namespace is in %s state, current state: %s", enumspb.NAMESPACE_STATE_REGISTERED.String(), nsState.String()))
+	}
+
+	if nsUpdateRequest.ReplicationConfig.State == enumspb.REPLICATION_STATE_HANDOVER {
+		if !existingNamespace.IsGlobalNamespace {
+			return serviceerror.NewInvalidArgument(fmt.Sprintf("%s can only be set for global namespace", enumspb.REPLICATION_STATE_HANDOVER))
+		}
+		// verify namespace has more than 1 replication clusters
+		replicationClusterCount := len(existingNamespace.Namespace.ReplicationConfig.Clusters)
+		if len(nsUpdateRequest.ReplicationConfig.Clusters) > 0 {
+			replicationClusterCount = len(nsUpdateRequest.ReplicationConfig.Clusters)
+		}
+		if replicationClusterCount < 2 {
+			return serviceerror.NewInvalidArgument(fmt.Sprintf("%s require more than one replication clusters", enumspb.REPLICATION_STATE_HANDOVER))
+		}
+	}
+	return nil
+}
+
+func validateStateUpdate(existingNamespace *persistence.GetNamespaceResponse, nsUpdateRequest *workflowservice.UpdateNamespaceRequest) error {
+	if nsUpdateRequest.UpdateInfo == nil {
+		return nil // no change
+	}
+	oldState := existingNamespace.Namespace.Info.State
+	newState := nsUpdateRequest.UpdateInfo.State
+	if newState == enumspb.NAMESPACE_STATE_UNSPECIFIED || oldState == newState {
+		return nil // no change
+	}
+
+	if existingNamespace.Namespace.ReplicationConfig != nil &&
+		existingNamespace.Namespace.ReplicationConfig.State == enumspb.REPLICATION_STATE_HANDOVER {
+		return serviceerror.NewInvalidArgument("cannot update namespace state while its replication state in REPLICATION_STATE_HANDOVER")
+	}
+
+	switch oldState {
+	case enumspb.NAMESPACE_STATE_REGISTERED:
+		switch newState {
+		case enumspb.NAMESPACE_STATE_DELETED, enumspb.NAMESPACE_STATE_DEPRECATED:
+			return nil
+		default:
+			return ErrInvalidNamespaceStateUpdate
+		}
+
+	case enumspb.NAMESPACE_STATE_DEPRECATED:
+		switch newState {
+		case enumspb.NAMESPACE_STATE_DELETED:
+			return nil
+		default:
+			return ErrInvalidNamespaceStateUpdate
+		}
+
+	case enumspb.NAMESPACE_STATE_DELETED:
+		return ErrInvalidNamespaceStateUpdate
+	default:
+		return ErrInvalidNamespaceStateUpdate
+	}
+
+	return nil
+}
