@@ -25,11 +25,11 @@
 package client
 
 import (
-	"errors"
 	"fmt"
 
 	commonpb "go.temporal.io/api/common/v1"
 	"go.temporal.io/api/serviceerror"
+
 	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/persistence"
 )
@@ -50,72 +50,59 @@ type (
 
 	FaultInjectionShardStore struct {
 		baseShardStore persistence.ShardStore
-		config         *config.FaultInjection
 		ErrorGenerator ErrorGenerator
 	}
 
 	FaultInjectionTaskStore struct {
 		baseTaskStore  persistence.TaskStore
-		config         *config.FaultInjection
 		ErrorGenerator ErrorGenerator
 	}
 
 	FaultInjectionMetadataStore struct {
 		baseMetadataStore persistence.MetadataStore
-		config            *config.FaultInjection
 		ErrorGenerator    ErrorGenerator
 	}
 
 	FaultInjectionClusterMetadataStore struct {
 		baseCMStore    persistence.ClusterMetadataStore
-		config         *config.FaultInjection
 		ErrorGenerator ErrorGenerator
 	}
 
 	FaultInjectionExecutionStore struct {
 		baseExecutionStore persistence.ExecutionStore
-		config             *config.FaultInjection
 		ErrorGenerator     ErrorGenerator
 	}
 
 	FaultInjectionQueue struct {
 		baseQueue      persistence.Queue
-		config         *config.FaultInjection
 		ErrorGenerator ErrorGenerator
 	}
 )
 
 // from errors.go ConvertError
 var defaultErrors = []FaultWeight{
-	// adding random error here to verify resiliency for API bugs
 	{
 		errFactory: func(msg string) error {
-			return errors.New(fmt.Sprintf("FaultInjection poison pill: %s", msg))
+			return serviceerror.NewUnavailable(fmt.Sprintf("serviceerror.NewUnavailable: %s", msg))
 		},
 		weight: 1,
 	},
 	{
 		errFactory: func(msg string) error {
-			return serviceerror.NewInternal(msg)
+			return &persistence.TimeoutError{Msg: fmt.Sprintf("persistence.TimeoutError: %s", msg)}
 		},
 		weight: 1,
 	},
 	{
 		errFactory: func(msg string) error {
-			return &persistence.TimeoutError{Msg: msg}
-		},
-		weight: 1,
-	},
-	{
-		errFactory: func(msg string) error {
-			return serviceerror.NewResourceExhausted(msg)
+			return serviceerror.NewResourceExhausted(fmt.Sprintf("serviceerror.NewResourceExhausted: %s", msg))
 		},
 		weight: 1,
 	},
 }
 
-func newErrorGenerator(config *config.FaultInjection, errorWeights []FaultWeight) ErrorGenerator {
-	return NewDefaultErrorGenerator(config.Rate, errorWeights)
+func newErrorGenerator(rate float64, errorWeights []FaultWeight) ErrorGenerator {
+	return NewDefaultErrorGenerator(rate, errorWeights)
 }
 
 func NewFaultInjectionDatastoreFactory(
@@ -123,9 +110,8 @@ func NewFaultInjectionDatastoreFactory(
 	baseFactory DataStoreFactory,
 ) *FaultInjectionDataStoreFactory {
 	errorGenerator := newErrorGenerator(
-		config,
+		config.Rate,
 		[]FaultWeight{
-			// from sqldb see NewShardStore
 			{
 				errFactory: func(data string) error { return fmt.Errorf("FaultInjectionDataStoreFactory: %s", data) },
 				weight:     1,
@@ -159,7 +145,7 @@ func (d *FaultInjectionDataStoreFactory) NewTaskStore() (persistence.TaskStore,
 		if err != nil {
 			return nil, err
 		}
-		d.TaskStore, err = NewFaultInjectionTaskStore(d.config, baseFactory)
+		d.TaskStore, err = NewFaultInjectionTaskStore(d.ErrorGenerator.Rate(), baseFactory)
 		if err != nil {
 			return nil, err
 		}
@@ -173,7 +159,7 @@ func (d *FaultInjectionDataStoreFactory) NewShardStore() (persistence.ShardStore
 		if err != nil {
 			return nil, err
 		}
-		d.ShardStore, err = NewFaultInjectionShardStore(d.config, baseFactory)
+		d.ShardStore, err = NewFaultInjectionShardStore(d.ErrorGenerator.Rate(), baseFactory)
 		if err != nil {
 			return nil, err
 		}
@@ -187,7 +173,7 @@ func (d *FaultInjectionDataStoreFactory) NewMetadataStore() (persistence.Metadat
 		if err != nil {
 			return nil, err
 		}
-		d.MetadataStore, err = NewFaultInjectionMetadataStore(d.config, baseStore)
+		d.MetadataStore, err = NewFaultInjectionMetadataStore(d.ErrorGenerator.Rate(), baseStore)
 		if err != nil {
 			return nil, err
 		}
@@ -201,7 +187,7 @@ func (d *FaultInjectionDataStoreFactory) NewExecutionStore() (persistence.Execut
 		if err != nil {
 			return nil, err
 		}
-		d.ExecutionStore, err = NewFaultInjectionExecutionStore(d.config, baseStore)
+		d.ExecutionStore, err = NewFaultInjectionExecutionStore(d.ErrorGenerator.Rate(), baseStore)
 		if err != nil {
 			return nil, err
 		}
@@ -216,7 +202,7 @@ func (d *FaultInjectionDataStoreFactory) NewQueue(queueType persistence.QueueTyp
 		if err != nil {
 			return baseQueue, err
 		}
-		d.Queue, err = NewFaultInjectionQueue(d.config, baseQueue)
+		d.Queue, err = NewFaultInjectionQueue(d.ErrorGenerator.Rate(), baseQueue)
 		if err != nil {
 			return nil, err
 		}
@@ -231,7 +217,7 @@ func (d *FaultInjectionDataStoreFactory) NewClusterMetadataStore() (persistence.
 		if err != nil {
 			return nil, err
 		}
-		d.ClusterMDStore, err = NewFaultInjectionClusterMetadataStore(d.config, baseStore)
+		d.ClusterMDStore, err = NewFaultInjectionClusterMetadataStore(d.ErrorGenerator.Rate(), baseStore)
 		if err != nil {
 			return nil, err
 		}
@@ -240,8 +226,8 @@ func (d *FaultInjectionDataStoreFactory) NewClusterMetadataStore() (persistence.
 	return d.ClusterMDStore, nil
 }
 
-func NewFaultInjectionQueue(config *config.FaultInjection, baseQueue persistence.Queue) (*FaultInjectionQueue, error) {
-	errorGenerator := newErrorGenerator(config,
+func NewFaultInjectionQueue(rate float64, baseQueue persistence.Queue) (*FaultInjectionQueue, error) {
+	errorGenerator := newErrorGenerator(rate,
 		append(defaultErrors,
 			FaultWeight{
 				errFactory: func(msg string) error {
@@ -257,7 +243,6 @@ func NewFaultInjectionQueue(config *config.FaultInjection, baseQueue persistence
 
 	return &FaultInjectionQueue{
 		baseQueue:      baseQueue,
-		config:         config,
 		ErrorGenerator: errorGenerator,
 	}, nil
 }
@@ -360,12 +345,12 @@ func (q *FaultInjectionQueue) UpdateRate(rate float64) {
 	q.ErrorGenerator.UpdateRate(rate)
 }
 
-func NewFaultInjectionExecutionStore(config *config.FaultInjection, executionStore persistence.ExecutionStore) (
+func NewFaultInjectionExecutionStore(rate float64, executionStore persistence.ExecutionStore) (
 	*FaultInjectionExecutionStore,
 	error,
 ) {
 	errorGenerator := newErrorGenerator(
-		config,
+		rate,
 		append(
 			defaultErrors,
 			FaultWeight{
@@ -381,7 +366,6 @@ func NewFaultInjectionExecutionStore(config *config.FaultInjection, executionSto
 	)
 	return &FaultInjectionExecutionStore{
 		baseExecutionStore: executionStore,
-		config:             config,
 		ErrorGenerator:     errorGenerator,
 	}, nil
 }
@@ -462,7 +446,7 @@ func (e *FaultInjectionExecutionStore) ListConcreteExecutions(request *persisten
 	return e.baseExecutionStore.ListConcreteExecutions(request)
 }
 
-func (e *FaultInjectionExecutionStore) AddTasks(request *persistence.AddTasksRequest) error {
+func (e *FaultInjectionExecutionStore) AddTasks(request *persistence.InternalAddTasksRequest) error {
 	if err := e.ErrorGenerator.Generate(); err != nil {
 		return err
 	}
@@ -470,7 +454,7 @@ func (e *FaultInjectionExecutionStore) AddTasks(request *persistence.AddTasksReq
 }
 
 func (e *FaultInjectionExecutionStore) GetTransferTask(request *persistence.GetTransferTaskRequest) (
-	*persistence.GetTransferTaskResponse,
+	*persistence.InternalGetTransferTaskResponse,
 	error,
 ) {
 	if err := e.ErrorGenerator.Generate(); err != nil {
@@ -480,7 +464,7 @@ func (e *FaultInjectionExecutionStore) GetTransferTask(request *persistence.GetT
 }
 
 func (e *FaultInjectionExecutionStore) GetTransferTasks(request *persistence.GetTransferTasksRequest) (
-	*persistence.GetTransferTasksResponse,
+	*persistence.InternalGetTransferTasksResponse,
 	error,
 ) {
 	if err := e.ErrorGenerator.Generate(); err != nil {
@@ -504,7 +488,7 @@ func (e *FaultInjectionExecutionStore) RangeCompleteTransferTask(request *persis
 }
 
 func (e *FaultInjectionExecutionStore) GetTimerTask(request *persistence.GetTimerTaskRequest) (
-	*persistence.GetTimerTaskResponse,
+	*persistence.InternalGetTimerTaskResponse,
 	error,
 ) {
 	if err := e.ErrorGenerator.Generate(); err != nil {
@@ -513,14 +497,14 @@ func (e *FaultInjectionExecutionStore) GetTimerTask(request *persistence.GetTime
 	return e.baseExecutionStore.GetTimerTask(request)
 }
 
-func (e *FaultInjectionExecutionStore) GetTimerIndexTasks(request *persistence.GetTimerIndexTasksRequest) (
-	*persistence.GetTimerIndexTasksResponse,
+func (e *FaultInjectionExecutionStore) GetTimerTasks(request *persistence.GetTimerTasksRequest) (
+	*persistence.InternalGetTimerTasksResponse,
 	error,
 ) {
 	if err := e.ErrorGenerator.Generate(); err != nil {
 		return nil, err
 	}
-	return e.baseExecutionStore.GetTimerIndexTasks(request)
+	return e.baseExecutionStore.GetTimerTasks(request)
 }
 
 func (e *FaultInjectionExecutionStore) CompleteTimerTask(request *persistence.CompleteTimerTaskRequest) error {
@@ -538,7 +522,7 @@ func (e *FaultInjectionExecutionStore) RangeCompleteTimerTask(request *persisten
 }
 
 func (e *FaultInjectionExecutionStore) GetReplicationTask(request *persistence.GetReplicationTaskRequest) (
-	*persistence.GetReplicationTaskResponse,
+	*persistence.InternalGetReplicationTaskResponse,
 	error,
 ) {
 	if err := e.ErrorGenerator.Generate(); err != nil {
@@ -548,7 +532,7 @@ func (e *FaultInjectionExecutionStore) GetReplicationTask(request *persistence.G
 }
 
 func (e *FaultInjectionExecutionStore) GetReplicationTasks(request *persistence.GetReplicationTasksRequest) (
-	*persistence.GetReplicationTasksResponse,
+	*persistence.InternalGetReplicationTasksResponse,
 	error,
 ) {
 	if err := e.ErrorGenerator.Generate(); err != nil {
@@ -579,7 +563,7 @@ func (e *FaultInjectionExecutionStore) PutReplicationTaskToDLQ(request *persiste
 }
 
 func (e *FaultInjectionExecutionStore) GetReplicationTasksFromDLQ(request *persistence.GetReplicationTasksFromDLQRequest) (
-	*persistence.GetReplicationTasksFromDLQResponse,
+	*persistence.InternalGetReplicationTasksFromDLQResponse,
 	error,
 ) {
 	if err := e.ErrorGenerator.Generate(); err != nil {
@@ -603,7 +587,7 @@ func (e *FaultInjectionExecutionStore) RangeDeleteReplicationTaskFromDLQ(request
 }
 
 func (e *FaultInjectionExecutionStore) GetVisibilityTask(request *persistence.GetVisibilityTaskRequest) (
-	*persistence.GetVisibilityTaskResponse,
+	*persistence.InternalGetVisibilityTaskResponse,
 	error,
 ) {
 	if err := e.ErrorGenerator.Generate(); err != nil {
@@ -613,7 +597,7 @@ func (e *FaultInjectionExecutionStore) GetVisibilityTask(request *persistence.Ge
 }
 
 func (e *FaultInjectionExecutionStore) GetVisibilityTasks(request *persistence.GetVisibilityTasksRequest) (
-	*persistence.GetVisibilityTasksResponse,
+	*persistence.InternalGetVisibilityTasksResponse,
 	error,
 ) {
 	if err := e.ErrorGenerator.Generate(); err != nil {
@@ -636,6 +620,40 @@ func (e *FaultInjectionExecutionStore) RangeCompleteVisibilityTask(request *pers
 	return e.baseExecutionStore.RangeCompleteVisibilityTask(request)
 }
 
+func (e *FaultInjectionExecutionStore) GetTieredStorageTask(request *persistence.GetTieredStorageTaskRequest) (
+	*persistence.InternalGetTieredStorageTaskResponse,
+	error,
+) {
+	if err := e.ErrorGenerator.Generate(); err != nil {
+		return nil, err
+	}
+	return e.baseExecutionStore.GetTieredStorageTask(request)
+}
+
+func (e *FaultInjectionExecutionStore) GetTieredStorageTasks(request *persistence.GetTieredStorageTasksRequest) (
+	*persistence.InternalGetTieredStorageTasksResponse,
+	error,
+) {
+	if err := e.ErrorGenerator.Generate(); err != nil {
+		return nil, err
+	}
+	return e.baseExecutionStore.GetTieredStorageTasks(request)
+}
+
+func (e *FaultInjectionExecutionStore) CompleteTieredStorageTask(request *persistence.CompleteTieredStorageTaskRequest) error {
+	if err := e.ErrorGenerator.Generate(); err != nil {
+		return err
+	}
+	return e.baseExecutionStore.CompleteTieredStorageTask(request)
+}
+
+func (e *FaultInjectionExecutionStore) RangeCompleteTieredStorageTask(request *persistence.RangeCompleteTieredStorageTaskRequest) error {
+	if err := e.ErrorGenerator.Generate(); err != nil {
+		return err
+	}
+	return e.baseExecutionStore.RangeCompleteTieredStorageTask(request)
+}
+
 func (e *FaultInjectionExecutionStore) AppendHistoryNodes(request *persistence.InternalAppendHistoryNodesRequest) error {
 	if err := e.ErrorGenerator.Generate(); err != nil {
 		return err
@@ -698,14 +716,13 @@ func (e *FaultInjectionExecutionStore) UpdateRate(rate float64) {
 	e.ErrorGenerator.UpdateRate(rate)
 }
 
-func NewFaultInjectionClusterMetadataStore(config *config.FaultInjection, baseStore persistence.ClusterMetadataStore) (
+func NewFaultInjectionClusterMetadataStore(rate float64, baseStore persistence.ClusterMetadataStore) (
 	*FaultInjectionClusterMetadataStore,
 	error,
 ) {
-	errorGenerator := newErrorGenerator(config, defaultErrors)
+	errorGenerator := newErrorGenerator(rate, defaultErrors)
 	return &FaultInjectionClusterMetadataStore{
 		baseCMStore:    baseStore,
-		config:         config,
 		ErrorGenerator: errorGenerator,
 	}, nil
 }
@@ -718,26 +735,60 @@ func (c *FaultInjectionClusterMetadataStore) GetName() string {
 	return c.baseCMStore.GetName()
 }
 
-func (c *FaultInjectionClusterMetadataStore) GetClusterMetadata() (
+func (c *FaultInjectionClusterMetadataStore) ListClusterMetadata(request *persistence.InternalListClusterMetadataRequest) (
+	*persistence.InternalListClusterMetadataResponse,
+	error,
+) {
+	if err := c.ErrorGenerator.Generate(); err != nil {
+		return nil, err
+	}
+	return c.baseCMStore.ListClusterMetadata(request)
+}
+
+func (c *FaultInjectionClusterMetadataStore) GetClusterMetadataV1() (*persistence.InternalGetClusterMetadataResponse, error) {
+	if err := c.ErrorGenerator.Generate(); err != nil {
+		return nil, err
+	}
+	return c.baseCMStore.GetClusterMetadataV1()
+}
+
+func (c *FaultInjectionClusterMetadataStore) GetClusterMetadata(request *persistence.InternalGetClusterMetadataRequest) (
 	*persistence.InternalGetClusterMetadataResponse,
 	error,
 ) {
 	if err := c.ErrorGenerator.Generate(); err != nil {
 		return nil, err
 	}
-	return c.baseCMStore.GetClusterMetadata()
+	return c.baseCMStore.GetClusterMetadata(request)
 }
 
-func (c *FaultInjectionClusterMetadataStore) SaveClusterMetadata(request *persistence.InternalSaveClusterMetadataRequest) (
-	bool,
-	error,
-) {
+func (c *FaultInjectionClusterMetadataStore) SaveClusterMetadataV1(
+	request *persistence.InternalSaveClusterMetadataRequest,
+) (bool, error) {
+	if err := c.ErrorGenerator.Generate(); err != nil {
+		return false, err
+	}
+	return c.baseCMStore.SaveClusterMetadataV1(request)
+}
+
+func (c *FaultInjectionClusterMetadataStore) SaveClusterMetadata(
+	request *persistence.InternalSaveClusterMetadataRequest,
+) (bool, error) {
 	if err := c.ErrorGenerator.Generate(); err != nil {
 		return false, err
 	}
 	return c.baseCMStore.SaveClusterMetadata(request)
 }
 
+func (c *FaultInjectionClusterMetadataStore) DeleteClusterMetadata(
+	request *persistence.InternalDeleteClusterMetadataRequest,
+) error {
+	if err := c.ErrorGenerator.Generate(); err != nil {
+		return err
+	}
+	return c.baseCMStore.DeleteClusterMetadata(request)
+}
+
 func (c *FaultInjectionClusterMetadataStore) GetClusterMembers(request *persistence.GetClusterMembersRequest) (
 	*persistence.GetClusterMembersResponse,
 	error,
@@ -767,13 +818,12 @@ func (c *FaultInjectionClusterMetadataStore) UpdateRate(rate float64) {
 }
 
 func NewFaultInjectionMetadataStore(
-	config *config.FaultInjection,
+	rate float64,
 	metadataStore persistence.MetadataStore,
 ) (*FaultInjectionMetadataStore, error) {
-	errorGenerator := newErrorGenerator(config, defaultErrors)
+	errorGenerator := newErrorGenerator(rate, defaultErrors)
 	return &FaultInjectionMetadataStore{
 		baseMetadataStore: metadataStore,
-		config:            config,
 		ErrorGenerator:    errorGenerator,
 	}, nil
 }
@@ -849,14 +899,13 @@ func (m *FaultInjectionMetadataStore) UpdateRate(rate float64) {
 }
 
 func NewFaultInjectionTaskStore(
-	config *config.FaultInjection,
+	rate float64,
 	baseTaskStore persistence.TaskStore,
 ) (*FaultInjectionTaskStore, error) {
-	errorGenerator := newErrorGenerator(config, defaultErrors)
+	errorGenerator := newErrorGenerator(rate, defaultErrors)
 
 	return &FaultInjectionTaskStore{
 		baseTaskStore:  baseTaskStore,
-		config:         config,
 		ErrorGenerator: errorGenerator,
 	}, nil
 }
@@ -883,7 +932,6 @@ func (t *FaultInjectionTaskStore) GetTaskQueue(request *persistence.InternalGetT
 	if err := t.ErrorGenerator.Generate(); err != nil {
 		return nil, err
 	}
-	// todomigryz newPersistedTypeMismatchError
 	return t.baseTaskStore.GetTaskQueue(request)
 }
 
@@ -963,7 +1011,7 @@ func (t *FaultInjectionTaskStore) UpdateRate(rate float64) {
 }
 
 func NewFaultInjectionShardStore(
-	config *config.FaultInjection,
+	rate float64,
 	baseShardStore persistence.ShardStore,
 ) (*FaultInjectionShardStore, error) {
 	errorWeights := append(
@@ -978,10 +1026,9 @@ func NewFaultInjectionShardStore(
 			weight: 1,
 		},
 	)
-	errorGenerator := newErrorGenerator(config, errorWeights)
+	errorGenerator := newErrorGenerator(rate, errorWeights)
 	return &FaultInjectionShardStore{
 		baseShardStore: baseShardStore,
-		config:         config,
 		ErrorGenerator: errorGenerator,
 	}, nil
 }
@@ -998,21 +1045,14 @@ func (s *FaultInjectionShardStore) GetClusterName() string {
 	return s.baseShardStore.GetClusterName()
 }
 
-func (s *FaultInjectionShardStore) CreateShard(request *persistence.InternalCreateShardRequest) error {
-	if err := s.ErrorGenerator.Generate(); err != nil {
-		return err
-	}
-	return s.baseShardStore.CreateShard(request)
-}
-
-func (s *FaultInjectionShardStore) GetShard(request *persistence.InternalGetShardRequest) (
-	*persistence.InternalGetShardResponse,
+func (s *FaultInjectionShardStore) GetOrCreateShard(request *persistence.InternalGetOrCreateShardRequest) (
+	*persistence.InternalGetOrCreateShardResponse,
 	error,
 ) {
 	if err := s.ErrorGenerator.Generate(); err != nil {
 		return nil, err
 	}
-	return s.baseShardStore.GetShard(request)
+	return s.baseShardStore.GetOrCreateShard(request)
 }
 
 func (s *FaultInjectionShardStore) UpdateShard(request *persistence.InternalUpdateShardRequest) error {
