@@ -29,6 +29,7 @@ import (
 	enumspb "go.temporal.io/api/enums/v1"
 	historypb "go.temporal.io/api/history/v1"
 	workflowpb "go.temporal.io/api/workflow/v1"
+
 	historyspb "go.temporal.io/server/api/history/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
@@ -36,7 +37,6 @@ import (
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/persistence/serialization"
 	"go.temporal.io/server/common/persistence/versionhistory"
-	"go.temporal.io/server/common/primitives/timestamp"
 )
 
 type (
@@ -45,7 +45,6 @@ type (
 		serializer            serialization.Serializer
 		persistence           ExecutionStore
 		logger                log.Logger
-		statsComputer         statsComputer
 		pagingTokenSerializer *jsonHistoryTokenSerializer
 		transactionSizeLimit  dynamicconfig.IntPropertyFn
 	}
@@ -64,7 +63,6 @@ func NewExecutionManager(
 		serializer:            serialization.NewSerializer(),
 		persistence:           persistence,
 		logger:                logger,
-		statsComputer:         statsComputer{},
 		pagingTokenSerializer: newJSONHistoryTokenSerializer(),
 		transactionSizeLimit:  transactionSizeLimit,
 	}
@@ -75,6 +73,7 @@ func (m *executionManagerImpl) GetName() string {
 }
 
 // The below three APIs are related to serialization/deserialization
+
 func (m *executionManagerImpl) GetWorkflowExecution(
 	request *GetWorkflowExecutionRequest,
 ) (*GetWorkflowExecutionResponse, error) {
@@ -86,11 +85,16 @@ func (m *executionManagerImpl) GetWorkflowExecution(
 	if err != nil {
 		return nil, err
 	}
+	if state.ExecutionInfo.ExecutionStats == nil {
+		state.ExecutionInfo.ExecutionStats = &persistencespb.ExecutionStats{
+			HistorySize: 0,
+		}
+	}
 
 	newResponse := &GetWorkflowExecutionResponse{
 		State:             state,
 		DBRecordVersion:   response.DBRecordVersion,
-		MutableStateStats: m.statsComputer.computeMutableStateStats(state),
+		MutableStateStats: *statusOfInternalWorkflow(response.State, nil),
 	}
 	return newResponse, nil
 }
@@ -120,34 +124,18 @@ func (m *executionManagerImpl) UpdateWorkflowExecution(
 	request *UpdateWorkflowExecutionRequest,
 ) (*UpdateWorkflowExecutionResponse, error) {
 
-	var updateWorkflowNewEvents []*InternalAppendHistoryNodesRequest
-	var newWorkflowNewEvents []*InternalAppendHistoryNodesRequest
-	var updateWorkflowHistoryDiff, newWorkflowHistoryDiff int64 = 0, 0
-	for _, workflowEvents := range request.UpdateWorkflowEvents {
-		newEvents, err := m.serializeWorkflowEvents(request.ShardID, workflowEvents)
-		newEvents.ShardID = request.ShardID
-		if err != nil {
-			return nil, err
-		}
-		if newEvents != nil {
-			updateWorkflowNewEvents = append(updateWorkflowNewEvents, newEvents)
-			updateWorkflowHistoryDiff += int64(len(newEvents.Node.Events.Data))
-		}
+	updateWorkflowNewEvents, updateWorkflowHistoryDiff, err := m.serializeWorkflowEventBatches(request.ShardID, request.UpdateWorkflowEvents)
+	if err != nil {
+		return nil, err
 	}
-	request.UpdateWorkflowMutation.ExecutionInfo.ExecutionStats.HistorySize += updateWorkflowHistoryDiff
+	request.UpdateWorkflowMutation.ExecutionInfo.ExecutionStats.HistorySize += int64(updateWorkflowHistoryDiff.SizeDiff)
 
-	for _, workflowEvents := range request.NewWorkflowEvents {
-		newEvents, err := m.serializeWorkflowEvents(request.ShardID, workflowEvents)
-		if err != nil {
-			return nil, err
-		}
-		if newEvents != nil {
-			newWorkflowNewEvents = append(newWorkflowNewEvents, newEvents)
-			newWorkflowHistoryDiff += int64(len(newEvents.Node.Events.Data))
-		}
+	newWorkflowNewEvents, newWorkflowHistoryDiff, err := m.serializeWorkflowEventBatches(request.ShardID, request.NewWorkflowEvents)
+	if err != nil {
+		return nil, err
 	}
 	if request.NewWorkflowSnapshot != nil {
-		request.NewWorkflowSnapshot.ExecutionInfo.ExecutionStats.HistorySize += newWorkflowHistoryDiff
+		request.NewWorkflowSnapshot.ExecutionInfo.ExecutionStats.HistorySize += int64(newWorkflowHistoryDiff.SizeDiff)
 	}
 
 	mutation := request.UpdateWorkflowMutation
@@ -193,15 +181,43 @@ func (m *executionManagerImpl) UpdateWorkflowExecution(
 	if err := m.persistence.UpdateWorkflowExecution(newRequest); err != nil {
 		return nil, err
 	}
-	msuss := m.statsComputer.computeMutableStateUpdateStats(request)
-	msuss.UpdateWorkflowHistorySizeDiff = updateWorkflowHistoryDiff
-	msuss.NewWorkflowHistorySizeDiff = newWorkflowHistoryDiff
-	return &UpdateWorkflowExecutionResponse{MutableStateUpdateSessionStats: msuss}, nil
+	return &UpdateWorkflowExecutionResponse{
+		UpdateMutableStateStats: *statusOfInternalWorkflowMutation(
+			&newRequest.UpdateWorkflowMutation,
+			updateWorkflowHistoryDiff,
+		),
+		NewMutableStateStats: statusOfInternalWorkflowSnapshot(
+			newRequest.NewWorkflowSnapshot,
+			newWorkflowHistoryDiff,
+		),
+	}, nil
 }
 
 func (m *executionManagerImpl) ConflictResolveWorkflowExecution(
 	request *ConflictResolveWorkflowExecutionRequest,
-) error {
+) (*ConflictResolveWorkflowExecutionResponse, error) {
+
+	resetWorkflowEventsNewEvents, resetWorkflowHistoryDiff, err := m.serializeWorkflowEventBatches(request.ShardID, request.ResetWorkflowEvents)
+	if err != nil {
+		return nil, err
+	}
+	request.ResetWorkflowSnapshot.ExecutionInfo.ExecutionStats.HistorySize += int64(resetWorkflowHistoryDiff.SizeDiff)
+
+	newWorkflowEventsNewEvents, newWorkflowHistoryDiff, err := m.serializeWorkflowEventBatches(request.ShardID, request.NewWorkflowEvents)
+	if err != nil {
+		return nil, err
+	}
+	if request.NewWorkflowSnapshot != nil {
+		request.NewWorkflowSnapshot.ExecutionInfo.ExecutionStats.HistorySize += int64(newWorkflowHistoryDiff.SizeDiff)
+	}
+
+	currentWorkflowEventsNewEvents, currentWorkflowHistoryDiff, err := m.serializeWorkflowEventBatches(request.ShardID, request.CurrentWorkflowEvents)
+	if err != nil {
+		return nil, err
+	}
+	if request.CurrentWorkflowMutation != nil {
+		request.CurrentWorkflowMutation.ExecutionInfo.ExecutionStats.HistorySize += int64(currentWorkflowHistoryDiff.SizeDiff)
+	}
 
 	if err := ValidateConflictResolveWorkflowModeState(
 		request.Mode,
@@ -209,25 +225,25 @@ func (m *executionManagerImpl) ConflictResolveWorkflowExecution(
 		request.NewWorkflowSnapshot,
 		request.CurrentWorkflowMutation,
 	); err != nil {
-		return err
+		return nil, err
 	}
 
 	serializedResetWorkflowSnapshot, err := m.SerializeWorkflowSnapshot(&request.ResetWorkflowSnapshot)
 	if err != nil {
-		return err
+		return nil, err
 	}
 	var serializedCurrentWorkflowMutation *InternalWorkflowMutation
 	if request.CurrentWorkflowMutation != nil {
 		serializedCurrentWorkflowMutation, err = m.SerializeWorkflowMutation(request.CurrentWorkflowMutation)
 		if err != nil {
-			return err
+			return nil, err
 		}
 	}
 	var serializedNewWorkflowMutation *InternalWorkflowSnapshot
 	if request.NewWorkflowSnapshot != nil {
 		serializedNewWorkflowMutation, err = m.SerializeWorkflowSnapshot(request.NewWorkflowSnapshot)
 		if err != nil {
-			return err
+			return nil, err
 		}
 	}
 
@@ -237,32 +253,45 @@ func (m *executionManagerImpl) ConflictResolveWorkflowExecution(
 
 		Mode: request.Mode,
 
-		ResetWorkflowSnapshot: *serializedResetWorkflowSnapshot,
+		ResetWorkflowSnapshot:        *serializedResetWorkflowSnapshot,
+		ResetWorkflowEventsNewEvents: resetWorkflowEventsNewEvents,
 
-		NewWorkflowSnapshot: serializedNewWorkflowMutation,
+		NewWorkflowSnapshot:        serializedNewWorkflowMutation,
+		NewWorkflowEventsNewEvents: newWorkflowEventsNewEvents,
 
-		CurrentWorkflowMutation: serializedCurrentWorkflowMutation,
+		CurrentWorkflowMutation:        serializedCurrentWorkflowMutation,
+		CurrentWorkflowEventsNewEvents: currentWorkflowEventsNewEvents,
 	}
-	return m.persistence.ConflictResolveWorkflowExecution(newRequest)
+
+	err = m.persistence.ConflictResolveWorkflowExecution(newRequest)
+	if err != nil {
+		return nil, err
+	}
+	return &ConflictResolveWorkflowExecutionResponse{
+		ResetMutableStateStats: *statusOfInternalWorkflowSnapshot(
+			&newRequest.ResetWorkflowSnapshot,
+			resetWorkflowHistoryDiff,
+		),
+		NewMutableStateStats: statusOfInternalWorkflowSnapshot(
+			newRequest.NewWorkflowSnapshot,
+			newWorkflowHistoryDiff,
+		),
+		CurrentMutableStateStats: statusOfInternalWorkflowMutation(
+			newRequest.CurrentWorkflowMutation,
+			currentWorkflowHistoryDiff,
+		),
+	}, nil
 }
 
 func (m *executionManagerImpl) CreateWorkflowExecution(
 	request *CreateWorkflowExecutionRequest,
 ) (*CreateWorkflowExecutionResponse, error) {
 
-	var newWorkflowNewEvents []*InternalAppendHistoryNodesRequest
-	var newHistoryDiff int64 = 0
-	for _, workflowEvents := range request.NewWorkflowEvents {
-		newEvents, err := m.serializeWorkflowEvents(request.ShardID, workflowEvents)
-		newEvents.ShardID = request.ShardID
-		if err != nil {
-			return nil, err
-		}
-		// newEvents cannot be nil or empty for new WorkflowExecution
-		newWorkflowNewEvents = append(newWorkflowNewEvents, newEvents)
-		newHistoryDiff += int64(len(newEvents.Node.Events.Data))
+	newWorkflowNewEvents, newHistoryDiff, err := m.serializeWorkflowEventBatches(request.ShardID, request.NewWorkflowEvents)
+	if err != nil {
+		return nil, err
 	}
-	request.NewWorkflowSnapshot.ExecutionInfo.ExecutionStats.HistorySize += newHistoryDiff
+	request.NewWorkflowSnapshot.ExecutionInfo.ExecutionStats.HistorySize += int64(newHistoryDiff.SizeDiff)
 
 	snapshot := request.NewWorkflowSnapshot
 	if err := ValidateCreateWorkflowModeState(
@@ -278,7 +307,6 @@ func (m *executionManagerImpl) CreateWorkflowExecution(
 		return nil, err
 	}
 
-	snapshot.ExecutionInfo.LastUpdateTime = timestamp.TimeNowPtrUtc()
 	serializedNewWorkflowSnapshot, err := m.SerializeWorkflowSnapshot(&snapshot)
 	if err != nil {
 		return nil, err
@@ -297,7 +325,35 @@ func (m *executionManagerImpl) CreateWorkflowExecution(
 	if _, err := m.persistence.CreateWorkflowExecution(newRequest); err != nil {
 		return nil, err
 	}
-	return &CreateWorkflowExecutionResponse{HistorySize: newHistoryDiff}, nil
+	return &CreateWorkflowExecutionResponse{
+		NewMutableStateStats: *statusOfInternalWorkflowSnapshot(
+			serializedNewWorkflowSnapshot,
+			newHistoryDiff,
+		),
+	}, nil
+}
+
+func (m *executionManagerImpl) serializeWorkflowEventBatches(
+	shardID int32,
+	eventBatches []*WorkflowEvents,
+) ([]*InternalAppendHistoryNodesRequest, *HistoryStatistics, error) {
+	var historyStatistics HistoryStatistics
+	if len(eventBatches) == 0 {
+		return nil, &historyStatistics, nil
+	}
+
+	workflowNewEvents := make([]*InternalAppendHistoryNodesRequest, 0, len(eventBatches))
+	for _, workflowEvents := range eventBatches {
+		newEvents, err := m.serializeWorkflowEvents(shardID, workflowEvents)
+		if err != nil {
+			return nil, nil, err
+		}
+		newEvents.ShardID = shardID
+		workflowNewEvents = append(workflowNewEvents, newEvents)
+		historyStatistics.SizeDiff += len(newEvents.Node.Events.Data)
+		historyStatistics.CountDiff += len(workflowEvents.Events)
+	}
+	return workflowNewEvents, &historyStatistics, nil
 }
 
 func (m *executionManagerImpl) serializeWorkflowEvents(
@@ -329,6 +385,24 @@ func (m *executionManagerImpl) SerializeWorkflowMutation(
 ) (*InternalWorkflowMutation, error) {
 
 	var err error
+
+	transferTasks, err := m.serializer.SerializeTransferTasks(input.TransferTasks)
+	if err != nil {
+		return nil, err
+	}
+	timerTasks, err := m.serializer.SerializeTimerTasks(input.TimerTasks)
+	if err != nil {
+		return nil, err
+	}
+	replicationTasks, err := m.serializer.SerializeReplicationTasks(input.ReplicationTasks)
+	if err != nil {
+		return nil, err
+	}
+	visibilityTasks, err := m.serializer.SerializeVisibilityTasks(input.VisibilityTasks)
+	if err != nil {
+		return nil, err
+	}
+
 	result := &InternalWorkflowMutation{
 		NamespaceID: input.ExecutionInfo.GetNamespaceId(),
 		WorkflowID:  input.ExecutionInfo.GetWorkflowId(),
@@ -352,15 +426,14 @@ func (m *executionManagerImpl) SerializeWorkflowMutation(
 		UpsertSignalRequestedIDs: input.UpsertSignalRequestedIDs,
 		ClearBufferedEvents:      input.ClearBufferedEvents,
 
-		TransferTasks:    input.TransferTasks,
-		ReplicationTasks: input.ReplicationTasks,
-		TimerTasks:       input.TimerTasks,
-		VisibilityTasks:  input.VisibilityTasks,
+		TransferTasks:    transferTasks,
+		TimerTasks:       timerTasks,
+		ReplicationTasks: replicationTasks,
+		VisibilityTasks:  visibilityTasks,
 
 		Condition:       input.Condition,
 		DBRecordVersion: input.DBRecordVersion,
 		NextEventID:     input.NextEventID,
-		StartVersion:    input.ExecutionInfo.StartVersion,
 	}
 
 	result.ExecutionInfo, err = m.serializer.WorkflowExecutionInfoToBlob(input.ExecutionInfo, enumspb.ENCODING_TYPE_PROTO3)
@@ -431,6 +504,24 @@ func (m *executionManagerImpl) SerializeWorkflowSnapshot(
 ) (*InternalWorkflowSnapshot, error) {
 
 	var err error
+
+	transferTasks, err := m.serializer.SerializeTransferTasks(input.TransferTasks)
+	if err != nil {
+		return nil, err
+	}
+	timerTasks, err := m.serializer.SerializeTimerTasks(input.TimerTasks)
+	if err != nil {
+		return nil, err
+	}
+	replicationTasks, err := m.serializer.SerializeReplicationTasks(input.ReplicationTasks)
+	if err != nil {
+		return nil, err
+	}
+	visibilityTasks, err := m.serializer.SerializeVisibilityTasks(input.VisibilityTasks)
+	if err != nil {
+		return nil, err
+	}
+
 	result := &InternalWorkflowSnapshot{
 		NamespaceID: input.ExecutionInfo.GetNamespaceId(),
 		WorkflowID:  input.ExecutionInfo.GetWorkflowId(),
@@ -443,17 +534,16 @@ func (m *executionManagerImpl) SerializeWorkflowSnapshot(
 		SignalInfos:         make(map[int64]*commonpb.DataBlob),
 
 		ExecutionState:     input.ExecutionState,
-		SignalRequestedIDs: input.SignalRequestedIDs,
-		TransferTasks:      input.TransferTasks,
-		ReplicationTasks:   input.ReplicationTasks,
-		TimerTasks:         input.TimerTasks,
-		VisibilityTasks:    input.VisibilityTasks,
+		SignalRequestedIDs: make(map[string]struct{}),
+
+		TransferTasks:    transferTasks,
+		TimerTasks:       timerTasks,
+		ReplicationTasks: replicationTasks,
+		VisibilityTasks:  visibilityTasks,
 
 		Condition:       input.Condition,
 		DBRecordVersion: input.DBRecordVersion,
 		NextEventID:     input.NextEventID,
-
-		StartVersion: input.ExecutionInfo.StartVersion,
 	}
 
 	result.ExecutionInfo, err = m.serializer.WorkflowExecutionInfoToBlob(input.ExecutionInfo, enumspb.ENCODING_TYPE_PROTO3)
@@ -504,6 +594,10 @@ func (m *executionManagerImpl) SerializeWorkflowSnapshot(
 		}
 		result.SignalInfos[key] = blob
 	}
+	for key := range input.SignalRequestedIDs {
+		result.SignalRequestedIDs[key] = struct{}{}
+	}
+
 	result.Checksum, err = m.serializer.ChecksumToBlob(input.Checksum, enumspb.ENCODING_TYPE_PROTO3)
 	if err != nil {
 		return nil, err
@@ -533,11 +627,10 @@ func (m *executionManagerImpl) GetCurrentExecution(
 	}
 
 	return &GetCurrentExecutionResponse{
-		RunID:            internalResp.RunID,
-		LastWriteVersion: internalResp.LastWriteVersion,
-		StartRequestID:   internalResp.ExecutionState.CreateRequestId,
-		State:            internalResp.ExecutionState.State,
-		Status:           internalResp.ExecutionState.Status,
+		RunID:          internalResp.RunID,
+		StartRequestID: internalResp.ExecutionState.CreateRequestId,
+		State:          internalResp.ExecutionState.State,
+		Status:         internalResp.ExecutionState.Status,
 	}, nil
 }
 
@@ -563,22 +656,69 @@ func (m *executionManagerImpl) ListConcreteExecutions(
 }
 
 func (m *executionManagerImpl) AddTasks(
-	request *AddTasksRequest,
+	input *AddTasksRequest,
 ) error {
-	return m.persistence.AddTasks(request)
+
+	transferTasks, err := m.serializer.SerializeTransferTasks(input.TransferTasks)
+	if err != nil {
+		return err
+	}
+	timerTasks, err := m.serializer.SerializeTimerTasks(input.TimerTasks)
+	if err != nil {
+		return err
+	}
+	replicationTasks, err := m.serializer.SerializeReplicationTasks(input.ReplicationTasks)
+	if err != nil {
+		return err
+	}
+	visibilityTasks, err := m.serializer.SerializeVisibilityTasks(input.VisibilityTasks)
+	if err != nil {
+		return err
+	}
+
+	return m.persistence.AddTasks(&InternalAddTasksRequest{
+		ShardID: input.ShardID,
+		RangeID: input.RangeID,
+
+		NamespaceID: input.NamespaceID,
+		WorkflowID:  input.WorkflowID,
+		RunID:       input.RunID,
+
+		TransferTasks:    transferTasks,
+		TimerTasks:       timerTasks,
+		ReplicationTasks: replicationTasks,
+		VisibilityTasks:  visibilityTasks,
+	})
 }
 
 // Transfer task related methods
+
 func (m *executionManagerImpl) GetTransferTask(
 	request *GetTransferTaskRequest,
 ) (*GetTransferTaskResponse, error) {
-	return m.persistence.GetTransferTask(request)
+	resp, err := m.persistence.GetTransferTask(request)
+	if err != nil {
+		return nil, err
+	}
+	tasks, err := m.serializer.DeserializeTransferTasks([]commonpb.DataBlob{resp.Task})
+	if err != nil {
+		return nil, err
+	}
+	return &GetTransferTaskResponse{Task: tasks[0]}, nil
 }
 
 func (m *executionManagerImpl) GetTransferTasks(
 	request *GetTransferTasksRequest,
 ) (*GetTransferTasksResponse, error) {
-	return m.persistence.GetTransferTasks(request)
+	resp, err := m.persistence.GetTransferTasks(request)
+	if err != nil {
+		return nil, err
+	}
+	tasks, err := m.serializer.DeserializeTransferTasks(resp.Tasks)
+	if err != nil {
+		return nil, err
+	}
+	return &GetTransferTasksResponse{Tasks: tasks, NextPageToken: resp.NextPageToken}, nil
 }
 
 func (m *executionManagerImpl) CompleteTransferTask(
@@ -593,42 +733,76 @@ func (m *executionManagerImpl) RangeCompleteTransferTask(
 	return m.persistence.RangeCompleteTransferTask(request)
 }
 
-// Visibility task related methods
-func (m *executionManagerImpl) GetVisibilityTask(
-	request *GetVisibilityTaskRequest,
-) (*GetVisibilityTaskResponse, error) {
-	return m.persistence.GetVisibilityTask(request)
+// Timer related methods.
+
+func (m *executionManagerImpl) GetTimerTask(
+	request *GetTimerTaskRequest,
+) (*GetTimerTaskResponse, error) {
+	resp, err := m.persistence.GetTimerTask(request)
+	if err != nil {
+		return nil, err
+	}
+	tasks, err := m.serializer.DeserializeTimerTasks([]commonpb.DataBlob{resp.Task})
+	if err != nil {
+		return nil, err
+	}
+	return &GetTimerTaskResponse{Task: tasks[0]}, nil
 }
 
-func (m *executionManagerImpl) GetVisibilityTasks(
-	request *GetVisibilityTasksRequest,
-) (*GetVisibilityTasksResponse, error) {
-	return m.persistence.GetVisibilityTasks(request)
+func (m *executionManagerImpl) GetTimerTasks(
+	request *GetTimerTasksRequest,
+) (*GetTimerTasksResponse, error) {
+	resp, err := m.persistence.GetTimerTasks(request)
+	if err != nil {
+		return nil, err
+	}
+	tasks, err := m.serializer.DeserializeTimerTasks(resp.Tasks)
+	if err != nil {
+		return nil, err
+	}
+	return &GetTimerTasksResponse{Tasks: tasks, NextPageToken: resp.NextPageToken}, nil
 }
 
-func (m *executionManagerImpl) CompleteVisibilityTask(
-	request *CompleteVisibilityTaskRequest,
+func (m *executionManagerImpl) CompleteTimerTask(
+	request *CompleteTimerTaskRequest,
 ) error {
-	return m.persistence.CompleteVisibilityTask(request)
+	return m.persistence.CompleteTimerTask(request)
 }
 
-func (m *executionManagerImpl) RangeCompleteVisibilityTask(
-	request *RangeCompleteVisibilityTaskRequest,
+func (m *executionManagerImpl) RangeCompleteTimerTask(
+	request *RangeCompleteTimerTaskRequest,
 ) error {
-	return m.persistence.RangeCompleteVisibilityTask(request)
+	return m.persistence.RangeCompleteTimerTask(request)
 }
 
 // Replication task related methods
+
 func (m *executionManagerImpl) GetReplicationTask(
 	request *GetReplicationTaskRequest,
 ) (*GetReplicationTaskResponse, error) {
-	return m.persistence.GetReplicationTask(request)
+	resp, err := m.persistence.GetReplicationTask(request)
+	if err != nil {
+		return nil, err
+	}
+	tasks, err := m.serializer.DeserializeReplicationTasks([]commonpb.DataBlob{resp.Task})
+	if err != nil {
+		return nil, err
+	}
+	return &GetReplicationTaskResponse{Task: tasks[0]}, nil
 }
 
 func (m *executionManagerImpl) GetReplicationTasks(
 	request *GetReplicationTasksRequest,
 ) (*GetReplicationTasksResponse, error) {
-	return m.persistence.GetReplicationTasks(request)
+	resp, err := m.persistence.GetReplicationTasks(request)
+	if err != nil {
+		return nil, err
+	}
+	tasks, err := m.serializer.DeserializeReplicationTasks(resp.Tasks)
+	if err != nil {
+		return nil, err
+	}
+	return &GetReplicationTasksResponse{Tasks: tasks, NextPageToken: resp.NextPageToken}, nil
 }
 
 func (m *executionManagerImpl) CompleteReplicationTask(
@@ -652,7 +826,15 @@ func (m *executionManagerImpl) PutReplicationTaskToDLQ(
 func (m *executionManagerImpl) GetReplicationTasksFromDLQ(
 	request *GetReplicationTasksFromDLQRequest,
 ) (*GetReplicationTasksFromDLQResponse, error) {
-	return m.persistence.GetReplicationTasksFromDLQ(request)
+	resp, err := m.persistence.GetReplicationTasksFromDLQ(request)
+	if err != nil {
+		return nil, err
+	}
+	tasks, err := m.serializer.DeserializeReplicationTasks(resp.Tasks)
+	if err != nil {
+		return nil, err
+	}
+	return &GetReplicationTasksFromDLQResponse{Tasks: tasks, NextPageToken: resp.NextPageToken}, nil
 }
 
 func (m *executionManagerImpl) DeleteReplicationTaskFromDLQ(
@@ -667,29 +849,88 @@ func (m *executionManagerImpl) RangeDeleteReplicationTaskFromDLQ(
 	return m.persistence.RangeDeleteReplicationTaskFromDLQ(request)
 }
 
-// Timer related methods.
-func (m *executionManagerImpl) GetTimerTask(
-	request *GetTimerTaskRequest,
-) (*GetTimerTaskResponse, error) {
-	return m.persistence.GetTimerTask(request)
+// Visibility task related methods
+
+func (m *executionManagerImpl) GetVisibilityTask(
+	request *GetVisibilityTaskRequest,
+) (*GetVisibilityTaskResponse, error) {
+	resp, err := m.persistence.GetVisibilityTask(request)
+	if err != nil {
+		return nil, err
+	}
+	tasks, err := m.serializer.DeserializeVisibilityTasks([]commonpb.DataBlob{resp.Task})
+	if err != nil {
+		return nil, err
+	}
+	return &GetVisibilityTaskResponse{Task: tasks[0]}, nil
 }
 
-func (m *executionManagerImpl) GetTimerIndexTasks(
-	request *GetTimerIndexTasksRequest,
-) (*GetTimerIndexTasksResponse, error) {
-	return m.persistence.GetTimerIndexTasks(request)
+func (m *executionManagerImpl) GetVisibilityTasks(
+	request *GetVisibilityTasksRequest,
+) (*GetVisibilityTasksResponse, error) {
+	resp, err := m.persistence.GetVisibilityTasks(request)
+	if err != nil {
+		return nil, err
+	}
+	tasks, err := m.serializer.DeserializeVisibilityTasks(resp.Tasks)
+	if err != nil {
+		return nil, err
+	}
+	return &GetVisibilityTasksResponse{Tasks: tasks, NextPageToken: resp.NextPageToken}, nil
 }
 
-func (m *executionManagerImpl) CompleteTimerTask(
-	request *CompleteTimerTaskRequest,
+func (m *executionManagerImpl) CompleteVisibilityTask(
+	request *CompleteVisibilityTaskRequest,
 ) error {
-	return m.persistence.CompleteTimerTask(request)
+	return m.persistence.CompleteVisibilityTask(request)
 }
 
-func (m *executionManagerImpl) RangeCompleteTimerTask(
-	request *RangeCompleteTimerTaskRequest,
+func (m *executionManagerImpl) RangeCompleteVisibilityTask(
+	request *RangeCompleteVisibilityTaskRequest,
 ) error {
-	return m.persistence.RangeCompleteTimerTask(request)
+	return m.persistence.RangeCompleteVisibilityTask(request)
+}
+
+// TieredStorage task related methods
+
+func (m *executionManagerImpl) GetTieredStorageTask(
+	request *GetTieredStorageTaskRequest,
+) (*GetTieredStorageTaskResponse, error) {
+	resp, err := m.persistence.GetTieredStorageTask(request)
+	if err != nil {
+		return nil, err
+	}
+	tasks, err := m.serializer.DeserializeTieredStorageTasks([]commonpb.DataBlob{resp.Task})
+	if err != nil {
+		return nil, err
+	}
+	return &GetTieredStorageTaskResponse{Task: tasks[0]}, nil
+}
+
+func (m *executionManagerImpl) GetTieredStorageTasks(
+	request *GetTieredStorageTasksRequest,
+) (*GetTieredStorageTasksResponse, error) {
+	resp, err := m.persistence.GetTieredStorageTasks(request)
+	if err != nil {
+		return nil, err
+	}
+	tasks, err := m.serializer.DeserializeTieredStorageTasks(resp.Tasks)
+	if err != nil {
+		return nil, err
+	}
+	return &GetTieredStorageTasksResponse{Tasks: tasks, NextPageToken: resp.NextPageToken}, nil
+}
+
+func (m *executionManagerImpl) CompleteTieredStorageTask(
+	request *CompleteTieredStorageTaskRequest,
+) error {
+	return m.persistence.CompleteTieredStorageTask(request)
+}
+
+func (m *executionManagerImpl) RangeCompleteTieredStorageTask(
+	request *RangeCompleteTieredStorageTaskRequest,
+) error {
+	return m.persistence.RangeCompleteTieredStorageTask(request)
 }
 
 func (m *executionManagerImpl) Close() {
