@@ -25,8 +25,6 @@
 package persistencetests
 
 import (
-	"log"
-	"os"
 	"runtime/debug"
 	"testing"
 	"time"
@@ -40,9 +38,11 @@ import (
 	historyspb "go.temporal.io/server/api/history/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
+	"go.temporal.io/server/common/definition"
 	p "go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/persistence/versionhistory"
 	"go.temporal.io/server/common/primitives/timestamp"
+	"go.temporal.io/server/service/history/tasks"
 )
 
 type (
@@ -66,9 +66,6 @@ func failOnPanic(t *testing.T) {
 // SetupSuite implementation
 func (s *ExecutionManagerSuiteForEventsV2) SetupSuite() {
 	defer failOnPanic(s.T())
-	if testing.Verbose() {
-		log.SetOutput(os.Stdout)
-	}
 }
 
 // TearDownSuite implementation
@@ -110,6 +107,11 @@ func (s *ExecutionManagerSuiteForEventsV2) TestWorkflowCreation() {
 		WorkflowId: "test-eventsv2-workflow",
 		RunId:      "aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa",
 	}
+	workflowKey := definition.NewWorkflowKey(
+		namespaceID,
+		workflowExecution.WorkflowId,
+		workflowExecution.RunId,
+	)
 
 	csum := s.newRandomChecksum()
 
@@ -136,10 +138,10 @@ func (s *ExecutionManagerSuiteForEventsV2) TestWorkflowCreation() {
 				State:           enumsspb.WORKFLOW_EXECUTION_STATE_RUNNING,
 				Status:          enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING,
 			},
-			TransferTasks: []p.Task{
-				&p.WorkflowTask{
+			TransferTasks: []tasks.Task{
+				&tasks.WorkflowTask{
+					WorkflowKey:         workflowKey,
 					TaskID:              s.GetNextSequenceNumber(),
-					NamespaceID:         namespaceID,
 					TaskQueue:           "taskQueue",
 					ScheduleID:          2,
 					VisibilityTimestamp: time.Now().UTC(),
@@ -237,10 +239,14 @@ func (s *ExecutionManagerSuiteForEventsV2) TestWorkflowCreationWithVersionHistor
 				Status:          enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING,
 			},
 			NextEventID: common.EmptyEventID,
-			TransferTasks: []p.Task{
-				&p.WorkflowTask{
+			TransferTasks: []tasks.Task{
+				&tasks.WorkflowTask{
+					WorkflowKey: definition.NewWorkflowKey(
+						namespaceID,
+						workflowExecution.WorkflowId,
+						workflowExecution.RunId,
+					),
 					TaskID:              s.GetNextSequenceNumber(),
-					NamespaceID:         namespaceID,
 					TaskQueue:           "taskQueue",
 					ScheduleID:          2,
 					VisibilityTimestamp: time.Now().UTC(),
@@ -301,6 +307,11 @@ func (s *ExecutionManagerSuiteForEventsV2) TestContinueAsNew() {
 		WorkflowId: "continue-as-new-workflow-test",
 		RunId:      "551c88d2-d9e6-404f-8131-9eec14f36643",
 	}
+	workflowKey := definition.NewWorkflowKey(
+		namespaceID,
+		workflowExecution.WorkflowId,
+		workflowExecution.RunId,
+	)
 
 	_, err0 := s.CreateWorkflowExecution(namespaceID, workflowExecution, "queue1", "wType", timestamp.DurationFromSeconds(20), timestamp.DurationFromSeconds(13), 3, 0, 2, nil)
 	s.NoError(err0)
@@ -319,9 +330,9 @@ func (s *ExecutionManagerSuiteForEventsV2) TestContinueAsNew() {
 		RunId:      "64c7e15a-3fd7-4182-9c6f-6f25a4fa2614",
 	}
 
-	newworkflowTask := &p.WorkflowTask{
+	newworkflowTask := &tasks.WorkflowTask{
+		WorkflowKey: workflowKey,
 		TaskID:      s.GetNextSequenceNumber(),
-		NamespaceID: updatedInfo.NamespaceId,
 		TaskQueue:   updatedInfo.TaskQueue,
 		ScheduleID:  int64(2),
 	}
@@ -332,7 +343,7 @@ func (s *ExecutionManagerSuiteForEventsV2) TestContinueAsNew() {
 			ExecutionInfo:       updatedInfo,
 			ExecutionState:      updatedState,
 			NextEventID:         int64(5),
-			TransferTasks:       []p.Task{newworkflowTask},
+			TransferTasks:       []tasks.Task{newworkflowTask},
 			TimerTasks:          nil,
 			Condition:           state0.NextEventId,
 			UpsertActivityInfos: nil,
