@@ -35,13 +35,11 @@ import (
 	"github.com/gogo/protobuf/types"
 	"github.com/pborman/uuid"
 	"github.com/stretchr/testify/suite"
-	"github.com/uber-go/tally"
+	"github.com/uber-go/tally/v4"
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	historypb "go.temporal.io/api/history/v1"
 	workflowpb "go.temporal.io/api/workflow/v1"
-	"go.temporal.io/server/common/persistence/visibility"
-	visibilityclient "go.temporal.io/server/common/persistence/visibility/client"
 
 	enumsspb "go.temporal.io/server/api/enums/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
@@ -52,6 +50,7 @@ import (
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/convert"
+	"go.temporal.io/server/common/definition"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
@@ -65,6 +64,7 @@ import (
 	"go.temporal.io/server/common/resolver"
 	"go.temporal.io/server/common/searchattribute"
 	"go.temporal.io/server/environment"
+	"go.temporal.io/server/service/history/tasks"
 )
 
 type (
@@ -80,10 +80,10 @@ type (
 		DBUsername      string
 		DBPassword      string
 		DBHost          string
-		DBPort          int    `yaml:"-"`
-		StoreType       string `yaml:"-"`
-		SchemaDir       string `yaml:"-"`
-		FaultInjection  *config.FaultInjection
+		DBPort          int                    `yaml:"-"`
+		StoreType       string                 `yaml:"-"`
+		SchemaDir       string                 `yaml:"-"`
+		FaultInjection  *config.FaultInjection `yaml:"faultinjection"`
 	}
 
 	// TestBase wraps the base setup needed to create workflows over persistence layer.
@@ -91,12 +91,12 @@ type (
 		suite.Suite
 		ShardMgr                  persistence.ShardManager
 		AbstractDataStoreFactory  client.AbstractDataStoreFactory
+		FaultInjection            *client.FaultInjectionDataStoreFactory
 		Factory                   client.Factory
 		ExecutionManager          persistence.ExecutionManager
 		TaskMgr                   persistence.TaskManager
 		ClusterMetadataManager    persistence.ClusterMetadataManager
 		MetadataManager           persistence.MetadataManager
-		VisibilityMgr             visibility.VisibilityManager
 		NamespaceReplicationQueue persistence.NamespaceReplicationQueue
 		ShardInfo                 *persistencespb.ShardInfo
 		TaskIDGenerator           TransferTaskIDGenerator
@@ -105,8 +105,7 @@ type (
 		ReadLevel                 int64
 		ReplicationReadLevel      int64
 		DefaultTestCluster        PersistenceTestCluster
-		VisibilityTestCluster     PersistenceTestCluster
-		logger                    log.Logger
+		Logger                    log.Logger
 	}
 
 	// PersistenceTestCluster exposes management operations on a database
@@ -181,26 +180,13 @@ func NewTestBase(options *TestBaseOptions) TestBase {
 
 func NewTestBaseForCluster(testCluster PersistenceTestCluster, logger log.Logger) TestBase {
 	return TestBase{
-		DefaultTestCluster:    testCluster,
-		VisibilityTestCluster: testCluster,
-		logger:                logger,
+		DefaultTestCluster: testCluster,
+		Logger:             logger,
 	}
 }
 
-// Config returns the persistence configuration for this test
-func (s *TestBase) Config() config.Persistence {
-	cfg := s.DefaultTestCluster.Config()
-	if s.DefaultTestCluster == s.VisibilityTestCluster {
-		return cfg
-	}
-	vCfg := s.VisibilityTestCluster.Config()
-	cfg.VisibilityStore = "visibility_ " + vCfg.VisibilityStore
-	cfg.DataStores[cfg.VisibilityStore] = vCfg.DataStores[vCfg.VisibilityStore]
-	return cfg
-}
-
 // Setup sets up the test base, must be called as part of SetupSuite
-func (s *TestBase) Setup(clusterMetadataConfig *config.ClusterMetadata) {
+func (s *TestBase) Setup(clusterMetadataConfig *cluster.Config) {
 	var err error
 	shardID := int32(10)
 	if clusterMetadataConfig == nil {
@@ -210,14 +196,11 @@ func (s *TestBase) Setup(clusterMetadataConfig *config.ClusterMetadata) {
 	clusterName := clusterMetadataConfig.CurrentClusterName
 
 	s.DefaultTestCluster.SetupTestDatabase()
-	if s.VisibilityTestCluster != s.DefaultTestCluster {
-		s.VisibilityTestCluster.SetupTestDatabase()
-	}
 
 	cfg := s.DefaultTestCluster.Config()
 	scope := tally.NewTestScope(common.HistoryServiceName, make(map[string]string))
-	metricsClient := metrics.NewClient(scope, metrics.GetMetricsServiceIdx(common.HistoryServiceName, s.logger))
-	factory := client.NewFactory(&cfg, resolver.NewNoopResolver(), nil, s.AbstractDataStoreFactory, clusterName, metricsClient, s.logger)
+	metricsClient := metrics.NewClient(&metrics.ClientConfig{}, scope, metrics.GetMetricsServiceIdx(common.HistoryServiceName, s.Logger))
+	factory := client.NewFactoryImpl(&cfg, resolver.NewNoopResolver(), nil, s.AbstractDataStoreFactory, clusterName, metricsClient, s.Logger)
 
 	s.TaskMgr, err = factory.NewTaskManager()
 	s.fatalOnError("NewTaskManager", err)
@@ -225,8 +208,8 @@ func (s *TestBase) Setup(clusterMetadataConfig *config.ClusterMetadata) {
 	s.ClusterMetadataManager, err = factory.NewClusterMetadataManager()
 	s.fatalOnError("NewClusterMetadataManager", err)
 
-	s.ClusterMetadata = cluster.NewTestClusterMetadata(clusterMetadataConfig)
-	s.SearchAttributesManager = persistence.NewSearchAttributesManager(clock.NewRealTimeSource(), s.ClusterMetadataManager)
+	s.ClusterMetadata = cluster.NewMetadataFromConfig(clusterMetadataConfig, s.ClusterMetadataManager, s.Logger)
+	s.SearchAttributesManager = searchattribute.NewManager(clock.NewRealTimeSource(), s.ClusterMetadataManager)
 
 	s.MetadataManager, err = factory.NewMetadataManager()
 	s.fatalOnError("NewMetadataManager", err)
@@ -238,19 +221,7 @@ func (s *TestBase) Setup(clusterMetadataConfig *config.ClusterMetadata) {
 	s.fatalOnError("NewExecutionManager", err)
 
 	s.Factory = factory
-
-	// SQL currently doesn't have support for visibility manager
-	vCfg := s.VisibilityTestCluster.Config()
-	s.VisibilityMgr, err = visibilityclient.NewVisibilityManager(
-		vCfg,
-		nil,
-		nil,
-		resolver.NewNoopResolver(),
-		s.logger,
-	)
-	if err != nil {
-		s.fatalOnError("NewVisibilityManager", err)
-	}
+	s.FaultInjection = factory.FaultInjection()
 
 	s.ReadLevel = 0
 	s.ReplicationReadLevel = 0
@@ -259,13 +230,16 @@ func (s *TestBase) Setup(clusterMetadataConfig *config.ClusterMetadata) {
 		RangeId:                 0,
 		TransferAckLevel:        0,
 		ReplicationAckLevel:     0,
-		TimerAckLevelTime:       &time.Time{},
+		TimerAckLevelTime:       timestamp.TimePtr(time.Unix(0, 0).UTC()),
 		ClusterTimerAckLevel:    map[string]*time.Time{},
 		ClusterTransferAckLevel: map[string]int64{clusterName: 0},
 	}
 
 	s.TaskIDGenerator = &TestTransferTaskIDGenerator{}
-	err = s.ShardMgr.CreateShard(&persistence.CreateShardRequest{ShardInfo: s.ShardInfo})
+	_, err = s.ShardMgr.GetOrCreateShard(&persistence.GetOrCreateShardRequest{
+		ShardID:          shardID,
+		InitialShardInfo: s.ShardInfo,
+	})
 	s.fatalOnError("CreateShard", err)
 
 	queue, err := factory.NewNamespaceReplicationQueue()
@@ -275,34 +249,25 @@ func (s *TestBase) Setup(clusterMetadataConfig *config.ClusterMetadata) {
 
 func (s *TestBase) fatalOnError(msg string, err error) {
 	if err != nil {
-		s.logger.Fatal(msg, tag.Error(err))
+		s.Logger.Fatal(msg, tag.Error(err))
 	}
 }
 
-// CreateShard is a utility method to create the shard using persistence layer
-func (s *TestBase) CreateShard(shardID int32, owner string, rangeID int64) error {
+// GetOrCreateShard is a utility method to get/create the shard using persistence layer
+func (s *TestBase) GetOrCreateShard(shardID int32, owner string, rangeID int64) (*persistencespb.ShardInfo, error) {
 	info := &persistencespb.ShardInfo{
 		ShardId: shardID,
 		Owner:   owner,
 		RangeId: rangeID,
 	}
-
-	return s.ShardMgr.CreateShard(&persistence.CreateShardRequest{
-		ShardInfo: info,
+	resp, err := s.ShardMgr.GetOrCreateShard(&persistence.GetOrCreateShardRequest{
+		ShardID:          shardID,
+		InitialShardInfo: info,
 	})
-}
-
-// GetShard is a utility method to get the shard using persistence layer
-func (s *TestBase) GetShard(shardID int32) (*persistencespb.ShardInfo, error) {
-	response, err := s.ShardMgr.GetShard(&persistence.GetShardRequest{
-		ShardID: shardID,
-	})
-
 	if err != nil {
 		return nil, err
 	}
-
-	return response.ShardInfo, nil
+	return resp.ShardInfo, nil
 }
 
 // UpdateShard is a utility method to update the shard using persistence layer
@@ -316,7 +281,7 @@ func (s *TestBase) UpdateShard(updatedInfo *persistencespb.ShardInfo, previousRa
 // CreateWorkflowExecutionWithBranchToken test util function
 func (s *TestBase) CreateWorkflowExecutionWithBranchToken(namespaceID string, workflowExecution commonpb.WorkflowExecution, taskQueue,
 	wType string, wTimeout *time.Duration, workflowTaskTimeout *time.Duration, nextEventID int64, lastProcessedEventID int64,
-	workflowTaskScheduleID int64, branchToken []byte, timerTasks []persistence.Task) (*persistence.CreateWorkflowExecutionResponse, error) {
+	workflowTaskScheduleID int64, branchToken []byte, timerTasks []tasks.Task) (*persistence.CreateWorkflowExecutionResponse, error) {
 	response, err := s.ExecutionManager.CreateWorkflowExecution(&persistence.CreateWorkflowExecutionRequest{
 		ShardID: s.ShardInfo.GetShardId(),
 		NewWorkflowSnapshot: persistence.WorkflowSnapshot{
@@ -334,6 +299,7 @@ func (s *TestBase) CreateWorkflowExecutionWithBranchToken(namespaceID string, wo
 				WorkflowTaskTimeout:        timestamp.DurationFromSeconds(1),
 				ExecutionStats:             &persistencespb.ExecutionStats{},
 				StartTime:                  timestamp.TimeNowPtrUtc(),
+				LastUpdateTime:             timestamp.TimeNowPtrUtc(),
 			},
 			ExecutionState: &persistencespb.WorkflowExecutionState{
 				RunId:           workflowExecution.GetRunId(),
@@ -342,10 +308,14 @@ func (s *TestBase) CreateWorkflowExecutionWithBranchToken(namespaceID string, wo
 				Status:          enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING,
 			},
 			NextEventID: nextEventID,
-			TransferTasks: []persistence.Task{
-				&persistence.WorkflowTask{
+			TransferTasks: []tasks.Task{
+				&tasks.WorkflowTask{
+					WorkflowKey: definition.NewWorkflowKey(
+						namespaceID,
+						workflowExecution.WorkflowId,
+						workflowExecution.RunId,
+					),
 					TaskID:              s.GetNextSequenceNumber(),
-					NamespaceID:         namespaceID,
 					TaskQueue:           taskQueue,
 					ScheduleID:          workflowTaskScheduleID,
 					VisibilityTimestamp: time.Now().UTC(),
@@ -361,7 +331,7 @@ func (s *TestBase) CreateWorkflowExecutionWithBranchToken(namespaceID string, wo
 }
 
 // CreateWorkflowExecution is a utility method to create workflow executions
-func (s *TestBase) CreateWorkflowExecution(namespaceID string, workflowExecution commonpb.WorkflowExecution, taskQueue, wType string, wTimeout *time.Duration, workflowTaskTimeout *time.Duration, nextEventID, lastProcessedEventID, workflowTaskScheduleID int64, timerTasks []persistence.Task) (*persistence.CreateWorkflowExecutionResponse, error) {
+func (s *TestBase) CreateWorkflowExecution(namespaceID string, workflowExecution commonpb.WorkflowExecution, taskQueue, wType string, wTimeout *time.Duration, workflowTaskTimeout *time.Duration, nextEventID, lastProcessedEventID, workflowTaskScheduleID int64, timerTasks []tasks.Task) (*persistence.CreateWorkflowExecutionResponse, error) {
 	return s.CreateWorkflowExecutionWithBranchToken(namespaceID, workflowExecution, taskQueue, wType, wTimeout, workflowTaskTimeout,
 		nextEventID, lastProcessedEventID, workflowTaskScheduleID, nil, timerTasks)
 }
@@ -371,12 +341,17 @@ func (s *TestBase) CreateWorkflowExecutionManyTasks(namespaceID string, workflow
 	taskQueue string, nextEventID int64, lastProcessedEventID int64,
 	workflowTaskScheduleIDs []int64, activityScheduleIDs []int64) (*persistence.CreateWorkflowExecutionResponse, error) {
 
-	transferTasks := []persistence.Task{}
+	workflowKey := definition.NewWorkflowKey(
+		namespaceID,
+		workflowExecution.WorkflowId,
+		workflowExecution.RunId,
+	)
+	transferTasks := []tasks.Task{}
 	for _, workflowTaskScheduleID := range workflowTaskScheduleIDs {
 		transferTasks = append(transferTasks,
-			&persistence.WorkflowTask{
+			&tasks.WorkflowTask{
+				WorkflowKey: workflowKey,
 				TaskID:      s.GetNextSequenceNumber(),
-				NamespaceID: namespaceID,
 				TaskQueue:   taskQueue,
 				ScheduleID:  workflowTaskScheduleID,
 			})
@@ -384,11 +359,12 @@ func (s *TestBase) CreateWorkflowExecutionManyTasks(namespaceID string, workflow
 
 	for _, activityScheduleID := range activityScheduleIDs {
 		transferTasks = append(transferTasks,
-			&persistence.ActivityTask{
-				TaskID:      s.GetNextSequenceNumber(),
-				NamespaceID: namespaceID,
-				TaskQueue:   taskQueue,
-				ScheduleID:  activityScheduleID,
+			&tasks.ActivityTask{
+				WorkflowKey:       workflowKey,
+				TaskID:            s.GetNextSequenceNumber(),
+				TargetNamespaceID: namespaceID,
+				TaskQueue:         taskQueue,
+				ScheduleID:        activityScheduleID,
 			})
 	}
 
@@ -427,7 +403,7 @@ func (s *TestBase) CreateWorkflowExecutionManyTasks(namespaceID string, workflow
 func (s *TestBase) CreateChildWorkflowExecution(namespaceID string, workflowExecution commonpb.WorkflowExecution,
 	parentNamespaceID string, parentExecution commonpb.WorkflowExecution, initiatedID int64, taskQueue, wType string,
 	wTimeout *time.Duration, workflowTaskTimeout *time.Duration, nextEventID int64, lastProcessedEventID int64,
-	workflowTaskScheduleID int64, timerTasks []persistence.Task) (*persistence.CreateWorkflowExecutionResponse, error) {
+	workflowTaskScheduleID int64, timerTasks []tasks.Task) (*persistence.CreateWorkflowExecutionResponse, error) {
 	response, err := s.ExecutionManager.CreateWorkflowExecution(&persistence.CreateWorkflowExecutionRequest{
 		ShardID: s.ShardInfo.GetShardId(),
 		NewWorkflowSnapshot: persistence.WorkflowSnapshot{
@@ -457,12 +433,16 @@ func (s *TestBase) CreateChildWorkflowExecution(namespaceID string, workflowExec
 				Status:          enumspb.WORKFLOW_EXECUTION_STATUS_RUNNING,
 			},
 			NextEventID: nextEventID,
-			TransferTasks: []persistence.Task{
-				&persistence.WorkflowTask{
-					TaskID:      s.GetNextSequenceNumber(),
-					NamespaceID: namespaceID,
-					TaskQueue:   taskQueue,
-					ScheduleID:  workflowTaskScheduleID,
+			TransferTasks: []tasks.Task{
+				&tasks.WorkflowTask{
+					WorkflowKey: definition.NewWorkflowKey(
+						namespaceID,
+						workflowExecution.WorkflowId,
+						workflowExecution.RunId,
+					),
+					TaskID:     s.GetNextSequenceNumber(),
+					TaskQueue:  taskQueue,
+					ScheduleID: workflowTaskScheduleID,
 				},
 			},
 			TimerTasks: timerTasks,
@@ -473,28 +453,14 @@ func (s *TestBase) CreateChildWorkflowExecution(namespaceID string, workflowExec
 	return response, err
 }
 
-// GetWorkflowExecutionInfoWithStats is a utility method to retrieve execution info with size stats
-func (s *TestBase) GetWorkflowExecutionInfoWithStats(namespaceID string, workflowExecution commonpb.WorkflowExecution) (
-	*persistence.MutableStateStats, *persistencespb.WorkflowMutableState, error) {
-	response, err := s.ExecutionManager.GetWorkflowExecution(&persistence.GetWorkflowExecutionRequest{
-		ShardID:     s.ShardInfo.GetShardId(),
-		NamespaceID: namespaceID,
-		Execution:   workflowExecution,
-	})
-	if err != nil {
-		return nil, nil, err
-	}
-
-	return response.MutableStateStats, response.State, nil
-}
-
 // GetWorkflowExecutionInfo is a utility method to retrieve execution info
 func (s *TestBase) GetWorkflowMutableState(namespaceID string, workflowExecution commonpb.WorkflowExecution) (
 	*persistencespb.WorkflowMutableState, error) {
 	response, err := s.ExecutionManager.GetWorkflowExecution(&persistence.GetWorkflowExecutionRequest{
 		ShardID:     s.ShardInfo.GetShardId(),
 		NamespaceID: namespaceID,
-		Execution:   workflowExecution,
+		WorkflowID:  workflowExecution.GetWorkflowId(),
+		RunID:       workflowExecution.GetRunId(),
 	})
 	if err != nil {
 		return nil, err
@@ -521,11 +487,15 @@ func (s *TestBase) GetCurrentWorkflowRunID(namespaceID, workflowID string) (stri
 func (s *TestBase) ContinueAsNewExecution(updatedInfo *persistencespb.WorkflowExecutionInfo, updatedState *persistencespb.WorkflowExecutionState, updatedNextEventID int64, condition int64,
 	newExecution commonpb.WorkflowExecution, nextEventID, workflowTaskScheduleID int64,
 	prevResetPoints *workflowpb.ResetPoints) error {
-	newworkflowTask := &persistence.WorkflowTask{
-		TaskID:      s.GetNextSequenceNumber(),
-		NamespaceID: updatedInfo.NamespaceId,
-		TaskQueue:   updatedInfo.TaskQueue,
-		ScheduleID:  workflowTaskScheduleID,
+	newworkflowTask := &tasks.WorkflowTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			updatedInfo.NamespaceId,
+			updatedInfo.WorkflowId,
+			updatedState.RunId,
+		),
+		TaskID:     s.GetNextSequenceNumber(),
+		TaskQueue:  updatedInfo.TaskQueue,
+		ScheduleID: workflowTaskScheduleID,
 	}
 
 	req := &persistence.UpdateWorkflowExecutionRequest{
@@ -534,7 +504,7 @@ func (s *TestBase) ContinueAsNewExecution(updatedInfo *persistencespb.WorkflowEx
 			ExecutionInfo:       updatedInfo,
 			ExecutionState:      updatedState,
 			NextEventID:         updatedNextEventID,
-			TransferTasks:       []persistence.Task{newworkflowTask},
+			TransferTasks:       []tasks.Task{newworkflowTask},
 			TimerTasks:          nil,
 			Condition:           condition,
 			UpsertActivityInfos: nil,
@@ -580,7 +550,7 @@ func (s *TestBase) ContinueAsNewExecution(updatedInfo *persistencespb.WorkflowEx
 // UpdateWorkflowExecution is a utility method to update workflow execution
 func (s *TestBase) UpdateWorkflowExecution(updatedInfo *persistencespb.WorkflowExecutionInfo,
 	updatedState *persistencespb.WorkflowExecutionState, nextEventID int64,
-	workflowTaskScheduleIDs []int64, activityScheduleIDs []int64, condition int64, timerTasks []persistence.Task,
+	workflowTaskScheduleIDs []int64, activityScheduleIDs []int64, condition int64, timerTasks []tasks.Task,
 	upsertActivityInfos []*persistencespb.ActivityInfo, deleteActivityInfos []int64,
 	upsertTimerInfos []*persistencespb.TimerInfo, deleteTimerInfos []string) error {
 	return s.UpdateWorkflowExecutionWithRangeID(updatedInfo, updatedState, nextEventID, workflowTaskScheduleIDs, activityScheduleIDs,
@@ -592,8 +562,15 @@ func (s *TestBase) UpdateWorkflowExecution(updatedInfo *persistencespb.WorkflowE
 // UpdateWorkflowExecutionAndFinish is a utility method to update workflow execution
 func (s *TestBase) UpdateWorkflowExecutionAndFinish(updatedInfo *persistencespb.WorkflowExecutionInfo,
 	updatedState *persistencespb.WorkflowExecutionState, nextEventID int64, condition int64) error {
-	var transferTasks []persistence.Task
-	transferTasks = append(transferTasks, &persistence.CloseExecutionTask{TaskID: s.GetNextSequenceNumber()})
+	var transferTasks []tasks.Task
+	transferTasks = append(transferTasks, &tasks.CloseExecutionTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			updatedInfo.NamespaceId,
+			updatedInfo.WorkflowId,
+			updatedState.RunId,
+		),
+		TaskID: s.GetNextSequenceNumber(),
+	})
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
 		ShardID: s.ShardInfo.GetShardId(),
 		RangeID: s.ShardInfo.GetRangeId(),
@@ -696,7 +673,7 @@ func (s *TestBase) DeleteSignalsRequestedState(updatedInfo *persistencespb.Workf
 // UpdateWorklowStateAndReplication is a utility method to update workflow execution
 func (s *TestBase) UpdateWorklowStateAndReplication(updatedInfo *persistencespb.WorkflowExecutionInfo,
 	updatedState *persistencespb.WorkflowExecutionState, nextEventID int64,
-	condition int64, txTasks []persistence.Task) error {
+	condition int64, txTasks []tasks.Task) error {
 	return s.UpdateWorkflowExecutionWithReplication(updatedInfo, updatedState, nextEventID, nil, nil,
 		s.ShardInfo.GetRangeId(), condition, nil, txTasks, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil, nil)
 }
@@ -704,14 +681,14 @@ func (s *TestBase) UpdateWorklowStateAndReplication(updatedInfo *persistencespb.
 // UpdateWorkflowExecutionWithRangeID is a utility method to update workflow execution
 func (s *TestBase) UpdateWorkflowExecutionWithRangeID(updatedInfo *persistencespb.WorkflowExecutionInfo,
 	updatedState *persistencespb.WorkflowExecutionState, nextEventID int64,
-	workflowTaskScheduleIDs []int64, activityScheduleIDs []int64, rangeID, condition int64, timerTasks []persistence.Task,
+	workflowTaskScheduleIDs []int64, activityScheduleIDs []int64, rangeID, condition int64, timerTasks []tasks.Task,
 	upsertActivityInfos []*persistencespb.ActivityInfo, deleteActivityInfos []int64, upsertTimerInfos []*persistencespb.TimerInfo,
 	deleteTimerInfos []string, upsertChildInfos []*persistencespb.ChildExecutionInfo, deleteChildInfos []int64,
 	upsertCancelInfos []*persistencespb.RequestCancelInfo, deleteCancelInfos []int64,
 	upsertSignalInfos []*persistencespb.SignalInfo, deleteSignalInfos []int64,
 	upsertSignalRequestedIDs []string, deleteSignalRequestedIDs []string) error {
 	return s.UpdateWorkflowExecutionWithReplication(updatedInfo, updatedState, nextEventID, workflowTaskScheduleIDs, activityScheduleIDs, rangeID,
-		condition, timerTasks, []persistence.Task{}, upsertActivityInfos, deleteActivityInfos, upsertTimerInfos, deleteTimerInfos,
+		condition, timerTasks, []tasks.Task{}, upsertActivityInfos, deleteActivityInfos, upsertTimerInfos, deleteTimerInfos,
 		upsertChildInfos, deleteChildInfos, upsertCancelInfos, deleteCancelInfos, upsertSignalInfos, deleteSignalInfos,
 		upsertSignalRequestedIDs, deleteSignalRequestedIDs)
 }
@@ -720,37 +697,43 @@ func (s *TestBase) UpdateWorkflowExecutionWithRangeID(updatedInfo *persistencesp
 func (s *TestBase) UpdateWorkflowExecutionWithReplication(updatedInfo *persistencespb.WorkflowExecutionInfo,
 	updatedState *persistencespb.WorkflowExecutionState, nextEventID int64,
 	workflowTaskScheduleIDs []int64, activityScheduleIDs []int64, rangeID,
-	condition int64, timerTasks []persistence.Task, txTasks []persistence.Task, upsertActivityInfos []*persistencespb.ActivityInfo,
+	condition int64, timerTasks []tasks.Task, txTasks []tasks.Task, upsertActivityInfos []*persistencespb.ActivityInfo,
 	deleteActivityInfos []int64, upsertTimerInfos []*persistencespb.TimerInfo, deleteTimerInfos []string,
 	upsertChildInfos []*persistencespb.ChildExecutionInfo, deleteChildInfos []int64, upsertCancelInfos []*persistencespb.RequestCancelInfo,
 	deleteCancelInfos []int64, upsertSignalInfos []*persistencespb.SignalInfo, deleteSignalInfos []int64, upsertSignalRequestedIDs []string,
 	deleteSignalRequestedIDs []string) error {
-	var transferTasks []persistence.Task
-	var replicationTasks []persistence.Task
+	var transferTasks []tasks.Task
+	var replicationTasks []tasks.Task
 	for _, task := range txTasks {
 		switch t := task.(type) {
-		case *persistence.WorkflowTask, *persistence.ActivityTask, *persistence.CloseExecutionTask, *persistence.CancelExecutionTask, *persistence.StartChildExecutionTask, *persistence.SignalExecutionTask:
+		case *tasks.WorkflowTask, *tasks.ActivityTask, *tasks.CloseExecutionTask, *tasks.CancelExecutionTask, *tasks.StartChildExecutionTask, *tasks.SignalExecutionTask:
 			transferTasks = append(transferTasks, t)
-		case *persistence.HistoryReplicationTask, *persistence.SyncActivityTask:
+		case *tasks.HistoryReplicationTask, *tasks.SyncActivityTask:
 			replicationTasks = append(replicationTasks, t)
 		default:
 			panic("Unknown transfer task type.")
 		}
 	}
+	workflowKey := definition.NewWorkflowKey(
+		updatedInfo.NamespaceId,
+		updatedInfo.WorkflowId,
+		updatedState.RunId,
+	)
 	for _, workflowTaskScheduleID := range workflowTaskScheduleIDs {
-		transferTasks = append(transferTasks, &persistence.WorkflowTask{
+		transferTasks = append(transferTasks, &tasks.WorkflowTask{
+			WorkflowKey: workflowKey,
 			TaskID:      s.GetNextSequenceNumber(),
-			NamespaceID: updatedInfo.NamespaceId,
 			TaskQueue:   updatedInfo.TaskQueue,
 			ScheduleID:  workflowTaskScheduleID})
 	}
 
 	for _, activityScheduleID := range activityScheduleIDs {
-		transferTasks = append(transferTasks, &persistence.ActivityTask{
-			TaskID:      s.GetNextSequenceNumber(),
-			NamespaceID: updatedInfo.NamespaceId,
-			TaskQueue:   updatedInfo.TaskQueue,
-			ScheduleID:  activityScheduleID})
+		transferTasks = append(transferTasks, &tasks.ActivityTask{
+			WorkflowKey:       workflowKey,
+			TaskID:            s.GetNextSequenceNumber(),
+			TargetNamespaceID: updatedInfo.NamespaceId,
+			TaskQueue:         updatedInfo.TaskQueue,
+			ScheduleID:        activityScheduleID})
 	}
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
 		ShardID: s.ShardInfo.GetShardId(),
@@ -786,7 +769,7 @@ func (s *TestBase) UpdateWorkflowExecutionWithReplication(updatedInfo *persisten
 
 // UpdateWorkflowExecutionWithTransferTasks is a utility method to update workflow execution
 func (s *TestBase) UpdateWorkflowExecutionWithTransferTasks(
-	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedState *persistencespb.WorkflowExecutionState, updatedNextEventID int64, condition int64, transferTasks []persistence.Task, upsertActivityInfo []*persistencespb.ActivityInfo) error {
+	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedState *persistencespb.WorkflowExecutionState, updatedNextEventID int64, condition int64, transferTasks []tasks.Task, upsertActivityInfo []*persistencespb.ActivityInfo) error {
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
 		ShardID: s.ShardInfo.GetShardId(),
 		UpdateWorkflowMutation: persistence.WorkflowMutation{
@@ -804,7 +787,7 @@ func (s *TestBase) UpdateWorkflowExecutionWithTransferTasks(
 
 // UpdateWorkflowExecutionForChildExecutionsInitiated is a utility method to update workflow execution
 func (s *TestBase) UpdateWorkflowExecutionForChildExecutionsInitiated(
-	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedNextEventID int64, condition int64, transferTasks []persistence.Task, childInfos []*persistencespb.ChildExecutionInfo) error {
+	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedNextEventID int64, condition int64, transferTasks []tasks.Task, childInfos []*persistencespb.ChildExecutionInfo) error {
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
 		ShardID: s.ShardInfo.GetShardId(),
 		UpdateWorkflowMutation: persistence.WorkflowMutation{
@@ -821,7 +804,7 @@ func (s *TestBase) UpdateWorkflowExecutionForChildExecutionsInitiated(
 
 // UpdateWorkflowExecutionForRequestCancel is a utility method to update workflow execution
 func (s *TestBase) UpdateWorkflowExecutionForRequestCancel(
-	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedNextEventID int64, condition int64, transferTasks []persistence.Task,
+	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedNextEventID int64, condition int64, transferTasks []tasks.Task,
 	upsertRequestCancelInfo []*persistencespb.RequestCancelInfo) error {
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
 		ShardID: s.ShardInfo.GetShardId(),
@@ -839,7 +822,7 @@ func (s *TestBase) UpdateWorkflowExecutionForRequestCancel(
 
 // UpdateWorkflowExecutionForSignal is a utility method to update workflow execution
 func (s *TestBase) UpdateWorkflowExecutionForSignal(
-	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedNextEventID int64, condition int64, transferTasks []persistence.Task,
+	updatedInfo *persistencespb.WorkflowExecutionInfo, updatedNextEventID int64, condition int64, transferTasks []tasks.Task,
 	upsertSignalInfos []*persistencespb.SignalInfo) error {
 	_, err := s.ExecutionManager.UpdateWorkflowExecution(&persistence.UpdateWorkflowExecutionRequest{
 		ShardID: s.ShardInfo.GetShardId(),
@@ -921,28 +904,6 @@ func (s *TestBase) UpdateAllMutableState(updatedMutableState *persistencespb.Wor
 	return err
 }
 
-// ConflictResolveWorkflowExecution is  utility method to reset mutable state
-func (s *TestBase) ConflictResolveWorkflowExecution(prevRunID string, prevLastWriteVersion int64, prevState enumsspb.WorkflowExecutionState,
-	info *persistencespb.WorkflowExecutionInfo, state *persistencespb.WorkflowExecutionState, nextEventID int64,
-	activityInfos []*persistencespb.ActivityInfo, timerInfos []*persistencespb.TimerInfo, childExecutionInfos []*persistencespb.ChildExecutionInfo,
-	requestCancelInfos []*persistencespb.RequestCancelInfo, signalInfos []*persistencespb.SignalInfo, ids []string) error {
-	return s.ExecutionManager.ConflictResolveWorkflowExecution(&persistence.ConflictResolveWorkflowExecutionRequest{
-		ShardID: s.ShardInfo.GetShardId(),
-		RangeID: s.ShardInfo.GetRangeId(),
-		ResetWorkflowSnapshot: persistence.WorkflowSnapshot{
-			ExecutionInfo:       info,
-			Condition:           nextEventID,
-			ActivityInfos:       convertActivityInfos(activityInfos),
-			TimerInfos:          convertTimerInfos(timerInfos),
-			ChildExecutionInfos: convertChildExecutionInfos(childExecutionInfos),
-			RequestCancelInfos:  convertRequestCancelInfos(requestCancelInfos),
-			SignalInfos:         convertSignalInfos(signalInfos),
-			SignalRequestedIDs:  convert.StringSliceToSet(ids),
-			Checksum:            testWorkflowChecksum,
-		},
-	})
-}
-
 // DeleteWorkflowExecution is a utility method to delete a workflow execution
 func (s *TestBase) DeleteWorkflowExecution(info *persistencespb.WorkflowExecutionInfo, state *persistencespb.WorkflowExecutionState) error {
 	return s.ExecutionManager.DeleteWorkflowExecution(&persistence.DeleteWorkflowExecutionRequest{
@@ -964,8 +925,8 @@ func (s *TestBase) DeleteCurrentWorkflowExecution(info *persistencespb.WorkflowE
 }
 
 // GetTransferTasks is a utility method to get tasks from transfer task queue
-func (s *TestBase) GetTransferTasks(batchSize int, getAll bool) ([]*persistencespb.TransferTaskInfo, error) {
-	result := []*persistencespb.TransferTaskInfo{}
+func (s *TestBase) GetTransferTasks(batchSize int, getAll bool) ([]tasks.Task, error) {
+	result := []tasks.Task{}
 	var token []byte
 
 Loop:
@@ -989,15 +950,15 @@ Loop:
 	}
 
 	for _, task := range result {
-		atomic.StoreInt64(&s.ReadLevel, task.GetTaskId())
+		atomic.StoreInt64(&s.ReadLevel, task.GetTaskID())
 	}
 
 	return result, nil
 }
 
 // GetReplicationTasks is a utility method to get tasks from replication task queue
-func (s *TestBase) GetReplicationTasks(batchSize int, getAll bool) ([]*persistencespb.ReplicationTaskInfo, error) {
-	result := []*persistencespb.ReplicationTaskInfo{}
+func (s *TestBase) GetReplicationTasks(batchSize int, getAll bool) ([]tasks.Task, error) {
+	result := []tasks.Task{}
 	var token []byte
 
 Loop:
@@ -1021,7 +982,7 @@ Loop:
 	}
 
 	for _, task := range result {
-		atomic.StoreInt64(&s.ReplicationReadLevel, task.GetTaskId())
+		atomic.StoreInt64(&s.ReplicationReadLevel, task.GetTaskID())
 	}
 
 	return result, nil
@@ -1124,16 +1085,16 @@ func (s *TestBase) CompleteReplicationTask(taskID int64) error {
 	})
 }
 
-// GetTimerIndexTasks is a utility method to get tasks from transfer task queue
-func (s *TestBase) GetTimerIndexTasks(batchSize int, getAll bool) ([]*persistencespb.TimerTaskInfo, error) {
-	result := []*persistencespb.TimerTaskInfo{}
+// GetTimerTasks is a utility method to get tasks from transfer task queue
+func (s *TestBase) GetTimerTasks(batchSize int, getAll bool) ([]tasks.Task, error) {
+	result := []tasks.Task{}
 	var token []byte
 
 Loop:
 	for {
-		response, err := s.ExecutionManager.GetTimerIndexTasks(&persistence.GetTimerIndexTasksRequest{
+		response, err := s.ExecutionManager.GetTimerTasks(&persistence.GetTimerTasksRequest{
 			ShardID:       s.ShardInfo.GetShardId(),
-			MinTimestamp:  time.Time{},
+			MinTimestamp:  time.Unix(0, 0).UTC(),
 			MaxTimestamp:  time.Unix(0, math.MaxInt64).UTC(),
 			BatchSize:     batchSize,
 			NextPageToken: token,
@@ -1143,7 +1104,7 @@ Loop:
 		}
 
 		token = response.NextPageToken
-		result = append(result, response.Timers...)
+		result = append(result, response.Tasks...)
 		if len(token) == 0 || !getAll {
 			break Loop
 		}
@@ -1301,13 +1262,6 @@ func (s *TestBase) TearDownWorkflowStore() {
 	s.ExecutionManager.Close()
 	s.NamespaceReplicationQueue.Stop()
 	s.Factory.Close()
-
-	// TODO VisibilityMgr/Store is created with a separated code path, this is incorrect and may cause leaking connection
-	// And Postgres requires all connection to be closed before dropping a database
-	// https://github.com/uber/cadence/issues/2854
-	// Remove the below line after the issue is fix
-	s.VisibilityMgr.Close()
-
 	s.DefaultTestCluster.TearDownTestDatabase()
 }
 
@@ -1335,39 +1289,39 @@ func (s *TestBase) ClearTasks() {
 
 // ClearTransferQueue completes all tasks in transfer queue
 func (s *TestBase) ClearTransferQueue() {
-	s.logger.Info("Clearing transfer tasks", tag.ShardRangeID(s.ShardInfo.GetRangeId()), tag.ReadLevel(s.GetTransferReadLevel()))
+	s.Logger.Info("Clearing transfer tasks", tag.ShardRangeID(s.ShardInfo.GetRangeId()), tag.ReadLevel(s.GetTransferReadLevel()))
 	tasks, err := s.GetTransferTasks(100, true)
 	if err != nil {
-		s.logger.Fatal("Error during cleanup", tag.Error(err))
+		s.Logger.Fatal("Error during cleanup", tag.Error(err))
 	}
 
 	counter := 0
 	for _, t := range tasks {
-		s.logger.Info("Deleting transfer task with ID", tag.TaskID(t.GetTaskId()))
-		s.NoError(s.CompleteTransferTask(t.GetTaskId()))
+		s.Logger.Info("Deleting transfer task with ID", tag.TaskID(t.GetTaskID()))
+		s.NoError(s.CompleteTransferTask(t.GetTaskID()))
 		counter++
 	}
 
-	s.logger.Info("Deleted transfer tasks.", tag.Counter(counter))
+	s.Logger.Info("Deleted transfer tasks.", tag.Counter(counter))
 	atomic.StoreInt64(&s.ReadLevel, 0)
 }
 
 // ClearReplicationQueue completes all tasks in replication queue
 func (s *TestBase) ClearReplicationQueue() {
-	s.logger.Info("Clearing replication tasks", tag.ShardRangeID(s.ShardInfo.GetRangeId()), tag.ReadLevel(s.GetReplicationReadLevel()))
+	s.Logger.Info("Clearing replication tasks", tag.ShardRangeID(s.ShardInfo.GetRangeId()), tag.ReadLevel(s.GetReplicationReadLevel()))
 	tasks, err := s.GetReplicationTasks(100, true)
 	if err != nil {
-		s.logger.Fatal("Error during cleanup", tag.Error(err))
+		s.Logger.Fatal("Error during cleanup", tag.Error(err))
 	}
 
 	counter := 0
 	for _, t := range tasks {
-		s.logger.Info("Deleting replication task with ID", tag.TaskID(t.GetTaskId()))
-		s.NoError(s.CompleteReplicationTask(t.GetTaskId()))
+		s.Logger.Info("Deleting replication task with ID", tag.TaskID(t.GetTaskID()))
+		s.NoError(s.CompleteReplicationTask(t.GetTaskID()))
 		counter++
 	}
 
-	s.logger.Info("Deleted replication tasks.", tag.Counter(counter))
+	s.Logger.Info("Deleted replication tasks.", tag.Counter(counter))
 	atomic.StoreInt64(&s.ReplicationReadLevel, 0)
 }
 
@@ -1389,7 +1343,7 @@ func (s *TestBase) validateTimeRange(t time.Time, expectedDuration time.Duration
 	currentTime := time.Now().UTC()
 	diff := time.Duration(currentTime.UnixNano() - t.UnixNano())
 	if diff > expectedDuration {
-		s.logger.Info("Check Current time, Application time, Differenrce", tag.Timestamp(t), tag.CursorTimestamp(currentTime), tag.Number(int64(diff)))
+		s.Logger.Info("Check Current time, Application time, Differenrce", tag.Timestamp(t), tag.CursorTimestamp(currentTime), tag.Number(int64(diff)))
 		return false
 	}
 	return true
