@@ -27,12 +27,15 @@
 package persistence
 
 import (
+	"context"
 	"math"
 	"time"
 
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
+
 	persistencespb "go.temporal.io/server/api/persistence/v1"
+	"go.temporal.io/server/service/history/tasks"
 )
 
 const (
@@ -53,8 +56,7 @@ type (
 		Closeable
 		GetName() string
 		GetClusterName() string
-		CreateShard(request *InternalCreateShardRequest) error
-		GetShard(request *InternalGetShardRequest) (*InternalGetShardResponse, error)
+		GetOrCreateShard(request *InternalGetOrCreateShardRequest) (*InternalGetOrCreateShardResponse, error)
 		UpdateShard(request *InternalUpdateShardRequest) error
 	}
 
@@ -92,8 +94,12 @@ type (
 	ClusterMetadataStore interface {
 		Closeable
 		GetName() string
-		GetClusterMetadata() (*InternalGetClusterMetadataResponse, error)
+		ListClusterMetadata(request *InternalListClusterMetadataRequest) (*InternalListClusterMetadataResponse, error)
+		GetClusterMetadataV1() (*InternalGetClusterMetadataResponse, error) //TODO: deprecate this after 1.15+
+		GetClusterMetadata(request *InternalGetClusterMetadataRequest) (*InternalGetClusterMetadataResponse, error)
+		SaveClusterMetadataV1(request *InternalSaveClusterMetadataRequest) (bool, error) //TODO: deprecate this after 1.15+
 		SaveClusterMetadata(request *InternalSaveClusterMetadataRequest) (bool, error)
+		DeleteClusterMetadata(request *InternalDeleteClusterMetadataRequest) error
 		// Membership APIs
 		GetClusterMembers(request *GetClusterMembersRequest) (*GetClusterMembersResponse, error)
 		UpsertClusterMembership(request *UpsertClusterMembershipRequest) error
@@ -118,36 +124,42 @@ type (
 		ListConcreteExecutions(request *ListConcreteExecutionsRequest) (*InternalListConcreteExecutionsResponse, error)
 
 		// Tasks related APIs
-		AddTasks(request *AddTasksRequest) error
+		AddTasks(request *InternalAddTasksRequest) error
 
 		// transfer tasks
-		GetTransferTask(request *GetTransferTaskRequest) (*GetTransferTaskResponse, error)
-		GetTransferTasks(request *GetTransferTasksRequest) (*GetTransferTasksResponse, error)
+		GetTransferTask(request *GetTransferTaskRequest) (*InternalGetTransferTaskResponse, error)
+		GetTransferTasks(request *GetTransferTasksRequest) (*InternalGetTransferTasksResponse, error)
 		CompleteTransferTask(request *CompleteTransferTaskRequest) error
 		RangeCompleteTransferTask(request *RangeCompleteTransferTaskRequest) error
 
 		// timer tasks
-		GetTimerTask(request *GetTimerTaskRequest) (*GetTimerTaskResponse, error)
-		GetTimerIndexTasks(request *GetTimerIndexTasksRequest) (*GetTimerIndexTasksResponse, error)
+		GetTimerTask(request *GetTimerTaskRequest) (*InternalGetTimerTaskResponse, error)
+		GetTimerTasks(request *GetTimerTasksRequest) (*InternalGetTimerTasksResponse, error)
 		CompleteTimerTask(request *CompleteTimerTaskRequest) error
 		RangeCompleteTimerTask(request *RangeCompleteTimerTaskRequest) error
 
 		// replication tasks
-		GetReplicationTask(request *GetReplicationTaskRequest) (*GetReplicationTaskResponse, error)
-		GetReplicationTasks(request *GetReplicationTasksRequest) (*GetReplicationTasksResponse, error)
+		GetReplicationTask(request *GetReplicationTaskRequest) (*InternalGetReplicationTaskResponse, error)
+		GetReplicationTasks(request *GetReplicationTasksRequest) (*InternalGetReplicationTasksResponse, error)
 		CompleteReplicationTask(request *CompleteReplicationTaskRequest) error
 		RangeCompleteReplicationTask(request *RangeCompleteReplicationTaskRequest) error
 		PutReplicationTaskToDLQ(request *PutReplicationTaskToDLQRequest) error
-		GetReplicationTasksFromDLQ(request *GetReplicationTasksFromDLQRequest) (*GetReplicationTasksFromDLQResponse, error)
+		GetReplicationTasksFromDLQ(request *GetReplicationTasksFromDLQRequest) (*InternalGetReplicationTasksFromDLQResponse, error)
 		DeleteReplicationTaskFromDLQ(request *DeleteReplicationTaskFromDLQRequest) error
 		RangeDeleteReplicationTaskFromDLQ(request *RangeDeleteReplicationTaskFromDLQRequest) error
 
 		// visibility tasks
-		GetVisibilityTask(request *GetVisibilityTaskRequest) (*GetVisibilityTaskResponse, error)
-		GetVisibilityTasks(request *GetVisibilityTasksRequest) (*GetVisibilityTasksResponse, error)
+		GetVisibilityTask(request *GetVisibilityTaskRequest) (*InternalGetVisibilityTaskResponse, error)
+		GetVisibilityTasks(request *GetVisibilityTasksRequest) (*InternalGetVisibilityTasksResponse, error)
 		CompleteVisibilityTask(request *CompleteVisibilityTaskRequest) error
 		RangeCompleteVisibilityTask(request *RangeCompleteVisibilityTaskRequest) error
 
+		// TieredStorage tasks
+		GetTieredStorageTask(request *GetTieredStorageTaskRequest) (*InternalGetTieredStorageTaskResponse, error)
+		GetTieredStorageTasks(request *GetTieredStorageTasksRequest) (*InternalGetTieredStorageTasksResponse, error)
+		CompleteTieredStorageTask(request *CompleteTieredStorageTaskRequest) error
+		RangeCompleteTieredStorageTask(request *RangeCompleteTieredStorageTaskRequest) error
+
 		// The below are history V2 APIs
 		// V2 regards history events growing as a tree, decoupled from workflow concepts
 
@@ -198,20 +210,17 @@ type (
 		Version int64
 	}
 
-	// InternalCreateShardRequest is used by ShardStore to create new shard
-	InternalCreateShardRequest struct {
-		ShardID   int32
-		RangeID   int64
-		ShardInfo *commonpb.DataBlob
+	// InternalGetOrCreateShardRequest is used by ShardStore to retrieve or create a shard.
+	// GetOrCreateShard should: if shard exists, return it. If not, call CreateShardInfo and
+	// create the shard with the returned value.
+	InternalGetOrCreateShardRequest struct {
+		ShardID          int32
+		CreateShardInfo  func() (rangeID int64, shardInfo *commonpb.DataBlob, err error)
+		LifecycleContext context.Context // cancelled when shard is unloaded
 	}
 
-	// InternalGetShardRequest is used by ShardStore to retrieve a shard
-	InternalGetShardRequest struct {
-		ShardID int32
-	}
-
-	// InternalGetShardResponse is the response to GetShard
-	InternalGetShardResponse struct {
+	// InternalGetOrCreateShardResponse is the response to GetShard
+	InternalGetOrCreateShardResponse struct {
 		ShardInfo *commonpb.DataBlob
 	}
 
@@ -283,6 +292,7 @@ type (
 		Items         []*InternalListTaskQueueItem
 		NextPageToken []byte
 	}
+
 	InternalListTaskQueueItem struct {
 		TaskQueue *commonpb.DataBlob // serialized PersistedTaskQueueInfo
 		RangeID   int64
@@ -347,13 +357,30 @@ type (
 		Mode ConflictResolveWorkflowMode
 
 		// workflow to be resetted
-		ResetWorkflowSnapshot InternalWorkflowSnapshot
-
+		ResetWorkflowSnapshot        InternalWorkflowSnapshot
+		ResetWorkflowEventsNewEvents []*InternalAppendHistoryNodesRequest
 		// maybe new workflow
-		NewWorkflowSnapshot *InternalWorkflowSnapshot
+		NewWorkflowSnapshot        *InternalWorkflowSnapshot
+		NewWorkflowEventsNewEvents []*InternalAppendHistoryNodesRequest
 
 		// current workflow
-		CurrentWorkflowMutation *InternalWorkflowMutation
+		CurrentWorkflowMutation        *InternalWorkflowMutation
+		CurrentWorkflowEventsNewEvents []*InternalAppendHistoryNodesRequest
+	}
+
+	// InternalAddTasksRequest is used to write new tasks
+	InternalAddTasksRequest struct {
+		ShardID int32
+		RangeID int64
+
+		NamespaceID string
+		WorkflowID  string
+		RunID       string
+
+		TransferTasks    map[tasks.Key]commonpb.DataBlob
+		TimerTasks       map[tasks.Key]commonpb.DataBlob
+		ReplicationTasks map[tasks.Key]commonpb.DataBlob
+		VisibilityTasks  map[tasks.Key]commonpb.DataBlob
 	}
 
 	// InternalWorkflowMutation is used as generic workflow execution state mutation for Persistence Interface
@@ -386,10 +413,10 @@ type (
 		NewBufferedEvents         *commonpb.DataBlob
 		ClearBufferedEvents       bool
 
-		TransferTasks    []Task
-		TimerTasks       []Task
-		ReplicationTasks []Task
-		VisibilityTasks  []Task
+		TransferTasks    map[tasks.Key]commonpb.DataBlob
+		TimerTasks       map[tasks.Key]commonpb.DataBlob
+		ReplicationTasks map[tasks.Key]commonpb.DataBlob
+		VisibilityTasks  map[tasks.Key]commonpb.DataBlob
 
 		Condition int64
 
@@ -418,10 +445,10 @@ type (
 		SignalInfos         map[int64]*commonpb.DataBlob
 		SignalRequestedIDs  map[string]struct{}
 
-		TransferTasks    []Task
-		TimerTasks       []Task
-		ReplicationTasks []Task
-		VisibilityTasks  []Task
+		TransferTasks    map[tasks.Key]commonpb.DataBlob
+		TimerTasks       map[tasks.Key]commonpb.DataBlob
+		ReplicationTasks map[tasks.Key]commonpb.DataBlob
+		VisibilityTasks  map[tasks.Key]commonpb.DataBlob
 
 		Condition int64
 
@@ -429,9 +456,8 @@ type (
 	}
 
 	InternalGetCurrentExecutionResponse struct {
-		RunID            string
-		ExecutionState   *persistencespb.WorkflowExecutionState
-		LastWriteVersion int64
+		RunID          string
+		ExecutionState *persistencespb.WorkflowExecutionState
 	}
 
 	// InternalHistoryNode represent a history node metadata
@@ -474,6 +500,53 @@ type (
 		NextPageToken []byte
 	}
 
+	InternalGetTransferTaskResponse struct {
+		Task commonpb.DataBlob
+	}
+
+	InternalGetTransferTasksResponse struct {
+		Tasks         []commonpb.DataBlob
+		NextPageToken []byte
+	}
+
+	InternalGetTimerTaskResponse struct {
+		Task commonpb.DataBlob
+	}
+
+	InternalGetTimerTasksResponse struct {
+		Tasks         []commonpb.DataBlob
+		NextPageToken []byte
+	}
+
+	InternalGetReplicationTaskResponse struct {
+		Task commonpb.DataBlob
+	}
+
+	InternalGetReplicationTasksResponse struct {
+		Tasks         []commonpb.DataBlob
+		NextPageToken []byte
+	}
+
+	InternalGetReplicationTasksFromDLQResponse = InternalGetReplicationTasksResponse
+
+	InternalGetVisibilityTaskResponse struct {
+		Task commonpb.DataBlob
+	}
+
+	InternalGetVisibilityTasksResponse struct {
+		Tasks         []commonpb.DataBlob
+		NextPageToken []byte
+	}
+
+	InternalGetTieredStorageTaskResponse struct {
+		Task commonpb.DataBlob
+	}
+
+	InternalGetTieredStorageTasksResponse struct {
+		Tasks         []commonpb.DataBlob
+		NextPageToken []byte
+	}
+
 	// InternalForkHistoryBranchRequest is used to fork a history branch
 	InternalForkHistoryBranchRequest struct {
 		// The base branch to fork from
@@ -607,6 +680,7 @@ type (
 		Name                string
 		Namespace           *commonpb.DataBlob
 		NotificationVersion int64
+		IsGlobal            bool
 	}
 
 	// InternalListNamespacesResponse is the response for GetNamespace
@@ -615,39 +689,43 @@ type (
 		NextPageToken []byte
 	}
 
-	// InternalInitializeImmutableClusterMetadataRequest is a request of InitializeImmutableClusterMetadata
-	// These values can only be set a single time upon cluster initialization.
-	InternalInitializeImmutableClusterMetadataRequest struct {
-		// Serialized ImmutableCusterMetadata to persist.
-		ImmutableClusterMetadata *commonpb.DataBlob
+	// InternalListClusterMetadataRequest is the request for ListClusterMetadata
+	InternalListClusterMetadataRequest struct {
+		PageSize      int
+		NextPageToken []byte
 	}
 
-	// InternalInitializeImmutableClusterMetadataResponse is a request of InitializeImmutableClusterMetadata
-	InternalInitializeImmutableClusterMetadataResponse struct {
-		// Serialized ImmutableCusterMetadata that is currently persisted.
-		PersistedImmutableMetadata *commonpb.DataBlob
-		RequestApplied             bool
+	// InternalListClusterMetadataResponse is the response for ListClusterMetadata
+	InternalListClusterMetadataResponse struct {
+		ClusterMetadata []*InternalGetClusterMetadataResponse
+		NextPageToken   []byte
 	}
 
-	// InternalGetImmutableClusterMetadataResponse is the response to GetImmutableClusterMetadata
-	// These values are set a single time upon cluster initialization.
-	InternalGetImmutableClusterMetadataResponse struct {
-		// Serialized ImmutableCusterMetadata.
-		ImmutableClusterMetadata *commonpb.DataBlob
+	// InternalGetClusterMetadataRequest is the request for GetClusterMetadata
+	InternalGetClusterMetadataRequest struct {
+		ClusterName string
 	}
 
+	// InternalGetClusterMetadataResponse is the response for GetClusterMetadata
 	InternalGetClusterMetadataResponse struct {
 		// Serialized MutableCusterMetadata.
 		ClusterMetadata *commonpb.DataBlob
 		Version         int64
 	}
 
+	// InternalSaveClusterMetadataRequest is the request for SaveClusterMetadata
 	InternalSaveClusterMetadataRequest struct {
+		ClusterName string
 		// Serialized MutableCusterMetadata.
 		ClusterMetadata *commonpb.DataBlob
 		Version         int64
 	}
 
+	// InternalDeleteClusterMetadataRequest is the request for DeleteClusterMetadata
+	InternalDeleteClusterMetadataRequest struct {
+		ClusterName string
+	}
+
 	// InternalUpsertClusterMembershipRequest is the request to UpsertClusterMembership
 	InternalUpsertClusterMembershipRequest struct {
 		ClusterMember
