@@ -31,9 +31,9 @@ import (
 	"math"
 	"time"
 
+	commonpb "go.temporal.io/api/common/v1"
 	"go.temporal.io/api/serviceerror"
 
-	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common/collection"
 	"go.temporal.io/server/common/persistence"
 	p "go.temporal.io/server/common/persistence"
@@ -42,7 +42,7 @@ import (
 )
 
 func (m *sqlExecutionStore) AddTasks(
-	request *p.AddTasksRequest,
+	request *p.InternalAddTasksRequest,
 ) error {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
@@ -54,12 +54,9 @@ func (m *sqlExecutionStore) AddTasks(
 			return applyTasks(ctx,
 				tx,
 				request.ShardID,
-				request.NamespaceID,
-				request.WorkflowID,
-				request.RunID,
 				request.TransferTasks,
-				request.ReplicationTasks,
 				request.TimerTasks,
+				request.ReplicationTasks,
 				request.VisibilityTasks,
 			)
 		})
@@ -67,7 +64,7 @@ func (m *sqlExecutionStore) AddTasks(
 
 func (m *sqlExecutionStore) GetTransferTask(
 	request *persistence.GetTransferTaskRequest,
-) (*persistence.GetTransferTaskResponse, error) {
+) (*persistence.InternalGetTransferTaskResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	rows, err := m.Db.SelectFromTransferTasks(ctx, sqlplugin.TransferTasksFilter{
@@ -78,27 +75,23 @@ func (m *sqlExecutionStore) GetTransferTask(
 		if err == sql.ErrNoRows {
 			return nil, serviceerror.NewNotFound(fmt.Sprintf("GetTransferTask operation failed. Task with ID %v not found. Error: %v", request.TaskID, err))
 		}
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetTransferTask operation failed. Failed to get record. TaskId: %v. Error: %v", request.TaskID, err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetTransferTask operation failed. Failed to get record. TaskId: %v. Error: %v", request.TaskID, err))
 	}
 
 	if len(rows) == 0 {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetTransferTask operation failed. Failed to get record. TaskId: %v", request.TaskID))
+		return nil, serviceerror.NewNotFound(fmt.Sprintf("GetTransferTask operation failed. Failed to get record. TaskId: %v", request.TaskID))
 	}
 
 	transferRow := rows[0]
-	transferInfo, err := serialization.TransferTaskInfoFromBlob(transferRow.Data, transferRow.DataEncoding)
-	if err != nil {
-		return nil, err
+	resp := &persistence.InternalGetTransferTaskResponse{
+		Task: *persistence.NewDataBlob(transferRow.Data, transferRow.DataEncoding),
 	}
-
-	resp := &persistence.GetTransferTaskResponse{TransferTaskInfo: transferInfo}
-
 	return resp, nil
 }
 
 func (m *sqlExecutionStore) GetTransferTasks(
 	request *p.GetTransferTasksRequest,
-) (*p.GetTransferTasksResponse, error) {
+) (*p.InternalGetTransferTasksResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	rows, err := m.Db.RangeSelectFromTransferTasks(ctx, sqlplugin.TransferTasksRangeFilter{
@@ -108,18 +101,13 @@ func (m *sqlExecutionStore) GetTransferTasks(
 	})
 	if err != nil {
 		if err != sql.ErrNoRows {
-			return nil, serviceerror.NewInternal(fmt.Sprintf("GetTransferTasks operation failed. Select failed. Error: %v", err))
+			return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetTransferTasks operation failed. Select failed. Error: %v", err))
 		}
 	}
-	resp := &p.GetTransferTasksResponse{Tasks: make([]*persistencespb.TransferTaskInfo, len(rows))}
+	resp := &p.InternalGetTransferTasksResponse{Tasks: make([]commonpb.DataBlob, len(rows))}
 	for i, row := range rows {
-		info, err := serialization.TransferTaskInfoFromBlob(row.Data, row.DataEncoding)
-		if err != nil {
-			return nil, err
-		}
-		resp.Tasks[i] = info
+		resp.Tasks[i] = *persistence.NewDataBlob(row.Data, row.DataEncoding)
 	}
-
 	return resp, nil
 }
 
@@ -132,7 +120,7 @@ func (m *sqlExecutionStore) CompleteTransferTask(
 		ShardID: request.ShardID,
 		TaskID:  request.TaskID,
 	}); err != nil {
-		return serviceerror.NewInternal(fmt.Sprintf("CompleteTransferTask operation failed. Error: %v", err))
+		return serviceerror.NewUnavailable(fmt.Sprintf("CompleteTransferTask operation failed. Error: %v", err))
 	}
 	return nil
 }
@@ -147,14 +135,14 @@ func (m *sqlExecutionStore) RangeCompleteTransferTask(
 		MinTaskID: request.ExclusiveBeginTaskID,
 		MaxTaskID: request.InclusiveEndTaskID,
 	}); err != nil {
-		return serviceerror.NewInternal(fmt.Sprintf("RangeCompleteTransferTask operation failed. Error: %v", err))
+		return serviceerror.NewUnavailable(fmt.Sprintf("RangeCompleteTransferTask operation failed. Error: %v", err))
 	}
 	return nil
 }
 
 func (m *sqlExecutionStore) GetTimerTask(
 	request *persistence.GetTimerTaskRequest,
-) (*persistence.GetTimerTaskResponse, error) {
+) (*persistence.InternalGetTimerTaskResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	rows, err := m.Db.SelectFromTimerTasks(ctx, sqlplugin.TimerTasksFilter{
@@ -166,27 +154,23 @@ func (m *sqlExecutionStore) GetTimerTask(
 		if err == sql.ErrNoRows {
 			return nil, serviceerror.NewNotFound(fmt.Sprintf("GetTimerTask operation failed. Task with ID %v not found. Error: %v", request.TaskID, err))
 		}
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetTimerTask operation failed. Failed to get record. TaskId: %v. Error: %v", request.TaskID, err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetTimerTask operation failed. Failed to get record. TaskId: %v. Error: %v", request.TaskID, err))
 	}
 
 	if len(rows) == 0 {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetTimerTask operation failed. Failed to get record. TaskId: %v", request.TaskID))
+		return nil, serviceerror.NewNotFound(fmt.Sprintf("GetTimerTask operation failed. Failed to get record. TaskId: %v", request.TaskID))
 	}
 
 	timerRow := rows[0]
-	timerInfo, err := serialization.TimerTaskInfoFromBlob(timerRow.Data, timerRow.DataEncoding)
-	if err != nil {
-		return nil, err
+	resp := &persistence.InternalGetTimerTaskResponse{
+		Task: *p.NewDataBlob(timerRow.Data, timerRow.DataEncoding),
 	}
-
-	resp := &persistence.GetTimerTaskResponse{TimerTaskInfo: timerInfo}
-
 	return resp, nil
 }
 
-func (m *sqlExecutionStore) GetTimerIndexTasks(
-	request *p.GetTimerIndexTasksRequest,
-) (*p.GetTimerIndexTasksResponse, error) {
+func (m *sqlExecutionStore) GetTimerTasks(
+	request *p.GetTimerTasksRequest,
+) (*p.InternalGetTimerTasksResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	pageToken := &timerTaskPageToken{TaskID: math.MinInt64, Timestamp: request.MinTimestamp}
@@ -205,29 +189,22 @@ func (m *sqlExecutionStore) GetTimerIndexTasks(
 	})
 
 	if err != nil && err != sql.ErrNoRows {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetTimerTasks operation failed. Select failed. Error: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetTimerTasks operation failed. Select failed. Error: %v", err))
 	}
 
-	resp := &p.GetTimerIndexTasksResponse{Timers: make([]*persistencespb.TimerTaskInfo, len(rows))}
+	resp := &p.InternalGetTimerTasksResponse{Tasks: make([]commonpb.DataBlob, len(rows))}
 	for i, row := range rows {
-		info, err := serialization.TimerTaskInfoFromBlob(row.Data, row.DataEncoding)
-		if err != nil {
-			return nil, err
-		}
-		resp.Timers[i] = info
+		resp.Tasks[i] = *p.NewDataBlob(row.Data, row.DataEncoding)
 	}
 
-	if len(resp.Timers) > request.BatchSize {
-		goVisibilityTimestamp := resp.Timers[request.BatchSize].VisibilityTime
-		if goVisibilityTimestamp == nil {
-			return nil, serviceerror.NewInternal(fmt.Sprintf("GetTimerTasks: time for page token is nil - TaskId '%v'", resp.Timers[request.BatchSize].TaskId))
-		}
-
+	// above use page size + 1 for query, so if this check is true
+	// there is definitely a next page
+	if len(resp.Tasks) > request.BatchSize {
 		pageToken = &timerTaskPageToken{
-			TaskID:    resp.Timers[request.BatchSize].GetTaskId(),
-			Timestamp: *goVisibilityTimestamp,
+			TaskID:    rows[request.BatchSize].TaskID,
+			Timestamp: rows[request.BatchSize].VisibilityTimestamp,
 		}
-		resp.Timers = resp.Timers[:request.BatchSize]
+		resp.Tasks = resp.Tasks[:request.BatchSize]
 		nextToken, err := pageToken.serialize()
 		if err != nil {
 			return nil, serviceerror.NewInternal(fmt.Sprintf("GetTimerTasks: error serializing page token: %v", err))
@@ -248,7 +225,7 @@ func (m *sqlExecutionStore) CompleteTimerTask(
 		VisibilityTimestamp: request.VisibilityTimestamp,
 		TaskID:              request.TaskID,
 	}); err != nil {
-		return serviceerror.NewInternal(fmt.Sprintf("CompleteTimerTask operation failed. Error: %v", err))
+		return serviceerror.NewUnavailable(fmt.Sprintf("CompleteTimerTask operation failed. Error: %v", err))
 	}
 	return nil
 }
@@ -265,14 +242,14 @@ func (m *sqlExecutionStore) RangeCompleteTimerTask(
 		MinVisibilityTimestamp: start,
 		MaxVisibilityTimestamp: end,
 	}); err != nil {
-		return serviceerror.NewInternal(fmt.Sprintf("CompleteTimerTask operation failed. Error: %v", err))
+		return serviceerror.NewUnavailable(fmt.Sprintf("CompleteTimerTask operation failed. Error: %v", err))
 	}
 	return nil
 }
 
 func (m *sqlExecutionStore) GetReplicationTask(
 	request *persistence.GetReplicationTaskRequest,
-) (*persistence.GetReplicationTaskResponse, error) {
+) (*persistence.InternalGetReplicationTaskResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	rows, err := m.Db.SelectFromReplicationTasks(ctx, sqlplugin.ReplicationTasksFilter{
@@ -283,27 +260,21 @@ func (m *sqlExecutionStore) GetReplicationTask(
 		if err == sql.ErrNoRows {
 			return nil, serviceerror.NewNotFound(fmt.Sprintf("GetReplicationTask operation failed. Task with ID %v not found. Error: %v", request.TaskID, err))
 		}
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetReplicationTask operation failed. Failed to get record. TaskId: %v. Error: %v", request.TaskID, err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetReplicationTask operation failed. Failed to get record. TaskId: %v. Error: %v", request.TaskID, err))
 	}
 
 	if len(rows) == 0 {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetReplicationTask operation failed. Failed to get record. TaskId: %v", request.TaskID))
+		return nil, serviceerror.NewNotFound(fmt.Sprintf("GetReplicationTask operation failed. Failed to get record. TaskId: %v", request.TaskID))
 	}
 
 	replicationRow := rows[0]
-	replicationInfo, err := serialization.ReplicationTaskInfoFromBlob(replicationRow.Data, replicationRow.DataEncoding)
-	if err != nil {
-		return nil, err
-	}
-
-	resp := &persistence.GetReplicationTaskResponse{ReplicationTaskInfo: replicationInfo}
-
+	resp := &persistence.InternalGetReplicationTaskResponse{Task: *p.NewDataBlob(replicationRow.Data, replicationRow.DataEncoding)}
 	return resp, nil
 }
 
 func (m *sqlExecutionStore) GetReplicationTasks(
 	request *p.GetReplicationTasksRequest,
-) (*p.GetReplicationTasksResponse, error) {
+) (*p.InternalGetReplicationTasksResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	readLevel, maxReadLevelInclusive, err := getReadLevels(request)
@@ -322,9 +293,9 @@ func (m *sqlExecutionStore) GetReplicationTasks(
 	case nil:
 		return m.populateGetReplicationTasksResponse(rows, request.MaxTaskID)
 	case sql.ErrNoRows:
-		return &p.GetReplicationTasksResponse{}, nil
+		return &p.InternalGetReplicationTasksResponse{}, nil
 	default:
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetReplicationTasks operation failed. Select failed: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetReplicationTasks operation failed. Select failed: %v", err))
 	}
 }
 
@@ -346,26 +317,21 @@ func getReadLevels(
 func (m *sqlExecutionStore) populateGetReplicationTasksResponse(
 	rows []sqlplugin.ReplicationTasksRow,
 	requestMaxReadLevel int64,
-) (*p.GetReplicationTasksResponse, error) {
+) (*p.InternalGetReplicationTasksResponse, error) {
 	if len(rows) == 0 {
-		return &p.GetReplicationTasksResponse{}, nil
+		return &p.InternalGetReplicationTasksResponse{}, nil
 	}
 
-	var tasks = make([]*persistencespb.ReplicationTaskInfo, len(rows))
+	var tasks = make([]commonpb.DataBlob, len(rows))
 	for i, row := range rows {
-		info, err := serialization.ReplicationTaskInfoFromBlob(row.Data, row.DataEncoding)
-		if err != nil {
-			return nil, err
-		}
-
-		tasks[i] = info
+		tasks[i] = *p.NewDataBlob(row.Data, row.DataEncoding)
 	}
 	var nextPageToken []byte
 	lastTaskID := rows[len(rows)-1].TaskID
 	if lastTaskID < requestMaxReadLevel {
 		nextPageToken = serializePageToken(lastTaskID)
 	}
-	return &p.GetReplicationTasksResponse{
+	return &p.InternalGetReplicationTasksResponse{
 		Tasks:         tasks,
 		NextPageToken: nextPageToken,
 	}, nil
@@ -374,26 +340,21 @@ func (m *sqlExecutionStore) populateGetReplicationTasksResponse(
 func (m *sqlExecutionStore) populateGetReplicationDLQTasksResponse(
 	rows []sqlplugin.ReplicationDLQTasksRow,
 	requestMaxReadLevel int64,
-) (*p.GetReplicationTasksResponse, error) {
+) (*p.InternalGetReplicationTasksResponse, error) {
 	if len(rows) == 0 {
-		return &p.GetReplicationTasksResponse{}, nil
+		return &p.InternalGetReplicationTasksResponse{}, nil
 	}
 
-	var tasks = make([]*persistencespb.ReplicationTaskInfo, len(rows))
+	var tasks = make([]commonpb.DataBlob, len(rows))
 	for i, row := range rows {
-		info, err := serialization.ReplicationTaskInfoFromBlob(row.Data, row.DataEncoding)
-		if err != nil {
-			return nil, err
-		}
-
-		tasks[i] = info
+		tasks[i] = *p.NewDataBlob(row.Data, row.DataEncoding)
 	}
 	var nextPageToken []byte
 	lastTaskID := rows[len(rows)-1].TaskID
 	if lastTaskID < requestMaxReadLevel {
 		nextPageToken = serializePageToken(lastTaskID)
 	}
-	return &p.GetReplicationTasksResponse{
+	return &p.InternalGetReplicationTasksResponse{
 		Tasks:         tasks,
 		NextPageToken: nextPageToken,
 	}, nil
@@ -408,7 +369,7 @@ func (m *sqlExecutionStore) CompleteReplicationTask(
 		ShardID: request.ShardID,
 		TaskID:  request.TaskID,
 	}); err != nil {
-		return serviceerror.NewInternal(fmt.Sprintf("CompleteReplicationTask operation failed. Error: %v", err))
+		return serviceerror.NewUnavailable(fmt.Sprintf("CompleteReplicationTask operation failed. Error: %v", err))
 	}
 	return nil
 }
@@ -423,7 +384,7 @@ func (m *sqlExecutionStore) RangeCompleteReplicationTask(
 		MinTaskID: 0,
 		MaxTaskID: request.InclusiveEndTaskID,
 	}); err != nil {
-		return serviceerror.NewInternal(fmt.Sprintf("RangeCompleteReplicationTask operation failed. Error: %v", err))
+		return serviceerror.NewUnavailable(fmt.Sprintf("RangeCompleteReplicationTask operation failed. Error: %v", err))
 	}
 	return nil
 }
@@ -451,7 +412,7 @@ func (m *sqlExecutionStore) PutReplicationTaskToDLQ(
 	// Tasks are immutable. So it's fine if we already persisted it before.
 	// This can happen when tasks are retried (ack and cleanup can have lag on source side).
 	if err != nil && !m.Db.IsDupEntryError(err) {
-		return serviceerror.NewInternal(fmt.Sprintf("Failed to create replication tasks. Error: %v", err))
+		return serviceerror.NewUnavailable(fmt.Sprintf("Failed to create replication tasks. Error: %v", err))
 	}
 
 	return nil
@@ -459,7 +420,7 @@ func (m *sqlExecutionStore) PutReplicationTaskToDLQ(
 
 func (m *sqlExecutionStore) GetReplicationTasksFromDLQ(
 	request *p.GetReplicationTasksFromDLQRequest,
-) (*p.GetReplicationTasksFromDLQResponse, error) {
+) (*p.InternalGetReplicationTasksFromDLQResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	readLevel, maxReadLevelInclusive, err := getReadLevels(&request.GetReplicationTasksRequest)
@@ -479,9 +440,9 @@ func (m *sqlExecutionStore) GetReplicationTasksFromDLQ(
 	case nil:
 		return m.populateGetReplicationDLQTasksResponse(rows, request.MaxTaskID)
 	case sql.ErrNoRows:
-		return &p.GetReplicationTasksResponse{}, nil
+		return &p.InternalGetReplicationTasksResponse{}, nil
 	default:
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetReplicationTasks operation failed. Select failed: %v", err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetReplicationTasks operation failed. Select failed: %v", err))
 	}
 }
 
@@ -518,7 +479,7 @@ func (m *sqlExecutionStore) RangeDeleteReplicationTaskFromDLQ(
 
 func (m *sqlExecutionStore) GetVisibilityTask(
 	request *persistence.GetVisibilityTaskRequest,
-) (*persistence.GetVisibilityTaskResponse, error) {
+) (*persistence.InternalGetVisibilityTaskResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	rows, err := m.Db.SelectFromVisibilityTasks(ctx, sqlplugin.VisibilityTasksFilter{
@@ -529,27 +490,21 @@ func (m *sqlExecutionStore) GetVisibilityTask(
 		if err == sql.ErrNoRows {
 			return nil, serviceerror.NewNotFound(fmt.Sprintf("GetVisibilityTask operation failed. Task with ID %v not found. Error: %v", request.TaskID, err))
 		}
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetVisibilityTask operation failed. Failed to get record. TaskId: %v. Error: %v", request.TaskID, err))
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetVisibilityTask operation failed. Failed to get record. TaskId: %v. Error: %v", request.TaskID, err))
 	}
 
 	if len(rows) == 0 {
-		return nil, serviceerror.NewInternal(fmt.Sprintf("GetVisibilityTask operation failed. Failed to get record. TaskId: %v", request.TaskID))
+		return nil, serviceerror.NewNotFound(fmt.Sprintf("GetVisibilityTask operation failed. Failed to get record. TaskId: %v", request.TaskID))
 	}
 
 	visibilityRow := rows[0]
-	visibilityInfo, err := serialization.VisibilityTaskInfoFromBlob(visibilityRow.Data, visibilityRow.DataEncoding)
-	if err != nil {
-		return nil, err
-	}
-
-	resp := &persistence.GetVisibilityTaskResponse{VisibilityTaskInfo: visibilityInfo}
-
+	resp := &persistence.InternalGetVisibilityTaskResponse{Task: *p.NewDataBlob(visibilityRow.Data, visibilityRow.DataEncoding)}
 	return resp, nil
 }
 
 func (m *sqlExecutionStore) GetVisibilityTasks(
 	request *p.GetVisibilityTasksRequest,
-) (*p.GetVisibilityTasksResponse, error) {
+) (*p.InternalGetVisibilityTasksResponse, error) {
 	ctx, cancel := newExecutionContext()
 	defer cancel()
 	rows, err := m.Db.RangeSelectFromVisibilityTasks(ctx, sqlplugin.VisibilityTasksRangeFilter{
@@ -559,18 +514,13 @@ func (m *sqlExecutionStore) GetVisibilityTasks(
 	})
 	if err != nil {
 		if err != sql.ErrNoRows {
-			return nil, serviceerror.NewInternal(fmt.Sprintf("GetVisibilityTasks operation failed. Select failed. Error: %v", err))
+			return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetVisibilityTasks operation failed. Select failed. Error: %v", err))
 		}
 	}
-	resp := &p.GetVisibilityTasksResponse{Tasks: make([]*persistencespb.VisibilityTaskInfo, len(rows))}
+	resp := &p.InternalGetVisibilityTasksResponse{Tasks: make([]commonpb.DataBlob, len(rows))}
 	for i, row := range rows {
-		info, err := serialization.VisibilityTaskInfoFromBlob(row.Data, row.DataEncoding)
-		if err != nil {
-			return nil, err
-		}
-		resp.Tasks[i] = info
+		resp.Tasks[i] = *p.NewDataBlob(row.Data, row.DataEncoding)
 	}
-
 	return resp, nil
 }
 
@@ -583,7 +533,7 @@ func (m *sqlExecutionStore) CompleteVisibilityTask(
 		ShardID: request.ShardID,
 		TaskID:  request.TaskID,
 	}); err != nil {
-		return serviceerror.NewInternal(fmt.Sprintf("CompleteVisibilityTask operation failed. Error: %v", err))
+		return serviceerror.NewUnavailable(fmt.Sprintf("CompleteVisibilityTask operation failed. Error: %v", err))
 	}
 	return nil
 }
@@ -598,7 +548,83 @@ func (m *sqlExecutionStore) RangeCompleteVisibilityTask(
 		MinTaskID: request.ExclusiveBeginTaskID,
 		MaxTaskID: request.InclusiveEndTaskID,
 	}); err != nil {
-		return serviceerror.NewInternal(fmt.Sprintf("RangeCompleteVisibilityTask operation failed. Error: %v", err))
+		return serviceerror.NewUnavailable(fmt.Sprintf("RangeCompleteVisibilityTask operation failed. Error: %v", err))
+	}
+	return nil
+}
+
+func (m *sqlExecutionStore) GetTieredStorageTask(
+	request *persistence.GetTieredStorageTaskRequest,
+) (*persistence.InternalGetTieredStorageTaskResponse, error) {
+	ctx, cancel := newExecutionContext()
+	defer cancel()
+	rows, err := m.Db.SelectFromTieredStorageTasks(ctx, sqlplugin.TieredStorageTasksFilter{
+		ShardID: request.ShardID,
+		TaskID:  request.TaskID,
+	})
+	if err != nil {
+		if err == sql.ErrNoRows {
+			return nil, serviceerror.NewNotFound(fmt.Sprintf("GetTieredStorageTask operation failed. Task with ID %v not found. Error: %v", request.TaskID, err))
+		}
+		return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetTieredStorageTask operation failed. Failed to get record. TaskId: %v. Error: %v", request.TaskID, err))
+	}
+
+	if len(rows) == 0 {
+		return nil, serviceerror.NewNotFound(fmt.Sprintf("GetTieredStorageTask operation failed. Failed to get record. TaskId: %v", request.TaskID))
+	}
+
+	TieredStorageRow := rows[0]
+	resp := &persistence.InternalGetTieredStorageTaskResponse{Task: *p.NewDataBlob(TieredStorageRow.Data, TieredStorageRow.DataEncoding)}
+	return resp, nil
+}
+
+func (m *sqlExecutionStore) GetTieredStorageTasks(
+	request *p.GetTieredStorageTasksRequest,
+) (*p.InternalGetTieredStorageTasksResponse, error) {
+	ctx, cancel := newExecutionContext()
+	defer cancel()
+	rows, err := m.Db.RangeSelectFromTieredStorageTasks(ctx, sqlplugin.TieredStorageTasksRangeFilter{
+		ShardID:   request.ShardID,
+		MinTaskID: request.MinTaskID,
+		MaxTaskID: request.MaxTaskID,
+	})
+	if err != nil {
+		if err != sql.ErrNoRows {
+			return nil, serviceerror.NewUnavailable(fmt.Sprintf("GetTieredStorageTasks operation failed. Select failed. Error: %v", err))
+		}
+	}
+	resp := &p.InternalGetTieredStorageTasksResponse{Tasks: make([]commonpb.DataBlob, len(rows))}
+	for i, row := range rows {
+		resp.Tasks[i] = *p.NewDataBlob(row.Data, row.DataEncoding)
+	}
+	return resp, nil
+}
+
+func (m *sqlExecutionStore) CompleteTieredStorageTask(
+	request *p.CompleteTieredStorageTaskRequest,
+) error {
+	ctx, cancel := newExecutionContext()
+	defer cancel()
+	if _, err := m.Db.DeleteFromTieredStorageTasks(ctx, sqlplugin.TieredStorageTasksFilter{
+		ShardID: request.ShardID,
+		TaskID:  request.TaskID,
+	}); err != nil {
+		return serviceerror.NewUnavailable(fmt.Sprintf("CompleteTieredStorageTask operation failed. Error: %v", err))
+	}
+	return nil
+}
+
+func (m *sqlExecutionStore) RangeCompleteTieredStorageTask(
+	request *p.RangeCompleteTieredStorageTaskRequest,
+) error {
+	ctx, cancel := newExecutionContext()
+	defer cancel()
+	if _, err := m.Db.RangeDeleteFromTieredStorageTasks(ctx, sqlplugin.TieredStorageTasksRangeFilter{
+		ShardID:   request.ShardID,
+		MinTaskID: request.ExclusiveBeginTaskID,
+		MaxTaskID: request.InclusiveEndTaskID,
+	}); err != nil {
+		return serviceerror.NewUnavailable(fmt.Sprintf("RangeCompleteTieredStorageTask operation failed. Error: %v", err))
 	}
 	return nil
 }
