@@ -114,9 +114,6 @@ func (s *TestCluster) Config() config.Persistence {
 			"test": {SQL: &cfg},
 		},
 		TransactionSizeLimit: dynamicconfig.GetIntPropertyFn(common.DefaultTransactionSizeLimit),
-		VisibilityConfig: &config.VisibilityConfig{
-			EnableSampling: dynamicconfig.GetBoolPropertyFn(false),
-		},
 	}
 }
 
