@@ -54,17 +54,17 @@ const (
 	readLockExecutionQuery  = lockExecutionQueryBase + ` FOR SHARE`
 
 	createCurrentExecutionQuery = `INSERT INTO current_executions
-(shard_id, namespace_id, workflow_id, run_id, create_request_id, state, status, start_version, last_write_version) VALUES
-(:shard_id, :namespace_id, :workflow_id, :run_id, :create_request_id, :state, :status, :start_version, :last_write_version)`
+(shard_id, namespace_id, workflow_id, run_id, create_request_id, state, status, last_write_version) VALUES
+(:shard_id, :namespace_id, :workflow_id, :run_id, :create_request_id, :state, :status, :last_write_version)`
 
 	deleteCurrentExecutionQuery = "DELETE FROM current_executions WHERE shard_id = $1 AND namespace_id = $2 AND workflow_id = $3 AND run_id = $4"
 
 	getCurrentExecutionQuery = `SELECT
-shard_id, namespace_id, workflow_id, run_id, create_request_id, state, status, start_version, last_write_version
+shard_id, namespace_id, workflow_id, run_id, create_request_id, state, status, last_write_version
 FROM current_executions WHERE shard_id = $1 AND namespace_id = $2 AND workflow_id = $3`
 
 	lockCurrentExecutionJoinExecutionsQuery = `SELECT
-ce.shard_id, ce.namespace_id, ce.workflow_id, ce.run_id, ce.create_request_id, ce.state, ce.status, ce.start_version, e.last_write_version
+ce.shard_id, ce.namespace_id, ce.workflow_id, ce.run_id, ce.create_request_id, ce.state, ce.status, e.last_write_version
 FROM current_executions ce
 INNER JOIN executions e ON e.shard_id = ce.shard_id AND e.namespace_id = ce.namespace_id AND e.workflow_id = ce.workflow_id AND e.run_id = ce.run_id
 WHERE ce.shard_id = $1 AND ce.namespace_id = $2 AND ce.workflow_id = $3 FOR UPDATE`
@@ -76,7 +76,6 @@ run_id = :run_id,
 create_request_id = :create_request_id,
 state = :state,
 status = :status,
-start_version = :start_version,
 last_write_version = :last_write_version
 WHERE
 shard_id = :shard_id AND
@@ -90,7 +89,7 @@ workflow_id = :workflow_id
 	getTransferTaskQuery = `SELECT task_id, data, data_encoding 
  FROM transfer_tasks WHERE shard_id = $1 AND task_id = $2`
 	getTransferTasksQuery = `SELECT task_id, data, data_encoding 
- FROM transfer_tasks WHERE shard_id = $1 AND task_id > $2 AND task_id <= $3 ORDER BY shard_id, task_id`
+ FROM transfer_tasks WHERE shard_id = $1 AND task_id > $2 AND task_id <= $3 ORDER BY task_id`
 
 	deleteTransferTaskQuery      = `DELETE FROM transfer_tasks WHERE shard_id = $1 AND task_id = $2`
 	rangeDeleteTransferTaskQuery = `DELETE FROM transfer_tasks WHERE shard_id = $1 AND task_id > $2 AND task_id <= $3`
@@ -137,11 +136,22 @@ ORDER BY task_id LIMIT $5`
 	getVisibilityTaskQuery = `SELECT task_id, data, data_encoding 
  FROM visibility_tasks WHERE shard_id = $1 AND task_id = $2`
 	getVisibilityTasksQuery = `SELECT task_id, data, data_encoding 
- FROM visibility_tasks WHERE shard_id = $1 AND task_id > $2 AND task_id <= $3 ORDER BY shard_id, task_id`
+ FROM visibility_tasks WHERE shard_id = $1 AND task_id > $2 AND task_id <= $3 ORDER BY task_id`
 
 	deleteVisibilityTaskQuery      = `DELETE FROM visibility_tasks WHERE shard_id = $1 AND task_id = $2`
 	rangeDeleteVisibilityTaskQuery = `DELETE FROM visibility_tasks WHERE shard_id = $1 AND task_id > $2 AND task_id <= $3`
 
+	createTieredStorageTasksQuery = `INSERT INTO tiered_storage_tasks(shard_id, task_id, data, data_encoding) 
+ VALUES(:shard_id, :task_id, :data, :data_encoding)`
+
+	getTieredStorageTaskQuery = `SELECT task_id, data, data_encoding 
+ FROM tiered_storage_tasks WHERE shard_id = $1 AND task_id = $2`
+	getTieredStorageTasksQuery = `SELECT task_id, data, data_encoding 
+ FROM tiered_storage_tasks WHERE shard_id = $1 AND task_id > $2 AND task_id <= $3 ORDER BY task_id`
+
+	deleteTieredStorageTaskQuery      = `DELETE FROM tiered_storage_tasks WHERE shard_id = $1 AND task_id = $2`
+	rangeDeleteTieredStorageTaskQuery = `DELETE FROM tiered_storage_tasks WHERE shard_id = $1 AND task_id > $2 AND task_id <= $3`
+
 	bufferedEventsColumns     = `shard_id, namespace_id, workflow_id, run_id, data, data_encoding`
 	createBufferedEventsQuery = `INSERT INTO buffered_events(` + bufferedEventsColumns + `)
 VALUES (:shard_id, :namespace_id, :workflow_id, :run_id, :data, :data_encoding)`
@@ -778,3 +788,76 @@ func (pdb *db) RangeDeleteFromVisibilityTasks(
 		filter.MaxTaskID,
 	)
 }
+
+// InsertIntoTieredStorageTasks inserts one or more rows into tiered_storage_tasks table
+func (pdb *db) InsertIntoTieredStorageTasks(
+	ctx context.Context,
+	rows []sqlplugin.TieredStorageTasksRow,
+) (sql.Result, error) {
+	return pdb.conn.NamedExecContext(ctx,
+		createTieredStorageTasksQuery,
+		rows,
+	)
+}
+
+// SelectFromTieredStorageTasks reads one or more rows from tiered_storage_tasks table
+func (pdb *db) SelectFromTieredStorageTasks(
+	ctx context.Context,
+	filter sqlplugin.TieredStorageTasksFilter,
+) ([]sqlplugin.TieredStorageTasksRow, error) {
+	var rows []sqlplugin.TieredStorageTasksRow
+	err := pdb.conn.SelectContext(ctx,
+		&rows,
+		getTieredStorageTaskQuery,
+		filter.ShardID,
+		filter.TaskID,
+	)
+	if err != nil {
+		return nil, err
+	}
+	return rows, nil
+}
+
+// RangeSelectFromTieredStorageTasks reads one or more rows from tiered_storage_tasks table
+func (pdb *db) RangeSelectFromTieredStorageTasks(
+	ctx context.Context,
+	filter sqlplugin.TieredStorageTasksRangeFilter,
+) ([]sqlplugin.TieredStorageTasksRow, error) {
+	var rows []sqlplugin.TieredStorageTasksRow
+	err := pdb.conn.SelectContext(ctx,
+		&rows,
+		getTieredStorageTasksQuery,
+		filter.ShardID,
+		filter.MinTaskID,
+		filter.MaxTaskID,
+	)
+	if err != nil {
+		return nil, err
+	}
+	return rows, nil
+}
+
+// DeleteFromTieredStorageTasks deletes one or more rows from tiered_storage_tasks table
+func (pdb *db) DeleteFromTieredStorageTasks(
+	ctx context.Context,
+	filter sqlplugin.TieredStorageTasksFilter,
+) (sql.Result, error) {
+	return pdb.conn.ExecContext(ctx,
+		deleteTieredStorageTaskQuery,
+		filter.ShardID,
+		filter.TaskID,
+	)
+}
+
+// RangeDeleteFromTieredStorageTasks deletes one or more rows from tiered_storage_tasks table
+func (pdb *db) RangeDeleteFromTieredStorageTasks(
+	ctx context.Context,
+	filter sqlplugin.TieredStorageTasksRangeFilter,
+) (sql.Result, error) {
+	return pdb.conn.ExecContext(ctx,
+		rangeDeleteTieredStorageTaskQuery,
+		filter.ShardID,
+		filter.MinTaskID,
+		filter.MaxTaskID,
+	)
+}
