@@ -60,6 +60,34 @@ const (
 	testMySQLVisibilitySchema = "../../../schema/mysql/v57/visibility/schema.sql"
 )
 
+func TestMySQLExecutionMutableStateStoreSuite(t *testing.T) {
+	cfg := NewMySQLConfig()
+	SetupMySQLDatabase(cfg)
+	SetupMySQLSchema(cfg)
+	logger := log.NewNoopLogger()
+	factory := sql.NewFactory(
+		*cfg,
+		resolver.NewNoopResolver(),
+		testMySQLClusterName,
+		logger,
+	)
+	shardStore, err := factory.NewShardStore()
+	if err != nil {
+		t.Fatalf("unable to create MySQL DB: %v", err)
+	}
+	executionStore, err := factory.NewExecutionStore()
+	if err != nil {
+		t.Fatalf("unable to create MySQL DB: %v", err)
+	}
+	defer func() {
+		factory.Close()
+		TearDownMySQLDatabase(cfg)
+	}()
+
+	s := NewExecutionMutableStateSuite(t, shardStore, executionStore, logger)
+	suite.Run(t, s)
+}
+
 func TestMySQLHistoryStoreSuite(t *testing.T) {
 	cfg := NewMySQLConfig()
 	SetupMySQLDatabase(cfg)
@@ -80,7 +108,7 @@ func TestMySQLHistoryStoreSuite(t *testing.T) {
 		TearDownMySQLDatabase(cfg)
 	}()
 
-	s := newHistoryEventsSuite(t, store, logger)
+	s := NewHistoryEventsSuite(t, store, logger)
 	suite.Run(t, s)
 }
 
