@@ -31,13 +31,15 @@ import (
 	"sync/atomic"
 	"time"
 
-	"go.temporal.io/server/common/persistence/serialization"
-	"go.temporal.io/server/common/persistence/visibility"
-
-	"github.com/uber-go/tally"
+	"github.com/uber-go/tally/v4"
 	"github.com/uber/tchannel-go"
 	"go.temporal.io/api/workflowservice/v1"
-	sdkclient "go.temporal.io/sdk/client"
+
+	"go.temporal.io/server/common/namespace"
+	"go.temporal.io/server/common/persistence/serialization"
+	"go.temporal.io/server/common/persistence/visibility/manager"
+	"go.temporal.io/server/common/resolver"
+	"go.temporal.io/server/common/sdk"
 	"go.temporal.io/server/common/searchattribute"
 
 	"go.temporal.io/server/api/adminservice/v1"
@@ -50,7 +52,6 @@ import (
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/archiver"
 	"go.temporal.io/server/common/archiver/provider"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/clock"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/dynamicconfig"
@@ -70,7 +71,7 @@ type (
 		persistenceBean persistenceClient.Bean,
 		searchAttributesProvider searchattribute.Provider,
 		logger log.Logger,
-	) (visibility.VisibilityManager, error)
+	) (manager.VisibilityManager, error)
 
 	// Impl contains all common resources shared across frontend / matching / history / worker
 	Impl struct {
@@ -90,7 +91,7 @@ type (
 
 		// other common resources
 
-		namespaceCache    cache.NamespaceCache
+		namespaceRegistry namespace.Registry
 		timeSource        clock.TimeSource
 		payloadSerializer serialization.Serializer
 		metricsClient     metrics.Client
@@ -107,7 +108,7 @@ type (
 
 		// internal services clients
 
-		sdkClient         sdkclient.Client
+		sdkClientFactory  sdk.ClientFactory
 		frontendRawClient workflowservice.WorkflowServiceClient
 		frontendClient    workflowservice.WorkflowServiceClient
 		matchingRawClient matchingservice.MatchingServiceClient
@@ -115,11 +116,11 @@ type (
 		historyRawClient  historyservice.HistoryServiceClient
 		historyClient     historyservice.HistoryServiceClient
 		clientBean        client.Bean
+		clientFactory     client.Factory
 
 		// persistence clients
-
-		persistenceBean persistenceClient.Bean
-		visibilityMgr   visibility.VisibilityManager
+		persistenceBean           persistenceClient.Bean
+		persistenceFaultInjection *persistenceClient.FaultInjectionDataStoreFactory
 
 		// loggers
 
@@ -142,15 +143,19 @@ var _ Resource = (*Impl)(nil)
 
 // New create a new resource containing common dependencies
 func New(
+	pLogger log.Logger,
 	params *BootstrapParams,
 	serviceName string,
+	dcClient dynamicconfig.Client,
 	persistenceMaxQPS dynamicconfig.IntPropertyFn,
 	persistenceGlobalMaxQPS dynamicconfig.IntPropertyFn,
 	throttledLoggerMaxRPS dynamicconfig.IntPropertyFn,
-	visibilityManagerInitializer VisibilityManagerInitializer,
+	datastoreFactory persistenceClient.AbstractDataStoreFactory,
+	persistenceServiceResolver resolver.ServiceResolver,
+	searchAttributesMapper searchattribute.Mapper,
 ) (impl *Impl, retError error) {
 
-	logger := log.With(params.Logger, tag.Service(serviceName))
+	logger := log.With(pLogger, tag.Service(serviceName))
 	throttledLogger := log.NewThrottledLogger(logger,
 		func() float64 { return float64(throttledLoggerMaxRPS()) })
 
@@ -164,9 +169,9 @@ func New(
 
 	ringpopChannel := params.RPCFactory.GetRingpopChannel()
 
-	persistenceBean, err := persistenceClient.NewBeanFromFactory(persistenceClient.NewFactory(
+	factory := persistenceClient.NewFactoryImpl(
 		&params.PersistenceConfig,
-		params.PersistenceServiceResolver,
+		persistenceServiceResolver,
 		func(...dynamicconfig.FilterOption) int {
 			if persistenceGlobalMaxQPS() > 0 {
 				// TODO: We have a bootstrap issue to correctly find memberCount.  Membership relies on
@@ -180,11 +185,13 @@ func New(
 			}
 			return persistenceMaxQPS()
 		},
-		params.AbstractDatastoreFactory,
+		datastoreFactory,
 		params.ClusterMetadataConfig.CurrentClusterName,
 		params.MetricsClient,
 		logger,
-	))
+	)
+
+	persistenceBean, err := persistenceClient.NewBeanFromFactory(factory)
 	if err != nil {
 		return nil, err
 	}
@@ -195,6 +202,8 @@ func New(
 		params.ClusterMetadataConfig.MasterClusterName,
 		params.ClusterMetadataConfig.CurrentClusterName,
 		params.ClusterMetadataConfig.ClusterInformation,
+		persistenceBean.GetClusterMetadataManager(),
+		logger,
 	)
 
 	membershipFactory, err := params.MembershipFactoryInitializer(persistenceBean, logger)
@@ -207,22 +216,20 @@ func New(
 		return nil, err
 	}
 
-	dynamicCollection := dynamicconfig.NewCollection(params.DynamicConfigClient, logger)
+	dynamicCollection := dynamicconfig.NewCollection(dcClient, logger)
 	factoryProvider := params.ClientFactoryProvider
 	if factoryProvider == nil {
 		factoryProvider = client.NewFactoryProvider()
 	}
-	clientBean, err := client.NewClientBean(
-		factoryProvider.NewFactory(
-			params.RPCFactory,
-			membershipMonitor,
-			params.MetricsClient,
-			dynamicCollection,
-			numShards,
-			logger,
-		),
-		clusterMetadata,
+	clientFactory := factoryProvider.NewFactory(
+		params.RPCFactory,
+		membershipMonitor,
+		params.MetricsClient,
+		dynamicCollection,
+		numShards,
+		logger,
 	)
+	clientBean, err := client.NewClientBean(clientFactory, clusterMetadata)
 	if err != nil {
 		return nil, err
 	}
@@ -247,21 +254,12 @@ func New(
 		return nil, err
 	}
 
-	saProvider := persistence.NewSearchAttributesManager(clock.NewRealTimeSource(), persistenceBean.GetClusterMetadataManager())
-	saManager := persistence.NewSearchAttributesManager(clock.NewRealTimeSource(), persistenceBean.GetClusterMetadataManager())
+	saProvider := searchattribute.NewManager(clock.NewRealTimeSource(), persistenceBean.GetClusterMetadataManager())
+	saManager := searchattribute.NewManager(clock.NewRealTimeSource(), persistenceBean.GetClusterMetadataManager())
 
-	visibilityMgr, err := visibilityManagerInitializer(
-		persistenceBean,
-		saProvider,
-		logger,
-	)
-	if err != nil {
-		return nil, err
-	}
-
-	namespaceCache := cache.NewNamespaceCache(
+	namespaceRegistry := namespace.NewRegistry(
 		persistenceBean.GetMetadataManager(),
-		clusterMetadata,
+		clusterMetadata.IsGlobalNamespaceEnabled(),
 		params.MetricsClient,
 		logger,
 	)
@@ -273,7 +271,7 @@ func New(
 		common.IsWhitelistServiceTransientError,
 	)
 
-	matchingRawClient, err := clientBean.GetMatchingClient(namespaceCache.GetNamespaceName)
+	matchingRawClient, err := clientBean.GetMatchingClient(namespaceRegistry.GetNamespaceName)
 	if err != nil {
 		return nil, err
 	}
@@ -295,13 +293,11 @@ func New(
 		Logger:           logger,
 		MetricsClient:    params.MetricsClient,
 		ClusterMetadata:  clusterMetadata,
-		NamespaceCache:   namespaceCache,
 	}
 	visibilityArchiverBootstrapContainer := &archiver.VisibilityBootstrapContainer{
 		Logger:          logger,
 		MetricsClient:   params.MetricsClient,
 		ClusterMetadata: clusterMetadata,
-		NamespaceCache:  namespaceCache,
 	}
 	if err := params.ArchiverProvider.RegisterBootstrapContainer(
 		serviceName,
@@ -323,11 +319,11 @@ func New(
 		clusterMetadata: clusterMetadata,
 		saProvider:      saProvider,
 		saManager:       saManager,
-		saMapper:        params.SearchAttributesMapper,
+		saMapper:        searchAttributesMapper,
 
 		// other common resources
 
-		namespaceCache:    namespaceCache,
+		namespaceRegistry: namespaceRegistry,
 		timeSource:        clock.NewRealTimeSource(),
 		payloadSerializer: serialization.NewSerializer(),
 		metricsClient:     params.MetricsClient,
@@ -344,7 +340,7 @@ func New(
 
 		// internal services clients
 
-		sdkClient:         params.SdkClient,
+		sdkClientFactory:  params.SdkClientFactory,
 		frontendRawClient: frontendRawClient,
 		frontendClient:    frontendClient,
 		matchingRawClient: matchingRawClient,
@@ -352,11 +348,12 @@ func New(
 		historyRawClient:  historyRawClient,
 		historyClient:     historyClient,
 		clientBean:        clientBean,
+		clientFactory:     clientFactory,
 
 		// persistence clients
 
-		persistenceBean: persistenceBean,
-		visibilityMgr:   visibilityMgr,
+		persistenceBean:           persistenceBean,
+		persistenceFaultInjection: factory.FaultInjection(),
 
 		// loggers
 
@@ -395,8 +392,9 @@ func (h *Impl) Start() {
 	h.metricsScope.Counter(metrics.RestartCount).Inc(1)
 	h.runtimeMetricsReporter.Start()
 
+	h.clusterMetadata.Start()
 	h.membershipMonitor.Start()
-	h.namespaceCache.Start()
+	h.namespaceRegistry.Start()
 
 	hostInfo, err := h.membershipMonitor.WhoAmI()
 	if err != nil {
@@ -421,14 +419,13 @@ func (h *Impl) Stop() {
 		return
 	}
 
-	h.namespaceCache.Stop()
+	h.namespaceRegistry.Stop()
 	h.membershipMonitor.Stop()
+	h.clusterMetadata.Stop()
 	h.ringpopChannel.Close()
 	h.runtimeMetricsReporter.Stop()
 	h.persistenceBean.Close()
-	if h.visibilityMgr != nil {
-		h.visibilityMgr.Close()
-	}
+	h.grpcListener.Close()
 }
 
 // GetServiceName return service name
@@ -453,9 +450,9 @@ func (h *Impl) GetClusterMetadata() cluster.Metadata {
 
 // other common resources
 
-// GetNamespaceCache return namespace cache
-func (h *Impl) GetNamespaceCache() cache.NamespaceCache {
-	return h.namespaceCache
+// GetNamespaceRegistry return namespace cache
+func (h *Impl) GetNamespaceRegistry() namespace.Registry {
+	return h.namespaceRegistry
 }
 
 // GetTimeSource return time source
@@ -512,14 +509,9 @@ func (h *Impl) GetWorkerServiceResolver() membership.ServiceResolver {
 
 // internal services clients
 
-// GetSDKClient return sdk client
-func (h *Impl) GetSDKClient() sdkclient.Client {
-	return h.sdkClient
-}
-
-// GetFrontendRawClient return frontend client without retry policy
-func (h *Impl) GetFrontendRawClient() workflowservice.WorkflowServiceClient {
-	return h.frontendRawClient
+// GetSDKClientFactory return sdk client
+func (h *Impl) GetSDKClientFactory() sdk.ClientFactory {
+	return h.sdkClientFactory
 }
 
 // GetFrontendClient return frontend client with retry policy
@@ -568,6 +560,11 @@ func (h *Impl) GetClientBean() client.Bean {
 	return h.clientBean
 }
 
+// GetClientFactory return RPC client factory
+func (h *Impl) GetClientFactory() client.Factory {
+	return h.clientFactory
+}
+
 // persistence clients
 
 // GetMetadataManager return metadata manager
@@ -585,11 +582,6 @@ func (h *Impl) GetTaskManager() persistence.TaskManager {
 	return h.persistenceBean.GetTaskManager()
 }
 
-// GetVisibilityManager return visibility manager
-func (h *Impl) GetVisibilityManager() visibility.VisibilityManager {
-	return h.visibilityMgr
-}
-
 // GetNamespaceReplicationQueue return namespace replication queue
 func (h *Impl) GetNamespaceReplicationQueue() persistence.NamespaceReplicationQueue {
 	return h.persistenceBean.GetNamespaceReplicationQueue()
@@ -638,3 +630,7 @@ func (h *Impl) GetSearchAttributesManager() searchattribute.Manager {
 func (h *Impl) GetSearchAttributesMapper() searchattribute.Mapper {
 	return h.saMapper
 }
+
+func (h *Impl) GetFaultInjection() *persistenceClient.FaultInjectionDataStoreFactory {
+	return h.persistenceFaultInjection
+}
