@@ -31,19 +31,14 @@ import (
 	"sync"
 	"time"
 
-	"github.com/uber-go/tally"
 	"go.temporal.io/server/common/log/tag"
+	"go.temporal.io/server/common/metrics"
 
 	"go.temporal.io/server/common/auth"
 	"go.temporal.io/server/common/config"
 	"go.temporal.io/server/common/log"
 )
 
-const (
-	metricCertsExpired  = "certificates_expired"
-	metricCertsExpiring = "certificates_expiring"
-)
-
 type CertProviderFactory func(
 	tlsSettings *config.GroupTLS,
 	workerTlsSettings *config.WorkerTLS,
@@ -71,13 +66,13 @@ type localStoreTlsProvider struct {
 	ticker *time.Ticker
 	logger log.Logger
 	stop   chan bool
-	scope  tally.Scope
+	scope  metrics.Scope
 }
 
 var _ TLSConfigProvider = (*localStoreTlsProvider)(nil)
 var _ CertExpirationChecker = (*localStoreTlsProvider)(nil)
 
-func NewLocalStoreTlsProvider(tlsConfig *config.RootTLS, scope tally.Scope, logger log.Logger, certProviderFactory CertProviderFactory,
+func NewLocalStoreTlsProvider(tlsConfig *config.RootTLS, scope metrics.Scope, logger log.Logger, certProviderFactory CertProviderFactory,
 ) (TLSConfigProvider, error) {
 
 	internodeProvider := certProviderFactory(&tlsConfig.Internode, nil, nil, tlsConfig.RefreshInterval, logger)
@@ -106,7 +101,6 @@ func NewLocalStoreTlsProvider(tlsConfig *config.RootTLS, scope tally.Scope, logg
 }
 
 func (s *localStoreTlsProvider) initialize() {
-
 	period := s.settings.ExpirationChecks.CheckInterval
 	if period != 0 {
 		s.stop = make(chan bool)
@@ -143,10 +137,13 @@ func (s *localStoreTlsProvider) GetInternodeClientConfig() (*tls.Config, error)
 func (s *localStoreTlsProvider) GetFrontendClientConfig() (*tls.Config, error) {
 
 	var client *config.ClientTLS
+	var useTLS bool
 	if isSystemWorker(s.settings) {
 		client = &s.settings.SystemWorker.Client
+		useTLS = true
 	} else {
 		client = &s.settings.Frontend.Client
+		useTLS = s.settings.Frontend.IsEnabled()
 	}
 	return s.getOrCreateConfig(
 		&s.cachedFrontendClientConfig,
@@ -154,7 +151,7 @@ func (s *localStoreTlsProvider) GetFrontendClientConfig() (*tls.Config, error) {
 			return newClientTLSConfig(s.workerCertProvider, client.ServerName,
 				s.settings.Frontend.Server.RequireClientAuth, true, !client.DisableHostVerification)
 		},
-		s.settings.Frontend.IsEnabled(),
+		useTLS,
 	)
 }
 
@@ -370,7 +367,6 @@ func (s *localStoreTlsProvider) timerCallback() {
 }
 
 func (s *localStoreTlsProvider) checkCertExpiration() {
-
 	defer func() {
 		var retError error
 		log.CapturePanic(s.logger, &retError)
@@ -395,8 +391,8 @@ func (s *localStoreTlsProvider) checkCertExpiration() {
 			return
 		}
 		if s.scope != nil {
-			s.scope.Gauge(metricCertsExpired).Update(float64(len(expired)))
-			s.scope.Gauge(metricCertsExpiring).Update(float64(len(expiring)))
+			s.scope.UpdateGauge(metrics.TlsCertsExpired, float64(len(expired)))
+			s.scope.UpdateGauge(metrics.TlsCertsExpiring, float64(len(expiring)))
 		}
 		s.logCerts(expired, true, errorTime)
 		s.logCerts(expiring, false, errorTime)
@@ -434,5 +430,7 @@ func mergeMaps(to CertExpirationMap, from CertExpirationMap) {
 }
 
 func isSystemWorker(tls *config.RootTLS) bool {
-	return tls.SystemWorker.CertData != "" || tls.SystemWorker.CertFile != ""
+	return tls.SystemWorker.CertData != "" || tls.SystemWorker.CertFile != "" ||
+		len(tls.SystemWorker.Client.RootCAData) > 0 || len(tls.SystemWorker.Client.RootCAFiles) > 0 ||
+		tls.SystemWorker.Client.ForceTLS
 }
