@@ -29,28 +29,28 @@ import (
 	"crypto/md5"
 	"fmt"
 
+	"google.golang.org/grpc"
+
 	"go.temporal.io/server/common/authorization"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
-	"google.golang.org/grpc"
-
-	"go.temporal.io/server/common/cache"
+	"go.temporal.io/server/common/namespace"
 )
 
 type (
 	NamespaceLogInterceptor struct {
-		namespaceCache cache.NamespaceCache
-		logger         log.Logger
+		namespaceRegistry namespace.Registry
+		logger            log.Logger
 	}
 )
 
 var _ grpc.UnaryServerInterceptor = (*NamespaceLogInterceptor)(nil).Intercept
 
-func NewNamespaceLogInterceptor(namespaceCache cache.NamespaceCache, logger log.Logger) *NamespaceLogInterceptor {
+func NewNamespaceLogInterceptor(namespaceRegistry namespace.Registry, logger log.Logger) *NamespaceLogInterceptor {
 
 	return &NamespaceLogInterceptor{
-		namespaceCache: namespaceCache,
-		logger:         logger,
+		namespaceRegistry: namespaceRegistry,
+		logger:            logger,
 	}
 }
 
@@ -63,7 +63,7 @@ func (nli *NamespaceLogInterceptor) Intercept(
 
 	if nli.logger != nil {
 		_, methodName := splitMethodName(info.FullMethod)
-		namespace := GetNamespace(nli.namespaceCache, req)
+		namespace := GetNamespace(nli.namespaceRegistry, req)
 		tlsInfo := authorization.TLSInfoFormContext(ctx)
 		var serverName string
 		var certThumbprint string
@@ -76,7 +76,7 @@ func (nli *NamespaceLogInterceptor) Intercept(
 		}
 		nli.logger.Debug(
 			"Frontend method invoked.",
-			tag.WorkflowNamespace(namespace),
+			tag.WorkflowNamespace(namespace.String()),
 			tag.Operation(methodName),
 			tag.ServerName(serverName),
 			tag.CertThumbprint(certThumbprint))
