@@ -30,7 +30,6 @@ import (
 	"fmt"
 	"math/rand"
 	"sort"
-	"strings"
 	"sync"
 	"time"
 
@@ -42,17 +41,15 @@ import (
 	"go.temporal.io/api/serviceerror"
 	"go.temporal.io/api/workflowservice/v1"
 
-	"go.temporal.io/server/common/number"
-
-	workflowspb "go.temporal.io/server/api/workflow/v1"
-	"go.temporal.io/server/common/metrics"
-
 	"go.temporal.io/server/api/historyservice/v1"
 	"go.temporal.io/server/api/matchingservice/v1"
+	workflowspb "go.temporal.io/server/api/workflow/v1"
 	"go.temporal.io/server/common/backoff"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
+	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/number"
 	"go.temporal.io/server/common/primitives/timestamp"
 	serviceerrors "go.temporal.io/server/common/serviceerror"
 )
@@ -141,8 +138,8 @@ func AwaitWaitGroup(wg *sync.WaitGroup, timeout time.Duration) bool {
 	}
 }
 
-// CreatePersistanceRetryPolicy creates a retry policy for persistence layer operations
-func CreatePersistanceRetryPolicy() backoff.RetryPolicy {
+// CreatePersistenceRetryPolicy creates a retry policy for persistence layer operations
+func CreatePersistenceRetryPolicy() backoff.RetryPolicy {
 	policy := backoff.NewExponentialRetryPolicy(retryPersistenceOperationInitialInterval)
 	policy.SetMaximumInterval(retryPersistenceOperationMaxInterval)
 	policy.SetExpirationInterval(retryPersistenceOperationExpirationInterval)
@@ -207,7 +204,7 @@ func CreateReplicationServiceBusyRetryPolicy() backoff.RetryPolicy {
 // IsPersistenceTransientError checks if the error is a transient persistence error
 func IsPersistenceTransientError(err error) bool {
 	switch err.(type) {
-	case *serviceerror.Internal,
+	case *serviceerror.Unavailable,
 		*serviceerror.ResourceExhausted:
 		return true
 	}
@@ -285,8 +282,8 @@ func WorkflowIDToHistoryShard(
 
 // PrettyPrintHistory prints history in human readable format
 func PrettyPrintHistory(history *historypb.History, logger log.Logger) {
-	fmt.Println("******************************************")
-	fmt.Println("History", proto.MarshalTextString(history))
+	fmt.Println("************** History *******************")
+	fmt.Println(proto.MarshalTextString(history))
 	fmt.Println("******************************************")
 }
 
@@ -552,7 +549,7 @@ func CheckEventBlobSizeLimit(
 	actualSize int,
 	warnLimit int,
 	errorLimit int,
-	namespaceID string,
+	namespace string,
 	workflowID string,
 	runID string,
 	scope metrics.Scope,
@@ -564,8 +561,8 @@ func CheckEventBlobSizeLimit(
 
 	if actualSize > warnLimit {
 		if logger != nil {
-			logger.Warn("Blob size exceeds limit.",
-				tag.WorkflowNamespaceID(namespaceID),
+			logger.Warn("Blob data size exceeds the warning limit.",
+				tag.WorkflowNamespace(namespace),
 				tag.WorkflowID(workflowID),
 				tag.WorkflowRunID(runID),
 				tag.WorkflowSize(int64(actualSize)),
@@ -623,22 +620,6 @@ func ValidateLongPollContextTimeoutIsSet(
 	return deadline, nil
 }
 
-// IsJustOrderByClause return true is query start with order by
-func IsJustOrderByClause(clause string) bool {
-	whereClause := strings.TrimSpace(clause)
-	whereClause = strings.ToLower(whereClause)
-	return strings.HasPrefix(whereClause, "order by")
-}
-
-// GetDefaultAdvancedVisibilityWritingMode get default advancedVisibilityWritingMode based on
-// whether related config exists in static config file.
-func GetDefaultAdvancedVisibilityWritingMode(isAdvancedVisConfigExist bool) string {
-	if isAdvancedVisConfigExist {
-		return AdvancedVisibilityWritingModeOn
-	}
-	return AdvancedVisibilityWritingModeOff
-}
-
 func GetPayloadsMapSize(data map[string]*commonpb.Payloads) int {
 	size := 0
 	for key, payloads := range data {
