@@ -46,9 +46,9 @@ import (
 	"go.temporal.io/server/api/historyservicemock/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/log"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/primitives/timestamp"
 	serviceerrors "go.temporal.io/server/common/serviceerror"
 )
@@ -60,12 +60,12 @@ type (
 
 		controller          *gomock.Controller
 		mockClusterMetadata *cluster.MockMetadata
-		mockNamespaceCache  *cache.MockNamespaceCache
+		mockNamespaceCache  *namespace.MockRegistry
 		mockAdminClient     *adminservicemock.MockAdminServiceClient
 		mockHistoryClient   *historyservicemock.MockHistoryServiceClient
 
-		namespaceID string
-		namespace   string
+		namespaceID namespace.ID
+		namespace   namespace.Name
 
 		serializer serialization.Serializer
 		logger     log.Logger
@@ -93,15 +93,15 @@ func (s *nDCHistoryResenderSuite) SetupTest() {
 	s.mockClusterMetadata = cluster.NewMockMetadata(s.controller)
 	s.mockAdminClient = adminservicemock.NewMockAdminServiceClient(s.controller)
 	s.mockHistoryClient = historyservicemock.NewMockHistoryServiceClient(s.controller)
-	s.mockNamespaceCache = cache.NewMockNamespaceCache(s.controller)
+	s.mockNamespaceCache = namespace.NewMockRegistry(s.controller)
 
 	s.logger = log.NewTestLogger()
 	s.mockClusterMetadata.EXPECT().IsGlobalNamespaceEnabled().Return(true).AnyTimes()
 
-	s.namespaceID = uuid.New()
+	s.namespaceID = namespace.ID(uuid.New())
 	s.namespace = "some random namespace name"
-	namespaceEntry := cache.NewGlobalNamespaceCacheEntryForTest(
-		&persistencespb.NamespaceInfo{Id: s.namespaceID, Name: s.namespace},
+	namespaceEntry := namespace.NewGlobalNamespaceForTest(
+		&persistencespb.NamespaceInfo{Id: s.namespaceID.String(), Name: s.namespace.String()},
 		&persistencespb.NamespaceConfig{Retention: timestamp.DurationFromDays(1)},
 		&persistencespb.NamespaceReplicationConfig{
 			ActiveClusterName: cluster.TestCurrentClusterName,
@@ -111,7 +111,6 @@ func (s *nDCHistoryResenderSuite) SetupTest() {
 			},
 		},
 		1234,
-		nil,
 	)
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(s.namespaceID).Return(namespaceEntry, nil).AnyTimes()
 	s.mockNamespaceCache.EXPECT().GetNamespace(s.namespace).Return(namespaceEntry, nil).AnyTimes()
@@ -166,7 +165,7 @@ func (s *nDCHistoryResenderSuite) TestSendSingleWorkflowHistory() {
 	s.mockAdminClient.EXPECT().GetWorkflowExecutionRawHistoryV2(
 		gomock.Any(),
 		&adminservice.GetWorkflowExecutionRawHistoryV2Request{
-			Namespace: s.namespace,
+			Namespace: s.namespace.String(),
 			Execution: &commonpb.WorkflowExecution{
 				WorkflowId: workflowID,
 				RunId:      runID,
@@ -188,7 +187,7 @@ func (s *nDCHistoryResenderSuite) TestSendSingleWorkflowHistory() {
 	s.mockAdminClient.EXPECT().GetWorkflowExecutionRawHistoryV2(
 		gomock.Any(),
 		&adminservice.GetWorkflowExecutionRawHistoryV2Request{
-			Namespace: s.namespace,
+			Namespace: s.namespace.String(),
 			Execution: &commonpb.WorkflowExecution{
 				WorkflowId: workflowID,
 				RunId:      runID,
@@ -210,7 +209,7 @@ func (s *nDCHistoryResenderSuite) TestSendSingleWorkflowHistory() {
 	s.mockHistoryClient.EXPECT().ReplicateEventsV2(
 		gomock.Any(),
 		&historyservice.ReplicateEventsV2Request{
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			WorkflowExecution: &commonpb.WorkflowExecution{
 				WorkflowId: workflowID,
 				RunId:      runID,
@@ -247,7 +246,7 @@ func (s *nDCHistoryResenderSuite) TestCreateReplicateRawEventsRequest() {
 	}
 
 	s.Equal(&historyservice.ReplicateEventsV2Request{
-		NamespaceId: s.namespaceID,
+		NamespaceId: s.namespaceID.String(),
 		WorkflowExecution: &commonpb.WorkflowExecution{
 			WorkflowId: workflowID,
 			RunId:      runID,
@@ -270,7 +269,7 @@ func (s *nDCHistoryResenderSuite) TestSendReplicationRawRequest() {
 		Version: 1,
 	}
 	request := &historyservice.ReplicateEventsV2Request{
-		NamespaceId: s.namespaceID,
+		NamespaceId: s.namespaceID.String(),
 		WorkflowExecution: &commonpb.WorkflowExecution{
 			WorkflowId: workflowID,
 			RunId:      runID,
@@ -295,7 +294,7 @@ func (s *nDCHistoryResenderSuite) TestSendReplicationRawRequest_Err() {
 		Version: 1,
 	}
 	request := &historyservice.ReplicateEventsV2Request{
-		NamespaceId: s.namespaceID,
+		NamespaceId: s.namespaceID.String(),
 		WorkflowExecution: &commonpb.WorkflowExecution{
 			WorkflowId: workflowID,
 			RunId:      runID,
@@ -308,7 +307,7 @@ func (s *nDCHistoryResenderSuite) TestSendReplicationRawRequest_Err() {
 	}
 	retryErr := serviceerrors.NewRetryReplication(
 		"",
-		s.namespaceID,
+		s.namespaceID.String(),
 		workflowID,
 		runID,
 		common.EmptyEventID,
@@ -341,7 +340,7 @@ func (s *nDCHistoryResenderSuite) TestGetHistory() {
 		NextPageToken: nextTokenOut,
 	}
 	s.mockAdminClient.EXPECT().GetWorkflowExecutionRawHistoryV2(gomock.Any(), &adminservice.GetWorkflowExecutionRawHistoryV2Request{
-		Namespace: s.namespace,
+		Namespace: s.namespace.String(),
 		Execution: &commonpb.WorkflowExecution{
 			WorkflowId: workflowID,
 			RunId:      runID,
