@@ -28,9 +28,6 @@ import (
 	"bytes"
 	"encoding/binary"
 	"errors"
-	"fmt"
-	"math/rand"
-	"strconv"
 	"time"
 
 	"go.temporal.io/server/common/convert"
@@ -342,7 +339,7 @@ func (s *integrationSuite) TestActivityHeartbeatDetailsDuringRetry() {
 	}
 
 	_, err = poller.PollAndProcessWorkflowTask(true, false)
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	s.True(workflowComplete)
 	s.Equal(3, activityExecutedCount)
@@ -506,7 +503,7 @@ func (s *integrationSuite) TestActivityRetry() {
 	}
 
 	_, err := poller.PollAndProcessWorkflowTask(false, false)
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	err = poller.PollAndProcessActivityTask(false)
 	s.True(err == nil || err == matching.ErrNoTasks, err)
@@ -654,7 +651,7 @@ func (s *integrationSuite) TestActivityRetry_Infinite() {
 	}
 
 	_, err := poller.PollAndProcessWorkflowTask(false, false)
-	s.True(err == nil, err)
+	s.NoError(err)
 
 	for i := 0; i <= activityExecutedLimit; i++ {
 		err = poller.PollAndProcessActivityTask(false)
@@ -767,454 +764,6 @@ func (s *integrationSuite) TestActivityHeartBeatWorkflow_Timeout() {
 	s.True(workflowComplete)
 }
 
-func (s *integrationSuite) TestActivityTimeouts() {
-	id := "integration-activity-timeout-test"
-	wt := "integration-activity-timeout-test-type"
-	tl := "integration-activity-timeout-test-taskqueue"
-	identity := "worker1"
-	activityName := "timeout_activity"
-
-	workflowType := &commonpb.WorkflowType{Name: wt}
-
-	taskQueue := &taskqueuepb.TaskQueue{Name: tl}
-
-	request := &workflowservice.StartWorkflowExecutionRequest{
-		RequestId:           uuid.New(),
-		Namespace:           s.namespace,
-		WorkflowId:          id,
-		WorkflowType:        workflowType,
-		TaskQueue:           taskQueue,
-		Input:               nil,
-		WorkflowRunTimeout:  timestamp.DurationPtr(300 * time.Second),
-		WorkflowTaskTimeout: timestamp.DurationPtr(2 * time.Second),
-		Identity:            identity,
-	}
-
-	we, err0 := s.engine.StartWorkflowExecution(NewContext(), request)
-	s.NoError(err0)
-
-	s.Logger.Info("StartWorkflowExecution", tag.WorkflowRunID(we.RunId))
-
-	workflowComplete := false
-	workflowFailed := false
-	activitiesScheduled := false
-	activitiesMap := map[int64]*historypb.HistoryEvent{}
-	failWorkflow := false
-	failReason := ""
-	var activityATimedout, activityBTimedout, activityCTimedout, activityDTimedout bool
-	wtHandler := func(execution *commonpb.WorkflowExecution, wt *commonpb.WorkflowType,
-		previousStartedEventID, startedEventID int64, history *historypb.History) ([]*commandpb.Command, error) {
-		if !activitiesScheduled {
-			activitiesScheduled = true
-			return []*commandpb.Command{{
-				CommandType: enumspb.COMMAND_TYPE_SCHEDULE_ACTIVITY_TASK,
-				Attributes: &commandpb.Command_ScheduleActivityTaskCommandAttributes{ScheduleActivityTaskCommandAttributes: &commandpb.ScheduleActivityTaskCommandAttributes{
-					ActivityId:             "A",
-					ActivityType:           &commonpb.ActivityType{Name: activityName},
-					TaskQueue:              &taskqueuepb.TaskQueue{Name: "NoWorker"},
-					Input:                  payloads.EncodeString("ScheduleToStart"),
-					ScheduleToCloseTimeout: timestamp.DurationPtr(35 * time.Second),
-					ScheduleToStartTimeout: timestamp.DurationPtr(3 * time.Second), // ActivityID A is expected to timeout using ScheduleToStart
-					StartToCloseTimeout:    timestamp.DurationPtr(30 * time.Second),
-					HeartbeatTimeout:       timestamp.DurationPtr(0 * time.Second),
-				},
-				}}, {
-				CommandType: enumspb.COMMAND_TYPE_SCHEDULE_ACTIVITY_TASK,
-				Attributes: &commandpb.Command_ScheduleActivityTaskCommandAttributes{ScheduleActivityTaskCommandAttributes: &commandpb.ScheduleActivityTaskCommandAttributes{
-					ActivityId:             "B",
-					ActivityType:           &commonpb.ActivityType{Name: activityName},
-					TaskQueue:              &taskqueuepb.TaskQueue{Name: tl},
-					Input:                  payloads.EncodeString("ScheduleToClose"),
-					ScheduleToCloseTimeout: timestamp.DurationPtr(7 * time.Second), // ActivityID B is expected to timeout using ScheduleClose
-					ScheduleToStartTimeout: timestamp.DurationPtr(5 * time.Second),
-					StartToCloseTimeout:    timestamp.DurationPtr(10 * time.Second),
-					HeartbeatTimeout:       timestamp.DurationPtr(0 * time.Second),
-				},
-				}}, {
-				CommandType: enumspb.COMMAND_TYPE_SCHEDULE_ACTIVITY_TASK,
-				Attributes: &commandpb.Command_ScheduleActivityTaskCommandAttributes{ScheduleActivityTaskCommandAttributes: &commandpb.ScheduleActivityTaskCommandAttributes{
-					ActivityId:             "C",
-					ActivityType:           &commonpb.ActivityType{Name: activityName},
-					TaskQueue:              &taskqueuepb.TaskQueue{Name: tl},
-					Input:                  payloads.EncodeString("StartToClose"),
-					ScheduleToCloseTimeout: timestamp.DurationPtr(15 * time.Second),
-					ScheduleToStartTimeout: timestamp.DurationPtr(1 * time.Second),
-					StartToCloseTimeout:    timestamp.DurationPtr(5 * time.Second), // ActivityID C is expected to timeout using StartToClose
-					HeartbeatTimeout:       timestamp.DurationPtr(0 * time.Second),
-					RetryPolicy: &commonpb.RetryPolicy{
-						MaximumAttempts: 1, // Disable retry and let it timeout.
-					},
-				},
-				}}, {
-				CommandType: enumspb.COMMAND_TYPE_SCHEDULE_ACTIVITY_TASK,
-				Attributes: &commandpb.Command_ScheduleActivityTaskCommandAttributes{ScheduleActivityTaskCommandAttributes: &commandpb.ScheduleActivityTaskCommandAttributes{
-					ActivityId:             "D",
-					ActivityType:           &commonpb.ActivityType{Name: activityName},
-					TaskQueue:              &taskqueuepb.TaskQueue{Name: tl},
-					Input:                  payloads.EncodeString("Heartbeat"),
-					ScheduleToCloseTimeout: timestamp.DurationPtr(35 * time.Second),
-					ScheduleToStartTimeout: timestamp.DurationPtr(20 * time.Second),
-					StartToCloseTimeout:    timestamp.DurationPtr(15 * time.Second),
-					HeartbeatTimeout:       timestamp.DurationPtr(3 * time.Second), // ActivityID D is expected to timeout using Heartbeat
-					RetryPolicy: &commonpb.RetryPolicy{
-						MaximumAttempts: 1, // Disable retry and let it timeout.
-					},
-				}}},
-			}, nil
-		} else if previousStartedEventID > 0 {
-			for _, event := range history.Events[previousStartedEventID:] {
-				if event.GetEventType() == enumspb.EVENT_TYPE_ACTIVITY_TASK_SCHEDULED {
-					activitiesMap[event.GetEventId()] = event
-				}
-
-				if event.GetEventType() == enumspb.EVENT_TYPE_ACTIVITY_TASK_TIMED_OUT {
-					timeoutEvent := event.GetActivityTaskTimedOutEventAttributes()
-					scheduledEvent, ok := activitiesMap[timeoutEvent.GetScheduledEventId()]
-					if !ok {
-						return []*commandpb.Command{{
-							CommandType: enumspb.COMMAND_TYPE_FAIL_WORKFLOW_EXECUTION,
-							Attributes: &commandpb.Command_FailWorkflowExecutionCommandAttributes{FailWorkflowExecutionCommandAttributes: &commandpb.FailWorkflowExecutionCommandAttributes{
-								Failure: failure.NewServerFailure("ScheduledEvent not found", false),
-							}},
-						}}, nil
-					}
-
-					switch timeoutEvent.GetFailure().GetTimeoutFailureInfo().GetTimeoutType() {
-					case enumspb.TIMEOUT_TYPE_SCHEDULE_TO_START:
-						if scheduledEvent.GetActivityTaskScheduledEventAttributes().GetActivityId() == "A" {
-							activityATimedout = true
-						} else {
-							failWorkflow = true
-							failReason = "ActivityID A is expected to timeout with ScheduleToStart but it was " + scheduledEvent.GetActivityTaskScheduledEventAttributes().GetActivityId()
-						}
-					case enumspb.TIMEOUT_TYPE_SCHEDULE_TO_CLOSE:
-						if scheduledEvent.GetActivityTaskScheduledEventAttributes().GetActivityId() == "B" {
-							activityBTimedout = true
-						} else {
-							failWorkflow = true
-							failReason = "ActivityID B is expected to timeout with ScheduleToClose but it was " + scheduledEvent.GetActivityTaskScheduledEventAttributes().GetActivityId()
-						}
-					case enumspb.TIMEOUT_TYPE_START_TO_CLOSE:
-						if scheduledEvent.GetActivityTaskScheduledEventAttributes().GetActivityId() == "C" {
-							activityCTimedout = true
-						} else {
-							failWorkflow = true
-							failReason = "ActivityID C is expected to timeout with StartToClose but it was " + scheduledEvent.GetActivityTaskScheduledEventAttributes().GetActivityId()
-						}
-					case enumspb.TIMEOUT_TYPE_HEARTBEAT:
-						if scheduledEvent.GetActivityTaskScheduledEventAttributes().GetActivityId() == "D" {
-							activityDTimedout = true
-						} else {
-							failWorkflow = true
-							failReason = "ActivityID D is expected to timeout with Heartbeat but it was " + scheduledEvent.GetActivityTaskScheduledEventAttributes().GetActivityId()
-						}
-					}
-				}
-			}
-		}
-
-		if failWorkflow {
-			s.Logger.Error("Failing workflow: " + failReason)
-			workflowFailed = true
-			return []*commandpb.Command{{
-				CommandType: enumspb.COMMAND_TYPE_FAIL_WORKFLOW_EXECUTION,
-				Attributes: &commandpb.Command_FailWorkflowExecutionCommandAttributes{FailWorkflowExecutionCommandAttributes: &commandpb.FailWorkflowExecutionCommandAttributes{
-					Failure: failure.NewServerFailure(failReason, false),
-				}},
-			}}, nil
-		}
-
-		if activityATimedout && activityBTimedout && activityCTimedout && activityDTimedout {
-			s.Logger.Info("Completing Workflow")
-			workflowComplete = true
-			return []*commandpb.Command{{
-				CommandType: enumspb.COMMAND_TYPE_COMPLETE_WORKFLOW_EXECUTION,
-				Attributes: &commandpb.Command_CompleteWorkflowExecutionCommandAttributes{CompleteWorkflowExecutionCommandAttributes: &commandpb.CompleteWorkflowExecutionCommandAttributes{
-					Result: payloads.EncodeString("Done"),
-				}},
-			}}, nil
-		}
-
-		return []*commandpb.Command{}, nil
-	}
-
-	atHandler := func(execution *commonpb.WorkflowExecution, activityType *commonpb.ActivityType,
-		activityID string, input *commonpb.Payloads, taskToken []byte) (*commonpb.Payloads, bool, error) {
-		s.Equal(id, execution.GetWorkflowId())
-		s.Equal(activityName, activityType.GetName())
-		var timeoutType string
-		err := payloads.Decode(input, &timeoutType)
-		s.NoError(err)
-		switch timeoutType {
-		case "ScheduleToStart":
-			s.Fail("Activity A not expected to be started")
-		case "ScheduleToClose":
-			s.Logger.Info("Sleeping activityB for 7 seconds")
-			time.Sleep(7 * time.Second)
-		case "StartToClose":
-			s.Logger.Info("Sleeping activityC for 8 seconds")
-			time.Sleep(8 * time.Second)
-		case "Heartbeat":
-			s.Logger.Info("Starting hearbeat activity")
-			go func() {
-				for i := 0; i < 6; i++ {
-					s.Logger.Info("Heartbeating for activity", tag.WorkflowActivityID(activityID), tag.Counter(i))
-					_, err := s.engine.RecordActivityTaskHeartbeat(NewContext(), &workflowservice.RecordActivityTaskHeartbeatRequest{
-						TaskToken: taskToken, Details: payloads.EncodeString(convert.IntToString(i))})
-					s.NoError(err)
-					time.Sleep(1 * time.Second)
-				}
-				s.Logger.Info("End Heartbeating")
-			}()
-			s.Logger.Info("Sleeping hearbeat activity")
-			time.Sleep(10 * time.Second)
-		}
-
-		return payloads.EncodeString("Activity Result"), false, nil
-	}
-
-	poller := &TaskPoller{
-		Engine:              s.engine,
-		Namespace:           s.namespace,
-		TaskQueue:           taskQueue,
-		Identity:            identity,
-		WorkflowTaskHandler: wtHandler,
-		ActivityTaskHandler: atHandler,
-		Logger:              s.Logger,
-		T:                   s.T(),
-	}
-
-	_, err := poller.PollAndProcessWorkflowTask(false, false)
-	s.True(err == nil || err == matching.ErrNoTasks)
-
-	for i := 0; i < 3; i++ {
-		go func() {
-			err = poller.PollAndProcessActivityTask(false)
-			s.Logger.Error("Activity Processing Completed", tag.Error(err))
-		}()
-	}
-
-	s.Logger.Info("Waiting for workflow to complete", tag.WorkflowRunID(we.RunId))
-	for i := 0; i < 10; i++ {
-		s.Logger.Info("Processing workflow task", tag.Counter(i))
-		_, err := poller.PollAndProcessWorkflowTask(false, false)
-		s.NoError(err, "Poll for workflow task failed")
-
-		if workflowComplete || workflowFailed {
-			break
-		}
-	}
-
-	s.True(workflowComplete)
-	s.False(workflowFailed)
-}
-
-func (s *integrationSuite) TestActivityHeartbeatTimeouts() {
-	id := "integration-activity-heartbeat-timeout-test"
-	wt := "integration-activity-heartbeat-timeout-test-type"
-	tl := "integration-activity-heartbeat-timeout-test-taskqueue"
-	identity := "worker1"
-	activityName := "timeout_activity"
-
-	workflowType := &commonpb.WorkflowType{Name: wt}
-
-	taskQueue := &taskqueuepb.TaskQueue{Name: tl}
-
-	request := &workflowservice.StartWorkflowExecutionRequest{
-		RequestId:           uuid.New(),
-		Namespace:           s.namespace,
-		WorkflowId:          id,
-		WorkflowType:        workflowType,
-		TaskQueue:           taskQueue,
-		Input:               nil,
-		WorkflowRunTimeout:  timestamp.DurationPtr(90 * time.Second),
-		WorkflowTaskTimeout: timestamp.DurationPtr(2 * time.Second),
-		Identity:            identity,
-	}
-
-	we, err0 := s.engine.StartWorkflowExecution(NewContext(), request)
-	s.NoError(err0)
-
-	s.Logger.Info("StartWorkflowExecution", tag.WorkflowRunID(we.RunId))
-
-	workflowComplete := false
-	activitiesScheduled := false
-	lastHeartbeatMap := make(map[int64]int)
-	failWorkflow := false
-	failReason := ""
-	activityCount := 10
-	activitiesTimedout := 0
-	wtHandler := func(execution *commonpb.WorkflowExecution, wt *commonpb.WorkflowType,
-		previousStartedEventID, startedEventID int64, history *historypb.History) ([]*commandpb.Command, error) {
-		if !activitiesScheduled {
-			activitiesScheduled = true
-			var commands []*commandpb.Command
-			for i := 0; i < activityCount; i++ {
-				aID := fmt.Sprintf("activity_%v", i)
-				d := &commandpb.Command{
-					CommandType: enumspb.COMMAND_TYPE_SCHEDULE_ACTIVITY_TASK,
-					Attributes: &commandpb.Command_ScheduleActivityTaskCommandAttributes{ScheduleActivityTaskCommandAttributes: &commandpb.ScheduleActivityTaskCommandAttributes{
-						ActivityId:             aID,
-						ActivityType:           &commonpb.ActivityType{Name: activityName},
-						TaskQueue:              &taskqueuepb.TaskQueue{Name: tl},
-						Input:                  payloads.EncodeString("Heartbeat"),
-						ScheduleToCloseTimeout: timestamp.DurationPtr(60 * time.Second),
-						StartToCloseTimeout:    timestamp.DurationPtr(60 * time.Second),
-						HeartbeatTimeout:       timestamp.DurationPtr(4 * time.Second),
-						RetryPolicy: &commonpb.RetryPolicy{
-							MaximumAttempts: 1,
-						},
-					}},
-				}
-
-				commands = append(commands, d)
-			}
-
-			return commands, nil
-		} else if previousStartedEventID > 0 {
-		ProcessLoop:
-			for _, event := range history.Events[previousStartedEventID:] {
-				if event.GetEventType() == enumspb.EVENT_TYPE_ACTIVITY_TASK_SCHEDULED {
-					lastHeartbeatMap[event.GetEventId()] = 0
-				}
-
-				if event.GetEventType() == enumspb.EVENT_TYPE_ACTIVITY_TASK_COMPLETED ||
-					event.GetEventType() == enumspb.EVENT_TYPE_ACTIVITY_TASK_FAILED {
-					failWorkflow = true
-					failReason = "Expected activities to timeout but seeing completion instead"
-				}
-
-				if event.GetEventType() == enumspb.EVENT_TYPE_ACTIVITY_TASK_TIMED_OUT {
-					timeoutEvent := event.GetActivityTaskTimedOutEventAttributes()
-					_, ok := lastHeartbeatMap[timeoutEvent.GetScheduledEventId()]
-					if !ok {
-						failWorkflow = true
-						failReason = "ScheduledEvent not found"
-						break ProcessLoop
-					}
-
-					switch timeoutEvent.GetFailure().GetTimeoutFailureInfo().GetTimeoutType() {
-					case enumspb.TIMEOUT_TYPE_HEARTBEAT:
-						activitiesTimedout++
-						scheduleID := timeoutEvent.GetScheduledEventId()
-						var details string
-						err := payloads.Decode(timeoutEvent.GetFailure().GetTimeoutFailureInfo().GetLastHeartbeatDetails(), &details)
-						s.NoError(err)
-						lastHeartbeat, _ := strconv.Atoi(details)
-						lastHeartbeatMap[scheduleID] = lastHeartbeat
-					default:
-						failWorkflow = true
-						failReason = fmt.Sprintf("Expected Heartbeat timeout but got another timeout: %v", timeoutEvent.GetFailure().GetTimeoutFailureInfo().GetTimeoutType())
-						break ProcessLoop
-					}
-				}
-			}
-		}
-
-		if failWorkflow {
-			s.Logger.Error("Failing workflow", tag.Value(failReason))
-			workflowComplete = true
-			return []*commandpb.Command{{
-				CommandType: enumspb.COMMAND_TYPE_FAIL_WORKFLOW_EXECUTION,
-				Attributes: &commandpb.Command_FailWorkflowExecutionCommandAttributes{FailWorkflowExecutionCommandAttributes: &commandpb.FailWorkflowExecutionCommandAttributes{
-					Failure: failure.NewServerFailure(failReason, false),
-				}},
-			}}, nil
-		}
-
-		if activitiesTimedout == activityCount {
-			s.Logger.Info("Completing Workflow")
-			workflowComplete = true
-			return []*commandpb.Command{{
-				CommandType: enumspb.COMMAND_TYPE_COMPLETE_WORKFLOW_EXECUTION,
-				Attributes: &commandpb.Command_CompleteWorkflowExecutionCommandAttributes{CompleteWorkflowExecutionCommandAttributes: &commandpb.CompleteWorkflowExecutionCommandAttributes{
-					Result: payloads.EncodeString("Done"),
-				}},
-			}}, nil
-		}
-
-		return []*commandpb.Command{}, nil
-	}
-
-	atHandler := func(execution *commonpb.WorkflowExecution, activityType *commonpb.ActivityType,
-		activityID string, input *commonpb.Payloads, taskToken []byte) (*commonpb.Payloads, bool, error) {
-		s.Logger.Info("Starting heartbeat activity", tag.WorkflowActivityID(activityID))
-		for i := 0; i < 10; i++ {
-			if !workflowComplete {
-				s.Logger.Info("Heartbeating for activity", tag.WorkflowActivityID(activityID), tag.Counter(i))
-				_, err := s.engine.RecordActivityTaskHeartbeat(NewContext(), &workflowservice.RecordActivityTaskHeartbeatRequest{
-					TaskToken: taskToken, Details: payloads.EncodeString(strconv.Itoa(i))})
-				if err != nil {
-					s.Logger.Error("Activity heartbeat failed", tag.WorkflowActivityID(activityID), tag.Counter(i), tag.Error(err))
-				}
-
-				secondsToSleep := rand.Intn(3)
-				s.Logger.Info("Activity is sleeping", tag.WorkflowActivityID(activityID), tag.Number(int64(secondsToSleep)))
-				time.Sleep(time.Duration(secondsToSleep) * time.Second)
-			}
-		}
-		s.Logger.Info("End Heartbeating", tag.WorkflowActivityID(activityID))
-
-		s.Logger.Info("Sleeping activity before completion (should timeout)", tag.WorkflowActivityID(activityID))
-		time.Sleep(8 * time.Second)
-
-		return payloads.EncodeString("Should never reach this point"), false, nil
-	}
-
-	poller := &TaskPoller{
-		Engine:              s.engine,
-		Namespace:           s.namespace,
-		TaskQueue:           taskQueue,
-		Identity:            identity,
-		WorkflowTaskHandler: wtHandler,
-		ActivityTaskHandler: atHandler,
-		Logger:              s.Logger,
-		T:                   s.T(),
-	}
-
-	_, err := poller.PollAndProcessWorkflowTask(false, false)
-	s.True(err == nil || err == matching.ErrNoTasks)
-
-	for i := 0; i < activityCount; i++ {
-		go func() {
-			err := poller.PollAndProcessActivityTask(false)
-			if err == nil {
-				s.Logger.Error("Activity expected to time out but got completed instead")
-			} else {
-				s.Logger.Info("Activity processing completed with error", tag.Error(err))
-			}
-		}()
-	}
-
-	s.Logger.Info("Waiting for workflow to complete", tag.WorkflowRunID(we.RunId))
-	for i := 0; i < 10; i++ {
-		s.Logger.Info("Processing workflow task", tag.Counter(i))
-		_, err := poller.PollAndProcessWorkflowTask(false, false)
-		s.NoError(err, "Poll for workflow task failed")
-
-		if workflowComplete {
-			break
-		}
-	}
-
-	if !workflowComplete || failWorkflow {
-		s.Logger.Error("Test completed with unexpected result.  Dumping workflow execution history: ")
-		s.printWorkflowHistory(s.namespace, &commonpb.WorkflowExecution{
-			WorkflowId: id,
-			RunId:      we.GetRunId(),
-		})
-	}
-
-	s.True(workflowComplete)
-	s.False(failWorkflow, failReason)
-	s.Equal(activityCount, activitiesTimedout)
-	s.Equal(activityCount, len(lastHeartbeatMap))
-	for aID, lastHeartbeat := range lastHeartbeatMap {
-		s.Logger.Info("Last heartbeat for activity with scheduleID", tag.Counter(int(aID)), tag.Number(int64(lastHeartbeat)))
-		s.Equal(9, lastHeartbeat)
-	}
-}
-
 func (s *integrationSuite) TestTryActivityCancellationFromWorkflow() {
 	id := "integration-activity-cancellation-test"
 	wt := "integration-activity-cancellation-test-type"
