@@ -52,7 +52,7 @@ CREATE TABLE current_executions(
   create_request_id VARCHAR(64) NOT NULL,
   state INTEGER NOT NULL,
   status INTEGER NOT NULL,
-  start_version BIGINT NOT NULL,
+  start_version BIGINT NOT NULL DEFAULT 0,
   last_write_version BIGINT NOT NULL,
   PRIMARY KEY (shard_id, namespace_id, workflow_id)
 );
@@ -136,6 +136,15 @@ CREATE TABLE visibility_tasks(
   PRIMARY KEY (shard_id, task_id)
 );
 
+CREATE TABLE tiered_storage_tasks(
+    shard_id      INTEGER     NOT NULL,
+    task_id       BIGINT      NOT NULL,
+    --
+    data          BYTEA       NOT NULL,
+    data_encoding VARCHAR(16) NOT NULL,
+    PRIMARY KEY (shard_id, task_id)
+);
+
 CREATE TABLE activity_info_maps (
 -- each row corresponds to one key of one map<string, ActivityInfo>
   shard_id INTEGER NOT NULL,
@@ -248,6 +257,7 @@ CREATE TABLE queue_metadata (
   PRIMARY KEY(queue_type)
 );
 
+-- TODO deprecate this table in v1.15+
 CREATE TABLE cluster_metadata (
   metadata_partition        INTEGER NOT NULL,
   data                      BYTEA NOT NULL,
@@ -256,6 +266,15 @@ CREATE TABLE cluster_metadata (
   PRIMARY KEY(metadata_partition)
 );
 
+CREATE TABLE cluster_metadata_info (
+  metadata_partition        INTEGER NOT NULL,
+  cluster_name              VARCHAR(255) NOT NULL,
+  data                      BYTEA NOT NULL,
+  data_encoding             VARCHAR(16) NOT NULL,
+  version                   BIGINT NOT NULL,
+  PRIMARY KEY(metadata_partition, cluster_name)
+);
+
 CREATE TABLE cluster_membership
 (
     membership_partition INTEGER NOT NULL,
