@@ -36,6 +36,7 @@ import (
 	"github.com/stretchr/testify/suite"
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
+	failurepb "go.temporal.io/api/failure/v1"
 	filterpb "go.temporal.io/api/filter/v1"
 	historypb "go.temporal.io/api/history/v1"
 	namespacepb "go.temporal.io/api/namespace/v1"
@@ -43,18 +44,20 @@ import (
 	"go.temporal.io/api/serviceerror"
 	taskqueuepb "go.temporal.io/api/taskqueue/v1"
 	"go.temporal.io/api/workflowservice/v1"
+
+	enumsspb "go.temporal.io/server/api/enums/v1"
+	"go.temporal.io/server/common/headers"
 	"go.temporal.io/server/common/payload"
-	"go.temporal.io/server/common/persistence/visibility"
+	"go.temporal.io/server/common/persistence/visibility/manager"
 	"go.temporal.io/server/common/searchattribute"
 
+	"go.temporal.io/server/api/historyservice/v1"
 	"go.temporal.io/server/api/historyservicemock/v1"
 	"go.temporal.io/server/api/matchingservicemock/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
-	tokenspb "go.temporal.io/server/api/token/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/archiver"
 	"go.temporal.io/server/common/archiver/provider"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	dc "go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/metrics"
@@ -81,7 +84,7 @@ type (
 
 		controller                   *gomock.Controller
 		mockResource                 *resource.Test
-		mockNamespaceCache           *cache.MockNamespaceCache
+		mockNamespaceCache           *namespace.MockRegistry
 		mockHistoryClient            *historyservicemock.MockHistoryServiceClient
 		mockClusterMetadata          *cluster.MockMetadata
 		mockSearchAttributesProvider *searchattribute.MockProvider
@@ -91,7 +94,7 @@ type (
 		mockProducer           *persistence.MockNamespaceReplicationQueue
 		mockMetadataMgr        *persistence.MockMetadataManager
 		mockExecutionManager   *persistence.MockExecutionManager
-		mockVisibilityMgr      *visibility.MockVisibilityManager
+		mockVisibilityMgr      *manager.MockVisibilityManager
 		mockArchivalMetadata   *archiver.MockArchivalMetadata
 		mockArchiverProvider   *provider.MockArchiverProvider
 		mockHistoryArchiver    *archiver.MockHistoryArchiver
@@ -99,8 +102,8 @@ type (
 
 		tokenSerializer common.TaskTokenSerializer
 
-		testNamespace   string
-		testNamespaceID string
+		testNamespace   namespace.Name
+		testNamespaceID namespace.ID
 	}
 )
 
@@ -132,7 +135,7 @@ func (s *workflowHandlerSuite) SetupTest() {
 	s.mockSearchAttributesMapper = s.mockResource.SearchAttributesMapper
 	s.mockMetadataMgr = s.mockResource.MetadataMgr
 	s.mockExecutionManager = s.mockResource.ExecutionMgr
-	s.mockVisibilityMgr = s.mockResource.VisibilityMgr
+	s.mockVisibilityMgr = manager.NewMockVisibilityManager(s.controller)
 	s.mockArchivalMetadata = s.mockResource.ArchivalMetadata
 	s.mockArchiverProvider = s.mockResource.ArchiverProvider
 	s.mockMatchingClient = s.mockResource.MatchingClient
@@ -152,12 +155,30 @@ func (s *workflowHandlerSuite) TearDownTest() {
 }
 
 func (s *workflowHandlerSuite) getWorkflowHandler(config *Config) *WorkflowHandler {
-	return NewWorkflowHandler(s.mockResource, config, s.mockProducer).(*WorkflowHandler)
+	return NewWorkflowHandler(
+		config,
+		s.mockProducer,
+		s.mockVisibilityMgr,
+		s.mockResource.Logger,
+		s.mockResource.GetThrottledLogger(),
+		s.mockResource.GetExecutionManager(),
+		s.mockResource.GetClusterMetadataManager(),
+		s.mockResource.GetMetadataManager(),
+		s.mockResource.GetHistoryClient(),
+		s.mockResource.GetMatchingClient(),
+		s.mockResource.GetArchiverProvider(),
+		s.mockResource.GetPayloadSerializer(),
+		s.mockResource.GetNamespaceRegistry(),
+		s.mockResource.GetSearchAttributesMapper(),
+		s.mockResource.GetSearchAttributesProvider(),
+		s.mockResource.GetClusterMetadata(),
+		s.mockResource.GetArchivalMetadata(),
+	)
 }
 
 func (s *workflowHandlerSuite) TestDisableListVisibilityByFilter() {
-	testNamespace := "test-namespace"
-	namespaceID := uuid.New()
+	testNamespace := namespace.Name("test-namespace")
+	namespaceID := namespace.ID(uuid.New())
 	config := s.newConfig()
 	config.DisableListVisibilityByFilter = dc.GetBoolPropertyFnFilteredByNamespace(true)
 
@@ -167,7 +188,7 @@ func (s *workflowHandlerSuite) TestDisableListVisibilityByFilter() {
 
 	// test list open by wid
 	listRequest := &workflowservice.ListOpenWorkflowExecutionsRequest{
-		Namespace: testNamespace,
+		Namespace: testNamespace.String(),
 		StartTimeFilter: &filterpb.StartTimeFilter{
 			EarliestTime: timestamp.TimePtr(time.Time{}),
 			LatestTime:   timestamp.TimePtr(time.Now().UTC()),
@@ -190,7 +211,7 @@ func (s *workflowHandlerSuite) TestDisableListVisibilityByFilter() {
 
 	// test list close by wid
 	listRequest2 := &workflowservice.ListClosedWorkflowExecutionsRequest{
-		Namespace: testNamespace,
+		Namespace: testNamespace.String(),
 		StartTimeFilter: &filterpb.StartTimeFilter{
 			EarliestTime: timestamp.TimePtr(time.Time{}),
 			LatestTime:   timestamp.TimePtr(time.Now().UTC()),
@@ -286,7 +307,10 @@ func (s *workflowHandlerSuite) TestStartWorkflowExecution_Failed_NamespaceNotSet
 	config.RPS = dc.GetIntPropertyFn(10)
 	wh := s.getWorkflowHandler(config)
 
+	s.mockNamespaceCache.EXPECT().GetNamespaceID(namespace.EmptyName).Return(namespace.EmptyID, serviceerror.NewNotFound("not found")).AnyTimes()
+
 	startWorkflowExecutionRequest := &workflowservice.StartWorkflowExecutionRequest{
+		// Namespace: "forget to specify",
 		WorkflowId: "workflow-id",
 		WorkflowType: &commonpb.WorkflowType{
 			Name: "workflow-type",
@@ -307,7 +331,8 @@ func (s *workflowHandlerSuite) TestStartWorkflowExecution_Failed_NamespaceNotSet
 	}
 	_, err := wh.StartWorkflowExecution(context.Background(), startWorkflowExecutionRequest)
 	s.Error(err)
-	s.Equal(errNamespaceNotSet, err)
+	var notFound *serviceerror.NotFound
+	s.ErrorAs(err, &notFound)
 }
 
 func (s *workflowHandlerSuite) TestStartWorkflowExecution_Failed_WorkflowIdNotSet() {
@@ -645,6 +670,190 @@ func (s *workflowHandlerSuite) TestRegisterNamespace_Success_NotEnabled() {
 	s.NoError(err)
 }
 
+func (s *workflowHandlerSuite) TestDeprecateNamespace_Success() {
+	s.mockClusterMetadata.EXPECT().IsGlobalNamespaceEnabled().Return(false)
+	s.mockClusterMetadata.EXPECT().GetAllClusterInfo().Return(cluster.TestAllClusterInfo).AnyTimes()
+	s.mockClusterMetadata.EXPECT().GetCurrentClusterName().Return(cluster.TestCurrentClusterName).AnyTimes()
+	s.mockArchivalMetadata.EXPECT().GetHistoryConfig().Return(archiver.NewArchivalConfig("enabled", dc.GetStringPropertyFn("enabled"), dc.GetBoolPropertyFn(true), "disabled", "some random URI")).Times(2)
+	s.mockArchivalMetadata.EXPECT().GetVisibilityConfig().Return(archiver.NewArchivalConfig("enabled", dc.GetStringPropertyFn("enabled"), dc.GetBoolPropertyFn(true), "disabled", "some random URI")).Times(2)
+	s.mockMetadataMgr.EXPECT().GetNamespace(gomock.Any()).Return(nil, serviceerror.NewNotFound("not found"))
+	s.mockMetadataMgr.EXPECT().CreateNamespace(gomock.Any()).Return(&persistence.CreateNamespaceResponse{
+		ID: testNamespaceID,
+	}, nil)
+
+	s.mockMetadataMgr.EXPECT().GetMetadata().Return(&persistence.GetMetadataResponse{}, nil)
+	s.mockMetadataMgr.EXPECT().GetNamespace(gomock.Any()).Return(&persistence.GetNamespaceResponse{
+		Namespace: &persistencespb.NamespaceDetail{
+			Info: &persistencespb.NamespaceInfo{
+				State: enumspb.NAMESPACE_STATE_REGISTERED,
+			},
+			Config: &persistencespb.NamespaceConfig{},
+			ReplicationConfig: &persistencespb.NamespaceReplicationConfig{
+				ActiveClusterName: cluster.TestCurrentClusterName,
+				Clusters:          []string{cluster.TestCurrentClusterName},
+			},
+		},
+	}, nil)
+	s.mockMetadataMgr.EXPECT().UpdateNamespace(gomock.Any()).DoAndReturn(func(request *persistence.UpdateNamespaceRequest) error {
+		s.Equal(enumspb.NAMESPACE_STATE_DEPRECATED, request.Namespace.Info.State)
+		return nil
+	})
+
+	wh := s.getWorkflowHandler(s.newConfig())
+
+	req := registerNamespaceRequest(enumspb.ARCHIVAL_STATE_UNSPECIFIED, "", enumspb.ARCHIVAL_STATE_UNSPECIFIED, "")
+
+	resp, err := wh.RegisterNamespace(context.Background(), req)
+	s.NoError(err)
+	s.NotNil(resp)
+	respDeprecate, errDeprecate := wh.UpdateNamespace(context.Background(), &workflowservice.UpdateNamespaceRequest{
+		Namespace: req.Namespace,
+		UpdateInfo: &namespacepb.UpdateNamespaceInfo{
+			State: enumspb.NAMESPACE_STATE_DEPRECATED,
+		},
+	})
+	s.NoError(errDeprecate)
+	s.NotNil(respDeprecate)
+}
+
+func (s *workflowHandlerSuite) TestDeprecateNamespace_Error() {
+	s.mockClusterMetadata.EXPECT().IsGlobalNamespaceEnabled().Return(false)
+	s.mockClusterMetadata.EXPECT().GetAllClusterInfo().Return(cluster.TestAllClusterInfo).AnyTimes()
+	s.mockClusterMetadata.EXPECT().GetCurrentClusterName().Return(cluster.TestCurrentClusterName).AnyTimes()
+	s.mockArchivalMetadata.EXPECT().GetHistoryConfig().Return(archiver.NewArchivalConfig("enabled", dc.GetStringPropertyFn("enabled"), dc.GetBoolPropertyFn(true), "disabled", "some random URI")).Times(2)
+	s.mockArchivalMetadata.EXPECT().GetVisibilityConfig().Return(archiver.NewArchivalConfig("enabled", dc.GetStringPropertyFn("enabled"), dc.GetBoolPropertyFn(true), "disabled", "some random URI")).Times(2)
+	s.mockMetadataMgr.EXPECT().GetNamespace(gomock.Any()).Return(nil, serviceerror.NewNotFound("not found"))
+	s.mockMetadataMgr.EXPECT().CreateNamespace(gomock.Any()).Return(&persistence.CreateNamespaceResponse{
+		ID: testNamespaceID,
+	}, nil)
+
+	s.mockMetadataMgr.EXPECT().GetMetadata().Return(&persistence.GetMetadataResponse{}, nil)
+	s.mockMetadataMgr.EXPECT().GetNamespace(gomock.Any()).Return(&persistence.GetNamespaceResponse{
+		Namespace: &persistencespb.NamespaceDetail{
+			Info: &persistencespb.NamespaceInfo{
+				State: enumspb.NAMESPACE_STATE_REGISTERED,
+			},
+			Config: &persistencespb.NamespaceConfig{},
+			ReplicationConfig: &persistencespb.NamespaceReplicationConfig{
+				ActiveClusterName: cluster.TestCurrentClusterName,
+				Clusters:          []string{cluster.TestCurrentClusterName},
+			},
+		},
+	}, nil)
+	s.mockMetadataMgr.EXPECT().UpdateNamespace(gomock.Any()).DoAndReturn(func(request *persistence.UpdateNamespaceRequest) error {
+		s.Equal(enumspb.NAMESPACE_STATE_DEPRECATED, request.Namespace.Info.State)
+		return serviceerror.NewInternal("db is down")
+	})
+
+	wh := s.getWorkflowHandler(s.newConfig())
+
+	req := registerNamespaceRequest(enumspb.ARCHIVAL_STATE_UNSPECIFIED, "", enumspb.ARCHIVAL_STATE_UNSPECIFIED, "")
+
+	resp, err := wh.RegisterNamespace(context.Background(), req)
+	s.NoError(err)
+	s.NotNil(resp)
+	respDeprecate, errDeprecate := wh.UpdateNamespace(context.Background(), &workflowservice.UpdateNamespaceRequest{
+		Namespace: req.Namespace,
+		UpdateInfo: &namespacepb.UpdateNamespaceInfo{
+			State: enumspb.NAMESPACE_STATE_DEPRECATED,
+		},
+	})
+	s.Error(errDeprecate)
+	s.Nil(respDeprecate)
+}
+
+func (s *workflowHandlerSuite) TestDeleteNamespace_Success() {
+	s.mockClusterMetadata.EXPECT().IsGlobalNamespaceEnabled().Return(false)
+	s.mockClusterMetadata.EXPECT().GetAllClusterInfo().Return(cluster.TestAllClusterInfo).AnyTimes()
+	s.mockClusterMetadata.EXPECT().GetCurrentClusterName().Return(cluster.TestCurrentClusterName).AnyTimes()
+	s.mockArchivalMetadata.EXPECT().GetHistoryConfig().Return(archiver.NewArchivalConfig("enabled", dc.GetStringPropertyFn("enabled"), dc.GetBoolPropertyFn(true), "disabled", "some random URI")).Times(2)
+	s.mockArchivalMetadata.EXPECT().GetVisibilityConfig().Return(archiver.NewArchivalConfig("enabled", dc.GetStringPropertyFn("enabled"), dc.GetBoolPropertyFn(true), "disabled", "some random URI")).Times(2)
+	s.mockMetadataMgr.EXPECT().GetNamespace(gomock.Any()).Return(nil, serviceerror.NewNotFound("not found"))
+	s.mockMetadataMgr.EXPECT().CreateNamespace(gomock.Any()).Return(&persistence.CreateNamespaceResponse{
+		ID: testNamespaceID,
+	}, nil)
+
+	s.mockMetadataMgr.EXPECT().GetMetadata().Return(&persistence.GetMetadataResponse{}, nil)
+	s.mockMetadataMgr.EXPECT().GetNamespace(gomock.Any()).Return(&persistence.GetNamespaceResponse{
+		Namespace: &persistencespb.NamespaceDetail{
+			Info: &persistencespb.NamespaceInfo{
+				State: enumspb.NAMESPACE_STATE_REGISTERED,
+			},
+			Config: &persistencespb.NamespaceConfig{},
+			ReplicationConfig: &persistencespb.NamespaceReplicationConfig{
+				ActiveClusterName: cluster.TestCurrentClusterName,
+				Clusters:          []string{cluster.TestCurrentClusterName},
+			},
+		},
+	}, nil)
+	s.mockMetadataMgr.EXPECT().UpdateNamespace(gomock.Any()).DoAndReturn(func(request *persistence.UpdateNamespaceRequest) error {
+		s.Equal(enumspb.NAMESPACE_STATE_DELETED, request.Namespace.Info.State)
+		return nil
+	})
+
+	wh := s.getWorkflowHandler(s.newConfig())
+
+	req := registerNamespaceRequest(enumspb.ARCHIVAL_STATE_UNSPECIFIED, "", enumspb.ARCHIVAL_STATE_UNSPECIFIED, "")
+
+	resp, err := wh.RegisterNamespace(context.Background(), req)
+	s.NoError(err)
+	s.NotNil(resp)
+	respDelete, errDelete := wh.UpdateNamespace(context.Background(), &workflowservice.UpdateNamespaceRequest{
+		Namespace: req.Namespace,
+		UpdateInfo: &namespacepb.UpdateNamespaceInfo{
+			State: enumspb.NAMESPACE_STATE_DELETED,
+		},
+	})
+	s.NoError(errDelete)
+	s.NotNil(respDelete)
+}
+
+func (s *workflowHandlerSuite) TestDeleteNamespace_Error() {
+	s.mockClusterMetadata.EXPECT().IsGlobalNamespaceEnabled().Return(false)
+	s.mockClusterMetadata.EXPECT().GetAllClusterInfo().Return(cluster.TestAllClusterInfo).AnyTimes()
+	s.mockClusterMetadata.EXPECT().GetCurrentClusterName().Return(cluster.TestCurrentClusterName).AnyTimes()
+	s.mockArchivalMetadata.EXPECT().GetHistoryConfig().Return(archiver.NewArchivalConfig("enabled", dc.GetStringPropertyFn("enabled"), dc.GetBoolPropertyFn(true), "disabled", "some random URI")).Times(2)
+	s.mockArchivalMetadata.EXPECT().GetVisibilityConfig().Return(archiver.NewArchivalConfig("enabled", dc.GetStringPropertyFn("enabled"), dc.GetBoolPropertyFn(true), "disabled", "some random URI")).Times(2)
+	s.mockMetadataMgr.EXPECT().GetNamespace(gomock.Any()).Return(nil, serviceerror.NewNotFound("not found"))
+	s.mockMetadataMgr.EXPECT().CreateNamespace(gomock.Any()).Return(&persistence.CreateNamespaceResponse{
+		ID: testNamespaceID,
+	}, nil)
+
+	s.mockMetadataMgr.EXPECT().GetMetadata().Return(&persistence.GetMetadataResponse{}, nil)
+	s.mockMetadataMgr.EXPECT().GetNamespace(gomock.Any()).Return(&persistence.GetNamespaceResponse{
+		Namespace: &persistencespb.NamespaceDetail{
+			Info: &persistencespb.NamespaceInfo{
+				State: enumspb.NAMESPACE_STATE_REGISTERED,
+			},
+			Config: &persistencespb.NamespaceConfig{},
+			ReplicationConfig: &persistencespb.NamespaceReplicationConfig{
+				ActiveClusterName: cluster.TestCurrentClusterName,
+				Clusters:          []string{cluster.TestCurrentClusterName},
+			},
+		},
+	}, nil)
+	s.mockMetadataMgr.EXPECT().UpdateNamespace(gomock.Any()).DoAndReturn(func(request *persistence.UpdateNamespaceRequest) error {
+		s.Equal(enumspb.NAMESPACE_STATE_DELETED, request.Namespace.Info.State)
+		return serviceerror.NewInternal("db is down")
+	})
+
+	wh := s.getWorkflowHandler(s.newConfig())
+
+	req := registerNamespaceRequest(enumspb.ARCHIVAL_STATE_UNSPECIFIED, "", enumspb.ARCHIVAL_STATE_UNSPECIFIED, "")
+
+	resp, err := wh.RegisterNamespace(context.Background(), req)
+	s.NoError(err)
+	s.NotNil(resp)
+	respDelete, errDelete := wh.UpdateNamespace(context.Background(), &workflowservice.UpdateNamespaceRequest{
+		Namespace: req.Namespace,
+		UpdateInfo: &namespacepb.UpdateNamespaceInfo{
+			State: enumspb.NAMESPACE_STATE_DELETED,
+		},
+	})
+	s.Error(errDelete)
+	s.Nil(respDelete)
+}
+
 func (s *workflowHandlerSuite) TestDescribeNamespace_Success_ArchivalDisabled() {
 	getNamespaceResp := persistenceGetNamespaceResponse(
 		&namespace.ArchivalState{State: enumspb.ARCHIVAL_STATE_DISABLED, URI: ""},
@@ -966,7 +1175,7 @@ func (s *workflowHandlerSuite) TestGetArchivedHistory_Failure_NamespaceCacheEntr
 }
 
 func (s *workflowHandlerSuite) TestGetArchivedHistory_Failure_ArchivalURIEmpty() {
-	namespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
+	namespaceEntry := namespace.NewLocalNamespaceForTest(
 		&persistencespb.NamespaceInfo{Name: "test-namespace"},
 		&persistencespb.NamespaceConfig{
 			HistoryArchivalState:    enumspb.ARCHIVAL_STATE_DISABLED,
@@ -974,8 +1183,7 @@ func (s *workflowHandlerSuite) TestGetArchivedHistory_Failure_ArchivalURIEmpty()
 			VisibilityArchivalState: enumspb.ARCHIVAL_STATE_DISABLED,
 			VisibilityArchivalUri:   "",
 		},
-		"",
-		nil)
+		"")
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(gomock.Any()).Return(namespaceEntry, nil).AnyTimes()
 
 	wh := s.getWorkflowHandler(s.newConfig())
@@ -986,7 +1194,7 @@ func (s *workflowHandlerSuite) TestGetArchivedHistory_Failure_ArchivalURIEmpty()
 }
 
 func (s *workflowHandlerSuite) TestGetArchivedHistory_Failure_InvalidURI() {
-	namespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
+	namespaceEntry := namespace.NewLocalNamespaceForTest(
 		&persistencespb.NamespaceInfo{Name: "test-namespace"},
 		&persistencespb.NamespaceConfig{
 			HistoryArchivalState:    enumspb.ARCHIVAL_STATE_ENABLED,
@@ -994,8 +1202,7 @@ func (s *workflowHandlerSuite) TestGetArchivedHistory_Failure_InvalidURI() {
 			VisibilityArchivalState: enumspb.ARCHIVAL_STATE_DISABLED,
 			VisibilityArchivalUri:   "",
 		},
-		"",
-		nil)
+		"")
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(gomock.Any()).Return(namespaceEntry, nil).AnyTimes()
 
 	wh := s.getWorkflowHandler(s.newConfig())
@@ -1006,7 +1213,7 @@ func (s *workflowHandlerSuite) TestGetArchivedHistory_Failure_InvalidURI() {
 }
 
 func (s *workflowHandlerSuite) TestGetArchivedHistory_Success_GetFirstPage() {
-	namespaceEntry := cache.NewLocalNamespaceCacheEntryForTest(
+	namespaceEntry := namespace.NewLocalNamespaceForTest(
 		&persistencespb.NamespaceInfo{Name: "test-namespace"},
 		&persistencespb.NamespaceConfig{
 			HistoryArchivalState:    enumspb.ARCHIVAL_STATE_ENABLED,
@@ -1014,8 +1221,7 @@ func (s *workflowHandlerSuite) TestGetArchivedHistory_Success_GetFirstPage() {
 			VisibilityArchivalState: enumspb.ARCHIVAL_STATE_DISABLED,
 			VisibilityArchivalUri:   "",
 		},
-		"",
-		nil)
+		"")
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(gomock.Any()).Return(namespaceEntry, nil).AnyTimes()
 
 	nextPageToken := []byte{'1', '2', '3'}
@@ -1053,8 +1259,8 @@ func (s *workflowHandlerSuite) TestGetArchivedHistory_Success_GetFirstPage() {
 }
 
 func (s *workflowHandlerSuite) TestGetHistory() {
-	namespaceID := uuid.New()
-	namespace := "namespace"
+	namespaceID := namespace.ID(uuid.New())
+	namespace := namespace.Name("namespace")
 	firstEventID := int64(100)
 	nextEventID := int64(102)
 	branchToken := []byte{1}
@@ -1062,7 +1268,7 @@ func (s *workflowHandlerSuite) TestGetHistory() {
 		WorkflowId: "wid",
 		RunId:      "rid",
 	}
-	shardID := common.WorkflowIDToHistoryShard(namespaceID, we.WorkflowId, numHistoryShards)
+	shardID := common.WorkflowIDToHistoryShard(namespaceID.String(), we.WorkflowId, numHistoryShards)
 	req := &persistence.ReadHistoryBranchRequest{
 		BranchToken:   branchToken,
 		MinEventID:    firstEventID,
@@ -1097,7 +1303,7 @@ func (s *workflowHandlerSuite) TestGetHistory() {
 	}, nil)
 
 	s.mockSearchAttributesProvider.EXPECT().GetSearchAttributes(gomock.Any(), false).Return(searchattribute.TestNameTypeMap, nil)
-	s.mockSearchAttributesMapper.EXPECT().GetAlias("CustomKeywordField", namespace).Return("AliasOfCustomKeyword", nil)
+	s.mockSearchAttributesMapper.EXPECT().GetAlias("CustomKeywordField", namespace.String()).Return("AliasOfCustomKeyword", nil)
 
 	wh := s.getWorkflowHandler(s.newConfig())
 
@@ -1121,7 +1327,110 @@ func (s *workflowHandlerSuite) TestGetHistory() {
 	s.EqualValues(`"random-data"`, history.Events[1].GetWorkflowExecutionStartedEventAttributes().GetSearchAttributes().GetIndexedFields()["TemporalChangeVersion"].GetData())
 }
 
+func (s *workflowHandlerSuite) TestGetWorkflowExecutionHistory() {
+	namespaceID := namespace.ID(uuid.New())
+	namespace := namespace.Name("namespace")
+	we := commonpb.WorkflowExecution{WorkflowId: "wid1", RunId: uuid.New()}
+	newRunID := uuid.New()
+
+	req := &workflowservice.GetWorkflowExecutionHistoryRequest{
+		Namespace:              namespace.String(),
+		Execution:              &we,
+		MaximumPageSize:        10,
+		NextPageToken:          nil,
+		WaitNewEvent:           true,
+		HistoryEventFilterType: enumspb.HISTORY_EVENT_FILTER_TYPE_CLOSE_EVENT,
+		SkipArchival:           true,
+	}
+
+	// set up mocks to simulate a failed workflow with a retry policy. the failure event is id 5.
+	branchToken := []byte{1, 2, 3}
+	shardID := common.WorkflowIDToHistoryShard(namespaceID.String(), we.WorkflowId, numHistoryShards)
+
+	s.mockNamespaceCache.EXPECT().GetNamespaceID(namespace).Return(namespaceID, nil).AnyTimes()
+	s.mockHistoryClient.EXPECT().PollMutableState(gomock.Any(), &historyservice.PollMutableStateRequest{
+		NamespaceId:         namespaceID.String(),
+		Execution:           &we,
+		ExpectedNextEventId: common.EndEventID,
+		CurrentBranchToken:  nil,
+	}).Return(&historyservice.PollMutableStateResponse{
+		Execution:           &we,
+		WorkflowType:        &commonpb.WorkflowType{Name: "mytype"},
+		NextEventId:         6,
+		LastFirstEventId:    5,
+		CurrentBranchToken:  branchToken,
+		VersionHistories:    nil,
+		WorkflowState:       enumsspb.WORKFLOW_EXECUTION_STATE_COMPLETED,
+		WorkflowStatus:      enumspb.WORKFLOW_EXECUTION_STATUS_FAILED,
+		LastFirstEventTxnId: 100,
+	}, nil).Times(2)
+
+	// GetWorkflowExecutionHistory will request the last event
+	s.mockExecutionManager.EXPECT().ReadHistoryBranch(&persistence.ReadHistoryBranchRequest{
+		BranchToken:   branchToken,
+		MinEventID:    5,
+		MaxEventID:    6,
+		PageSize:      10,
+		NextPageToken: nil,
+		ShardID:       shardID,
+	}).Return(&persistence.ReadHistoryBranchResponse{
+		HistoryEvents: []*historypb.HistoryEvent{
+			{
+				EventId:   int64(5),
+				EventType: enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_FAILED,
+				Attributes: &historypb.HistoryEvent_WorkflowExecutionFailedEventAttributes{
+					WorkflowExecutionFailedEventAttributes: &historypb.WorkflowExecutionFailedEventAttributes{
+						Failure:                      &failurepb.Failure{Message: "this workflow failed"},
+						RetryState:                   enumspb.RETRY_STATE_IN_PROGRESS,
+						WorkflowTaskCompletedEventId: 4,
+						NewExecutionRunId:            newRunID,
+					},
+				},
+			},
+		},
+		NextPageToken: []byte{},
+		Size:          1,
+	}, nil).Times(2)
+
+	s.mockExecutionManager.EXPECT().TrimHistoryBranch(gomock.Any()).Return(nil, nil).AnyTimes()
+	s.mockSearchAttributesProvider.EXPECT().GetSearchAttributes(gomock.Any(), false).Return(searchattribute.TestNameTypeMap, nil).AnyTimes()
+
+	wh := s.getWorkflowHandler(s.newConfig())
+
+	oldGoSDKVersion := "1.9.1"
+	newGoSDKVersion := "1.10.1"
+
+	// new sdk: should see failed event
+	ctx := headers.SetVersionsForTests(context.Background(), newGoSDKVersion, headers.ClientNameGoSDK, headers.SupportedServerVersions, headers.AllFeatures)
+	resp, err := wh.GetWorkflowExecutionHistory(ctx, req)
+	s.NoError(err)
+	s.False(resp.Archived)
+	event := resp.History.Events[0]
+	s.Equal(int64(5), event.EventId)
+	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_FAILED, event.EventType)
+	attrs := event.GetWorkflowExecutionFailedEventAttributes()
+	s.Equal("this workflow failed", attrs.Failure.Message)
+	s.Equal(newRunID, attrs.NewExecutionRunId)
+	s.Equal(enumspb.RETRY_STATE_IN_PROGRESS, attrs.RetryState)
+
+	// old sdk: should see continued-as-new event
+	// TODO: We can remove this once we no longer support SDK versions prior to around September 2021.
+	// See comment in workflowHandler.go:GetWorkflowExecutionHistory
+	ctx = headers.SetVersionsForTests(context.Background(), oldGoSDKVersion, headers.ClientNameGoSDK, headers.SupportedServerVersions, "")
+	resp, err = wh.GetWorkflowExecutionHistory(ctx, req)
+	s.NoError(err)
+	s.False(resp.Archived)
+	event = resp.History.Events[0]
+	s.Equal(int64(5), event.EventId)
+	s.Equal(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_CONTINUED_AS_NEW, event.EventType)
+	attrs2 := event.GetWorkflowExecutionContinuedAsNewEventAttributes()
+	s.Equal(newRunID, attrs2.NewExecutionRunId)
+	s.Equal("this workflow failed", attrs2.Failure.Message)
+}
+
 func (s *workflowHandlerSuite) TestListArchivedVisibility_Failure_InvalidRequest() {
+	s.mockArchivalMetadata.EXPECT().GetVisibilityConfig().Return(archiver.NewDisabledArchvialConfig())
+
 	wh := s.getWorkflowHandler(s.newConfig())
 
 	resp, err := wh.ListArchivedWorkflowExecutions(context.Background(), &workflowservice.ListArchivedWorkflowExecutionsRequest{})
@@ -1151,13 +1460,12 @@ func (s *workflowHandlerSuite) TestListArchivedVisibility_Failure_NamespaceCache
 }
 
 func (s *workflowHandlerSuite) TestListArchivedVisibility_Failure_NamespaceNotConfiguredForArchival() {
-	s.mockNamespaceCache.EXPECT().GetNamespace(gomock.Any()).Return(cache.NewLocalNamespaceCacheEntryForTest(
+	s.mockNamespaceCache.EXPECT().GetNamespace(gomock.Any()).Return(namespace.NewLocalNamespaceForTest(
 		nil,
 		&persistencespb.NamespaceConfig{
 			VisibilityArchivalState: enumspb.ARCHIVAL_STATE_DISABLED,
 		},
 		"",
-		nil,
 	), nil)
 	s.mockArchivalMetadata.EXPECT().GetVisibilityConfig().Return(archiver.NewArchivalConfig("enabled", dc.GetStringPropertyFn("enabled"), dc.GetBoolPropertyFn(true), "disabled", "random URI")).Times(2)
 
@@ -1169,14 +1477,13 @@ func (s *workflowHandlerSuite) TestListArchivedVisibility_Failure_NamespaceNotCo
 }
 
 func (s *workflowHandlerSuite) TestListArchivedVisibility_Failure_InvalidURI() {
-	s.mockNamespaceCache.EXPECT().GetNamespace(gomock.Any()).Return(cache.NewLocalNamespaceCacheEntryForTest(
+	s.mockNamespaceCache.EXPECT().GetNamespace(gomock.Any()).Return(namespace.NewLocalNamespaceForTest(
 		&persistencespb.NamespaceInfo{Name: "test-namespace"},
 		&persistencespb.NamespaceConfig{
 			VisibilityArchivalState: enumspb.ARCHIVAL_STATE_DISABLED,
 			VisibilityArchivalUri:   "uri without scheme",
 		},
 		"",
-		nil,
 	), nil)
 	s.mockArchivalMetadata.EXPECT().GetVisibilityConfig().Return(archiver.NewArchivalConfig("enabled", dc.GetStringPropertyFn("enabled"), dc.GetBoolPropertyFn(true), "disabled", "random URI")).Times(2)
 
@@ -1188,14 +1495,13 @@ func (s *workflowHandlerSuite) TestListArchivedVisibility_Failure_InvalidURI() {
 }
 
 func (s *workflowHandlerSuite) TestListArchivedVisibility_Success() {
-	s.mockNamespaceCache.EXPECT().GetNamespace(gomock.Any()).Return(cache.NewLocalNamespaceCacheEntryForTest(
+	s.mockNamespaceCache.EXPECT().GetNamespace(gomock.Any()).Return(namespace.NewLocalNamespaceForTest(
 		&persistencespb.NamespaceInfo{Name: "test-namespace"},
 		&persistencespb.NamespaceConfig{
 			VisibilityArchivalState: enumspb.ARCHIVAL_STATE_ENABLED,
 			VisibilityArchivalUri:   testVisibilityArchivalURI,
 		},
 		"",
-		nil,
 	), nil).AnyTimes()
 	s.mockArchivalMetadata.EXPECT().GetVisibilityConfig().Return(archiver.NewArchivalConfig("enabled", dc.GetStringPropertyFn("enabled"), dc.GetBoolPropertyFn(true), "disabled", "random URI")).Times(2)
 	s.mockVisibilityArchiver.EXPECT().Query(gomock.Any(), gomock.Any(), gomock.Any(), gomock.Any()).Return(&archiver.QueryVisibilityResponse{}, nil)
@@ -1226,10 +1532,10 @@ func (s *workflowHandlerSuite) TestListWorkflowExecutions() {
 	wh := s.getWorkflowHandler(config)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceID(gomock.Any()).Return(s.testNamespaceID, nil).AnyTimes()
-	s.mockVisibilityMgr.EXPECT().ListWorkflowExecutions(gomock.Any()).Return(&visibility.ListWorkflowExecutionsResponse{}, nil)
+	s.mockVisibilityMgr.EXPECT().ListWorkflowExecutions(gomock.Any()).Return(&manager.ListWorkflowExecutionsResponse{}, nil)
 
 	listRequest := &workflowservice.ListWorkflowExecutionsRequest{
-		Namespace: s.testNamespace,
+		Namespace: s.testNamespace.String(),
 		PageSize:  int32(config.ESIndexMaxResultWindow()),
 	}
 	ctx := context.Background()
@@ -1251,10 +1557,10 @@ func (s *workflowHandlerSuite) TestScanWorkflowExecutions() {
 	wh := s.getWorkflowHandler(config)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceID(gomock.Any()).Return(s.testNamespaceID, nil).AnyTimes()
-	s.mockVisibilityMgr.EXPECT().ScanWorkflowExecutions(gomock.Any()).Return(&visibility.ListWorkflowExecutionsResponse{}, nil)
+	s.mockVisibilityMgr.EXPECT().ScanWorkflowExecutions(gomock.Any()).Return(&manager.ListWorkflowExecutionsResponse{}, nil)
 
 	scanRequest := &workflowservice.ScanWorkflowExecutionsRequest{
-		Namespace: s.testNamespace,
+		Namespace: s.testNamespace.String(),
 		PageSize:  int32(config.ESIndexMaxResultWindow()),
 	}
 	ctx := context.Background()
@@ -1266,7 +1572,7 @@ func (s *workflowHandlerSuite) TestScanWorkflowExecutions() {
 	s.Equal(query, scanRequest.GetQuery())
 
 	listRequest := &workflowservice.ListWorkflowExecutionsRequest{
-		Namespace: s.testNamespace,
+		Namespace: s.testNamespace.String(),
 		PageSize:  int32(config.ESIndexMaxResultWindow() + 1),
 		Query:     query,
 	}
@@ -1280,10 +1586,10 @@ func (s *workflowHandlerSuite) TestCountWorkflowExecutions() {
 	wh := s.getWorkflowHandler(config)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceID(gomock.Any()).Return(s.testNamespaceID, nil).AnyTimes()
-	s.mockVisibilityMgr.EXPECT().CountWorkflowExecutions(gomock.Any()).Return(&visibility.CountWorkflowExecutionsResponse{Count: 5}, nil)
+	s.mockVisibilityMgr.EXPECT().CountWorkflowExecutions(gomock.Any()).Return(&manager.CountWorkflowExecutionsResponse{Count: 5}, nil)
 
 	countRequest := &workflowservice.CountWorkflowExecutionsRequest{
-		Namespace: s.testNamespace,
+		Namespace: s.testNamespace.String(),
 	}
 	ctx := context.Background()
 
@@ -1349,145 +1655,10 @@ func (s *workflowHandlerSuite) TestVerifyHistoryIsComplete() {
 	}
 }
 
-func (s *workflowHandlerSuite) TestTokenNamespaceEnforcementDisabled() {
-	s.executeTokenTestCases("wrong-namespace", false, false, false)
-}
-
-func (s *workflowHandlerSuite) TestTokenNamespaceEnforcementEnabledMismatch() {
-	s.executeTokenTestCases("wrong-namespace", true, true, true)
-}
-
-func (s *workflowHandlerSuite) TestTokenNamespaceEnforcementEnabledMatch() {
-	s.executeTokenTestCases(s.testNamespace, true, false, false)
-}
-
-func (s *workflowHandlerSuite) executeTokenTestCases(tokenNamespace string, enforceNamespaceMatch bool,
-	isErrorExpected bool, isNilExpected bool) {
-	ctx := context.Background()
-	wh := s.setupTokenNamespaceTest(tokenNamespace, enforceNamespaceMatch)
-
-	req1 := s.newRespondActivityTaskCompletedRequest(uuid.New())
-	resp1, err := wh.RespondActivityTaskCompleted(ctx, req1)
-	s.checkResponse(err, resp1, isErrorExpected, isNilExpected)
-
-	req2 := s.newRespondActivityTaskFailedRequest(uuid.New())
-	resp2, err := wh.RespondActivityTaskFailed(ctx, req2)
-	s.checkResponse(err, resp2, isErrorExpected, isNilExpected)
-
-	req3 := s.newRespondActivityTaskCanceledRequest(uuid.New())
-	resp3, err := wh.RespondActivityTaskCanceled(ctx, req3)
-	s.checkResponse(err, resp3, isErrorExpected, isNilExpected)
-
-	req4 := s.newRespondWorkflowTaskCompletedRequest(uuid.New())
-	resp4, err := wh.RespondWorkflowTaskCompleted(ctx, req4)
-	s.checkResponse(err, resp4, isErrorExpected, isNilExpected)
-
-	req5 := s.newRespondWorkflowTaskFailedRequest(uuid.New())
-	resp5, err := wh.RespondWorkflowTaskFailed(ctx, req5)
-	s.checkResponse(err, resp5, isErrorExpected, isNilExpected)
-
-	req6 := s.newRespondQueryTaskCompletedRequest(uuid.New())
-	resp6, err := wh.RespondQueryTaskCompleted(ctx, req6)
-	s.checkResponse(err, resp6, isErrorExpected, isNilExpected)
-}
-
-func (s *workflowHandlerSuite) checkResponse(err error, response interface{},
-	isErrorExpected bool, isNilExpected bool) {
-	if isErrorExpected {
-		s.Error(err)
-	} else {
-		s.NoError(err)
-	}
-	if isNilExpected {
-		s.Nil(response)
-	} else {
-		s.NotNil(response)
-	}
-}
-
 func (s *workflowHandlerSuite) newConfig() *Config {
 	return NewConfig(dc.NewCollection(dc.NewNoopClient(), s.mockResource.GetLogger()), numHistoryShards, "", false)
 }
 
-func (s *workflowHandlerSuite) newRespondActivityTaskCompletedRequest(tokenNamespaceId string) *workflowservice.RespondActivityTaskCompletedRequest {
-	return &workflowservice.RespondActivityTaskCompletedRequest{
-		Namespace: s.testNamespace,
-		TaskToken: s.newSerializedToken(tokenNamespaceId),
-	}
-}
-
-func (s *workflowHandlerSuite) newRespondActivityTaskFailedRequest(tokenNamespaceId string) *workflowservice.RespondActivityTaskFailedRequest {
-	return &workflowservice.RespondActivityTaskFailedRequest{
-		Namespace: s.testNamespace,
-		TaskToken: s.newSerializedToken(tokenNamespaceId),
-	}
-}
-
-func (s *workflowHandlerSuite) newRespondActivityTaskCanceledRequest(tokenNamespaceId string) *workflowservice.RespondActivityTaskCanceledRequest {
-	return &workflowservice.RespondActivityTaskCanceledRequest{
-		Namespace: s.testNamespace,
-		TaskToken: s.newSerializedToken(tokenNamespaceId),
-	}
-}
-
-func (s *workflowHandlerSuite) newRespondWorkflowTaskCompletedRequest(tokenNamespaceId string) *workflowservice.RespondWorkflowTaskCompletedRequest {
-	return &workflowservice.RespondWorkflowTaskCompletedRequest{
-		Namespace: s.testNamespace,
-		TaskToken: s.newSerializedToken(tokenNamespaceId),
-	}
-}
-
-func (s *workflowHandlerSuite) newRespondWorkflowTaskFailedRequest(tokenNamespaceId string) *workflowservice.RespondWorkflowTaskFailedRequest {
-	return &workflowservice.RespondWorkflowTaskFailedRequest{
-		Namespace: s.testNamespace,
-		TaskToken: s.newSerializedToken(tokenNamespaceId),
-	}
-}
-
-func (s *workflowHandlerSuite) newRespondQueryTaskCompletedRequest(tokenNamespaceId string) *workflowservice.RespondQueryTaskCompletedRequest {
-	return &workflowservice.RespondQueryTaskCompletedRequest{
-		Namespace: s.testNamespace,
-		TaskToken: s.newSerializedQueryTaskToken(tokenNamespaceId),
-	}
-}
-
-func (s *workflowHandlerSuite) newSerializedToken(namespaceId string) []byte {
-	token, _ := s.tokenSerializer.Serialize(&tokenspb.Task{
-		NamespaceId: namespaceId,
-	})
-	return token
-}
-
-func (s *workflowHandlerSuite) newSerializedQueryTaskToken(namespaceId string) []byte {
-	token, _ := s.tokenSerializer.SerializeQueryTaskToken(&tokenspb.QueryTask{
-		NamespaceId: namespaceId,
-		TaskQueue:   "some-task-queue",
-		TaskId:      "some-task-id",
-	})
-	return token
-}
-
-func newNamespaceCacheEntry(namespaceName string) *cache.NamespaceCacheEntry {
-	info := &persistencespb.NamespaceInfo{
-		Name: namespaceName,
-	}
-	return cache.NewLocalNamespaceCacheEntryForTest(info, nil, "", nil)
-}
-
-func (s *workflowHandlerSuite) setupTokenNamespaceTest(tokenNamespace string, enforce bool) *WorkflowHandler {
-	s.mockNamespaceCache.EXPECT().GetNamespaceByID(gomock.Any()).Return(newNamespaceCacheEntry(tokenNamespace), nil).AnyTimes()
-	ctx := context.Background()
-	s.mockHistoryClient.EXPECT().RespondActivityTaskCompleted(ctx, gomock.Any()).Return(nil, nil).AnyTimes()
-	s.mockHistoryClient.EXPECT().RespondActivityTaskFailed(ctx, gomock.Any()).Return(nil, nil).AnyTimes()
-	s.mockHistoryClient.EXPECT().RespondActivityTaskCanceled(ctx, gomock.Any()).Return(nil, nil).AnyTimes()
-	s.mockHistoryClient.EXPECT().RespondWorkflowTaskCompleted(ctx, gomock.Any()).Return(nil, nil).AnyTimes()
-	s.mockHistoryClient.EXPECT().RespondWorkflowTaskFailed(ctx, gomock.Any()).Return(nil, nil).AnyTimes()
-	s.mockMatchingClient.EXPECT().RespondQueryTaskCompleted(ctx, gomock.Any()).Return(nil, nil).AnyTimes()
-	cfg := s.newConfig()
-	cfg.EnableTokenNamespaceEnforcement = dc.GetBoolPropertyFn(enforce)
-	return s.getWorkflowHandler(cfg)
-}
-
 func updateRequest(
 	historyArchivalURI string,
 	historyArchivalState enumspb.ArchivalState,
