@@ -39,12 +39,12 @@ import (
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/backoff"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/failure"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/common/searchattribute"
 	"go.temporal.io/server/service/history/configs"
@@ -53,7 +53,7 @@ import (
 
 type (
 	commandAttrValidator struct {
-		namespaceCache                  cache.NamespaceCache
+		namespaceRegistry               namespace.Registry
 		config                          *configs.Config
 		maxIDLengthLimit                int
 		searchAttributesValidator       *searchattribute.Validator
@@ -89,12 +89,12 @@ const (
 )
 
 func newCommandAttrValidator(
-	namespaceCache cache.NamespaceCache,
+	namespaceRegistry namespace.Registry,
 	config *configs.Config,
 	searchAttributesValidator *searchattribute.Validator,
 ) *commandAttrValidator {
 	return &commandAttrValidator{
-		namespaceCache:                  namespaceCache,
+		namespaceRegistry:               namespaceRegistry,
 		config:                          config,
 		maxIDLengthLimit:                config.MaxIDLengthLimit(),
 		searchAttributesValidator:       searchAttributesValidator,
@@ -165,7 +165,7 @@ func (c *workflowSizeChecker) failWorkflowIfPayloadSizeExceedsLimit(
 		Failure: failure.NewServerFailure(message, true),
 	}
 
-	if _, err := c.mutableState.AddFailWorkflowEvent(c.completedID, enumspb.RETRY_STATE_NON_RETRYABLE_FAILURE, attributes); err != nil {
+	if _, err := c.mutableState.AddFailWorkflowEvent(c.completedID, enumspb.RETRY_STATE_NON_RETRYABLE_FAILURE, attributes, ""); err != nil {
 		return false, err
 	}
 
@@ -199,7 +199,7 @@ func (c *workflowSizeChecker) failWorkflowIfMemoSizeExceedsLimit(
 		Failure: failure.NewServerFailure(message, true),
 	}
 
-	if _, err := c.mutableState.AddFailWorkflowEvent(c.completedID, enumspb.RETRY_STATE_NON_RETRYABLE_FAILURE, attributes); err != nil {
+	if _, err := c.mutableState.AddFailWorkflowEvent(c.completedID, enumspb.RETRY_STATE_NON_RETRYABLE_FAILURE, attributes, ""); err != nil {
 		return false, err
 	}
 
@@ -208,23 +208,23 @@ func (c *workflowSizeChecker) failWorkflowIfMemoSizeExceedsLimit(
 
 func (c *workflowSizeChecker) failWorkflowIfSearchAttributesSizeExceedsLimit(
 	searchAttributes *commonpb.SearchAttributes,
-	namespace string,
+	namespace namespace.Name,
 	commandTypeTag metrics.Tag,
 ) (bool, error) {
 	c.metricsScope.Tagged(commandTypeTag).RecordDistribution(metrics.SearchAttributesSize, searchAttributes.Size())
 
-	err := c.searchAttributesValidator.ValidateSize(searchAttributes, namespace)
+	err := c.searchAttributesValidator.ValidateSize(searchAttributes, namespace.String())
 	if err == nil {
 		return false, nil
 	}
 
-	c.logger.Warn("Search attributes size exceeds limits. Fail workflow.", tag.Error(err), tag.WorkflowNamespace(namespace))
+	c.logger.Warn("Search attributes size exceeds limits. Fail workflow.", tag.Error(err), tag.WorkflowNamespace(namespace.String()))
 
 	attributes := &commandpb.FailWorkflowExecutionCommandAttributes{
 		Failure: failure.NewServerFailure(err.Error(), true),
 	}
 
-	if _, err := c.mutableState.AddFailWorkflowEvent(c.completedID, enumspb.RETRY_STATE_NON_RETRYABLE_FAILURE, attributes); err != nil {
+	if _, err := c.mutableState.AddFailWorkflowEvent(c.completedID, enumspb.RETRY_STATE_NON_RETRYABLE_FAILURE, attributes, ""); err != nil {
 		return false, err
 	}
 
@@ -232,8 +232,8 @@ func (c *workflowSizeChecker) failWorkflowIfSearchAttributesSizeExceedsLimit(
 }
 
 func (v *commandAttrValidator) validateActivityScheduleAttributes(
-	namespaceID string,
-	targetNamespaceID string,
+	namespaceID namespace.ID,
+	targetNamespaceID namespace.ID,
 	attributes *commandpb.ScheduleActivityTaskCommandAttributes,
 	runTimeout time.Duration,
 ) error {
@@ -430,8 +430,8 @@ func (v *commandAttrValidator) validateCancelWorkflowExecutionAttributes(
 }
 
 func (v *commandAttrValidator) validateCancelExternalWorkflowExecutionAttributes(
-	namespaceID string,
-	targetNamespaceID string,
+	namespaceID namespace.ID,
+	targetNamespaceID namespace.ID,
 	initiatedChildExecutionsInSession map[string]struct{},
 	attributes *commandpb.RequestCancelExternalWorkflowExecutionCommandAttributes,
 ) error {
@@ -467,8 +467,8 @@ func (v *commandAttrValidator) validateCancelExternalWorkflowExecutionAttributes
 }
 
 func (v *commandAttrValidator) validateSignalExternalWorkflowExecutionAttributes(
-	namespaceID string,
-	targetNamespaceID string,
+	namespaceID namespace.ID,
+	targetNamespaceID namespace.ID,
 	attributes *commandpb.SignalExternalWorkflowExecutionCommandAttributes,
 ) error {
 
@@ -507,7 +507,7 @@ func (v *commandAttrValidator) validateSignalExternalWorkflowExecutionAttributes
 }
 
 func (v *commandAttrValidator) validateUpsertWorkflowSearchAttributes(
-	namespace string,
+	namespace namespace.Name,
 	attributes *commandpb.UpsertWorkflowSearchAttributesCommandAttributes,
 	visibilityIndexName string,
 ) error {
@@ -524,11 +524,11 @@ func (v *commandAttrValidator) validateUpsertWorkflowSearchAttributes(
 		return serviceerror.NewInvalidArgument("IndexedFields is empty on command.")
 	}
 
-	return v.searchAttributesValidator.Validate(attributes.GetSearchAttributes(), namespace, visibilityIndexName)
+	return v.searchAttributesValidator.Validate(attributes.GetSearchAttributes(), namespace.String(), visibilityIndexName)
 }
 
 func (v *commandAttrValidator) validateContinueAsNewWorkflowExecutionAttributes(
-	namespace string,
+	namespace namespace.Name,
 	attributes *commandpb.ContinueAsNewWorkflowExecutionCommandAttributes,
 	executionInfo *persistencespb.WorkflowExecutionInfo,
 	visibilityIndexName string,
@@ -574,13 +574,13 @@ func (v *commandAttrValidator) validateContinueAsNewWorkflowExecutionAttributes(
 		attributes.WorkflowTaskTimeout = timestamp.DurationPtr(timestamp.DurationValue(executionInfo.DefaultWorkflowTaskTimeout))
 	}
 
-	return v.searchAttributesValidator.Validate(attributes.GetSearchAttributes(), namespace, visibilityIndexName)
+	return v.searchAttributesValidator.Validate(attributes.GetSearchAttributes(), namespace.String(), visibilityIndexName)
 }
 
 func (v *commandAttrValidator) validateStartChildExecutionAttributes(
-	namespaceID string,
-	targetNamespaceID string,
-	targetNamespace string,
+	namespaceID namespace.ID,
+	targetNamespaceID namespace.ID,
+	targetNamespace namespace.Name,
 	attributes *commandpb.StartChildWorkflowExecutionCommandAttributes,
 	parentInfo *persistencespb.WorkflowExecutionInfo,
 	defaultWorkflowTaskTimeoutFn dynamicconfig.DurationPropertyFnWithNamespaceFilter,
@@ -638,7 +638,7 @@ func (v *commandAttrValidator) validateStartChildExecutionAttributes(
 		return err
 	}
 
-	if err := v.searchAttributesValidator.Validate(attributes.GetSearchAttributes(), targetNamespace, visibilityIndexName); err != nil {
+	if err := v.searchAttributesValidator.Validate(attributes.GetSearchAttributes(), targetNamespace.String(), visibilityIndexName); err != nil {
 		return err
 	}
 
@@ -661,7 +661,7 @@ func (v *commandAttrValidator) validateStartChildExecutionAttributes(
 
 	attributes.WorkflowTaskTimeout = timestamp.DurationPtr(
 		common.OverrideWorkflowTaskTimeout(
-			targetNamespace,
+			targetNamespace.String(),
 			timestamp.DurationValue(attributes.GetWorkflowTaskTimeout()),
 			timestamp.DurationValue(attributes.GetWorkflowRunTimeout()),
 			defaultWorkflowTaskTimeoutFn,
@@ -729,8 +729,8 @@ func (v *commandAttrValidator) validateWorkflowRetryPolicy(
 }
 
 func (v *commandAttrValidator) validateCrossNamespaceCall(
-	namespaceID string,
-	targetNamespaceID string,
+	namespaceID namespace.ID,
+	targetNamespaceID namespace.ID,
 ) error {
 
 	// same name, no check needed
@@ -742,12 +742,12 @@ func (v *commandAttrValidator) validateCrossNamespaceCall(
 		return serviceerror.NewInvalidArgument("cross namespace commands are not allowed")
 	}
 
-	namespaceEntry, err := v.namespaceCache.GetNamespaceByID(namespaceID)
+	namespaceEntry, err := v.namespaceRegistry.GetNamespaceByID(namespaceID)
 	if err != nil {
 		return err
 	}
 
-	targetNamespaceEntry, err := v.namespaceCache.GetNamespaceByID(targetNamespaceID)
+	targetNamespaceEntry, err := v.namespaceRegistry.GetNamespaceByID(targetNamespaceID)
 	if err != nil {
 		return err
 	}
@@ -757,8 +757,8 @@ func (v *commandAttrValidator) validateCrossNamespaceCall(
 		return nil
 	}
 
-	namespaceClusters := namespaceEntry.GetReplicationConfig().Clusters
-	targetNamespaceClusters := targetNamespaceEntry.GetReplicationConfig().Clusters
+	namespaceClusters := namespaceEntry.ClusterNames()
+	targetNamespaceClusters := targetNamespaceEntry.ClusterNames()
 
 	// one is local namespace, another one is global namespace or both global namespace
 	// treat global namespace with one replication cluster as local namespace
@@ -772,8 +772,65 @@ func (v *commandAttrValidator) validateCrossNamespaceCall(
 }
 
 func (v *commandAttrValidator) createCrossNamespaceCallError(
-	namespaceEntry *cache.NamespaceCacheEntry,
-	targetNamespaceEntry *cache.NamespaceCacheEntry,
+	namespaceEntry *namespace.Namespace,
+	targetNamespaceEntry *namespace.Namespace,
 ) error {
-	return serviceerror.NewInvalidArgument(fmt.Sprintf("unable to process cross namespace command between %v and %v", namespaceEntry.GetInfo().Name, targetNamespaceEntry.GetInfo().Name))
+	return serviceerror.NewInvalidArgument(fmt.Sprintf("unable to process cross namespace command between %v and %v", namespaceEntry.Name(), targetNamespaceEntry.Name()))
+}
+
+func (v *commandAttrValidator) validateCommandSequence(
+	commands []*commandpb.Command,
+) error {
+	encounterTerminalCommand := false
+
+	for _, command := range commands {
+		if encounterTerminalCommand {
+			return serviceerror.NewInvalidArgument(fmt.Sprintf(
+				"encouter invalid commands sequence: %v",
+				strings.Join(v.commandTypes(commands), ", "),
+			))
+		}
+
+		switch command.GetCommandType() {
+		case enumspb.COMMAND_TYPE_SCHEDULE_ACTIVITY_TASK:
+			// noop
+		case enumspb.COMMAND_TYPE_REQUEST_CANCEL_ACTIVITY_TASK:
+			// noop
+		case enumspb.COMMAND_TYPE_START_TIMER:
+			// noop
+		case enumspb.COMMAND_TYPE_CANCEL_TIMER:
+			// noop
+		case enumspb.COMMAND_TYPE_RECORD_MARKER:
+			// noop
+		case enumspb.COMMAND_TYPE_REQUEST_CANCEL_EXTERNAL_WORKFLOW_EXECUTION:
+			// noop
+		case enumspb.COMMAND_TYPE_SIGNAL_EXTERNAL_WORKFLOW_EXECUTION:
+			// noop
+		case enumspb.COMMAND_TYPE_START_CHILD_WORKFLOW_EXECUTION:
+			// noop
+		case enumspb.COMMAND_TYPE_UPSERT_WORKFLOW_SEARCH_ATTRIBUTES:
+			// noop
+		case enumspb.COMMAND_TYPE_CONTINUE_AS_NEW_WORKFLOW_EXECUTION:
+			encounterTerminalCommand = true
+		case enumspb.COMMAND_TYPE_COMPLETE_WORKFLOW_EXECUTION:
+			encounterTerminalCommand = true
+		case enumspb.COMMAND_TYPE_FAIL_WORKFLOW_EXECUTION:
+			encounterTerminalCommand = true
+		case enumspb.COMMAND_TYPE_CANCEL_WORKFLOW_EXECUTION:
+			encounterTerminalCommand = true
+		default:
+			return serviceerror.NewInvalidArgument(fmt.Sprintf("unknown command type: %v", command.GetCommandType()))
+		}
+	}
+	return nil
+}
+
+func (v *commandAttrValidator) commandTypes(
+	commands []*commandpb.Command,
+) []string {
+	result := make([]string, len(commands))
+	for index, command := range commands {
+		result[index] = command.GetCommandType().String()
+	}
+	return result
 }
