@@ -33,8 +33,6 @@ import (
 	"github.com/pborman/uuid"
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
-	"github.com/uber-go/tally"
-	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	"go.temporal.io/api/serviceerror"
 
@@ -43,12 +41,12 @@ import (
 	"go.temporal.io/server/api/historyservice/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/clock"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/definition"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/primitives/timestamp"
 	serviceerrors "go.temporal.io/server/common/serviceerror"
@@ -67,14 +65,14 @@ type (
 		mockShard           *shard.ContextTest
 		mockTxProcessor     *MocktransferQueueProcessor
 		mockTimerProcessor  *MocktimerQueueProcessor
-		mockNamespaceCache  *cache.MockNamespaceCache
+		mockNamespaceCache  *namespace.MockRegistry
 		mockClusterMetadata *cluster.MockMetadata
 		mockMutableState    *workflow.MockMutableState
 
 		mockExecutionMgr *persistence.MockExecutionManager
 
 		logger       log.Logger
-		historyCache *workflow.Cache
+		historyCache *workflow.CacheImpl
 
 		nDCActivityReplicator *nDCActivityReplicatorImpl
 	}
@@ -123,7 +121,7 @@ func (s *activityReplicatorSuite) SetupTest() {
 
 	s.logger = s.mockShard.GetLogger()
 
-	s.historyCache = workflow.NewCache(s.mockShard)
+	s.historyCache = workflow.NewCache(s.mockShard).(*workflow.CacheImpl)
 	engine := &historyEngineImpl{
 		currentClusterName: s.mockClusterMetadata.GetCurrentClusterName(),
 		shard:              s.mockShard,
@@ -136,13 +134,13 @@ func (s *activityReplicatorSuite) SetupTest() {
 		timeSource:         s.mockShard.GetTimeSource(),
 		eventNotifier: events.NewNotifier(
 			clock.NewRealTimeSource(),
-			metrics.NewClient(tally.NoopScope, metrics.History),
-			func(string, string) int32 { return 1 },
+			metrics.NewNoopMetricsClient(),
+			func(namespace.ID, string) int32 { return 1 },
 		),
 		txProcessor:    s.mockTxProcessor,
 		timerProcessor: s.mockTimerProcessor,
 	}
-	s.mockShard.SetEngine(engine)
+	s.mockShard.SetEngineForTesting(engine)
 
 	s.nDCActivityReplicator = newNDCActivityReplicator(
 		s.mockShard,
@@ -153,6 +151,7 @@ func (s *activityReplicatorSuite) SetupTest() {
 
 func (s *activityReplicatorSuite) TearDownTest() {
 	s.controller.Finish()
+	s.mockShard.StopForTest()
 }
 
 func (s *activityReplicatorSuite) TestRefreshTask_DiffCluster() {
@@ -461,7 +460,7 @@ func (s *activityReplicatorSuite) TestVersionHistory_IncomingIsSuperSet_Resend()
 	)
 	s.Equal(serviceerrors.NewRetryReplication(
 		resendMissingEventMessage,
-		namespaceID,
+		namespaceID.String(),
 		workflowID,
 		runID,
 		scheduleID-1,
@@ -582,7 +581,7 @@ func (s *activityReplicatorSuite) TestVersionHistory_Diverge_IncomingLarger() {
 	)
 	s.Equal(serviceerrors.NewRetryReplication(
 		resendHigherVersionMessage,
-		namespaceID,
+		namespaceID.String(),
 		workflowID,
 		runID,
 		scheduleID,
@@ -594,28 +593,26 @@ func (s *activityReplicatorSuite) TestVersionHistory_Diverge_IncomingLarger() {
 }
 
 func (s *activityReplicatorSuite) TestSyncActivity_WorkflowNotFound() {
-	namespace := "some random namespace name"
+	namespaceName := namespace.Name("some random namespace name")
 	namespaceID := tests.NamespaceID
 	workflowID := "some random workflow ID"
 	runID := uuid.New()
 	version := int64(100)
 
 	request := &historyservice.SyncActivityRequest{
-		NamespaceId: namespaceID,
+		NamespaceId: namespaceID.String(),
 		WorkflowId:  workflowID,
 		RunId:       runID,
 	}
 	s.mockExecutionMgr.EXPECT().GetWorkflowExecution(&persistence.GetWorkflowExecutionRequest{
 		ShardID:     s.mockShard.GetShardID(),
-		NamespaceID: namespaceID,
-		Execution: commonpb.WorkflowExecution{
-			WorkflowId: workflowID,
-			RunId:      runID,
-		},
+		NamespaceID: namespaceID.String(),
+		WorkflowID:  workflowID,
+		RunID:       runID,
 	}).Return(nil, serviceerror.NewNotFound(""))
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(
-		cache.NewGlobalNamespaceCacheEntryForTest(
-			&persistencespb.NamespaceInfo{Id: namespaceID, Name: namespace},
+		namespace.NewGlobalNamespaceForTest(
+			&persistencespb.NamespaceInfo{Id: namespaceID.String(), Name: namespaceName.String()},
 			&persistencespb.NamespaceConfig{Retention: timestamp.DurationFromDays(1)},
 			&persistencespb.NamespaceReplicationConfig{
 				ActiveClusterName: cluster.TestCurrentClusterName,
@@ -625,7 +622,6 @@ func (s *activityReplicatorSuite) TestSyncActivity_WorkflowNotFound() {
 				},
 			},
 			version,
-			nil,
 		), nil,
 	).AnyTimes()
 
@@ -634,7 +630,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_WorkflowNotFound() {
 }
 
 func (s *activityReplicatorSuite) TestSyncActivity_WorkflowClosed() {
-	namespace := tests.Namespace
+	namespaceName := tests.Namespace
 	namespaceID := tests.NamespaceID
 	workflowID := tests.WorkflowID
 	runID := uuid.New()
@@ -664,7 +660,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_WorkflowClosed() {
 		},
 	}
 
-	key := definition.NewWorkflowIdentifier(namespaceID, workflowID, runID)
+	key := definition.NewWorkflowKey(namespaceID.String(), workflowID, runID)
 	weContext := workflow.NewMockContext(s.controller)
 	weContext.EXPECT().LoadWorkflowExecution().Return(s.mockMutableState, nil)
 	weContext.EXPECT().Lock(gomock.Any(), workflow.CallerTypeAPI).Return(nil)
@@ -673,7 +669,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_WorkflowClosed() {
 	s.NoError(err)
 
 	request := &historyservice.SyncActivityRequest{
-		NamespaceId:    namespaceID,
+		NamespaceId:    namespaceID.String(),
 		WorkflowId:     workflowID,
 		RunId:          runID,
 		Version:        version,
@@ -689,8 +685,8 @@ func (s *activityReplicatorSuite) TestSyncActivity_WorkflowClosed() {
 	).AnyTimes()
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(
-		cache.NewGlobalNamespaceCacheEntryForTest(
-			&persistencespb.NamespaceInfo{Id: namespaceID, Name: namespace},
+		namespace.NewGlobalNamespaceForTest(
+			&persistencespb.NamespaceInfo{Id: namespaceID.String(), Name: namespaceName.String()},
 			&persistencespb.NamespaceConfig{Retention: timestamp.DurationFromDays(1)},
 			&persistencespb.NamespaceReplicationConfig{
 				ActiveClusterName: cluster.TestCurrentClusterName,
@@ -700,7 +696,6 @@ func (s *activityReplicatorSuite) TestSyncActivity_WorkflowClosed() {
 				},
 			},
 			lastWriteVersion,
-			nil,
 		), nil,
 	).AnyTimes()
 
@@ -709,7 +704,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_WorkflowClosed() {
 }
 
 func (s *activityReplicatorSuite) TestSyncActivity_ActivityNotFound() {
-	namespace := tests.Namespace
+	namespaceName := tests.Namespace
 	namespaceID := tests.NamespaceID
 	workflowID := tests.WorkflowID
 	runID := uuid.New()
@@ -739,7 +734,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityNotFound() {
 		},
 	}
 
-	key := definition.NewWorkflowIdentifier(namespaceID, workflowID, runID)
+	key := definition.NewWorkflowKey(namespaceID.String(), workflowID, runID)
 	weContext := workflow.NewMockContext(s.controller)
 	weContext.EXPECT().LoadWorkflowExecution().Return(s.mockMutableState, nil)
 	weContext.EXPECT().Lock(gomock.Any(), workflow.CallerTypeAPI).Return(nil)
@@ -748,7 +743,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityNotFound() {
 	s.NoError(err)
 
 	request := &historyservice.SyncActivityRequest{
-		NamespaceId:    namespaceID,
+		NamespaceId:    namespaceID.String(),
 		WorkflowId:     workflowID,
 		RunId:          runID,
 		Version:        version,
@@ -765,8 +760,8 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityNotFound() {
 	s.mockMutableState.EXPECT().GetActivityInfo(scheduleID).Return(nil, false)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(
-		cache.NewGlobalNamespaceCacheEntryForTest(
-			&persistencespb.NamespaceInfo{Id: namespaceID, Name: namespace},
+		namespace.NewGlobalNamespaceForTest(
+			&persistencespb.NamespaceInfo{Id: namespaceID.String(), Name: namespaceName.String()},
 			&persistencespb.NamespaceConfig{Retention: timestamp.DurationFromDays(1)},
 			&persistencespb.NamespaceReplicationConfig{
 				ActiveClusterName: cluster.TestCurrentClusterName,
@@ -776,7 +771,6 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityNotFound() {
 				},
 			},
 			lastWriteVersion,
-			nil,
 		), nil,
 	).AnyTimes()
 
@@ -785,7 +779,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityNotFound() {
 }
 
 func (s *activityReplicatorSuite) TestSyncActivity_ActivityFound_Zombie() {
-	namespace := tests.Namespace
+	namespaceName := tests.Namespace
 	namespaceID := tests.NamespaceID
 	workflowID := tests.WorkflowID
 	runID := uuid.New()
@@ -815,7 +809,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityFound_Zombie() {
 		},
 	}
 
-	key := definition.NewWorkflowIdentifier(namespaceID, workflowID, runID)
+	key := definition.NewWorkflowKey(namespaceID.String(), workflowID, runID)
 	weContext := workflow.NewMockContext(s.controller)
 	weContext.EXPECT().LoadWorkflowExecution().Return(s.mockMutableState, nil)
 	weContext.EXPECT().Lock(gomock.Any(), workflow.CallerTypeAPI).Return(nil)
@@ -824,7 +818,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityFound_Zombie() {
 	s.NoError(err)
 
 	request := &historyservice.SyncActivityRequest{
-		NamespaceId:    namespaceID,
+		NamespaceId:    namespaceID.String(),
 		WorkflowId:     workflowID,
 		RunId:          runID,
 		Version:        version,
@@ -856,8 +850,8 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityFound_Zombie() {
 	).Return(nil)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(
-		cache.NewGlobalNamespaceCacheEntryForTest(
-			&persistencespb.NamespaceInfo{Id: namespaceID, Name: namespace},
+		namespace.NewGlobalNamespaceForTest(
+			&persistencespb.NamespaceInfo{Id: namespaceID.String(), Name: namespaceName.String()},
 			&persistencespb.NamespaceConfig{Retention: timestamp.DurationFromDays(1)},
 			&persistencespb.NamespaceReplicationConfig{
 				ActiveClusterName: cluster.TestCurrentClusterName,
@@ -867,7 +861,6 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityFound_Zombie() {
 				},
 			},
 			lastWriteVersion,
-			nil,
 		), nil,
 	).AnyTimes()
 
@@ -876,7 +869,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityFound_Zombie() {
 }
 
 func (s *activityReplicatorSuite) TestSyncActivity_ActivityFound_NonZombie() {
-	namespace := tests.Namespace
+	namespaceName := tests.Namespace
 	namespaceID := tests.NamespaceID
 	workflowID := tests.WorkflowID
 	runID := uuid.New()
@@ -906,7 +899,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityFound_NonZombie() {
 		},
 	}
 
-	key := definition.NewWorkflowIdentifier(namespaceID, workflowID, runID)
+	key := definition.NewWorkflowKey(namespaceID.String(), workflowID, runID)
 	weContext := workflow.NewMockContext(s.controller)
 	weContext.EXPECT().LoadWorkflowExecution().Return(s.mockMutableState, nil)
 	weContext.EXPECT().Lock(gomock.Any(), workflow.CallerTypeAPI).Return(nil)
@@ -915,7 +908,7 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityFound_NonZombie() {
 	s.NoError(err)
 
 	request := &historyservice.SyncActivityRequest{
-		NamespaceId:    namespaceID,
+		NamespaceId:    namespaceID.String(),
 		WorkflowId:     workflowID,
 		RunId:          runID,
 		Version:        version,
@@ -947,8 +940,8 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityFound_NonZombie() {
 	).Return(nil)
 
 	s.mockNamespaceCache.EXPECT().GetNamespaceByID(namespaceID).Return(
-		cache.NewGlobalNamespaceCacheEntryForTest(
-			&persistencespb.NamespaceInfo{Id: namespaceID, Name: namespace},
+		namespace.NewGlobalNamespaceForTest(
+			&persistencespb.NamespaceInfo{Id: namespaceID.String(), Name: namespaceName.String()},
 			&persistencespb.NamespaceConfig{Retention: timestamp.DurationFromDays(1)},
 			&persistencespb.NamespaceReplicationConfig{
 				ActiveClusterName: cluster.TestCurrentClusterName,
@@ -958,7 +951,6 @@ func (s *activityReplicatorSuite) TestSyncActivity_ActivityFound_NonZombie() {
 				},
 			},
 			lastWriteVersion,
-			nil,
 		), nil,
 	).AnyTimes()
 
