@@ -105,6 +105,7 @@ func (s *nDCConflictResolverSuite) SetupTest() {
 
 func (s *nDCConflictResolverSuite) TearDownTest() {
 	s.controller.Finish()
+	s.mockShard.StopForTest()
 }
 
 func (s *nDCConflictResolverSuite) TestRebuild() {
@@ -141,7 +142,7 @@ func (s *nDCConflictResolverSuite) TestRebuild() {
 		RunId: s.runID,
 	}).AnyTimes()
 
-	workflowIdentifier := definition.NewWorkflowIdentifier(
+	workflowKey := definition.NewWorkflowKey(
 		s.namespaceID,
 		s.workflowID,
 		s.runID,
@@ -162,11 +163,11 @@ func (s *nDCConflictResolverSuite) TestRebuild() {
 	s.mockStateBuilder.EXPECT().rebuild(
 		ctx,
 		gomock.Any(),
-		workflowIdentifier,
+		workflowKey,
 		branchToken1,
 		lastEventID1,
 		version,
-		workflowIdentifier,
+		workflowKey,
 		branchToken1,
 		requestID,
 	).Return(mockRebuildMutableState, historySize, nil)
@@ -238,7 +239,7 @@ func (s *nDCConflictResolverSuite) TestPrepareMutableState_Rebuild() {
 		RunId: s.runID,
 	}).AnyTimes()
 
-	workflowIdentifier := definition.NewWorkflowIdentifier(
+	workflowKey := definition.NewWorkflowKey(
 		s.namespaceID,
 		s.workflowID,
 		s.runID,
@@ -259,11 +260,11 @@ func (s *nDCConflictResolverSuite) TestPrepareMutableState_Rebuild() {
 	s.mockStateBuilder.EXPECT().rebuild(
 		ctx,
 		gomock.Any(),
-		workflowIdentifier,
+		workflowKey,
 		branchToken1,
 		lastEventID1,
 		version,
-		workflowIdentifier,
+		workflowKey,
 		branchToken1,
 		gomock.Any(),
 	).Return(mockRebuildMutableState, historySize, nil)
