@@ -28,15 +28,25 @@ package history
 
 import (
 	"context"
+	"fmt"
+	"sync"
+
+	"go.temporal.io/server/api/historyservice/v1"
+	"go.temporal.io/server/client/admin"
+	"go.temporal.io/server/client/history"
+	"go.temporal.io/server/common"
+	"go.temporal.io/server/common/xdc"
 
 	"go.temporal.io/api/serviceerror"
 
 	"go.temporal.io/server/api/adminservice/v1"
+	enumsspb "go.temporal.io/server/api/enums/v1"
 	replicationspb "go.temporal.io/server/api/replication/v1"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tasks"
 )
 
 var (
@@ -67,12 +77,17 @@ type (
 	}
 
 	replicationDLQHandlerImpl struct {
-		taskExecutors map[string]replicationTaskExecutor
-		shard         shard.Context
-		logger        log.Logger
+		taskExecutorsLock sync.Mutex
+		taskExecutors     map[string]replicationTaskExecutor
+		shard             shard.Context
+		logger            log.Logger
 	}
 )
 
+func newLazyReplicationDLQHandler(shard shard.Context) replicationDLQHandler {
+	return newReplicationDLQHandler(shard, make(map[string]replicationTaskExecutor))
+}
+
 func newReplicationDLQHandler(
 	shard shard.Context,
 	taskExecutors map[string]replicationTaskExecutor,
@@ -87,6 +102,7 @@ func newReplicationDLQHandler(
 		logger:        shard.GetLogger(),
 	}
 }
+
 func (r *replicationDLQHandlerImpl) getMessages(
 	ctx context.Context,
 	sourceCluster string,
@@ -129,20 +145,37 @@ func (r *replicationDLQHandlerImpl) readMessagesWithAckLevel(
 	}
 	pageToken = resp.NextPageToken
 
-	remoteAdminClient := r.shard.GetService().GetClientBean().GetRemoteAdminClient(sourceCluster)
+	remoteAdminClient := r.shard.GetRemoteAdminClient(sourceCluster)
 	taskInfo := make([]*replicationspb.ReplicationTaskInfo, 0, len(resp.Tasks))
 	for _, task := range resp.Tasks {
-		taskInfo = append(taskInfo, &replicationspb.ReplicationTaskInfo{
-			NamespaceId:  task.GetNamespaceId(),
-			WorkflowId:   task.GetWorkflowId(),
-			RunId:        task.GetRunId(),
-			TaskType:     task.GetTaskType(),
-			TaskId:       task.GetTaskId(),
-			Version:      task.GetVersion(),
-			FirstEventId: task.GetFirstEventId(),
-			NextEventId:  task.GetNextEventId(),
-			ScheduledId:  task.GetScheduledId(),
-		})
+		switch task := task.(type) {
+		case *tasks.SyncActivityTask:
+			taskInfo = append(taskInfo, &replicationspb.ReplicationTaskInfo{
+				NamespaceId:  task.NamespaceID,
+				WorkflowId:   task.WorkflowID,
+				RunId:        task.RunID,
+				TaskType:     enumsspb.TASK_TYPE_REPLICATION_SYNC_ACTIVITY,
+				TaskId:       task.TaskID,
+				Version:      task.GetVersion(),
+				FirstEventId: 0,
+				NextEventId:  0,
+				ScheduledId:  task.ScheduledID,
+			})
+		case *tasks.HistoryReplicationTask:
+			taskInfo = append(taskInfo, &replicationspb.ReplicationTaskInfo{
+				NamespaceId:  task.NamespaceID,
+				WorkflowId:   task.WorkflowID,
+				RunId:        task.RunID,
+				TaskType:     enumsspb.TASK_TYPE_REPLICATION_HISTORY,
+				TaskId:       task.TaskID,
+				Version:      task.Version,
+				FirstEventId: task.FirstEventID,
+				NextEventId:  task.NextEventID,
+				ScheduledId:  0,
+			})
+		default:
+			panic(fmt.Sprintf("Unknown repication task type: %v", task))
+		}
 	}
 
 	if len(taskInfo) == 0 {
@@ -198,10 +231,6 @@ func (r *replicationDLQHandlerImpl) mergeMessages(
 	pageToken []byte,
 ) ([]byte, error) {
 
-	if _, ok := r.taskExecutors[sourceCluster]; !ok {
-		return nil, errInvalidCluster
-	}
-
 	tasks, ackLevel, token, err := r.readMessagesWithAckLevel(
 		ctx,
 		sourceCluster,
@@ -210,8 +239,13 @@ func (r *replicationDLQHandlerImpl) mergeMessages(
 		pageToken,
 	)
 
+	taskExecutor, err := r.getOrCreateTaskExecutor(sourceCluster)
+	if err != nil {
+		return nil, err
+	}
+
 	for _, task := range tasks {
-		if _, err := r.taskExecutors[sourceCluster].execute(
+		if _, err := taskExecutor.execute(
 			task,
 			true,
 		); err != nil {
@@ -240,3 +274,48 @@ func (r *replicationDLQHandlerImpl) mergeMessages(
 	}
 	return token, nil
 }
+
+func (r *replicationDLQHandlerImpl) getOrCreateTaskExecutor(clusterName string) (replicationTaskExecutor, error) {
+	r.taskExecutorsLock.Lock()
+	defer r.taskExecutorsLock.Unlock()
+	if executor, ok := r.taskExecutors[clusterName]; ok {
+		return executor, nil
+	}
+	engine, err := r.shard.GetEngine()
+	if err != nil {
+		return nil, err
+	}
+	adminClient := r.shard.GetRemoteAdminClient(clusterName)
+	adminRetryableClient := admin.NewRetryableClient(
+		adminClient,
+		common.CreateReplicationServiceBusyRetryPolicy(),
+		common.IsResourceExhausted,
+	)
+	historyClient := r.shard.GetHistoryClient()
+	historyRetryableClient := history.NewRetryableClient(
+		historyClient,
+		common.CreateReplicationServiceBusyRetryPolicy(),
+		common.IsResourceExhausted,
+	)
+	resender := xdc.NewNDCHistoryResender(
+		r.shard.GetNamespaceRegistry(),
+		adminRetryableClient,
+		func(ctx context.Context, request *historyservice.ReplicateEventsV2Request) error {
+			_, err := historyRetryableClient.ReplicateEventsV2(ctx, request)
+			return err
+		},
+		r.shard.GetPayloadSerializer(),
+		r.shard.GetConfig().StandbyTaskReReplicationContextTimeout,
+		r.shard.GetLogger(),
+	)
+	taskExecutor := newReplicationTaskExecutor(
+		r.shard,
+		r.shard.GetNamespaceRegistry(),
+		resender,
+		engine,
+		r.shard.GetMetricsClient(),
+		r.shard.GetLogger(),
+	)
+	r.taskExecutors[clusterName] = taskExecutor
+	return taskExecutor, nil
+}
