@@ -32,7 +32,6 @@ import (
 	"github.com/pborman/uuid"
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
-	"github.com/uber-go/tally"
 	commonpb "go.temporal.io/api/common/v1"
 
 	"go.temporal.io/server/api/adminservicemock/v1"
@@ -43,9 +42,9 @@ import (
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	replicationspb "go.temporal.io/server/api/replication/v1"
 	"go.temporal.io/server/client"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/resource"
 	serviceerrors "go.temporal.io/server/common/serviceerror"
@@ -67,7 +66,7 @@ type (
 		mockEngine         *shard.MockEngine
 		config             *configs.Config
 		historyClient      *historyservicemock.MockHistoryServiceClient
-		mockNamespaceCache *cache.MockNamespaceCache
+		mockNamespaceCache *namespace.MockRegistry
 		mockClientBean     *client.MockBean
 		adminClient        *adminservicemock.MockAdminServiceClient
 		clusterMetadata    *cluster.MockMetadata
@@ -118,11 +117,10 @@ func (s *replicationTaskExecutorSuite) SetupTest() {
 	s.nDCHistoryResender = xdc.NewMockNDCHistoryResender(s.controller)
 
 	s.historyClient = historyservicemock.NewMockHistoryServiceClient(s.controller)
-	metricsClient := metrics.NewClient(tally.NoopScope, metrics.History)
+	metricsClient := metrics.NewNoopMetricsClient()
 	s.clusterMetadata.EXPECT().GetCurrentClusterName().Return(cluster.TestCurrentClusterName).AnyTimes()
 
 	s.replicationTaskHandler = newReplicationTaskExecutor(
-		s.currentCluster,
 		s.mockShard,
 		s.mockNamespaceCache,
 		s.nDCHistoryResender,
@@ -134,13 +132,14 @@ func (s *replicationTaskExecutorSuite) SetupTest() {
 
 func (s *replicationTaskExecutorSuite) TearDownTest() {
 	s.controller.Finish()
+	s.mockShard.StopForTest()
 }
 
 func (s *replicationTaskExecutorSuite) TestFilterTask_Apply() {
-	namespaceID := uuid.New()
+	namespaceID := namespace.ID(uuid.New())
 	s.mockNamespaceCache.EXPECT().
 		GetNamespaceByID(namespaceID).
-		Return(cache.NewGlobalNamespaceCacheEntryForTest(
+		Return(namespace.NewGlobalNamespaceForTest(
 			nil,
 			nil,
 			&persistencespb.NamespaceReplicationConfig{Clusters: []string{
@@ -148,7 +147,6 @@ func (s *replicationTaskExecutorSuite) TestFilterTask_Apply() {
 				cluster.TestAlternativeClusterName,
 			}},
 			0,
-			s.clusterMetadata,
 		), nil)
 	ok, err := s.replicationTaskHandler.filterTask(namespaceID, false)
 	s.NoError(err)
@@ -156,15 +154,14 @@ func (s *replicationTaskExecutorSuite) TestFilterTask_Apply() {
 }
 
 func (s *replicationTaskExecutorSuite) TestFilterTask_NotApply() {
-	namespaceID := uuid.New()
+	namespaceID := namespace.ID(uuid.New())
 	s.mockNamespaceCache.EXPECT().
 		GetNamespaceByID(namespaceID).
-		Return(cache.NewGlobalNamespaceCacheEntryForTest(
+		Return(namespace.NewGlobalNamespaceForTest(
 			nil,
 			nil,
 			&persistencespb.NamespaceReplicationConfig{Clusters: []string{cluster.TestAlternativeClusterName}},
 			0,
-			s.clusterMetadata,
 		), nil)
 	ok, err := s.replicationTaskHandler.filterTask(namespaceID, false)
 	s.NoError(err)
@@ -172,7 +169,7 @@ func (s *replicationTaskExecutorSuite) TestFilterTask_NotApply() {
 }
 
 func (s *replicationTaskExecutorSuite) TestFilterTask_Error() {
-	namespaceID := uuid.New()
+	namespaceID := namespace.ID(uuid.New())
 	s.mockNamespaceCache.EXPECT().
 		GetNamespaceByID(namespaceID).
 		Return(nil, fmt.Errorf("random error"))
@@ -182,21 +179,21 @@ func (s *replicationTaskExecutorSuite) TestFilterTask_Error() {
 }
 
 func (s *replicationTaskExecutorSuite) TestFilterTask_EnforceApply() {
-	namespaceID := uuid.New()
+	namespaceID := namespace.ID(uuid.New())
 	ok, err := s.replicationTaskHandler.filterTask(namespaceID, true)
 	s.NoError(err)
 	s.True(ok)
 }
 
 func (s *replicationTaskExecutorSuite) TestProcessTaskOnce_SyncActivityReplicationTask() {
-	namespaceID := uuid.New()
+	namespaceID := namespace.ID(uuid.New())
 	workflowID := uuid.New()
 	runID := uuid.New()
 	task := &replicationspb.ReplicationTask{
 		TaskType: enumsspb.REPLICATION_TASK_TYPE_SYNC_ACTIVITY_TASK,
 		Attributes: &replicationspb.ReplicationTask_SyncActivityTaskAttributes{
 			SyncActivityTaskAttributes: &replicationspb.SyncActivityTaskAttributes{
-				NamespaceId:        namespaceID,
+				NamespaceId:        namespaceID.String(),
 				WorkflowId:         workflowID,
 				RunId:              runID,
 				Version:            1234,
@@ -212,7 +209,7 @@ func (s *replicationTaskExecutorSuite) TestProcessTaskOnce_SyncActivityReplicati
 		},
 	}
 	request := &historyservice.SyncActivityRequest{
-		NamespaceId:        namespaceID,
+		NamespaceId:        namespaceID.String(),
 		WorkflowId:         workflowID,
 		RunId:              runID,
 		Version:            1234,
@@ -232,14 +229,14 @@ func (s *replicationTaskExecutorSuite) TestProcessTaskOnce_SyncActivityReplicati
 }
 
 func (s *replicationTaskExecutorSuite) TestProcessTaskOnce_SyncActivityReplicationTask_Resend() {
-	namespaceID := uuid.New()
+	namespaceID := namespace.ID(uuid.New())
 	workflowID := uuid.New()
 	runID := uuid.New()
 	task := &replicationspb.ReplicationTask{
 		TaskType: enumsspb.REPLICATION_TASK_TYPE_SYNC_ACTIVITY_TASK,
 		Attributes: &replicationspb.ReplicationTask_SyncActivityTaskAttributes{
 			SyncActivityTaskAttributes: &replicationspb.SyncActivityTaskAttributes{
-				NamespaceId:        namespaceID,
+				NamespaceId:        namespaceID.String(),
 				WorkflowId:         workflowID,
 				RunId:              runID,
 				Version:            1234,
@@ -255,7 +252,7 @@ func (s *replicationTaskExecutorSuite) TestProcessTaskOnce_SyncActivityReplicati
 		},
 	}
 	request := &historyservice.SyncActivityRequest{
-		NamespaceId:        namespaceID,
+		NamespaceId:        namespaceID.String(),
 		WorkflowId:         workflowID,
 		RunId:              runID,
 		Version:            1234,
@@ -271,7 +268,7 @@ func (s *replicationTaskExecutorSuite) TestProcessTaskOnce_SyncActivityReplicati
 
 	resendErr := serviceerrors.NewRetryReplication(
 		"some random error message",
-		namespaceID,
+		namespaceID.String(),
 		workflowID,
 		runID,
 		123,
@@ -295,14 +292,14 @@ func (s *replicationTaskExecutorSuite) TestProcessTaskOnce_SyncActivityReplicati
 }
 
 func (s *replicationTaskExecutorSuite) TestProcess_HistoryReplicationTask() {
-	namespaceID := uuid.New()
+	namespaceID := namespace.ID(uuid.New())
 	workflowID := uuid.New()
 	runID := uuid.New()
 	task := &replicationspb.ReplicationTask{
 		TaskType: enumsspb.REPLICATION_TASK_TYPE_HISTORY_V2_TASK,
 		Attributes: &replicationspb.ReplicationTask_HistoryTaskV2Attributes{
 			HistoryTaskV2Attributes: &replicationspb.HistoryTaskV2Attributes{
-				NamespaceId:         namespaceID,
+				NamespaceId:         namespaceID.String(),
 				WorkflowId:          workflowID,
 				RunId:               runID,
 				VersionHistoryItems: []*historyspb.VersionHistoryItem{{EventId: 233, Version: 2333}},
@@ -312,7 +309,7 @@ func (s *replicationTaskExecutorSuite) TestProcess_HistoryReplicationTask() {
 		},
 	}
 	request := &historyservice.ReplicateEventsV2Request{
-		NamespaceId: namespaceID,
+		NamespaceId: namespaceID.String(),
 		WorkflowExecution: &commonpb.WorkflowExecution{
 			WorkflowId: workflowID,
 			RunId:      runID,
@@ -328,14 +325,14 @@ func (s *replicationTaskExecutorSuite) TestProcess_HistoryReplicationTask() {
 }
 
 func (s *replicationTaskExecutorSuite) TestProcess_HistoryReplicationTask_Resend() {
-	namespaceID := uuid.New()
+	namespaceID := namespace.ID(uuid.New())
 	workflowID := uuid.New()
 	runID := uuid.New()
 	task := &replicationspb.ReplicationTask{
 		TaskType: enumsspb.REPLICATION_TASK_TYPE_HISTORY_V2_TASK,
 		Attributes: &replicationspb.ReplicationTask_HistoryTaskV2Attributes{
 			HistoryTaskV2Attributes: &replicationspb.HistoryTaskV2Attributes{
-				NamespaceId:         namespaceID,
+				NamespaceId:         namespaceID.String(),
 				WorkflowId:          workflowID,
 				RunId:               runID,
 				VersionHistoryItems: []*historyspb.VersionHistoryItem{{EventId: 233, Version: 2333}},
@@ -345,7 +342,7 @@ func (s *replicationTaskExecutorSuite) TestProcess_HistoryReplicationTask_Resend
 		},
 	}
 	request := &historyservice.ReplicateEventsV2Request{
-		NamespaceId: namespaceID,
+		NamespaceId: namespaceID.String(),
 		WorkflowExecution: &commonpb.WorkflowExecution{
 			WorkflowId: workflowID,
 			RunId:      runID,
@@ -357,7 +354,7 @@ func (s *replicationTaskExecutorSuite) TestProcess_HistoryReplicationTask_Resend
 
 	resendErr := serviceerrors.NewRetryReplication(
 		"some random error message",
-		namespaceID,
+		namespaceID.String(),
 		workflowID,
 		runID,
 		123,
