@@ -185,6 +185,13 @@ func (s *ContextImpl) GetEngine() (Engine, error) {
 	return s.engine, nil
 }
 
+func (s *ContextImpl) GetMaxTaskIDForCurrentRangeID() int64 {
+	s.rLock()
+	defer s.rUnlock()
+	// maxTransferSequenceNumber is the exclusive upper bound of task ID for current range.
+	return s.maxTransferSequenceNumber - 1
+}
+
 func (s *ContextImpl) GenerateTransferTaskID() (int64, error) {
 	s.wLock()
 	defer s.wUnlock()
@@ -1076,7 +1083,8 @@ func (s *ContextImpl) allocateTaskIDsLocked(
 	return s.allocateTimerIDsLocked(
 		namespaceEntry,
 		workflowID,
-		timerTasks)
+		timerTasks,
+		transferMaxReadLevel)
 }
 
 func (s *ContextImpl) allocateTransferIDsLocked(
@@ -1103,6 +1111,7 @@ func (s *ContextImpl) allocateTimerIDsLocked(
 	namespaceEntry *namespace.Namespace,
 	workflowID string,
 	timerTasks []tasks.Task,
+	transferMaxReadLevel *int64,
 ) error {
 
 	// assign IDs for the timer tasks. They need to be assigned under shard lock.
@@ -1133,6 +1142,7 @@ func (s *ContextImpl) allocateTimerIDsLocked(
 			return err
 		}
 		task.SetTaskID(seqNum)
+		*transferMaxReadLevel = seqNum
 		visibilityTs := task.GetVisibilityTime()
 		s.contextTaggedLogger.Debug("Assigning new timer",
 			tag.Timestamp(visibilityTs), tag.TaskID(task.GetTaskID()), tag.AckLevel(s.shardInfo.TimerAckLevelTime))
@@ -1257,6 +1267,10 @@ func (s *ContextImpl) start() {
 	s.transitionLocked(contextRequestAcquire{})
 }
 
+func (s *ContextImpl) Unload() {
+	s.stop()
+}
+
 // stop should only be called by the controller.
 func (s *ContextImpl) stop() {
 	s.wLock()
