@@ -31,13 +31,14 @@ import (
 	"github.com/pborman/uuid"
 
 	"go.temporal.io/server/api/matchingservice/v1"
-	persistencespb "go.temporal.io/server/api/persistence/v1"
-	"go.temporal.io/server/common/collection"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
+	"go.temporal.io/server/common/timer"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tasks"
 )
 
 type (
@@ -58,11 +59,10 @@ func newTimerQueueActiveProcessor(
 	historyService *historyEngineImpl,
 	matchingClient matchingservice.MatchingServiceClient,
 	taskAllocator taskAllocator,
-	queueTaskProcessor queueTaskProcessor,
 	logger log.Logger,
 ) *timerQueueActiveProcessorImpl {
 
-	currentClusterName := shard.GetService().GetClusterMetadata().GetCurrentClusterName()
+	currentClusterName := shard.GetClusterMetadata().GetCurrentClusterName()
 	timeNow := func() time.Time {
 		return shard.GetCurrentTime(currentClusterName)
 	}
@@ -70,12 +70,8 @@ func newTimerQueueActiveProcessor(
 		return shard.UpdateTimerClusterAckLevel(currentClusterName, ackLevel.VisibilityTimestamp)
 	}
 	logger = log.With(logger, tag.ClusterName(currentClusterName))
-	timerTaskFilter := func(taskInfo queueTaskInfo) (bool, error) {
-		timer, ok := taskInfo.(*persistencespb.TimerTaskInfo)
-		if !ok {
-			return false, errUnexpectedQueueTask
-		}
-		return taskAllocator.verifyActiveTask(timer.GetNamespaceId(), timer)
+	timerTaskFilter := func(task tasks.Task) (bool, error) {
+		return taskAllocator.verifyActiveTask(namespace.ID(task.GetNamespaceID()), task)
 	}
 
 	timerQueueAckMgr := newTimerQueueAckMgr(
@@ -89,9 +85,7 @@ func newTimerQueueActiveProcessor(
 		currentClusterName,
 	)
 
-	timerGate := NewLocalTimerGate(shard.GetTimeSource())
-
-	redispatchQueue := collection.NewConcurrentQueue()
+	timerGate := timer.NewLocalGate(shard.GetTimeSource())
 
 	processor := &timerQueueActiveProcessorImpl{
 		shard:              shard,
@@ -108,32 +102,15 @@ func newTimerQueueActiveProcessor(
 		logger,
 		historyService.metricsClient,
 		shard.GetConfig(),
+		matchingClient,
 	)
-	timerQueueTaskInitializer := func(taskInfo queueTaskInfo) queueTask {
-		return newTimerQueueTask(
-			shard,
-			taskInfo,
-			historyService.metricsClient.Scope(
-				getTimerTaskMetricScope(taskInfo.GetTaskType(), true),
-			),
-			initializeLoggerForTask(shard.GetShardID(), taskInfo, logger),
-			timerTaskFilter,
-			processor.taskExecutor,
-			redispatchQueue,
-			shard.GetTimeSource(),
-			shard.GetConfig().TimerTaskMaxRetryCount,
-			timerQueueAckMgr,
-		)
-	}
+
 	processor.timerQueueProcessorBase = newTimerQueueProcessorBase(
 		metrics.TimerActiveQueueProcessorScope,
 		shard,
 		historyService,
 		processor,
-		queueTaskProcessor,
 		timerQueueAckMgr,
-		redispatchQueue,
-		timerQueueTaskInitializer,
 		timerGate,
 		shard.GetConfig().TimerProcessorMaxPollRPS,
 		logger,
@@ -152,11 +129,10 @@ func newTimerQueueFailoverProcessor(
 	maxLevel time.Time,
 	matchingClient matchingservice.MatchingServiceClient,
 	taskAllocator taskAllocator,
-	queueTaskProcessor queueTaskProcessor,
 	logger log.Logger,
 ) (func(ackLevel timerKey) error, *timerQueueActiveProcessorImpl) {
 
-	currentClusterName := shard.GetService().GetClusterMetadata().GetCurrentClusterName()
+	currentClusterName := shard.GetClusterMetadata().GetCurrentClusterName()
 	timeNow := func() time.Time {
 		// should use current cluster's time when doing namespace failover
 		return shard.GetCurrentTime(currentClusterName)
@@ -186,12 +162,8 @@ func newTimerQueueFailoverProcessor(
 		tag.WorkflowNamespaceIDs(namespaceIDs),
 		tag.FailoverMsg("from: "+standbyClusterName),
 	)
-	timerTaskFilter := func(taskInfo queueTaskInfo) (bool, error) {
-		timer, ok := taskInfo.(*persistencespb.TimerTaskInfo)
-		if !ok {
-			return false, errUnexpectedQueueTask
-		}
-		return taskAllocator.verifyFailoverActiveTask(namespaceIDs, timer.GetNamespaceId(), timer)
+	timerTaskFilter := func(task tasks.Task) (bool, error) {
+		return taskAllocator.verifyFailoverActiveTask(namespaceIDs, namespace.ID(task.GetNamespaceID()), task)
 	}
 
 	timerQueueAckMgr := newTimerQueueFailoverAckMgr(
@@ -205,9 +177,7 @@ func newTimerQueueFailoverProcessor(
 		logger,
 	)
 
-	timerGate := NewLocalTimerGate(shard.GetTimeSource())
-
-	redispatchQueue := collection.NewConcurrentQueue()
+	timerGate := timer.NewLocalGate(shard.GetTimeSource())
 
 	processor := &timerQueueActiveProcessorImpl{
 		shard:              shard,
@@ -224,32 +194,15 @@ func newTimerQueueFailoverProcessor(
 		logger,
 		historyService.metricsClient,
 		shard.GetConfig(),
+		matchingClient,
 	)
-	timerQueueTaskInitializer := func(taskInfo queueTaskInfo) queueTask {
-		return newTimerQueueTask(
-			shard,
-			taskInfo,
-			historyService.metricsClient.Scope(
-				getTimerTaskMetricScope(taskInfo.GetTaskType(), true),
-			),
-			initializeLoggerForTask(shard.GetShardID(), taskInfo, logger),
-			timerTaskFilter,
-			processor.taskExecutor,
-			redispatchQueue,
-			shard.GetTimeSource(),
-			shard.GetConfig().TimerTaskMaxRetryCount,
-			timerQueueAckMgr,
-		)
-	}
+
 	processor.timerQueueProcessorBase = newTimerQueueProcessorBase(
 		metrics.TimerActiveQueueProcessorScope,
 		shard,
 		historyService,
 		processor,
-		queueTaskProcessor,
 		timerQueueAckMgr,
-		redispatchQueue,
-		timerQueueTaskInitializer,
 		timerGate,
 		shard.GetConfig().TimerProcessorFailoverMaxPollRPS,
 		logger,
@@ -282,7 +235,7 @@ func (t *timerQueueActiveProcessorImpl) getReadLevel() timerKey {
 // NotifyNewTimers - Notify the processor about the new active timer events arrival.
 // This should be called each time new timer events arrives, otherwise timers maybe fired unexpected.
 func (t *timerQueueActiveProcessorImpl) notifyNewTimers(
-	timerTasks []persistence.Task,
+	timerTasks []tasks.Task,
 ) {
 	t.timerQueueProcessorBase.notifyNewTimers(timerTasks)
 }
@@ -290,11 +243,7 @@ func (t *timerQueueActiveProcessorImpl) notifyNewTimers(
 func (t *timerQueueActiveProcessorImpl) complete(
 	taskInfo *taskInfo,
 ) {
-	timerTask, ok := taskInfo.task.(*persistencespb.TimerTaskInfo)
-	if !ok {
-		return
-	}
-	t.timerQueueProcessorBase.complete(timerTask)
+	t.timerQueueProcessorBase.complete(taskInfo.Task)
 }
 
 func (t *timerQueueActiveProcessorImpl) process(
@@ -302,6 +251,6 @@ func (t *timerQueueActiveProcessorImpl) process(
 	taskInfo *taskInfo,
 ) (int, error) {
 	// TODO: task metricScope should be determined when creating taskInfo
-	metricScope := getTimerTaskMetricScope(taskInfo.task.GetTaskType(), true)
-	return metricScope, t.taskExecutor.execute(ctx, taskInfo.task, taskInfo.shouldProcessTask)
+	metricScope := getTimerTaskMetricScope(taskInfo.Task, true)
+	return metricScope, t.taskExecutor.execute(ctx, taskInfo.Task, taskInfo.shouldProcessTask)
 }
