@@ -28,14 +28,15 @@ import (
 	"context"
 	"time"
 
-	persistencespb "go.temporal.io/server/api/persistence/v1"
-	"go.temporal.io/server/common/collection"
+	"go.temporal.io/server/client"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/log/tag"
 	"go.temporal.io/server/common/metrics"
-	"go.temporal.io/server/common/persistence"
+	"go.temporal.io/server/common/namespace"
+	"go.temporal.io/server/common/timer"
 	"go.temporal.io/server/common/xdc"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tasks"
 )
 
 const (
@@ -48,7 +49,7 @@ type (
 		timerTaskFilter         taskFilter
 		logger                  log.Logger
 		metricsClient           metrics.Client
-		timerGate               RemoteTimerGate
+		timerGate               timer.RemoteGate
 		timerQueueProcessorBase *timerQueueProcessorBase
 		taskExecutor            queueTaskExecutor
 	}
@@ -60,8 +61,8 @@ func newTimerQueueStandbyProcessor(
 	clusterName string,
 	taskAllocator taskAllocator,
 	nDCHistoryResender xdc.NDCHistoryResender,
-	queueTaskProcessor queueTaskProcessor,
 	logger log.Logger,
+	clientBean client.Bean,
 ) *timerQueueStandbyProcessorImpl {
 
 	timeNow := func() time.Time {
@@ -71,15 +72,11 @@ func newTimerQueueStandbyProcessor(
 		return shard.UpdateTimerClusterAckLevel(clusterName, ackLevel.VisibilityTimestamp)
 	}
 	logger = log.With(logger, tag.ClusterName(clusterName))
-	timerTaskFilter := func(taskInfo queueTaskInfo) (bool, error) {
-		timer, ok := taskInfo.(*persistencespb.TimerTaskInfo)
-		if !ok {
-			return false, errUnexpectedQueueTask
-		}
-		return taskAllocator.verifyStandbyTask(clusterName, timer.GetNamespaceId(), timer)
+	timerTaskFilter := func(task tasks.Task) (bool, error) {
+		return taskAllocator.verifyStandbyTask(clusterName, namespace.ID(task.GetNamespaceID()), task)
 	}
 
-	timerGate := NewRemoteTimerGate()
+	timerGate := timer.NewRemoteGate()
 	timerGate.SetCurrentTime(shard.GetCurrentTime(clusterName))
 	timerQueueAckMgr := newTimerQueueAckMgr(
 		metrics.TimerStandbyQueueProcessorScope,
@@ -92,8 +89,6 @@ func newTimerQueueStandbyProcessor(
 		clusterName,
 	)
 
-	redispatchQueue := collection.NewConcurrentQueue()
-
 	processor := &timerQueueStandbyProcessorImpl{
 		shard:           shard,
 		timerTaskFilter: timerTaskFilter,
@@ -108,35 +103,16 @@ func newTimerQueueStandbyProcessor(
 			historyService.metricsClient,
 			clusterName,
 			shard.GetConfig(),
+			clientBean,
 		),
 	}
 
-	timerQueueTaskInitializer := func(taskInfo queueTaskInfo) queueTask {
-		return newTimerQueueTask(
-			shard,
-			taskInfo,
-			historyService.metricsClient.Scope(
-				getTimerTaskMetricScope(taskInfo.GetTaskType(), false),
-			),
-			initializeLoggerForTask(shard.GetShardID(), taskInfo, logger),
-			timerTaskFilter,
-			processor.taskExecutor,
-			redispatchQueue,
-			shard.GetTimeSource(),
-			shard.GetConfig().TimerTaskMaxRetryCount,
-			timerQueueAckMgr,
-		)
-	}
-
 	processor.timerQueueProcessorBase = newTimerQueueProcessorBase(
 		metrics.TimerStandbyQueueProcessorScope,
 		shard,
 		historyService,
 		processor,
-		queueTaskProcessor,
 		timerQueueAckMgr,
-		redispatchQueue,
-		timerQueueTaskInitializer,
 		timerGate,
 		shard.GetConfig().TimerProcessorMaxPollRPS,
 		logger,
@@ -186,7 +162,7 @@ func (t *timerQueueStandbyProcessorImpl) getReadLevel() timerKey {
 // NotifyNewTimers - Notify the processor about the new standby timer events arrival.
 // This should be called each time new timer events arrives, otherwise timers maybe fired unexpected.
 func (t *timerQueueStandbyProcessorImpl) notifyNewTimers(
-	timerTasks []persistence.Task,
+	timerTasks []tasks.Task,
 ) {
 
 	t.timerQueueProcessorBase.notifyNewTimers(timerTasks)
@@ -195,11 +171,7 @@ func (t *timerQueueStandbyProcessorImpl) notifyNewTimers(
 func (t *timerQueueStandbyProcessorImpl) complete(
 	taskInfo *taskInfo,
 ) {
-	timerTask, ok := taskInfo.task.(*persistencespb.TimerTaskInfo)
-	if !ok {
-		return
-	}
-	t.timerQueueProcessorBase.complete(timerTask)
+	t.timerQueueProcessorBase.complete(taskInfo.Task)
 }
 
 func (t *timerQueueStandbyProcessorImpl) process(
@@ -207,6 +179,6 @@ func (t *timerQueueStandbyProcessorImpl) process(
 	taskInfo *taskInfo,
 ) (int, error) {
 	// TODO: task metricScope should be determined when creating taskInfo
-	metricScope := getTimerTaskMetricScope(taskInfo.task.GetTaskType(), false)
-	return metricScope, t.taskExecutor.execute(ctx, taskInfo.task, taskInfo.shouldProcessTask)
+	metricScope := getTimerTaskMetricScope(taskInfo.Task, false)
+	return metricScope, t.taskExecutor.execute(ctx, taskInfo.Task, taskInfo.shouldProcessTask)
 }
