@@ -33,7 +33,6 @@ import (
 	"github.com/pborman/uuid"
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
-	"github.com/uber-go/tally"
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	taskqueuepb "go.temporal.io/api/taskqueue/v1"
@@ -44,12 +43,14 @@ import (
 	enumsspb "go.temporal.io/server/api/enums/v1"
 	"go.temporal.io/server/api/historyservice/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
+	"go.temporal.io/server/client"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/clock"
 	"go.temporal.io/server/common/cluster"
+	"go.temporal.io/server/common/definition"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/metrics"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence"
 	"go.temporal.io/server/common/persistence/versionhistory"
 	"go.temporal.io/server/common/primitives/timestamp"
@@ -57,6 +58,7 @@ import (
 	"go.temporal.io/server/service/history/consts"
 	"go.temporal.io/server/service/history/events"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tasks"
 	"go.temporal.io/server/service/history/tests"
 	"go.temporal.io/server/service/history/workflow"
 )
@@ -72,14 +74,14 @@ type (
 		mockTxProcessor          *MocktransferQueueProcessor
 		mockReplicationProcessor *MockReplicatorQueueProcessor
 		mockTimerProcessor       *MocktimerQueueProcessor
-		mockNamespaceCache       *cache.MockNamespaceCache
+		mockNamespaceCache       *namespace.MockRegistry
 		mockClusterMetadata      *cluster.MockMetadata
 		mockAdminClient          *adminservicemock.MockAdminServiceClient
 		mockNDCHistoryResender   *xdc.MockNDCHistoryResender
 
 		logger               log.Logger
-		namespaceID          string
-		namespaceEntry       *cache.NamespaceCacheEntry
+		namespaceID          namespace.ID
+		namespaceEntry       *namespace.Namespace
 		version              int64
 		clusterName          string
 		now                  time.Time
@@ -88,6 +90,7 @@ type (
 		discardDuration      time.Duration
 
 		timerQueueStandbyTaskExecutor *timerQueueStandbyTaskExecutor
+		mockBean                      *client.MockBean
 	}
 )
 
@@ -106,7 +109,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) SetupTest() {
 	config := tests.NewDynamicConfig()
 	s.namespaceID = tests.NamespaceID
 	s.namespaceEntry = tests.GlobalNamespaceEntry
-	s.version = s.namespaceEntry.GetFailoverVersion()
+	s.version = s.namespaceEntry.FailoverVersion()
 	s.clusterName = cluster.TestAlternativeClusterName
 	s.now = time.Now().UTC()
 	s.timeSource = clock.NewEventTimeSource().Update(s.now)
@@ -123,7 +126,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) SetupTest() {
 	s.mockReplicationProcessor.EXPECT().notifyNewTask().AnyTimes()
 	s.mockTimerProcessor.EXPECT().NotifyNewTimers(gomock.Any(), gomock.Any()).AnyTimes()
 
-	s.mockShard = shard.NewTestContext(
+	s.mockShard = shard.NewTestContextWithTimeSource(
 		s.controller,
 		&persistence.ShardInfoWithFailover{
 			ShardInfo: &persistencespb.ShardInfo{
@@ -132,8 +135,9 @@ func (s *timerQueueStandbyTaskExecutorSuite) SetupTest() {
 				TransferAckLevel: 0,
 			}},
 		config,
+		s.timeSource,
 	)
-	s.mockShard.EventsCache = events.NewEventsCache(
+	s.mockShard.SetEventsCacheForTesting(events.NewEventsCache(
 		s.mockShard.GetShardID(),
 		s.mockShard.GetConfig().EventsCacheInitialSize(),
 		s.mockShard.GetConfig().EventsCacheMaxSize(),
@@ -142,8 +146,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) SetupTest() {
 		false,
 		s.mockShard.GetLogger(),
 		s.mockShard.GetMetricsClient(),
-	)
-	s.mockShard.Resource.TimeSource = s.timeSource
+	))
 
 	// ack manager will use the namespace information
 	s.mockNamespaceCache = s.mockShard.Resource.NamespaceCache
@@ -160,7 +163,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) SetupTest() {
 
 	historyCache := workflow.NewCache(s.mockShard)
 	h := &historyEngineImpl{
-		currentClusterName: s.mockShard.GetService().GetClusterMetadata().GetCurrentClusterName(),
+		currentClusterName: s.mockShard.Resource.GetClusterMetadata().GetCurrentClusterName(),
 		shard:              s.mockShard,
 		clusterMetadata:    s.mockClusterMetadata,
 		executionManager:   s.mockExecutionMgr,
@@ -168,11 +171,13 @@ func (s *timerQueueStandbyTaskExecutorSuite) SetupTest() {
 		logger:             s.logger,
 		tokenSerializer:    common.NewProtoTaskTokenSerializer(),
 		metricsClient:      s.mockShard.GetMetricsClient(),
-		eventNotifier:      events.NewNotifier(s.timeSource, metrics.NewClient(tally.NoopScope, metrics.History), func(string, string) int32 { return 1 }),
+		eventNotifier:      events.NewNotifier(s.timeSource, metrics.NewNoopMetricsClient(), func(namespace.ID, string) int32 { return 1 }),
 		txProcessor:        s.mockTxProcessor,
 		timerProcessor:     s.mockTimerProcessor,
 	}
-	s.mockShard.SetEngine(h)
+	s.mockShard.SetEngineForTesting(h)
+	s.mockBean = client.NewMockBean(s.controller)
+	s.mockBean.EXPECT().GetRemoteAdminClient("standby").Return(s.mockAdminClient)
 
 	s.timerQueueStandbyTaskExecutor = newTimerQueueStandbyTaskExecutor(
 		s.mockShard,
@@ -182,12 +187,14 @@ func (s *timerQueueStandbyTaskExecutorSuite) SetupTest() {
 		s.mockShard.GetMetricsClient(),
 		s.clusterName,
 		config,
-		// newTaskAllocator(s.mockShard),
+		s.mockBean,
 	).(*timerQueueStandbyTaskExecutor)
+
 }
 
 func (s *timerQueueStandbyTaskExecutorSuite) TearDownTest() {
 	s.controller.Finish()
+	s.mockShard.StopForTest()
 }
 
 func (s *timerQueueStandbyTaskExecutorSuite) TestProcessUserTimerTimeout_Pending() {
@@ -210,7 +217,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessUserTimerTimeout_Pending
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -237,19 +244,17 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessUserTimerTimeout_Pending
 	s.NoError(err)
 	s.True(modified)
 	task := mutableState.InsertTimerTasks[0]
-	protoTaskTime := task.(*persistence.UserTimerTask).GetVisibilityTime()
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_USER_TIMER,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_START_TO_CLOSE,
-		VisibilityTime:  &protoTaskTime,
-		EventId:         event.EventId,
+
+	timerTask := &tasks.UserTimerTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Version:             s.version,
+		TaskID:              int64(100),
+		VisibilityTimestamp: task.(*tasks.UserTimerTask).VisibilityTimestamp,
+		EventID:             event.EventId,
 	}
 
 	persistenceMutableState := s.createPersistenceMutableState(mutableState, event.GetEventId(), event.GetVersion())
@@ -261,13 +266,21 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessUserTimerTimeout_Pending
 
 	s.mockShard.SetCurrentTime(s.clusterName, s.now.Add(s.fetchHistoryDuration))
 	s.mockAdminClient.EXPECT().RefreshWorkflowTasks(gomock.Any(), &adminservice.RefreshWorkflowTasksRequest{
-		Namespace: s.namespaceEntry.GetInfo().Name,
+		Namespace: s.namespaceEntry.Name().String(),
 		Execution: &commonpb.WorkflowExecution{
-			WorkflowId: timerTask.GetWorkflowId(),
-			RunId:      timerTask.GetRunId(),
+			WorkflowId: timerTask.WorkflowID,
+			RunId:      timerTask.RunID,
 		},
 	}).Return(&adminservice.RefreshWorkflowTasksResponse{}, nil)
-	s.mockNDCHistoryResender.EXPECT().SendSingleWorkflowHistory(timerTask.GetNamespaceId(), timerTask.GetWorkflowId(), timerTask.GetRunId(), nextEventID, s.version, int64(0), int64(0)).Return(nil)
+	s.mockNDCHistoryResender.EXPECT().SendSingleWorkflowHistory(
+		namespace.ID(timerTask.NamespaceID),
+		timerTask.WorkflowID,
+		timerTask.RunID,
+		nextEventID,
+		s.version,
+		int64(0),
+		int64(0),
+	).Return(nil)
 	err = s.timerQueueStandbyTaskExecutor.execute(context.Background(), timerTask, true)
 	s.Equal(consts.ErrTaskRetry, err)
 
@@ -294,7 +307,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessUserTimerTimeout_Success
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -320,19 +333,17 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessUserTimerTimeout_Success
 	s.NoError(err)
 	s.True(modified)
 	task := mutableState.InsertTimerTasks[0]
-	protoTaskTime := task.(*persistence.UserTimerTask).GetVisibilityTime()
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_USER_TIMER,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_START_TO_CLOSE,
-		VisibilityTime:  &protoTaskTime,
-		EventId:         event.EventId,
+
+	timerTask := &tasks.UserTimerTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Version:             s.version,
+		TaskID:              int64(100),
+		VisibilityTimestamp: task.(*tasks.UserTimerTask).VisibilityTimestamp,
+		EventID:             event.EventId,
 	}
 
 	event = addTimerFiredEvent(mutableState, timerID)
@@ -361,7 +372,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessUserTimerTimeout_Multipl
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -391,19 +402,17 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessUserTimerTimeout_Multipl
 	s.NoError(err)
 	s.True(modified)
 	task := mutableState.InsertTimerTasks[0]
-	protoTaskTime := task.(*persistence.UserTimerTask).GetVisibilityTime()
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_USER_TIMER,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_START_TO_CLOSE,
-		VisibilityTime:  &protoTaskTime,
-		EventId:         event.EventId,
+
+	timerTask := &tasks.UserTimerTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Version:             s.version,
+		TaskID:              int64(100),
+		VisibilityTimestamp: task.(*tasks.UserTimerTask).VisibilityTimestamp,
+		EventID:             event.EventId,
 	}
 
 	event = addTimerFiredEvent(mutableState, timerID1)
@@ -432,7 +441,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessActivityTimeout_Pending(
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -462,19 +471,19 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessActivityTimeout_Pending(
 	s.NoError(err)
 	s.True(modified)
 	task := mutableState.InsertTimerTasks[0]
-	protoTaskTime := task.(*persistence.ActivityTimeoutTask).GetVisibilityTime()
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_ACTIVITY_TIMEOUT,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_SCHEDULE_TO_CLOSE,
-		VisibilityTime:  &protoTaskTime,
-		EventId:         event.EventId,
+
+	timerTask := &tasks.ActivityTimeoutTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Attempt:             1,
+		Version:             s.version,
+		TaskID:              int64(100),
+		TimeoutType:         enumspb.TIMEOUT_TYPE_SCHEDULE_TO_CLOSE,
+		VisibilityTimestamp: task.(*tasks.ActivityTimeoutTask).VisibilityTimestamp,
+		EventID:             event.EventId,
 	}
 
 	persistenceMutableState := s.createPersistenceMutableState(mutableState, scheduledEvent.GetEventId(), scheduledEvent.GetVersion())
@@ -486,16 +495,16 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessActivityTimeout_Pending(
 
 	s.mockShard.SetCurrentTime(s.clusterName, s.now.Add(s.fetchHistoryDuration))
 	s.mockAdminClient.EXPECT().RefreshWorkflowTasks(gomock.Any(), &adminservice.RefreshWorkflowTasksRequest{
-		Namespace: s.namespaceEntry.GetInfo().Name,
+		Namespace: s.namespaceEntry.Name().String(),
 		Execution: &commonpb.WorkflowExecution{
-			WorkflowId: timerTask.GetWorkflowId(),
-			RunId:      timerTask.GetRunId(),
+			WorkflowId: timerTask.WorkflowID,
+			RunId:      timerTask.RunID,
 		},
 	}).Return(&adminservice.RefreshWorkflowTasksResponse{}, nil)
 	s.mockNDCHistoryResender.EXPECT().SendSingleWorkflowHistory(
-		timerTask.GetNamespaceId(),
-		timerTask.GetWorkflowId(),
-		timerTask.GetRunId(),
+		namespace.ID(timerTask.NamespaceID),
+		timerTask.WorkflowID,
+		timerTask.RunID,
 		nextEventID,
 		s.version,
 		int64(0),
@@ -523,7 +532,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessActivityTimeout_Success(
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -554,19 +563,19 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessActivityTimeout_Success(
 	s.NoError(err)
 	s.True(modified)
 	task := mutableState.InsertTimerTasks[0]
-	protoTaskTime := task.(*persistence.ActivityTimeoutTask).GetVisibilityTime()
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_ACTIVITY_TIMEOUT,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_SCHEDULE_TO_CLOSE,
-		VisibilityTime:  &protoTaskTime,
-		EventId:         event.GetEventId(),
+
+	timerTask := &tasks.ActivityTimeoutTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Attempt:             1,
+		Version:             s.version,
+		TaskID:              int64(100),
+		TimeoutType:         enumspb.TIMEOUT_TYPE_SCHEDULE_TO_CLOSE,
+		VisibilityTimestamp: task.(*tasks.ActivityTimeoutTask).VisibilityTimestamp,
+		EventID:             event.GetEventId(),
 	}
 
 	completeEvent := addActivityTaskCompletedEvent(mutableState, scheduledEvent.GetEventId(), startedEvent.GetEventId(), nil, identity)
@@ -594,7 +603,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessActivityTimeout_Heartbea
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -628,19 +637,20 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessActivityTimeout_Heartbea
 	s.NoError(err)
 	s.True(modified)
 	task := mutableState.InsertTimerTasks[0]
-	s.Equal(enumspb.TIMEOUT_TYPE_HEARTBEAT, task.(*persistence.ActivityTimeoutTask).TimeoutType)
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_ACTIVITY_TIMEOUT,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_HEARTBEAT,
-		VisibilityTime:  &time.Time{},
-		EventId:         scheduledEvent.GetEventId(),
+	s.Equal(enumspb.TIMEOUT_TYPE_HEARTBEAT, task.(*tasks.ActivityTimeoutTask).TimeoutType)
+
+	timerTask := &tasks.ActivityTimeoutTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Attempt:             1,
+		Version:             s.version,
+		TaskID:              int64(100),
+		TimeoutType:         enumspb.TIMEOUT_TYPE_HEARTBEAT,
+		VisibilityTimestamp: time.Unix(946684800, 0).Add(-100 * time.Second), // see pendingActivityTimerHeartbeats from mutable state
+		EventID:             scheduledEvent.GetEventId(),
 	}
 
 	persistenceMutableState := s.createPersistenceMutableState(mutableState, startedEvent.GetEventId(), startedEvent.GetVersion())
@@ -665,7 +675,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessActivityTimeout_Multiple
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -705,19 +715,19 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessActivityTimeout_Multiple
 	modified, err := timerSequence.CreateNextActivityTimer()
 	s.NoError(err)
 	s.True(modified)
-	protoTime := activityInfo2.LastHeartbeatUpdateTime.Add(-5 * time.Second)
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_ACTIVITY_TIMEOUT,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_HEARTBEAT,
-		VisibilityTime:  &protoTime,
-		EventId:         scheduledEvent2.GetEventId(),
+
+	timerTask := &tasks.ActivityTimeoutTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Attempt:             1,
+		Version:             s.version,
+		TaskID:              int64(100),
+		TimeoutType:         enumspb.TIMEOUT_TYPE_HEARTBEAT,
+		VisibilityTimestamp: activityInfo2.LastHeartbeatUpdateTime.Add(-5 * time.Second),
+		EventID:             scheduledEvent2.GetEventId(),
 	}
 
 	completeEvent1 := addActivityTaskCompletedEvent(mutableState, scheduledEvent1.GetEventId(), startedEvent1.GetEventId(), nil, identity)
@@ -768,7 +778,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessActivityTimeout_Multiple
 				},
 				UpdateWorkflowEvents: []*persistence.WorkflowEvents{},
 			}, input)
-			return &persistence.UpdateWorkflowExecutionResponse{MutableStateUpdateSessionStats: &persistence.MutableStateUpdateSessionStats{}}, nil
+			return tests.UpdateWorkflowExecutionResponse, nil
 		})
 
 	s.mockShard.SetCurrentTime(s.clusterName, s.now)
@@ -791,7 +801,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTaskTimeout_Pend
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -808,19 +818,18 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTaskTimeout_Pend
 	mutableState.FlushBufferedEvents()
 	nextEventID := startedEvent.GetEventId()
 
-	protoTime := s.now
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_WORKFLOW_TASK_TIMEOUT,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_START_TO_CLOSE,
-		VisibilityTime:  &protoTime,
-		EventId:         di.ScheduleID,
+	timerTask := &tasks.WorkflowTaskTimeoutTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		ScheduleAttempt:     1,
+		Version:             s.version,
+		TaskID:              int64(100),
+		TimeoutType:         enumspb.TIMEOUT_TYPE_START_TO_CLOSE,
+		VisibilityTimestamp: s.now,
+		EventID:             di.ScheduleID,
 	}
 
 	persistenceMutableState := s.createPersistenceMutableState(mutableState, startedEvent.GetEventId(), startedEvent.GetVersion())
@@ -832,16 +841,16 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTaskTimeout_Pend
 
 	s.mockShard.SetCurrentTime(s.clusterName, s.now.Add(s.fetchHistoryDuration))
 	s.mockAdminClient.EXPECT().RefreshWorkflowTasks(gomock.Any(), &adminservice.RefreshWorkflowTasksRequest{
-		Namespace: s.namespaceEntry.GetInfo().Name,
+		Namespace: s.namespaceEntry.Name().String(),
 		Execution: &commonpb.WorkflowExecution{
-			WorkflowId: timerTask.GetWorkflowId(),
-			RunId:      timerTask.GetRunId(),
+			WorkflowId: timerTask.WorkflowID,
+			RunId:      timerTask.RunID,
 		},
 	}).Return(&adminservice.RefreshWorkflowTasksResponse{}, nil)
 	s.mockNDCHistoryResender.EXPECT().SendSingleWorkflowHistory(
-		timerTask.GetNamespaceId(),
-		timerTask.GetWorkflowId(),
-		timerTask.GetRunId(),
+		namespace.ID(timerTask.NamespaceID),
+		timerTask.WorkflowID,
+		timerTask.RunID,
 		nextEventID,
 		s.version,
 		int64(0),
@@ -864,18 +873,18 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTaskTimeout_Sche
 
 	workflowTaskScheduleID := int64(16384)
 
-	protoTaskTime := s.now
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_WORKFLOW_TASK_TIMEOUT,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_SCHEDULE_TO_START,
-		VisibilityTime:  &protoTaskTime,
-		EventId:         workflowTaskScheduleID,
+	timerTask := &tasks.WorkflowTaskTimeoutTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		ScheduleAttempt:     1,
+		Version:             s.version,
+		TaskID:              int64(100),
+		TimeoutType:         enumspb.TIMEOUT_TYPE_SCHEDULE_TO_START,
+		VisibilityTimestamp: s.now,
+		EventID:             workflowTaskScheduleID,
 	}
 
 	s.mockShard.SetCurrentTime(s.clusterName, s.now)
@@ -897,7 +906,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTaskTimeout_Succ
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -915,19 +924,18 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTaskTimeout_Succ
 	// Flush buffered events so real IDs get assigned
 	mutableState.FlushBufferedEvents()
 
-	protoTime := s.now
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_WORKFLOW_TASK_TIMEOUT,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_START_TO_CLOSE,
-		VisibilityTime:  &protoTime,
-		EventId:         di.ScheduleID,
+	timerTask := &tasks.WorkflowTaskTimeoutTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		ScheduleAttempt:     1,
+		Version:             s.version,
+		TaskID:              int64(100),
+		TimeoutType:         enumspb.TIMEOUT_TYPE_START_TO_CLOSE,
+		VisibilityTimestamp: s.now,
+		EventID:             di.ScheduleID,
 	}
 
 	persistenceMutableState := s.createPersistenceMutableState(mutableState, event.GetEventId(), event.GetVersion())
@@ -952,7 +960,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowBackoffTimer_Pen
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -967,17 +975,16 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowBackoffTimer_Pen
 	mutableState.FlushBufferedEvents()
 	nextEventID := event.GetEventId()
 
-	protoTaskTime := s.now
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_WORKFLOW_BACKOFF_TIMER,
-		VisibilityTime:  &protoTaskTime,
+	timerTask := &tasks.WorkflowBackoffTimerTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Version:             s.version,
+		TaskID:              int64(100),
+		VisibilityTimestamp: s.now,
+		WorkflowBackoffType: enumsspb.WORKFLOW_BACKOFF_TYPE_RETRY,
 	}
 
 	persistenceMutableState := s.createPersistenceMutableState(mutableState, event.GetEventId(), event.GetVersion())
@@ -989,16 +996,16 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowBackoffTimer_Pen
 
 	s.mockShard.SetCurrentTime(s.clusterName, time.Now().UTC().Add(s.fetchHistoryDuration))
 	s.mockAdminClient.EXPECT().RefreshWorkflowTasks(gomock.Any(), &adminservice.RefreshWorkflowTasksRequest{
-		Namespace: s.namespaceEntry.GetInfo().Name,
+		Namespace: s.namespaceEntry.Name().String(),
 		Execution: &commonpb.WorkflowExecution{
-			WorkflowId: timerTask.GetWorkflowId(),
-			RunId:      timerTask.GetRunId(),
+			WorkflowId: timerTask.WorkflowID,
+			RunId:      timerTask.RunID,
 		},
 	}).Return(&adminservice.RefreshWorkflowTasksResponse{}, nil)
 	s.mockNDCHistoryResender.EXPECT().SendSingleWorkflowHistory(
-		timerTask.GetNamespaceId(),
-		timerTask.GetWorkflowId(),
-		timerTask.GetRunId(),
+		namespace.ID(timerTask.NamespaceID),
+		timerTask.WorkflowID,
+		timerTask.RunID,
 		nextEventID,
 		s.version,
 		int64(0),
@@ -1026,7 +1033,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowBackoffTimer_Suc
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -1041,17 +1048,16 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowBackoffTimer_Suc
 	// Flush buffered events so real IDs get assigned
 	mutableState.FlushBufferedEvents()
 
-	protoTaskTime := s.now
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_WORKFLOW_BACKOFF_TIMER,
-		VisibilityTime:  &protoTaskTime,
+	timerTask := &tasks.WorkflowBackoffTimerTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Version:             s.version,
+		TaskID:              int64(100),
+		VisibilityTimestamp: s.now,
+		WorkflowBackoffType: enumsspb.WORKFLOW_BACKOFF_TYPE_CRON,
 	}
 
 	persistenceMutableState := s.createPersistenceMutableState(mutableState, di.ScheduleID, di.Version)
@@ -1076,7 +1082,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTimeout_Pending(
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -1095,18 +1101,15 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTimeout_Pending(
 	mutableState.FlushBufferedEvents()
 	nextEventID := completionEvent.GetEventId()
 
-	protoTaskTime := s.now
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_WORKFLOW_RUN_TIMEOUT,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_START_TO_CLOSE,
-		VisibilityTime:  &protoTaskTime,
+	timerTask := &tasks.WorkflowTimeoutTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Version:             s.version,
+		TaskID:              int64(100),
+		VisibilityTimestamp: s.now,
 	}
 
 	persistenceMutableState := s.createPersistenceMutableState(mutableState, completionEvent.GetEventId(), completionEvent.GetVersion())
@@ -1118,16 +1121,16 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTimeout_Pending(
 
 	s.mockShard.SetCurrentTime(s.clusterName, s.now.Add(s.fetchHistoryDuration))
 	s.mockAdminClient.EXPECT().RefreshWorkflowTasks(gomock.Any(), &adminservice.RefreshWorkflowTasksRequest{
-		Namespace: s.namespaceEntry.GetInfo().Name,
+		Namespace: s.namespaceEntry.Name().String(),
 		Execution: &commonpb.WorkflowExecution{
-			WorkflowId: timerTask.GetWorkflowId(),
-			RunId:      timerTask.GetRunId(),
+			WorkflowId: timerTask.WorkflowID,
+			RunId:      timerTask.RunID,
 		},
 	}).Return(&adminservice.RefreshWorkflowTasksResponse{}, nil)
 	s.mockNDCHistoryResender.EXPECT().SendSingleWorkflowHistory(
-		timerTask.GetNamespaceId(),
-		timerTask.GetWorkflowId(),
-		timerTask.GetRunId(),
+		namespace.ID(timerTask.NamespaceID),
+		timerTask.WorkflowID,
+		timerTask.RunID,
 		nextEventID,
 		s.version,
 		int64(0),
@@ -1155,7 +1158,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTimeout_Success(
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -1174,18 +1177,15 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTimeout_Success(
 	// Flush buffered events so real IDs get assigned
 	mutableState.FlushBufferedEvents()
 
-	protoTaskTime := s.now
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_WORKFLOW_RUN_TIMEOUT,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_START_TO_CLOSE,
-		VisibilityTime:  &protoTaskTime,
+	timerTask := &tasks.WorkflowTimeoutTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Version:             s.version,
+		TaskID:              int64(100),
+		VisibilityTimestamp: s.now,
 	}
 
 	persistenceMutableState := s.createPersistenceMutableState(mutableState, event.GetEventId(), event.GetVersion())
@@ -1197,7 +1197,6 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessWorkflowTimeout_Success(
 }
 
 func (s *timerQueueStandbyTaskExecutorSuite) TestProcessRetryTimeout() {
-
 	execution := commonpb.WorkflowExecution{
 		WorkflowId: "some random workflow ID",
 		RunId:      uuid.New(),
@@ -1210,7 +1209,7 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessRetryTimeout() {
 		execution,
 		&historyservice.StartWorkflowExecutionRequest{
 			Attempt:     1,
-			NamespaceId: s.namespaceID,
+			NamespaceId: s.namespaceID.String(),
 			StartRequest: &workflowservice.StartWorkflowExecutionRequest{
 				WorkflowType:        &commonpb.WorkflowType{Name: workflowType},
 				TaskQueue:           &taskqueuepb.TaskQueue{Name: taskQueueName},
@@ -1221,18 +1220,17 @@ func (s *timerQueueStandbyTaskExecutorSuite) TestProcessRetryTimeout() {
 	)
 	s.Nil(err)
 
-	protoTaskTime := s.now
-	s.NoError(err)
-	timerTask := &persistencespb.TimerTaskInfo{
-		ScheduleAttempt: 1,
-		Version:         s.version,
-		NamespaceId:     s.namespaceID,
-		WorkflowId:      execution.GetWorkflowId(),
-		RunId:           execution.GetRunId(),
-		TaskId:          int64(100),
-		TaskType:        enumsspb.TASK_TYPE_ACTIVITY_RETRY_TIMER,
-		TimeoutType:     enumspb.TIMEOUT_TYPE_START_TO_CLOSE,
-		VisibilityTime:  &protoTaskTime,
+	timerTask := &tasks.ActivityRetryTimerTask{
+		WorkflowKey: definition.NewWorkflowKey(
+			s.namespaceID.String(),
+			execution.GetWorkflowId(),
+			execution.GetRunId(),
+		),
+		Attempt:             1,
+		Version:             s.version,
+		TaskID:              int64(100),
+		VisibilityTimestamp: s.now,
+		EventID:             int64(16384),
 	}
 
 	s.mockShard.SetCurrentTime(s.clusterName, s.now)
