@@ -30,16 +30,15 @@ import (
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 
-	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/backoff"
-	"go.temporal.io/server/common/cache"
+	"go.temporal.io/server/common/definition"
 	"go.temporal.io/server/common/log"
 	"go.temporal.io/server/common/metrics"
-	"go.temporal.io/server/common/persistence"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/searchattribute"
 	"go.temporal.io/server/service/history/configs"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tasks"
 	"go.temporal.io/server/service/history/workflow"
 
 	"go.temporal.io/server/service/worker/archiver"
@@ -49,7 +48,7 @@ type (
 	timerQueueTaskExecutorBase struct {
 		shard                    shard.Context
 		historyService           *historyEngineImpl
-		cache                    *workflow.Cache
+		cache                    workflow.Cache
 		logger                   log.Logger
 		metricsClient            metrics.Client
 		config                   *configs.Config
@@ -71,13 +70,13 @@ func newTimerQueueTaskExecutorBase(
 		logger:                   logger,
 		metricsClient:            metricsClient,
 		config:                   config,
-		searchAttributesProvider: shard.GetService().GetSearchAttributesProvider(),
+		searchAttributesProvider: shard.GetSearchAttributesProvider(),
 	}
 }
 
 func (t *timerQueueTaskExecutorBase) executeDeleteHistoryEventTask(
 	ctx context.Context,
-	task *persistencespb.TimerTaskInfo,
+	task *tasks.DeleteHistoryEventTask,
 ) (retError error) {
 	var cancel context.CancelFunc
 	ctx, cancel = context.WithTimeout(ctx, taskTimeout)
@@ -108,23 +107,23 @@ func (t *timerQueueTaskExecutorBase) executeDeleteHistoryEventTask(
 	if err != nil {
 		return err
 	}
-	ok, err := verifyTaskVersion(t.shard, t.logger, task.GetNamespaceId(), lastWriteVersion, task.Version, task)
+	ok, err := verifyTaskVersion(t.shard, t.logger, namespace.ID(task.NamespaceID), lastWriteVersion, task.Version, task)
 	if err != nil || !ok {
 		return err
 	}
 
-	namespaceCacheEntry, err := t.shard.GetNamespaceCache().GetNamespaceByID(task.GetNamespaceId())
+	namespaceRegistryEntry, err := t.shard.GetNamespaceRegistry().GetNamespaceByID(namespace.ID(task.NamespaceID))
 	if err != nil {
 		return err
 	}
-	clusterConfiguredForHistoryArchival := t.shard.GetService().GetArchivalMetadata().GetHistoryConfig().ClusterConfiguredForArchival()
-	namespaceConfiguredForHistoryArchival := namespaceCacheEntry.GetConfig().HistoryArchivalState == enumspb.ARCHIVAL_STATE_ENABLED
+	clusterConfiguredForHistoryArchival := t.shard.GetArchivalMetadata().GetHistoryConfig().ClusterConfiguredForArchival()
+	namespaceConfiguredForHistoryArchival := namespaceRegistryEntry.HistoryArchivalState().State == enumspb.ARCHIVAL_STATE_ENABLED
 	archiveHistory := clusterConfiguredForHistoryArchival && namespaceConfiguredForHistoryArchival
 
 	// TODO: @ycyang once archival backfill is in place cluster:paused && namespace:enabled should be a nop rather than a delete
 	if archiveHistory {
 		t.metricsClient.IncCounter(metrics.HistoryProcessDeleteHistoryEventScope, metrics.WorkflowCleanupArchiveCount)
-		return t.archiveWorkflow(task, weContext, mutableState, namespaceCacheEntry)
+		return t.archiveWorkflow(task, weContext, mutableState, namespaceRegistryEntry)
 	}
 
 	t.metricsClient.IncCounter(metrics.HistoryProcessDeleteHistoryEventScope, metrics.WorkflowCleanupDeleteCount)
@@ -132,24 +131,24 @@ func (t *timerQueueTaskExecutorBase) executeDeleteHistoryEventTask(
 }
 
 func (t *timerQueueTaskExecutorBase) deleteWorkflow(
-	task *persistencespb.TimerTaskInfo,
+	task *tasks.DeleteHistoryEventTask,
 	workflowContext workflow.Context,
 	msBuilder workflow.MutableState,
 ) error {
-
-	if err := t.deleteCurrentWorkflowExecution(task); err != nil {
+	branchToken, err := msBuilder.GetCurrentBranchToken()
+	if err != nil {
 		return err
 	}
 
-	if err := t.deleteWorkflowExecution(task); err != nil {
-		return err
-	}
-
-	if err := t.deleteWorkflowHistory(task, msBuilder); err != nil {
-		return err
-	}
-
-	if err := t.deleteWorkflowVisibility(task); err != nil {
+	if err := t.shard.DeleteWorkflowExecution(
+		definition.WorkflowKey{
+			NamespaceID: task.NamespaceID,
+			WorkflowID:  task.WorkflowID,
+			RunID:       task.RunID,
+		},
+		branchToken,
+		task.Version,
+	); err != nil {
 		return err
 	}
 
@@ -160,10 +159,10 @@ func (t *timerQueueTaskExecutorBase) deleteWorkflow(
 }
 
 func (t *timerQueueTaskExecutorBase) archiveWorkflow(
-	task *persistencespb.TimerTaskInfo,
+	task *tasks.DeleteHistoryEventTask,
 	workflowContext workflow.Context,
 	msBuilder workflow.MutableState,
-	namespaceCacheEntry *cache.NamespaceCacheEntry,
+	namespaceRegistryEntry *namespace.Namespace,
 ) error {
 	branchToken, err := msBuilder.GetCurrentBranchToken()
 	if err != nil {
@@ -176,13 +175,13 @@ func (t *timerQueueTaskExecutorBase) archiveWorkflow(
 
 	req := &archiver.ClientRequest{
 		ArchiveRequest: &archiver.ArchiveRequest{
-			NamespaceID:          task.GetNamespaceId(),
-			WorkflowID:           task.GetWorkflowId(),
-			RunID:                task.GetRunId(),
-			Namespace:            namespaceCacheEntry.GetInfo().Name,
+			NamespaceID:          task.NamespaceID,
+			WorkflowID:           task.WorkflowID,
+			RunID:                task.RunID,
+			Namespace:            namespaceRegistryEntry.Name().String(),
 			ShardID:              t.shard.GetShardID(),
 			Targets:              []archiver.ArchivalTarget{archiver.ArchiveTargetHistory},
-			HistoryURI:           namespaceCacheEntry.GetConfig().HistoryArchivalUri,
+			HistoryURI:           namespaceRegistryEntry.HistoryArchivalState().URI,
 			NextEventID:          msBuilder.GetNextEventID(),
 			BranchToken:          branchToken,
 			CloseFailoverVersion: closeFailoverVersion,
@@ -212,108 +211,40 @@ func (t *timerQueueTaskExecutorBase) archiveWorkflow(
 
 	// delete visibility record here regardless if it's been archived inline or not
 	// since the entire record is included as part of the archive request.
-	if err := t.deleteWorkflowVisibility(task); err != nil {
-		return err
-	}
 
-	if err := t.deleteCurrentWorkflowExecution(task); err != nil {
-		return err
-	}
-
-	if err := t.deleteWorkflowExecution(task); err != nil {
-		return err
-	}
-
-	// delete workflow history if history archival is not needed or history as been archived inline
+	// delete workflow history if history archival is not needed or history has been archived inline
 	if resp.HistoryArchivedInline {
+		// branchToken was retrieved above
 		t.metricsClient.IncCounter(metrics.HistoryProcessDeleteHistoryEventScope, metrics.WorkflowCleanupDeleteHistoryInlineCount)
-		if err := t.deleteWorkflowHistory(task, msBuilder); err != nil {
-			return err
-		}
+	} else {
+		// branchToken == nil means don't delete history
+		branchToken = nil
 	}
+
+	if err := t.shard.DeleteWorkflowExecution(
+		definition.WorkflowKey{
+			NamespaceID: task.NamespaceID,
+			WorkflowID:  task.WorkflowID,
+			RunID:       task.RunID,
+		},
+		branchToken,
+		task.Version,
+	); err != nil {
+		return err
+	}
+
 	// calling clear here to force accesses of mutable state to read database
 	// if this is not called then callers will get mutable state even though its been removed from database
 	workflowContext.Clear()
 	return nil
 }
 
-func (t *timerQueueTaskExecutorBase) deleteWorkflowExecution(
-	task *persistencespb.TimerTaskInfo,
-) error {
-
-	op := func() error {
-		return t.shard.GetExecutionManager().DeleteWorkflowExecution(&persistence.DeleteWorkflowExecutionRequest{
-			ShardID:     t.shard.GetShardID(),
-			NamespaceID: task.GetNamespaceId(),
-			WorkflowID:  task.GetWorkflowId(),
-			RunID:       task.GetRunId(),
-		})
-	}
-	return backoff.Retry(op, workflow.PersistenceOperationRetryPolicy, common.IsPersistenceTransientError)
-}
-
-func (t *timerQueueTaskExecutorBase) deleteCurrentWorkflowExecution(
-	task *persistencespb.TimerTaskInfo,
-) error {
-
-	op := func() error {
-		return t.shard.GetExecutionManager().DeleteCurrentWorkflowExecution(&persistence.DeleteCurrentWorkflowExecutionRequest{
-			ShardID:     t.shard.GetShardID(),
-			NamespaceID: task.GetNamespaceId(),
-			WorkflowID:  task.GetWorkflowId(),
-			RunID:       task.GetRunId(),
-		})
-	}
-	return backoff.Retry(op, workflow.PersistenceOperationRetryPolicy, common.IsPersistenceTransientError)
-}
-
-func (t *timerQueueTaskExecutorBase) deleteWorkflowHistory(
-	task *persistencespb.TimerTaskInfo,
-	msBuilder workflow.MutableState,
-) error {
-
-	op := func() error {
-		branchToken, err := msBuilder.GetCurrentBranchToken()
-		if err != nil {
-			return err
-		}
-		return t.shard.GetExecutionManager().DeleteHistoryBranch(&persistence.DeleteHistoryBranchRequest{
-			BranchToken: branchToken,
-			ShardID:     t.shard.GetShardID(),
-		})
-
-	}
-	return backoff.Retry(op, workflow.PersistenceOperationRetryPolicy, common.IsPersistenceTransientError)
-}
-
-func (t *timerQueueTaskExecutorBase) deleteWorkflowVisibility(
-	task *persistencespb.TimerTaskInfo,
-) error {
-
-	return t.shard.AddTasks(&persistence.AddTasksRequest{
-		ShardID: t.shard.GetShardID(),
-		// RangeID is set by shard
-		NamespaceID: task.GetNamespaceId(),
-		WorkflowID:  task.GetWorkflowId(),
-		RunID:       task.GetRunId(),
-
-		TransferTasks:    nil,
-		TimerTasks:       nil,
-		ReplicationTasks: nil,
-		VisibilityTasks: []persistence.Task{&persistence.DeleteExecutionVisibilityTask{
-			// TaskID is set by shard
-			VisibilityTimestamp: t.shard.GetTimeSource().Now(),
-			Version:             task.GetVersion(),
-		}},
-	})
-}
-
 func (t *timerQueueTaskExecutorBase) getNamespaceIDAndWorkflowExecution(
-	task *persistencespb.TimerTaskInfo,
-) (string, commonpb.WorkflowExecution) {
+	task tasks.Task,
+) (namespace.ID, commonpb.WorkflowExecution) {
 
-	return task.GetNamespaceId(), commonpb.WorkflowExecution{
-		WorkflowId: task.GetWorkflowId(),
-		RunId:      task.GetRunId(),
+	return namespace.ID(task.GetNamespaceID()), commonpb.WorkflowExecution{
+		WorkflowId: task.GetWorkflowID(),
+		RunId:      task.GetRunID(),
 	}
 }
