@@ -39,6 +39,7 @@ import (
 	"go.temporal.io/server/api/historyservice/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/clock"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/primitives/timestamp"
 )
 
@@ -400,12 +401,14 @@ func (b *HistoryBuilder) AddActivityTaskTimedOutEvent(
 func (b *HistoryBuilder) AddCompletedWorkflowEvent(
 	workflowTaskCompletedEventID int64,
 	command *commandpb.CompleteWorkflowExecutionCommandAttributes,
+	newExecutionRunID string,
 ) *historypb.HistoryEvent {
 	event := b.createNewHistoryEvent(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_COMPLETED, b.timeSource.Now())
 	event.Attributes = &historypb.HistoryEvent_WorkflowExecutionCompletedEventAttributes{
 		WorkflowExecutionCompletedEventAttributes: &historypb.WorkflowExecutionCompletedEventAttributes{
 			WorkflowTaskCompletedEventId: workflowTaskCompletedEventID,
 			Result:                       command.Result,
+			NewExecutionRunId:            newExecutionRunID,
 		},
 	}
 
@@ -416,6 +419,7 @@ func (b *HistoryBuilder) AddFailWorkflowEvent(
 	workflowTaskCompletedEventID int64,
 	retryState enumspb.RetryState,
 	command *commandpb.FailWorkflowExecutionCommandAttributes,
+	newExecutionRunID string,
 ) *historypb.HistoryEvent {
 	event := b.createNewHistoryEvent(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_FAILED, b.timeSource.Now())
 	event.Attributes = &historypb.HistoryEvent_WorkflowExecutionFailedEventAttributes{
@@ -423,6 +427,7 @@ func (b *HistoryBuilder) AddFailWorkflowEvent(
 			WorkflowTaskCompletedEventId: workflowTaskCompletedEventID,
 			Failure:                      command.Failure,
 			RetryState:                   retryState,
+			NewExecutionRunId:            newExecutionRunID,
 		},
 	}
 
@@ -431,11 +436,13 @@ func (b *HistoryBuilder) AddFailWorkflowEvent(
 
 func (b *HistoryBuilder) AddTimeoutWorkflowEvent(
 	retryState enumspb.RetryState,
+	newExecutionRunID string,
 ) *historypb.HistoryEvent {
 	event := b.createNewHistoryEvent(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_TIMED_OUT, b.timeSource.Now())
 	event.Attributes = &historypb.HistoryEvent_WorkflowExecutionTimedOutEventAttributes{
 		WorkflowExecutionTimedOutEventAttributes: &historypb.WorkflowExecutionTimedOutEventAttributes{
-			RetryState: retryState,
+			RetryState:        retryState,
+			NewExecutionRunId: newExecutionRunID,
 		},
 	}
 
@@ -475,15 +482,12 @@ func (b *HistoryBuilder) AddContinuedAsNewEvent(
 		WorkflowRunTimeout:           command.WorkflowRunTimeout,
 		WorkflowTaskTimeout:          command.WorkflowTaskTimeout,
 		BackoffStartInterval:         command.BackoffStartInterval,
-		Initiator:                    enumspb.CONTINUE_AS_NEW_INITIATOR_WORKFLOW,
+		Initiator:                    command.Initiator,
 		Failure:                      command.Failure,
 		LastCompletionResult:         command.LastCompletionResult,
 		Memo:                         command.Memo,
 		SearchAttributes:             command.SearchAttributes,
 	}
-	if len(command.CronSchedule) != 0 {
-		attributes.Initiator = enumspb.CONTINUE_AS_NEW_INITIATOR_CRON_SCHEDULE
-	}
 	event.Attributes = &historypb.HistoryEvent_WorkflowExecutionContinuedAsNewEventAttributes{
 		WorkflowExecutionContinuedAsNewEventAttributes: attributes,
 	}
@@ -631,7 +635,7 @@ func (b *HistoryBuilder) AddRequestCancelExternalWorkflowExecutionInitiatedEvent
 func (b *HistoryBuilder) AddRequestCancelExternalWorkflowExecutionFailedEvent(
 	workflowTaskCompletedEventID int64,
 	initiatedEventID int64,
-	namespace string,
+	namespace namespace.Name,
 	workflowID string,
 	runID string,
 	cause enumspb.CancelExternalWorkflowExecutionFailedCause,
@@ -641,7 +645,7 @@ func (b *HistoryBuilder) AddRequestCancelExternalWorkflowExecutionFailedEvent(
 		RequestCancelExternalWorkflowExecutionFailedEventAttributes: &historypb.RequestCancelExternalWorkflowExecutionFailedEventAttributes{
 			WorkflowTaskCompletedEventId: workflowTaskCompletedEventID,
 			InitiatedEventId:             initiatedEventID,
-			Namespace:                    namespace,
+			Namespace:                    namespace.String(),
 			WorkflowExecution: &commonpb.WorkflowExecution{
 				WorkflowId: workflowID,
 				RunId:      runID,
@@ -656,7 +660,7 @@ func (b *HistoryBuilder) AddRequestCancelExternalWorkflowExecutionFailedEvent(
 
 func (b *HistoryBuilder) AddExternalWorkflowExecutionCancelRequested(
 	initiatedEventID int64,
-	namespace string,
+	namespace namespace.Name,
 	workflowID string,
 	runID string,
 ) *historypb.HistoryEvent {
@@ -664,7 +668,7 @@ func (b *HistoryBuilder) AddExternalWorkflowExecutionCancelRequested(
 	event.Attributes = &historypb.HistoryEvent_ExternalWorkflowExecutionCancelRequestedEventAttributes{
 		ExternalWorkflowExecutionCancelRequestedEventAttributes: &historypb.ExternalWorkflowExecutionCancelRequestedEventAttributes{
 			InitiatedEventId: initiatedEventID,
-			Namespace:        namespace,
+			Namespace:        namespace.String(),
 			WorkflowExecution: &commonpb.WorkflowExecution{
 				WorkflowId: workflowID,
 				RunId:      runID,
@@ -692,6 +696,7 @@ func (b *HistoryBuilder) AddSignalExternalWorkflowExecutionInitiatedEvent(
 			Input:             command.Input,
 			Control:           command.Control,
 			ChildWorkflowOnly: command.ChildWorkflowOnly,
+			Header:            command.Header,
 		},
 	}
 
@@ -716,7 +721,7 @@ func (b *HistoryBuilder) AddUpsertWorkflowSearchAttributesEvent(
 func (b *HistoryBuilder) AddSignalExternalWorkflowExecutionFailedEvent(
 	workflowTaskCompletedEventID int64,
 	initiatedEventID int64,
-	namespace string,
+	namespace namespace.Name,
 	workflowID string,
 	runID string,
 	control string,
@@ -727,7 +732,7 @@ func (b *HistoryBuilder) AddSignalExternalWorkflowExecutionFailedEvent(
 		SignalExternalWorkflowExecutionFailedEventAttributes: &historypb.SignalExternalWorkflowExecutionFailedEventAttributes{
 			WorkflowTaskCompletedEventId: workflowTaskCompletedEventID,
 			InitiatedEventId:             initiatedEventID,
-			Namespace:                    namespace,
+			Namespace:                    namespace.String(),
 			WorkflowExecution: &commonpb.WorkflowExecution{
 				WorkflowId: workflowID,
 				RunId:      runID,
@@ -742,7 +747,7 @@ func (b *HistoryBuilder) AddSignalExternalWorkflowExecutionFailedEvent(
 
 func (b *HistoryBuilder) AddExternalWorkflowExecutionSignaled(
 	initiatedEventID int64,
-	namespace string,
+	namespace namespace.Name,
 	workflowID string,
 	runID string,
 	control string,
@@ -751,7 +756,7 @@ func (b *HistoryBuilder) AddExternalWorkflowExecutionSignaled(
 	event.Attributes = &historypb.HistoryEvent_ExternalWorkflowExecutionSignaledEventAttributes{
 		ExternalWorkflowExecutionSignaledEventAttributes: &historypb.ExternalWorkflowExecutionSignaledEventAttributes{
 			InitiatedEventId: initiatedEventID,
-			Namespace:        namespace,
+			Namespace:        namespace.String(),
 			WorkflowExecution: &commonpb.WorkflowExecution{
 				WorkflowId: workflowID,
 				RunId:      runID,
@@ -785,6 +790,7 @@ func (b *HistoryBuilder) AddWorkflowExecutionSignaledEvent(
 	signalName string,
 	input *commonpb.Payloads,
 	identity string,
+	header *commonpb.Header,
 ) *historypb.HistoryEvent {
 	event := b.createNewHistoryEvent(enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_SIGNALED, b.timeSource.Now())
 	event.Attributes = &historypb.HistoryEvent_WorkflowExecutionSignaledEventAttributes{
@@ -792,6 +798,7 @@ func (b *HistoryBuilder) AddWorkflowExecutionSignaledEvent(
 			SignalName: signalName,
 			Input:      input,
 			Identity:   identity,
+			Header:     header,
 		},
 	}
 
@@ -830,7 +837,7 @@ func (b *HistoryBuilder) AddStartChildWorkflowExecutionInitiatedEvent(
 
 func (b *HistoryBuilder) AddChildWorkflowExecutionStartedEvent(
 	initiatedID int64,
-	namespace string,
+	namespace namespace.Name,
 	execution *commonpb.WorkflowExecution,
 	workflowType *commonpb.WorkflowType,
 	header *commonpb.Header,
@@ -839,7 +846,7 @@ func (b *HistoryBuilder) AddChildWorkflowExecutionStartedEvent(
 	event.Attributes = &historypb.HistoryEvent_ChildWorkflowExecutionStartedEventAttributes{
 		ChildWorkflowExecutionStartedEventAttributes: &historypb.ChildWorkflowExecutionStartedEventAttributes{
 			InitiatedEventId:  initiatedID,
-			Namespace:         namespace,
+			Namespace:         namespace.String(),
 			WorkflowExecution: execution,
 			WorkflowType:      workflowType,
 			Header:            header,
@@ -853,7 +860,7 @@ func (b *HistoryBuilder) AddStartChildWorkflowExecutionFailedEvent(
 	workflowTaskCompletedEventID int64,
 	initiatedID int64,
 	cause enumspb.StartChildWorkflowExecutionFailedCause,
-	namespace string,
+	namespace namespace.Name,
 	workflowID string,
 	workflowType *commonpb.WorkflowType,
 	control string,
@@ -863,7 +870,7 @@ func (b *HistoryBuilder) AddStartChildWorkflowExecutionFailedEvent(
 		StartChildWorkflowExecutionFailedEventAttributes: &historypb.StartChildWorkflowExecutionFailedEventAttributes{
 			WorkflowTaskCompletedEventId: workflowTaskCompletedEventID,
 			InitiatedEventId:             initiatedID,
-			Namespace:                    namespace,
+			Namespace:                    namespace.String(),
 			WorkflowId:                   workflowID,
 			WorkflowType:                 workflowType,
 			Control:                      control,
@@ -877,7 +884,7 @@ func (b *HistoryBuilder) AddStartChildWorkflowExecutionFailedEvent(
 func (b *HistoryBuilder) AddChildWorkflowExecutionCompletedEvent(
 	initiatedID int64,
 	startedID int64,
-	namespace string,
+	namespace namespace.Name,
 	execution *commonpb.WorkflowExecution,
 	workflowType *commonpb.WorkflowType,
 	result *commonpb.Payloads,
@@ -887,7 +894,7 @@ func (b *HistoryBuilder) AddChildWorkflowExecutionCompletedEvent(
 		ChildWorkflowExecutionCompletedEventAttributes: &historypb.ChildWorkflowExecutionCompletedEventAttributes{
 			InitiatedEventId:  initiatedID,
 			StartedEventId:    startedID,
-			Namespace:         namespace,
+			Namespace:         namespace.String(),
 			WorkflowExecution: execution,
 			WorkflowType:      workflowType,
 			Result:            result,
@@ -900,7 +907,7 @@ func (b *HistoryBuilder) AddChildWorkflowExecutionCompletedEvent(
 func (b *HistoryBuilder) AddChildWorkflowExecutionFailedEvent(
 	initiatedID int64,
 	startedID int64,
-	namespace string,
+	namespace namespace.Name,
 	execution *commonpb.WorkflowExecution,
 	workflowType *commonpb.WorkflowType,
 	failure *failurepb.Failure,
@@ -911,7 +918,7 @@ func (b *HistoryBuilder) AddChildWorkflowExecutionFailedEvent(
 		ChildWorkflowExecutionFailedEventAttributes: &historypb.ChildWorkflowExecutionFailedEventAttributes{
 			InitiatedEventId:  initiatedID,
 			StartedEventId:    startedID,
-			Namespace:         namespace,
+			Namespace:         namespace.String(),
 			WorkflowExecution: execution,
 			WorkflowType:      workflowType,
 			Failure:           failure,
@@ -925,7 +932,7 @@ func (b *HistoryBuilder) AddChildWorkflowExecutionFailedEvent(
 func (b *HistoryBuilder) AddChildWorkflowExecutionCanceledEvent(
 	initiatedID int64,
 	startedID int64,
-	namespace string,
+	namespace namespace.Name,
 	execution *commonpb.WorkflowExecution,
 	workflowType *commonpb.WorkflowType,
 	details *commonpb.Payloads,
@@ -935,7 +942,7 @@ func (b *HistoryBuilder) AddChildWorkflowExecutionCanceledEvent(
 		ChildWorkflowExecutionCanceledEventAttributes: &historypb.ChildWorkflowExecutionCanceledEventAttributes{
 			InitiatedEventId:  initiatedID,
 			StartedEventId:    startedID,
-			Namespace:         namespace,
+			Namespace:         namespace.String(),
 			WorkflowExecution: execution,
 			WorkflowType:      workflowType,
 			Details:           details,
@@ -948,7 +955,7 @@ func (b *HistoryBuilder) AddChildWorkflowExecutionCanceledEvent(
 func (b *HistoryBuilder) AddChildWorkflowExecutionTerminatedEvent(
 	initiatedID int64,
 	startedID int64,
-	namespace string,
+	namespace namespace.Name,
 	execution *commonpb.WorkflowExecution,
 	workflowType *commonpb.WorkflowType,
 ) *historypb.HistoryEvent {
@@ -957,7 +964,7 @@ func (b *HistoryBuilder) AddChildWorkflowExecutionTerminatedEvent(
 		ChildWorkflowExecutionTerminatedEventAttributes: &historypb.ChildWorkflowExecutionTerminatedEventAttributes{
 			InitiatedEventId:  initiatedID,
 			StartedEventId:    startedID,
-			Namespace:         namespace,
+			Namespace:         namespace.String(),
 			WorkflowExecution: execution,
 			WorkflowType:      workflowType,
 		},
@@ -969,7 +976,7 @@ func (b *HistoryBuilder) AddChildWorkflowExecutionTerminatedEvent(
 func (b *HistoryBuilder) AddChildWorkflowExecutionTimedOutEvent(
 	initiatedID int64,
 	startedID int64,
-	namespace string,
+	namespace namespace.Name,
 	execution *commonpb.WorkflowExecution,
 	workflowType *commonpb.WorkflowType,
 	retryState enumspb.RetryState,
@@ -979,7 +986,7 @@ func (b *HistoryBuilder) AddChildWorkflowExecutionTimedOutEvent(
 		ChildWorkflowExecutionTimedOutEventAttributes: &historypb.ChildWorkflowExecutionTimedOutEventAttributes{
 			InitiatedEventId:  initiatedID,
 			StartedEventId:    startedID,
-			Namespace:         namespace,
+			Namespace:         namespace.String(),
 			WorkflowExecution: execution,
 			WorkflowType:      workflowType,
 			RetryState:        retryState,
