@@ -32,7 +32,7 @@ import (
 	"github.com/pborman/uuid"
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
-	"github.com/uber-go/tally"
+	"github.com/uber-go/tally/v4"
 	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	historypb "go.temporal.io/api/history/v1"
@@ -42,11 +42,11 @@ import (
 	historyspb "go.temporal.io/server/api/history/v1"
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/definition"
 	"go.temporal.io/server/common/dynamicconfig"
 	"go.temporal.io/server/common/failure"
 	"go.temporal.io/server/common/log"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/payload"
 	"go.temporal.io/server/common/payloads"
 	"go.temporal.io/server/common/persistence"
@@ -107,7 +107,7 @@ func (s *mutableStateSuite) SetupTest() {
 	// set the checksum probabilities to 100% for exercising during test
 	s.mockConfig.MutableStateChecksumGenProbability = func(namespace string) int { return 100 }
 	s.mockConfig.MutableStateChecksumVerifyProbability = func(namespace string) int { return 100 }
-	s.mockShard.EventsCache = s.mockEventsCache
+	s.mockShard.SetEventsCacheForTesting(s.mockEventsCache)
 
 	s.testScope = s.mockShard.Resource.MetricsScope.(tally.TestScope)
 	s.logger = s.mockShard.GetLogger()
@@ -117,6 +117,7 @@ func (s *mutableStateSuite) SetupTest() {
 
 func (s *mutableStateSuite) TearDownTest() {
 	s.controller.Finish()
+	s.mockShard.StopForTest()
 }
 
 func (s *mutableStateSuite) TestTransientWorkflowTaskCompletionFirstBatchReplicated_ReplicateWorkflowTaskCompleted() {
@@ -592,14 +593,13 @@ func (s *mutableStateSuite) prepareTransientWorkflowTaskCompletionFirstBatchRepl
 	return newWorkflowTaskScheduleEvent, newWorkflowTaskStartedEvent
 }
 
-func (s *mutableStateSuite) newNamespaceCacheEntry() *cache.NamespaceCacheEntry {
-	return cache.NewNamespaceCacheEntryForTest(
+func (s *mutableStateSuite) newNamespaceCacheEntry() *namespace.Namespace {
+	return namespace.NewNamespaceForTest(
 		&persistencespb.NamespaceInfo{Name: "mutableStateTest"},
 		&persistencespb.NamespaceConfig{},
 		true,
 		&persistencespb.NamespaceReplicationConfig{},
 		1,
-		nil,
 	)
 }
 
@@ -614,7 +614,7 @@ func (s *mutableStateSuite) buildWorkflowMutableState() *persistencespb.Workflow
 
 	startTime := timestamp.TimePtr(time.Date(2020, 8, 22, 1, 2, 3, 4, time.UTC))
 	info := &persistencespb.WorkflowExecutionInfo{
-		NamespaceId:                namespaceID,
+		NamespaceId:                namespaceID.String(),
 		WorkflowId:                 we.GetWorkflowId(),
 		TaskQueue:                  tl,
 		WorkflowTypeName:           "wType",
@@ -677,10 +677,9 @@ func (s *mutableStateSuite) buildWorkflowMutableState() *persistencespb.Workflow
 			Version:               failoverVersion,
 			InitiatedId:           80,
 			InitiatedEventBatchId: 20,
-			InitiatedEvent:        &historypb.HistoryEvent{},
 			StartedId:             common.EmptyEventID,
 			CreateRequestId:       uuid.New(),
-			Namespace:             tests.NamespaceID,
+			Namespace:             tests.Namespace.String(),
 			WorkflowTypeName:      "code.uber.internal/test/foobar",
 		},
 	}
@@ -693,6 +692,9 @@ func (s *mutableStateSuite) buildWorkflowMutableState() *persistencespb.Workflow
 			RequestId:             uuid.New(),
 			Name:                  "test-signal-75",
 			Input:                 payloads.EncodeString("signal-input-75"),
+			Header: &commonpb.Header{
+				Fields: map[string]*commonpb.Payload{"signal-header-key-75": payload.EncodeString("signal-header-value-75")},
+			},
 		},
 	}
 
