@@ -36,9 +36,9 @@ import (
 	historypb "go.temporal.io/api/history/v1"
 	"go.temporal.io/api/serviceerror"
 
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/cluster"
 	"go.temporal.io/server/common/log"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/persistence/versionhistory"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/service/history/shard"
@@ -49,7 +49,7 @@ type (
 
 	MutableStateRebuilder interface {
 		ApplyEvents(
-			namespaceID string,
+			namespaceID namespace.ID,
 			requestID string,
 			execution commonpb.WorkflowExecution,
 			history []*historypb.HistoryEvent,
@@ -58,10 +58,10 @@ type (
 	}
 
 	MutableStateRebuilderImpl struct {
-		shard           shard.Context
-		clusterMetadata cluster.Metadata
-		namespaceCache  cache.NamespaceCache
-		logger          log.Logger
+		shard             shard.Context
+		clusterMetadata   cluster.Metadata
+		namespaceRegistry namespace.Registry
+		logger            log.Logger
 
 		mutableState          MutableState
 		taskGeneratorProvider taskGeneratorProvider
@@ -84,8 +84,8 @@ func NewMutableStateRebuilder(
 
 	return &MutableStateRebuilderImpl{
 		shard:                 shard,
-		clusterMetadata:       shard.GetService().GetClusterMetadata(),
-		namespaceCache:        shard.GetNamespaceCache(),
+		clusterMetadata:       shard.GetClusterMetadata(),
+		namespaceRegistry:     shard.GetNamespaceRegistry(),
 		logger:                logger,
 		mutableState:          mutableState,
 		taskGeneratorProvider: taskGeneratorProvider,
@@ -93,7 +93,7 @@ func NewMutableStateRebuilder(
 }
 
 func (b *MutableStateRebuilderImpl) ApplyEvents(
-	namespaceID string,
+	namespaceID namespace.ID,
 	requestID string,
 	execution commonpb.WorkflowExecution,
 	history []*historypb.HistoryEvent,
@@ -129,20 +129,21 @@ func (b *MutableStateRebuilderImpl) ApplyEvents(
 			)); err != nil {
 				return nil, err
 			}
+			b.mutableState.GetExecutionInfo().LastEventTaskId = event.GetTaskId()
 		}
 
 		switch event.GetEventType() {
 		case enumspb.EVENT_TYPE_WORKFLOW_EXECUTION_STARTED:
 			attributes := event.GetWorkflowExecutionStartedEventAttributes()
-			var parentNamespaceID string
+			var parentNamespaceID namespace.ID
 			if attributes.GetParentWorkflowNamespace() != "" {
-				parentNamespaceEntry, err := b.namespaceCache.GetNamespace(
-					attributes.GetParentWorkflowNamespace(),
+				parentNamespaceEntry, err := b.namespaceRegistry.GetNamespace(
+					namespace.Name(attributes.GetParentWorkflowNamespace()),
 				)
 				if err != nil {
 					return nil, err
 				}
-				parentNamespaceID = parentNamespaceEntry.GetInfo().Id
+				parentNamespaceID = parentNamespaceEntry.ID()
 			}
 
 			if err := b.mutableState.ReplicateWorkflowExecutionStartedEvent(
@@ -622,7 +623,6 @@ func (b *MutableStateRebuilderImpl) ApplyEvents(
 
 			err := b.mutableState.ReplicateWorkflowExecutionContinuedAsNewEvent(
 				firstEvent.GetEventId(),
-				namespaceID,
 				event,
 			)
 			if err != nil {
