@@ -28,23 +28,28 @@ import (
 	"time"
 
 	persistencespb "go.temporal.io/server/api/persistence/v1"
-	"go.temporal.io/server/common/persistence"
+	"go.temporal.io/server/common/definition"
+	"go.temporal.io/server/service/history/tasks"
 )
 
 // NOTE: do not use make(type, len(input))
 // since this will assume initial length being len(inputs)
 // always use make(type, 0, len(input))
 func convertSyncActivityInfos(
+	now time.Time,
+	workflowKey definition.WorkflowKey,
 	activityInfos map[int64]*persistencespb.ActivityInfo,
 	inputs map[int64]struct{},
-) []persistence.Task {
-	outputs := make([]persistence.Task, 0, len(inputs))
+) []tasks.Task {
+	outputs := make([]tasks.Task, 0, len(inputs))
 	for item := range inputs {
 		activityInfo, ok := activityInfos[item]
 		if ok {
-			outputs = append(outputs, &persistence.SyncActivityTask{
-				Version:     activityInfo.Version,
-				ScheduledID: activityInfo.ScheduleId,
+			outputs = append(outputs, &tasks.SyncActivityTask{
+				WorkflowKey:         workflowKey,
+				Version:             activityInfo.Version,
+				ScheduledID:         activityInfo.ScheduleId,
+				VisibilityTimestamp: now,
 			})
 		}
 	}
@@ -54,9 +59,9 @@ func convertSyncActivityInfos(
 func setTaskInfo(
 	version int64,
 	timestamp time.Time,
-	transferTasks []persistence.Task,
-	timerTasks []persistence.Task,
-	visibilityTasks []persistence.Task,
+	transferTasks []tasks.Task,
+	timerTasks []tasks.Task,
+	visibilityTasks []tasks.Task,
 ) {
 	// set both the task version, as well as the Timestamp on the transfer tasks
 	for _, task := range transferTasks {
