@@ -33,9 +33,10 @@ import (
 	"go.temporal.io/api/serviceerror"
 
 	"go.temporal.io/server/common"
-	"go.temporal.io/server/common/cache"
 	"go.temporal.io/server/common/clock"
 	"go.temporal.io/server/common/log"
+	"go.temporal.io/server/common/namespace"
+	"go.temporal.io/server/common/persistence/visibility"
 	"go.temporal.io/server/common/primitives/timestamp"
 	"go.temporal.io/server/service/history/configs"
 	"go.temporal.io/server/service/history/events"
@@ -47,25 +48,25 @@ type (
 	}
 
 	TaskRefresherImpl struct {
-		config         *configs.Config
-		namespaceCache cache.NamespaceCache
-		eventsCache    events.Cache
-		logger         log.Logger
+		config            *configs.Config
+		namespaceRegistry namespace.Registry
+		eventsCache       events.Cache
+		logger            log.Logger
 	}
 )
 
 func NewTaskRefresher(
 	config *configs.Config,
-	namespaceCache cache.NamespaceCache,
+	namespaceRegistry namespace.Registry,
 	eventsCache events.Cache,
 	logger log.Logger,
 ) *TaskRefresherImpl {
 
 	return &TaskRefresherImpl{
-		config:         config,
-		namespaceCache: namespaceCache,
-		eventsCache:    eventsCache,
-		logger:         logger,
+		config:            config,
+		namespaceRegistry: namespaceRegistry,
+		eventsCache:       eventsCache,
+		logger:            logger,
 	}
 }
 
@@ -75,7 +76,7 @@ func (r *TaskRefresherImpl) RefreshTasks(
 ) error {
 
 	taskGenerator := NewTaskGenerator(
-		r.namespaceCache,
+		r.namespaceRegistry,
 		r.logger,
 		mutableState,
 	)
@@ -152,7 +153,7 @@ func (r *TaskRefresherImpl) RefreshTasks(
 		return err
 	}
 
-	if r.config.AdvancedVisibilityWritingMode() != common.AdvancedVisibilityWritingModeOff {
+	if r.config.AdvancedVisibilityWritingMode() != visibility.AdvancedVisibilityWritingModeOff {
 		if err := r.refreshTasksForWorkflowSearchAttr(
 			now,
 			mutableState,
@@ -300,7 +301,7 @@ Loop:
 
 		scheduleEvent, err := r.eventsCache.GetEvent(
 			events.EventKey{
-				NamespaceID: executionInfo.NamespaceId,
+				NamespaceID: namespace.ID(executionInfo.NamespaceId),
 				WorkflowID:  executionInfo.WorkflowId,
 				RunID:       executionState.RunId,
 				EventID:     activityInfo.ScheduleId,
@@ -384,7 +385,7 @@ Loop:
 
 		scheduleEvent, err := r.eventsCache.GetEvent(
 			events.EventKey{
-				NamespaceID: executionInfo.NamespaceId,
+				NamespaceID: namespace.ID(executionInfo.NamespaceId),
 				WorkflowID:  executionInfo.WorkflowId,
 				RunID:       executionState.RunId,
 				EventID:     childWorkflowInfo.InitiatedId,
@@ -426,7 +427,7 @@ func (r *TaskRefresherImpl) refreshTasksForRequestCancelExternalWorkflow(
 	for _, requestCancelInfo := range pendingRequestCancelInfos {
 		initiateEvent, err := r.eventsCache.GetEvent(
 			events.EventKey{
-				NamespaceID: executionInfo.NamespaceId,
+				NamespaceID: namespace.ID(executionInfo.NamespaceId),
 				WorkflowID:  executionInfo.WorkflowId,
 				RunID:       executionState.RunId,
 				EventID:     requestCancelInfo.GetInitiatedId(),
@@ -468,7 +469,7 @@ func (r *TaskRefresherImpl) refreshTasksForSignalExternalWorkflow(
 	for _, signalInfo := range pendingSignalInfos {
 		initiateEvent, err := r.eventsCache.GetEvent(
 			events.EventKey{
-				NamespaceID: executionInfo.NamespaceId,
+				NamespaceID: namespace.ID(executionInfo.NamespaceId),
 				WorkflowID:  executionInfo.WorkflowId,
 				RunID:       executionState.RunId,
 				EventID:     signalInfo.GetInitiatedId(),
