@@ -33,24 +33,25 @@ import (
 	"github.com/pborman/uuid"
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
+	commonpb "go.temporal.io/api/common/v1"
 	enumspb "go.temporal.io/api/enums/v1"
 	historypb "go.temporal.io/api/history/v1"
 	taskqueuepb "go.temporal.io/api/taskqueue/v1"
 
-	"go.temporal.io/server/common/primitives/timestamp"
-	"go.temporal.io/server/service/history/consts"
-	"go.temporal.io/server/service/history/tests"
-	"go.temporal.io/server/service/history/workflow"
-
 	persistencespb "go.temporal.io/server/api/persistence/v1"
 	"go.temporal.io/server/common"
 	"go.temporal.io/server/common/collection"
 	"go.temporal.io/server/common/definition"
 	"go.temporal.io/server/common/failure"
 	"go.temporal.io/server/common/log"
+	"go.temporal.io/server/common/namespace"
 	"go.temporal.io/server/common/payloads"
 	"go.temporal.io/server/common/persistence"
+	"go.temporal.io/server/common/primitives/timestamp"
+	"go.temporal.io/server/service/history/consts"
 	"go.temporal.io/server/service/history/shard"
+	"go.temporal.io/server/service/history/tests"
+	"go.temporal.io/server/service/history/workflow"
 )
 
 type (
@@ -66,7 +67,7 @@ type (
 		mockTransaction  *workflow.MockTransaction
 
 		logger       log.Logger
-		namespaceID  string
+		namespaceID  namespace.ID
 		workflowID   string
 		baseRunID    string
 		currentRunID string
@@ -126,6 +127,7 @@ func (s *workflowResetterSuite) SetupTest() {
 
 func (s *workflowResetterSuite) TearDownTest() {
 	s.controller.Finish()
+	s.mockShard.StopForTest()
 }
 
 func (s *workflowResetterSuite) TestPersistToDB_CurrentTerminated() {
@@ -144,7 +146,7 @@ func (s *workflowResetterSuite) TestPersistToDB_CurrentTerminated() {
 		ExecutionInfo: &persistencespb.WorkflowExecutionInfo{},
 	}
 	currentEventsSeq := []*persistence.WorkflowEvents{{
-		NamespaceID: s.namespaceID,
+		NamespaceID: s.namespaceID.String(),
 		WorkflowID:  s.workflowID,
 		RunID:       s.currentRunID,
 		BranchToken: []byte("some random current branch token"),
@@ -170,7 +172,7 @@ func (s *workflowResetterSuite) TestPersistToDB_CurrentTerminated() {
 		ExecutionInfo: &persistencespb.WorkflowExecutionInfo{},
 	}
 	resetEventsSeq := []*persistence.WorkflowEvents{{
-		NamespaceID: s.namespaceID,
+		NamespaceID: s.namespaceID.String(),
 		WorkflowID:  s.workflowID,
 		RunID:       s.resetRunID,
 		BranchToken: []byte("some random reset branch token"),
@@ -230,7 +232,7 @@ func (s *workflowResetterSuite) TestPersistToDB_CurrentNotTerminated() {
 		ExecutionInfo: &persistencespb.WorkflowExecutionInfo{},
 	}
 	resetEventsSeq := []*persistence.WorkflowEvents{{
-		NamespaceID: s.namespaceID,
+		NamespaceID: s.namespaceID.String(),
 		WorkflowID:  s.workflowID,
 		RunID:       s.resetRunID,
 		BranchToken: []byte("some random reset branch token"),
@@ -245,7 +247,7 @@ func (s *workflowResetterSuite) TestPersistToDB_CurrentNotTerminated() {
 	resetContext.EXPECT().GetHistorySize().Return(int64(123)).AnyTimes()
 	resetContext.EXPECT().CreateWorkflowExecution(
 		gomock.Any(),
-		persistence.CreateWorkflowModeContinueAsNew,
+		persistence.CreateWorkflowModeWorkflowIDReuse,
 		s.currentRunID,
 		currentLastWriteVersion,
 		resetMutableState,
@@ -276,23 +278,23 @@ func (s *workflowResetterSuite) TestReplayResetWorkflow() {
 	s.mockExecutionMgr.EXPECT().ForkHistoryBranch(&persistence.ForkHistoryBranchRequest{
 		ForkBranchToken: baseBranchToken,
 		ForkNodeID:      baseNodeID,
-		Info:            persistence.BuildHistoryGarbageCleanupInfo(s.namespaceID, s.workflowID, s.resetRunID),
+		Info:            persistence.BuildHistoryGarbageCleanupInfo(s.namespaceID.String(), s.workflowID, s.resetRunID),
 		ShardID:         shardID,
 	}).Return(&persistence.ForkHistoryBranchResponse{NewBranchToken: resetBranchToken}, nil)
 
 	s.mockStateRebuilder.EXPECT().rebuild(
 		ctx,
 		gomock.Any(),
-		definition.NewWorkflowIdentifier(
-			s.namespaceID,
+		definition.NewWorkflowKey(
+			s.namespaceID.String(),
 			s.workflowID,
 			s.baseRunID,
 		),
 		baseBranchToken,
 		baseRebuildLastEventID,
 		baseRebuildLastEventVersion,
-		definition.NewWorkflowIdentifier(
-			s.namespaceID,
+		definition.NewWorkflowKey(
+			s.namespaceID.String(),
 			s.workflowID,
 			s.resetRunID,
 		),
@@ -485,7 +487,7 @@ func (s *workflowResetterSuite) TestGenerateBranchToken() {
 	s.mockExecutionMgr.EXPECT().ForkHistoryBranch(&persistence.ForkHistoryBranchRequest{
 		ForkBranchToken: baseBranchToken,
 		ForkNodeID:      baseNodeID,
-		Info:            persistence.BuildHistoryGarbageCleanupInfo(s.namespaceID, s.workflowID, s.resetRunID),
+		Info:            persistence.BuildHistoryGarbageCleanupInfo(s.namespaceID.String(), s.workflowID, s.resetRunID),
 		ShardID:         shardID,
 	}).Return(&persistence.ForkHistoryBranchResponse{NewBranchToken: resetBranchToken}, nil)
 
@@ -683,8 +685,8 @@ func (s *workflowResetterSuite) TestReapplyContinueAsNewWorkflowEvents_WithConti
 	resetContext.EXPECT().LoadWorkflowExecution().Return(resetMutableState, nil)
 	resetMutableState.EXPECT().GetNextEventID().Return(newNextEventID).AnyTimes()
 	resetMutableState.EXPECT().GetCurrentBranchToken().Return(newBranchToken, nil).AnyTimes()
-	resetContextCacheKey := definition.NewWorkflowIdentifier(s.namespaceID, s.workflowID, newRunID)
-	_, _ = s.workflowResetter.historyCache.PutIfNotExist(resetContextCacheKey, resetContext)
+	resetContextCacheKey := definition.NewWorkflowKey(s.namespaceID.String(), s.workflowID, newRunID)
+	_, _ = s.workflowResetter.historyCache.(*workflow.CacheImpl).PutIfNotExist(resetContextCacheKey, resetContext)
 
 	mutableState := workflow.NewMockMutableState(s.controller)
 
@@ -770,6 +772,7 @@ func (s *workflowResetterSuite) TestReapplyEvents() {
 			SignalName: "some random signal name",
 			Input:      payloads.EncodeString("some random signal input"),
 			Identity:   "some random signal identity",
+			Header:     &commonpb.Header{Fields: map[string]*commonpb.Payload{"myheader": {Data: []byte("myheader")}}},
 		}},
 	}
 	event2 := &historypb.HistoryEvent{
@@ -797,6 +800,7 @@ func (s *workflowResetterSuite) TestReapplyEvents() {
 				attr.GetSignalName(),
 				attr.GetInput(),
 				attr.GetIdentity(),
+				attr.GetHeader(),
 			).Return(&historypb.HistoryEvent{}, nil)
 		}
 	}
