@@ -87,7 +87,7 @@ func NewScavenger(
 			metricsClient,
 			metrics.ExecutionsScavengerScope,
 		),
-		rateLimiter: quotas.NewDefaultOutgoingDynamicRateLimiter(
+		rateLimiter: quotas.NewDefaultOutgoingRateLimiter(
 			func() float64 { return float64(rateOverall) },
 		),
 		metrics: metricsClient,
@@ -151,7 +151,7 @@ func (s *Scavenger) run() {
 			s.logger,
 			s,
 			quotas.NewMultiRateLimiter([]quotas.RateLimiter{
-				quotas.NewDefaultOutgoingDynamicRateLimiter(
+				quotas.NewDefaultOutgoingRateLimiter(
 					func() float64 { return float64(ratePerShard) },
 				),
 				s.rateLimiter,
@@ -166,6 +166,9 @@ func (s *Scavenger) run() {
 }
 
 func (s *Scavenger) awaitExecutor() {
+	// gauge value persists, so we want to reset it to 0
+	defer s.metrics.UpdateGauge(metrics.ExecutionsScavengerScope, metrics.ExecutionsOutstandingCount, float64(0))
+
 	outstanding := s.executor.TaskCount()
 	for outstanding > 0 {
 		select {
